﻿using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationCGTS;
using Ato.EN.IntegrationServices.CodeGenerationFITR;
using DataContracts;

namespace Ato.CD.Inbound.FITR201802
{
    internal class CrossFormValidatorCGTS : ICrossFormValidator
    {
        private FITR2018 ParentDocument { get; }

        private CGTS2018 ChildDocument { get; }

        private List<Context> Contexts { get; }

        internal CrossFormValidatorCGTS(FITR2018 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (CGTS2018)childDocument.ConsumedReport;
            Contexts = ChildDocument.GetContexts();
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN402009(response);
            VRATOGEN438022(response);           
            VRATOGEN438024(response);
            VRATOGEN438025(response);           
            VRATOGEN438027(response);
            VRATOCGTS402165(response);
            VRATOCGTS402171(response);
            VRATOCGTS402178(response);
            VRATOCGTS402226(response);
            VRATOCGTS402230(response);
            VRATOCGTS402240(response);
            
            VRATOGEN438000(response);
            VRATOGEN438001(response);
            VRATOFITR434088(response);
            VRATOFITR434036(response);

            return response;
        }


        #region VR.ATO.FITR.434036
        private void VRATOFITR434036(List<ProcessMessageDocument> response)
        {
            /*
            IF (COUNT(SCHEDULE = ""CGTS"") = 1) AND [FITR23] <> [CGTS131]
            RETURN VALIDATION MESSAGE
            ENDIF
            [CGTS131] = CGTS:RP: bafpr1.02.00:Income.CapitalGainsNet.Amount
            [FITR23] = FITR:RP: bafpr1.02.00:Income.CapitalGainsNet.Amount
            */

            bool assertion = ParentDocument.FITR23.HasValue && ParentDocument.FITR23.Value != ChildDocument.CGTS131.GetValueOrDefault();
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434036",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gains in return must equal total net capital gain in CGT schedule.",
                    Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434036" } }
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "FITR23",
                    Value = ParentDocument.FITR23.Value.ToString()
                });

                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.FITR.434088

        /*  VR.ATO.FITR.434088
                    Net capital losses carried forward to later income years must equal amount in CGT schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "CGTS") = 1) AND  [FITR207] <> ([CGTS124]+[CGTS125])
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS124] = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [CGTS125] = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [FITR207] = FITR:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('CGTS') = 1) AND  ^FITR207 <> (^CGTS124+^CGTS125)

        Data Elements:
            
                    RP.Closing(Instant):^FITR207 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Closing(Instant):^CGTS124 = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Closing(Instant):^CGTS125 = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        protected void VRATOFITR434088(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;
            
            assertion = ParentDocument.FITR207 != ChildDocument.CGTS124.GetValueOrDefault() + ChildDocument.CGTS125.GetValueOrDefault();

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FITR.434289",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in CGT schedule",
                    LongDescription = @"Net capital losses carried forward to later income years' on the main form must equal the total of the 'Unapplied net capital losses carried forward'  in the Capital Gains Tax schedule",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434088" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR207", Value = report.FITR207.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434088

        #region VR.ATO.GEN.402009
        /* VR.ATO.GEN.402009
            IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                RETURN VALIDATION MESSAGE
            ENDIF 
        */
        private void VRATOGEN402009(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009
        #region VR.ATO.GEN.438022
        /*  VR.ATO.GEN.438022
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed

                Legacy Rule Format:
                   IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "A - Small business active asset reduction") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS127] = NULL
                        RETURN VALIDATION MESSAGE
                    ENDIF

                [CGTS127] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount
        */
        private void VRATOGEN438022(List<ProcessMessageDocument> response)
        {
            bool assertion = ParentDocument.FITR204Collection != null &&
                             ParentDocument.FITR204Collection.Any(
                                 s => s.Equals("A - Small business active asset reduction")) &&
                             !ChildDocument.CGTS127.HasValue;
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438022",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business active asset reduction amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438022" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438022

        #region VR.ATO.GEN.438024
        /*  VR.ATO.GEN.438024
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed

                Legacy Rule Format:
                   IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "C - Small business roll-over") AND 
                            (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS129] = NULL
                        RETURN VALIDATION MESSAGE
                    ENDIF

                [CGTS129] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount
        */
        private void VRATOGEN438024(List<ProcessMessageDocument> response)
        {
            const string SmallBusinessExceptionCode = "C - Small business roll-over";
            bool assertion = ParentDocument.FITR204Collection != null &&
                             ParentDocument.FITR204Collection.Any(s => s.Equals(SmallBusinessExceptionCode)) &&
                             !ChildDocument.CGTS129.HasValue;
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438024",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438024" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438024
        #region VR.ATO.GEN.438025
        /*  VR.ATO.GEN.438025
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "D - Small business 15 year exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS138] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS138] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount

            Technical Business Rule Format:
                    (^SMSFAR328 = 'D - Small business 15 year exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS138 = NULL

        Data Elements:
            
                    RP:^SMSFAR328 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS138 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount
                    */
        private void VRATOGEN438025(List<ProcessMessageDocument> response)
        {

            const string SmallBusinessExceptionCode = "D - Small business 15 year exemption";

            bool assertion = ParentDocument.FITR204Collection != null &&
                             ParentDocument.FITR204Collection.Any(s => s.Equals(SmallBusinessExceptionCode)) &&
                             !ChildDocument.CGTS138.HasValue;
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business 15 year exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438025" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438025
       
        #region VR.ATO.GEN.438027
        /*  VR.ATO.GEN.438027
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "F - Scrip for scrip roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS45] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS45] = CGTS:RP:tns:Capital.Gains.ScriptForScriptRolloverDeferral.Amount

            Technical Business Rule Format:
                    (^SMSFAR328 = 'F - Scrip for scrip roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS45 = NULL

        Data Elements:
            
                    RP:^SMSFAR328 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS45 = CGTS:RP:tns:Capital.Gains.ScriptForScriptRolloverDeferral.Amount
                    */
        private void VRATOGEN438027(List<ProcessMessageDocument> response)
        {

            const string SmallBusinessExceptionCode = "F - Scrip for scrip roll-over";
            bool assertion = ParentDocument.FITR204Collection!= null &&
                             ParentDocument.FITR204Collection.Any(s => s.Equals(SmallBusinessExceptionCode)) &&
                             !ChildDocument.CGTS45.HasValue;
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438027",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438027" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438027
        #region VR.ATO.CGTS.402165
        /* VR.ATO.CGTS.402165

             IF (PARENT RETURN <> "CTR") AND ([CGTS122] <> NULL)  
               RETURN VALIDATION MESSAGE
             ENDIF

            [CGTS122] = CGTS:RP.Applied:tns:Capital.Losses.TransferredTotal.Amount
    
        */
        private void VRATOCGTS402165(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.CGTS122.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402165",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total losses transferred in applied is only valid for a company",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.TransferredTotal.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402165" } }
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS122",
                    Value = ChildDocument.CGTS122.Value.ToString()
                });

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402165
        #region VR.ATO.CGTS.402171
        /* VR.ATO.CGTS.402171

            IF PARENT RETURN = "SMSFAR","FITR","TRT" AND([CGTS131] > 0) AND([CGTS125] > 0)
                RETURN VALIDATION MESSAGE
            ENDIF

            [CGTS125] = CGTS:RP.CGTOther.Unapplied:rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
            [CGTS131] = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount        
    
        */
        private void VRATOCGTS402171(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.CGTS131.GetValueOrDefault() > 0 && ChildDocument.CGTS125.GetValueOrDefault() > 0;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402171",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"A capital loss must not be carried forward when a positive net capital gain amount has been provided",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402171" } }
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS131",
                    Value = ChildDocument.CGTS131.GetValueOrDefault().ToString()
                });

                processMessage.Parameters.Add(new ProcessMessageParameter
                {
                    Name = "CGTS125",
                    Value = ChildDocument.CGTS125.GetValueOrDefault().ToString()
                });

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402171
        #region VR.ATO.CGTS.402178
        /* VR.ATO.CGTS.402178

        IF PARENT RETURN = "SMSFAR","FITR" AND
            ([CGTS128] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            [CGTS128] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount   
        */
        private void VRATOCGTS402178(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.CGTS128.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402178",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business retirement exemption is not valid for Self Managed Super Funds",
                    Location = "/xbrli:xbrl/tns:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402178" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402178
        #region VR.ATO.CGTS.402226
        /*  VR.ATO.CGTS.402226
          
            IF (PARENT RETURN:RP:pyin.xx.xx:Report.Amendment.Indicator = TRUE) AND ([CGTS150] <> NULL AND [CGTS151] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            [CGTS150] = CGTS:RP:pyin.02.37:Report.TargetFinancial.Year
            [CGTS151] = CGTS:RP:bafot.02.26:Miscellaneous.RequestedAmendment.Amount
           
        */
        private void VRATOCGTS402226(List<ProcessMessageDocument> response)
        {
            bool assertion = ParentDocument.FITR171.GetValueOrDefault() && ChildDocument.CGTS150.HasValue && ChildDocument.CGTS151.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402226",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year earnout right created and Amount to be amended cannot be supplied when Parent return Amendment indicator is present.",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402226" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402226       
        #region VR.ATO.CGTS.402230
        /*  VR.ATO.CGTS.402230
          
          IF PARENT RETURN  <> "TRTAMI" AND ([CGTS153] <> NULL OR [CGTS154] <> NULL OR [CGTS155] <> NULL)
            RETURN VALIDATION MESSAGE
          ENDIF
           
        */
        private void VRATOCGTS402230(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrEmpty(ChildDocument.CGTS153) || ChildDocument.CGTS154.HasValue || ChildDocument.CGTS155.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402230",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Multi-class treatment fields incorrectly supplied",
                    Location = "/xbrli:xbrl/tns:IncomeTax.AttributionManagedInvestmentTrustClass.Description",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402230" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402230
        #region VR.ATO.CGTS.402240
        /* VR.ATO.CGTS.402240

            IF([CGTS150] <> NULL) AND([CGTS150] = (PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year))
               RETURN VALIDATION MESSAGE
            ENDIF

            [CGTS150] = CGTS:RP:pyin.02.37:Report.TargetFinancial.Year         
        */
        private void VRATOCGTS402240(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.CGTS150.HasValue && ChildDocument.CGTS150 == ParentDocument.FITR1.GetValueOrDefault();

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402240",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year earnout right created incorrect",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402240" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402240
        #region VR.ATO.GEN.438000
        /*  VR.ATO.GEN.438000
            IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOGEN438000(List<ProcessMessageDocument> response)
        {
            response.AddRange(from context in Contexts
                              where context.StartDate.GetValueOrDefault() != ParentDocument.RPStartDate.GetValueOrDefault()
                select new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438000",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } }
                });
        }
        #endregion // VR.ATO.CGTS.438000
        #region VR.ATO.GEN.438001
        /*  VR.ATO.GEN.438001
            IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOGEN438001(List<ProcessMessageDocument> response)
        {
            response.AddRange(from context in Contexts
                              where context.EndDate.GetValueOrDefault() != ParentDocument.RPEndDate.GetValueOrDefault()
                select new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } }
                });
        }
        #endregion // VR.ATO.CGTS.438001
    }
}