﻿using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationLS;
using Ato.EN.IntegrationServices.CodeGenerationFITR;
using DataContracts;

namespace Ato.CD.Inbound.FITR201802
{
    internal class CrossFormValidatorLS : ICrossFormValidator
    {
        private FITR2018 ParentDocument { get; }

        private LS2017 ChildDocument { get; }

        private List<Context> Contexts { get; }

        internal CrossFormValidatorLS(FITR2018 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (LS2017)childDocument.ConsumedReport;
            Contexts = GetContexts();
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN402009(response);
            VRATOGEN402010(response);
            VRATOLS414017(response);
            VRATOLS414022(response);
            VRATOGEN428045(response);
            VRATOFITR434086(response);
            VRATOFITR434087(response);

            return response;
        }

        #region VR.ATO.LS.414017
        /* VR.ATO.LS.414022
            IF (PARENT RETURN <> SET("CTR", "TRT")) AND (([LS15] <>NULL) OR ([LS16] <> NULL) OR ([LS17] <> NULL) OR ([LS18] <> NULL) OR ([LS19] <> NULL) OR ([LS53] <> NULL))
                RETURN VALIDATION MESSAGE
            ENDIF
            [LS15] = LS:RP.Y0:rvctc3.xx.xx:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            [LS16] = LS:RP.Y0-1:rvctc3.xx.xx:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            [LS17] = LS:RP.Y0-2:rvctc3.xx.xx:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            [LS18] = LS:RP.Y0-3:rvctc3.xx.xx:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            [LS19] = LS:RP.Y0-4:rvctc3.xx.xx:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            [LS53] = LS:RP.Y0-5+:rvctc3.xx.xx:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
        */
        private void VRATOLS414017(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.LS15.HasValue || ChildDocument.LS16.HasValue ||
                             ChildDocument.LS17.HasValue || ChildDocument.LS18.HasValue ||
                             ChildDocument.LS19.HasValue || ChildDocument.LS53.HasValue;
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.LS.414017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Continuity of majority ownership test only required for companies or widely held trusts",
                    Location = "/xbrli:xbrl/ls:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.LS.414017" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.LS.414017
        #region VR.ATO.LS.414022
        /* VR.ATO.LS.414022
            IF (PARENT RETURN<>"CTR") AND ([LS25] <> NULL OR [LS26] <> NULL OR [LS27] <> NULL OR [LS28] <> NULL)
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [LS25] = LS:RP:rvctc3.xx.xx:Tax.Losses.ChangeOfOwnershipOrControlAfterReferenceTime.Indicator
            [LS26] = LS:RP:rvctc3.xx.xx:Tax.Losses.MaximumNetAssetValueTestPassedAtChangeoverTime.Indicator
            [LS27] = LS:RP:rvctc3.xx.xx:Tax.Losses.UnrealisedNetLossAtChangeoverTime.Indicator
            [LS28] = LS:RP:rvctc3.xx.xx:Tax.Losses.UnrealisedNetLossAtChangeoverTime.Amount
        */
        private void VRATOLS414022(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.LS25.HasValue || ChildDocument.LS26.HasValue ||
                             ChildDocument.LS27.HasValue || ChildDocument.LS28.HasValue;
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.LS.414022",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unrealised losses are not required",
                    Location = "/xbrli:xbrl/ls:Tax.Losses.ChangeOfOwnershipOrControlAfterReferenceTime.Indicator",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.LS.414022" } },
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.LS.414022

        #region VR.ATO.GEN.402009
        /* VR.ATO.GEN.402009
            IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                RETURN VALIDATION MESSAGE
            ENDIF 
        */
        private void VRATOGEN402009(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009

        #region VR.ATO.FITR.434086

        /*  VR.ATO.FITR.434086
                    Tax losses carried forward to later income years in schedule must equal amount in main form

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND [LS50] <> [FITR76]  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FITR76] = FITR:RP.Closing:rvctc3.02.02:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
                    [LS50] = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND ^LS50 <> ^FITR76

        Data Elements:
            
                    RP.Closing:^FITR76 = tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
            
                    RP.Closing:^LS50 = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    */
        private void VRATOFITR434086(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.FITR.434086

            assertion = ParentDocument.FITR76.GetValueOrDefault() != ChildDocument.LS50.GetValueOrDefault();
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax losses carried forward to later income years in schedule must equal amount in main form",
                    LongDescription = @"The total of the 'Tax losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Tax losses carried forward to later income years' on the main form",
                    Location = "/xbrli:xbrl/tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434086" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434086

        #region VR.ATO.FITR.434087

        /*  VR.ATO.FITR.434087
                    Net capital losses carried forward to later income years must equal amount in Losses schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND [LS52] <> [FITR207]  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FITR207] = FITR:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [LS52] = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND ^LS52 <> ^FITR207

            Data Elements:
            
                    RP.Closing(Instant):^FITR207 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Closing(Instant):^LS52 = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        private void VRATOFITR434087(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ParentDocument.FITR207 != ChildDocument.LS52.GetValueOrDefault();
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434087",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in Losses schedule",
                    LongDescription = @"The total of the 'Net capital losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434087" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434087

        #region VR.ATO.GEN.402010
        //IF(RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier<> NULLORBLANK) AND
        //(PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier<> NULLORBLANK)
        // AND(RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier<> PARENT 
        // RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier)
        //RETURN VALIDATION MESSAGE
        //    ENDIF

        private void VRATOGEN402010(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;
            
            assertion = (!string.IsNullOrWhiteSpace(ParentDocument.FITR7) &&
                         !string.IsNullOrWhiteSpace(ChildDocument.LS3)) &&
                        !(ParentDocument.FITR7.Equals(ChildDocument.LS3));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.402010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied ABN does not match the ABN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402010" } },
                };
                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.GEN.428045
        //IF RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text
        //    IN TUPLE(xbrli\organisationname1.xx.xx:OrganisationNameDetails) <> NULL
        //    AND RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text
        //    IN TUPLE(xbrli\organisationname1.xx.xx:OrganisationNameDetails) <> 
        //(PARENT RETURN:RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text
        //    IN TUPLE(xbrli\organisationname2.xx.xx:OrganisationNameDetails)
        //WHERE(TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = "MN")
        //AND(TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.Currency.Code = "C")) 
        //RETURN VALIDATION MESSAGE
        //    ENDIF


        private void VRATOGEN428045(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion =(!string.IsNullOrWhiteSpace(ParentDocument.FITR6) &&
                        !string.IsNullOrWhiteSpace(ChildDocument.LS2)) &&
                        (!ParentDocument.FITR6.Equals(ChildDocument.LS2));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428045",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation Name in schedule should match Organisation Name in parent return",
                    //LongDescription = @"Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428045" } },
                };
                response.Add(processMessage);
            }
        }


        #endregion

        private List<Context> GetContexts() => new List<Context>()
            .AddContext(ChildDocument.RPStartDate, ChildDocument.RPEndDate, ChildDocument.RPOccurrenceIndex, ChildDocument.RPId)
            .AddContext(ChildDocument.RPY0StartDate, ChildDocument.RPY0EndDate, ChildDocument.RPY0OccurrenceIndex, ChildDocument.RPY0Id)
            .AddContext(ChildDocument.RPY0Minus1StartDate, ChildDocument.RPY0Minus1EndDate, ChildDocument.RPY0Minus1OccurrenceIndex, ChildDocument.RPY0Minus1Id)
            .AddContext(ChildDocument.RPY0Minus2StartDate, ChildDocument.RPY0Minus2EndDate, ChildDocument.RPY0Minus2OccurrenceIndex, ChildDocument.RPY0Minus2Id)
            .AddContext(ChildDocument.RPY0Minus3StartDate, ChildDocument.RPY0Minus3EndDate, ChildDocument.RPY0Minus3OccurrenceIndex, ChildDocument.RPY0Minus3Id)
            .AddContext(ChildDocument.RPY0Minus4StartDate, ChildDocument.RPY0Minus4EndDate, ChildDocument.RPY0Minus4OccurrenceIndex, ChildDocument.RPY0Minus4Id)
            .AddContext(ChildDocument.RPY0Minus5PStartDate, ChildDocument.RPY0Minus5PEndDate, ChildDocument.RPY0Minus5POccurrenceIndex, ChildDocument.RPY0Minus5PId)
            .AddContext(ChildDocument.RPALLStartDate, ChildDocument.RPALLEndDate, ChildDocument.RPALLOccurrenceIndex, ChildDocument.RPALLId);
    }
}
