﻿using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationFITR;
using Ato.EN.IntegrationServices.CodeGenerationPSS;
using DataContracts;

namespace Ato.CD.Inbound.FITR201802
{
    internal class CrossFormValidatorPSS : ICrossFormValidator
    {
        private FITR2018 ParentDocument { get; }

        private PSS2018 ChildDocument { get; }

        internal CrossFormValidatorPSS(FITR2018 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument =  (PSS2018)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOPSS000035(response);
            VRATOPSS000036(response);
            VRATOFITR434045(response);
            VRATOFITR434046(response);
            VRATOFITR434056(response);
            VRATOFITR434080(response);

            return response;
        }

        #region VRATOFITR434056
        /*  VR.ATO.FITR.434056
                 Other income must not be less than Gross payment amounts on Payment Summary schedule

             Legacy Rule Format:
                 IF (COUNT(SCHEDULE = "PSS") = 1) AND SUM([PSS18]) > [FITR47]   
                     RETURN VALIDATION MESSAGE
                 ENDIF

                 [FITR47] = FITR:RP:bafpr1.02.04:Income.Other.Amount
                 [PSS18] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount

         Technical Business Rule Format:
                 (CountDocument('PSS') = 1) AND Sum(^PSS18) > ^FITR47

     Data Elements:

                 RP:^FITR47 = tns:Income.Other.Amount

                 RP:^PSS18 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                 */
        private void VRATOFITR434056(List<ProcessMessageDocument> response)
        {
            bool assertion =  ChildDocument.RPPayerPSSeqNumContextCollection != null && 
                              ChildDocument.RPPayerPSSeqNumContextCollection.Sum(x => x.PSS18).GetValueOrDefault() > 
                              ParentDocument.FITR47.GetValueOrDefault();
            
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434056",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other income must not be less than Gross payment amounts on Payment Summary schedule",
                    LongDescription = @"The 'Other income' amount must not be less than the sum of all 'Gross payment' amounts on the Payment Summary schedule",
                    Location = "/xbrli:xbrl/tns:Income.Other.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier",
                        Value = "VR.ATO.FITR.434056" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR47", Value = report.FITR47.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion  

        #region VR.ATO.FITR.434080

        /*  VR.ATO.FITR.434080
                Credit: FRW (excluding capital gains) must not be less than the total of all FRW amounts in Payment Summary schedule

            Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND SUM([PSS22]) > ([FITR71] + 1)  
                    RETURN VALIDATION MESSAGE
                ENDIF

                [FITR71] = FITR:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
                [PSS22] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount

        Technical Business Rule Format:
                (CountDocument('PSS') = 1) AND Sum(^PSS22) > (^FITR71 + 1)

                Data Elements:

                RP:^FITR71 = tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

                RP:^PSS22 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
                */
        private void VRATOFITR434080(List<ProcessMessageDocument> response)
        {
            bool assertion = false;
            assertion = ChildDocument.RPPayerPSSeqNumContextCollection != null && 
                        ChildDocument.RPPayerPSSeqNumContextCollection.Sum(x => x.PSS22).GetValueOrDefault() > (ParentDocument.FITR71 + 1);
                
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438069",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Credit: FRW (excluding capital gains) must not be less than the total of all FRW amounts in Payment Summary schedule",
                    LongDescription = "The 'Credit: foreign resident withholding (excluding capital gains)' amount on the main form must not be less than the sum of all 'Foreign resident withholding' amounts on the Payment Summary schedule",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.FITR.434080"}
                    }
                });
            }
            

        }
        #endregion      

        #region VR.ATO.PSS.000035
        /*  VR.ATO.PSS.000035
            IF PARENT RETURN <> “IITR”
               AND ([PSS20] <> NULLORBLANK OR [PSS21] <> NULLORBLANK OR [PSS31] <> NULLORBLANK OR [PSS24] <> NULLORBLANK OR [PSS25] <> NULLORBLANK OR [PSS32] <> NULLORBLANK)
                 RETURN VALIDATION MESSAGE
              ENDIF

              [PSS20] = PSS:RP.Payer.{PSSeqNum}:tns:Remuneration.VoluntaryAgreementGross.Amount
              [PSS21] = PSS:RP.Payer.{PSSeqNum}:tns:Remuneration.LabourHireArrangementPaymentGross.Amount
              [PSS24] = PSS:RP.Payer.{PSSeqNum}:tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
              [PSS25] = PSS:RP.Payer.{PSSeqNum}:tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
              [PSS31] = PSS:RP.Payer.{PSSeqNum}:tns:Remuneration.PersonalServicesIncome.AttributedGross.Amount
              [PSS32] = PSS:RP.Payer.{PSSeqNum}:tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncome.Amount   
       */
        private void VRATOPSS000035(List<ProcessMessageDocument> response)
        {
            if (ChildDocument.RPPayerPSSeqNumContextCollection != null)
                response.AddRange(ChildDocument.RPPayerPSSeqNumContextCollection
                    .Where(p => p.PSS20.HasValue || p.PSS21.HasValue || p.PSS24.HasValue || p.PSS25.HasValue ||
                                p.PSS31.HasValue || p.PSS32.HasValue).Select(p =>
                        new ProcessMessageDocument
                        {
                            Code = "CMN.ATO.PSS.000035",
                            Severity = ProcessMessageSeverity.Error,
                            Description = "Payment Type not applicable to Non-individual income tax return",
                            Location = $"/xbrli:xbrl/tns:Remuneration.PaymentToForeignResidentGross.Amount[contextRef=\"{p.Id}\"]",
                            Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000035" } }
                        }
                    ));


        }
        #endregion // VR.ATO.PSS.000035
        #region VR.ATO.PSS.000036
        /*  VR.ATO.PSS.000036
            WHERE IN CONTEXT (RP.Payer.{PSSeqNum)
            IF (RP.Payer.{PSSeqNum}.entity.identifier.TFN <> PARENT RETURN:RP.entity.identifier.TFN)
               RETURN VALIDATION MESSAGE
            ENDIF
       */
        private void VRATOPSS000036(List<ProcessMessageDocument> response)
        {
            if (ChildDocument.RPPayerPSSeqNumContextCollection != null)
                response.AddRange(ChildDocument.RPPayerPSSeqNumContextCollection
                    .Where(p => p.IdentifierTFN != ParentDocument.RPIdentifierTFN)
                    .Select((p, i) => new { RPPayer = p, Index = i }).Select(p => new ProcessMessageDocument
                    {
                        Code = "CMN.ATO.PSS.000036",
                        Severity = ProcessMessageSeverity.Error,
                        Description = "Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                        Location = p.Index == 0 ? "xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier" : $"xbrli:xbrl/xbrli:context[{p.Index + 1}]/xbrli:entity/xbrli:identifier",
                        Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000036" } }
                    }));

        }
        #endregion // VR.ATO.PSS.000036

        #region VR.ATO.FITR.434045
        //IF(COUNT(SCHEDULE = "PSS") = 1) AND[FITR32] > 0 AND(([FITR72] + 1) < SUM([PSS23]))  
        //RETURN VALIDATION MESSAGE
        //    ENDIF

        //        [FITR32] = FITR:RP:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
        //    [FITR72] = FITR:RP:rvctc2.02.02:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldTFNNotQuotedAndABNNotQuoted.Amount
        //    [PSS23] = PSS:RP.Payer.{PSSeqNum
        //:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
        private void VRATOFITR434045(List<ProcessMessageDocument> response)
        {
          
            bool assertion = ChildDocument.RPPayerPSSeqNumContextCollection != null && ParentDocument.FITR32 > 0 && 
                            (ParentDocument.FITR72 + 1) < ChildDocument.RPPayerPSSeqNumContextCollection.Sum(x => x.PSS23).GetValueOrDefault();
           
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434045",
                    Severity = ProcessMessageSeverity.Error,
                    Description =
                        "Tax withheld amount must not be less than the sum of amounts present in the schedule",
                    LongDescription =
                        "The 'Credit for tax withheld where ABN/TFN not quoted' amount must equal the sum of the Tax withheld amounts on the Payment Summary schedule",
                    Location =
                        "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldTFNNotQuotedAndABNNotQuoted.Amount",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.FITR.434045"}
                    }
                });
            }
            
        }
        #endregion

        #region VR.ATO.FITR.434046
        /*  VR.ATO.FITR.434046
                    Gross payment amount must equal the sum of amounts in the Payment Summary schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") = 1) AND SUM([PSS19]) <> [FITR32]  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FITR32] = FITR:RP:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [PSS19] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') = 1) AND Sum(^PSS19) <> ^FITR32

        Data Elements:
            
                    RP:^FITR32 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP:^PSS19 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        private void VRATOFITR434046(List<ProcessMessageDocument> response)
        {
            bool  assertion = ChildDocument.RPPayerPSSeqNumContextCollection!= null && 
                              ChildDocument.RPPayerPSSeqNumContextCollection.Sum(x => x.PSS19).GetValueOrDefault() != 
                              ParentDocument.FITR32.GetValueOrDefault();
           
                if (assertion)
                {
                    response.Add(new ProcessMessageDocument
                    {
                        Code = "CMN.ATO.GEN.434046",
                        Severity = ProcessMessageSeverity.Error,
                        Description =
                            "Gross payment amount must equal the sum of amounts in the Payment Summary schedule",
                        //LongDescription = "The 'Credit for tax withheld where ABN/TFN not quoted' amount must equal the sum of the Tax withheld amounts on the Payment Summary schedule",
                        Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount",
                        Parameters = new ProcessMessageParameters
                        {
                            new ProcessMessageParameter {Name = "RuleIdentifier", Value = "VR.ATO.FITR.434046"}
                        }
                    });
                }
            }
        #endregion
    }
}