using System.Collections.Generic;
using System.Linq;
using DataContracts;

namespace Ato.EN.IntegrationServices.CodeGenerationIEE
{
    public partial class IEE2018Validator
    {
        #region Manual Rules - Rules marked for manual coding
        #region VR.ATO.IEE.410189

        /*  VR.ATO.IEE.410189
                    More than four periods must not be specified for a trustee or partner

                Legacy Rule Format:
                    IF COUNT([IEE43]) > 4
                        RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    Count(^IEE43) > 4

        Data Elements:

                    RP:^IEE43 = tns:Period.Start.Date WHERE [Period.Type.Code = 'NonResidentPartPeriod']
                    */
        protected void VRATOIEE410189(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            bool assertion = report.TrusteeOrPartnerCollection != null && report.TrusteeOrPartnerCollection.Exists(c =>
                                 c.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection != null &&
                                 c.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Count > 4);
            if (assertion)
            {
                int trusteeOrPartnerOccurrence = 1 + report.TrusteeOrPartnerCollection.FindIndex(c =>
                                                     c.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection
                                                         .Count > 4);
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410189",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "More than four periods must not be specified for a trustee or partner",
                    LongDescription = "More than four periods must not be specified for trustee or partner being a non-resident for tax purposes",
                    Location = $"/xbrli:xbrl/tns:TrusteeOrPartner{OccurrenceIndex(trusteeOrPartnerOccurrence)}/tns:PeriodDetails[5]/tns:Period.Start.Date",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410189" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410189

        #region VR.ATO.IEE.410269

        /*  VR.ATO.IEE.410269
                    Trustee/partner Declaration Family name and Given name or non individual name must match at least one Trustee or partner Family name and Given name or non individual name.

                Legacy Rule Format:
                    WHERE IN TUPLE(iee.0002.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner)
                    IF ([IEE203] <> NULL AND ([IEE203] <> ANY OCCURRENCE OF([IEE28]) OR [IEE204] <> (ANY OCCURRENCE OF([IEE30]) WHERE [IEE28] = [IEE203])))
                     OR ([IEE88] <> NULL AND [IEE88] <> ANY OCCURRENCE OF([IEE32]))
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE28] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE30] = IEE:RP:pyde.02.00:PersonNameDetails.GivenName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE32] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE88] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
                    [IEE203] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
                    [IEE204] = IEE:RP:pyde.02.00:PersonNameDetails.GivenName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)

            Technical Business Rule Format:
                    (^IEE203 <> NULL AND (AnyOccurrence(^IEE28, ^IEE28 <> ^IEE203)) OR AnyOccurrence(^IEE30, ^IEE30 <> ^IEE204) WHERE ^IEE28 = ^IEE203))) OR (^IEE88 <> NULL AND AnyOccurrence(^IEE32, ^IEE32 <> ^IEE88))

        Data Elements:
            
                    RP:^IEE28 = tns:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            
                    RP:^IEE30 = tns:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            
                    RP:^IEE32 = tns:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    RP:^IEE88 = tns:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    RP:^IEE203 = tns:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    RP:^IEE204 = tns:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
        protected void VRATOIEE410269(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            if (report.DeclarationOfTrusteeOrPartnerCollection == null) return;

            IEE2018.DeclarationOfTrusteeOrPartner violator = null;

            if (report.TrusteeOrPartnerCollection == null)
            {
                violator = report.DeclarationOfTrusteeOrPartnerCollection[0];
            }
            else foreach (IEE2018.DeclarationOfTrusteeOrPartner d in report.DeclarationOfTrusteeOrPartnerCollection)
                {
                    if (d.IEE203 != null && report.TrusteeOrPartnerCollection.Find(t => t.IEE28 == d.IEE203 && t.IEE30 == d.IEE204) == null || d.IEE88 != null && report.TrusteeOrPartnerCollection.Find(t => t.IEE32 == d.IEE88) == null)
                    {
                        violator = d;
                        break;
                    }
                }

            if (violator != null)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410269",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Invalid Trustee/partner Declaration Family name and Given name or non individual name.",
                    LongDescription = "Trustee/partner Declaration Family name and Given name or non individual name must match at least one Trustee or partner Family name and Given name or non individual name.",
                    Location = $"/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner{OccurrenceIndex(violator.OccurrenceIndex)}/tns:Declaration/tns:Declaration.StatementType.Code",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410269" },
                        new ProcessMessageParameter { Name = "IEE88", Value = violator.IEE88 },
                        new ProcessMessageParameter { Name = "IEE203", Value = violator.IEE203 },
                        new ProcessMessageParameter { Name = "IEE204", Value = violator.IEE204 }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410269

        #region VR.ATO.IEE.410249

        /*  VR.ATO.IEE.410249
                    Non-individual declaration Name Type Code must be 'MN' and Currency Code must be 'C'

                Legacy Rule Format:
                    WHERE IN TUPLE (iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner\orgname2.xx.xx:OrganisationNameDetails) 
                    IF (OrganisationNameDetails.OrganisationalNameType.Code <> "MN") OR (OrganisationNameDetails.Currency.Code <> "C")
                      RETURN VALIDATION MESSAGE 
                    ENDIF

            Technical Business Rule Format:
                    ^IEE88 = BLANK

        Data Elements:
            
                    RP:^IEE88 = tns:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
        protected void VRATOIEE410249(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            ProcessMessageDocument processMessage;
            var violator = report.DeclarationOfTrusteeOrPartnerCollection?.Find(x =>
               x.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPExists == false &&
               x.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollectionCount > 0);
            if (violator != null)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410249",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual declaration Name Type Code must be 'MN' and Currency Code must be 'C'",
                    LongDescription = @"Each non-individual name of a trustee or partner declaration must include a Name Type Code of 'MN' and a Currency Code of 'C'",
                    Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(violator.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(violator.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410249" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE88", Value = violator.IEE88 });

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410249



        #region VR.ATO.IEE.410258

        /*  VR.ATO.IEE.410258

                Legacy Rule Format:
                    WHERE IN TUPLE(iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner\declaration2.xx.xx:Declaration)
                    IF COUNT (TUPLE(prsnstrcnm1.xx.xx:PersonUnstructuredName) > 0)
                      RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^DeclarationOfTrusteeOrPartner <> NULL AND ^Declaration <> NULL AND Count(^PersonUnstructuredName) > 0

        Data Elements:
            
                    ^DeclarationOfTrusteeOrPartner = tns:DeclarationOfTrusteeOrPartner
            
                    ^Declaration = tns:Declaration
            
                    ^PersonUnstructuredName = tns:PersonUnstructuredName
                    */
        protected void VRATOIEE410258(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            IEE2018.DeclarationOfTrusteeOrPartner violator =
                report.DeclarationOfTrusteeOrPartnerCollection?.Find(d => d.IEE196 != null || d.IEE197 != null);

            //assertion = report.DeclarationOfTrusteeOrPartnerCollectionExists != false && declarationOfTrusteeOrPartner.RpDeclarationOfTrusteeOrPartner_DeclarationCollectionExists != false && Count(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCount) > 0);
            if (violator != null)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410258",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Person Unstructured Name must not be present",
                    Location = $"/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner{OccurrenceIndex(violator.OccurrenceIndex)}/tns:Declaration/tns:PersonUnstructuredName{OccurrenceIndex(violator.DeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPLastOccurrenceIndex)}/tns:PersonUnstructuredName.Usage.Code",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410258" },
                        new ProcessMessageParameter { Name = "IEE196", Value = violator.IEE196 },
                        new ProcessMessageParameter { Name = "IEE197", Value = violator.IEE197 }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410258      

        #region VR.ATO.GEN.410116

        /*  VR.ATO.GEN.410116
                    Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

                Legacy Rule Format:
                    IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                        (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE97 <> BLANK) AND     (NotInSet(^IEE97, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

        Data Elements:
            
                    RP:^IEE97 = tns:Period.Type.Code WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
                    */
        protected void VRATOGEN410116(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            // implemented in RequestConsumerValidateUnsupportedRules
        }
        #endregion // VR.ATO.GEN.410116

        #region VR.ATO.GEN.410116

        /*  VR.ATO.GEN.410116
                    Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

                Legacy Rule Format:
                    IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                        (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE98 <> BLANK) AND     (NotInSet(^IEE98, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

        Data Elements:
            
                    RP:^IEE98 = tns:Period.Type.Code WHERE [Period.Type.Code='NonResidentPartPeriod']
                    */
        protected void VRATOGEN410116_(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            // implemented in RequestConsumerValidateUnsupportedRules
        }
        #endregion // VR.ATO.GEN.410116

        #region VR.ATO.GEN.410116

        /*  VR.ATO.GEN.410116
                    Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

                Legacy Rule Format:
                    IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                        (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE103 <> BLANK) AND     (NotInSet(^IEE103, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

        Data Elements:
            
                    RP:^IEE103 = tns:Period.Type.Code WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
                    */
        protected void VRATOGEN410116__(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            // implemented in RequestConsumerValidateUnsupportedRules
        }
        #endregion // VR.ATO.GEN.410116

        #region VR.ATO.GEN.410116

        /*  VR.ATO.GEN.410116
                    Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

                Legacy Rule Format:
                    IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                        (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE99 <> BLANK) AND     (NotInSet(^IEE99, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

        Data Elements:
            
                    RP:^IEE99 = tns:Period.Type.Code WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
                    */
        protected void VRATOGEN410116___(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            // implemented in RequestConsumerValidateUnsupportedRules
        }
        #endregion // VR.ATO.GEN.410116

        #region VR.ATO.GEN.410116

        /*  VR.ATO.GEN.410116
                    Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

                Legacy Rule Format:
                    IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                        (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE102 <> BLANK) AND     (NotInSet(^IEE102, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

        Data Elements:
            
                    RP:^IEE102 = tns:Period.Type.Code WHERE [Period.Type.Code='NonResidentPartPeriod']
                    */
        protected void VRATOGEN410116____(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            // implemented in RequestConsumerValidateUnsupportedRules
        }
        #endregion // VR.ATO.GEN.410116

        #region VR.ATO.IEE.410139

        /*  VR.ATO.IEE.410139
                    Address Currency Code must be 'C'

                Legacy Rule Format:
                    IN TUPLE((iee.0003.lodge.request.xx.xx:TrusteeOrPartner) OR (iee.0003.lodge.request.xx.xx:FamilyTrust) OR (iee.0003.lodge.request.xx.xx:SpecifiedIndividual)) 
                    IN TUPLE(address2.xx.xx:AddressDetails) 
                    IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Currency.Code <> "C")
                        RETURN VALIDATION MESSAGE 
                    ENDIF

            Technical Business Rule Format:
                    (^IEE130 <> BLANK) AND (^IEE130 <> 'C')

        Data Elements:
            
                    RP:^IEE130 = tns:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
        protected void VRATOIEE410139(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            // implemented in RequestConsumerValidateUnsupportedRules
        }
        #endregion // VR.ATO.IEE.410139

        #region VR.ATO.IEE.410139

        /*  VR.ATO.IEE.410139
                    Address Currency Code must be 'C'

                Legacy Rule Format:
                    IN TUPLE((iee.0003.lodge.request.xx.xx:TrusteeOrPartner) OR (iee.0003.lodge.request.xx.xx:FamilyTrust) OR (iee.0003.lodge.request.xx.xx:SpecifiedIndividual)) 
                    IN TUPLE(address2.xx.xx:AddressDetails) 
                    IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Currency.Code <> "C")
                        RETURN VALIDATION MESSAGE 
                    ENDIF

            Technical Business Rule Format:
                    (^IEE130 <> BLANK) AND (^IEE130 <> 'C')

        Data Elements:
            
                    RP:^IEE130 = tns:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
        protected void VRATOIEE410139_(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            // implemented in RequestConsumerValidateUnsupportedRules
        }
        #endregion // VR.ATO.IEE.410139

        #region VR.ATO.IEE.410139

        /*  VR.ATO.IEE.410139
                    Address Currency Code must be 'C'

                Legacy Rule Format:
                    IN TUPLE((iee.0003.lodge.request.xx.xx:TrusteeOrPartner) OR (iee.0003.lodge.request.xx.xx:FamilyTrust) OR (iee.0003.lodge.request.xx.xx:SpecifiedIndividual)) 
                    IN TUPLE(address2.xx.xx:AddressDetails) 
                    IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Currency.Code <> "C")
                        RETURN VALIDATION MESSAGE 
                    ENDIF

            Technical Business Rule Format:
                    (^IEE130 <> BLANK) AND (^IEE130 <> 'C')

        Data Elements:
            
                    RP:^IEE130 = tns:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
        protected void VRATOIEE410139__(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            // implemented in RequestConsumerValidateUnsupportedRules
        }
        #endregion // VR.ATO.IEE.410139

        #region VR.ATO.GEN.432016

        /*  VR.ATO.GEN.432016

                Legacy Rule Format:
                    WHERE IN TUPLE(electroniccontacttelephone1.xx.xx:ElectronicContactTelephone)
                    IF pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number = NULL OR pyde.xx.xx:ElectronicContact.Telephone.Usage.Code <> "03" OR pyde.xx.xx:Electronic Contact.Telephone.ServiceLine.Code <> SET ("01", "02")
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^ECT <> NULL) AND (^IEE190 = NULL OR ^IEE187 <> '03' OR NotInSet(^IEE188, '"01", "02"'))

        Data Elements:
            
                    INT:^IEE190 = tns:ElectronicContact.Telephone.Minimal.Number WHERE [ElectronicContact.Telephone.Usage.Code = '03']
            
                    INT:^IEE187 = tns:ElectronicContact.Telephone.Usage.Code WHERE [ElectronicContact.Telephone.Usage.Code = '03']
            
                    INT:^IEE188 = tns:ElectronicContact.Telephone.ServiceLine.Code WHERE [ElectronicContact.Telephone.Usage.Code = '03']
            
                    ^ECT = tns:ElectronicContactTelephone
                    */
        protected void VRATOGEN432016(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            bool assertion = report.ElectronicContactTelephoneCollectionExists &&
                             (report.IEE190 == null || report.IEE187 != @"03" || !IsMatch(report.IEE188, @"^(01|02)$",
                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase));
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.432016",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Telephone must include number, usage code '03' and service line code '01' or '02'.",
                    LongDescription = "A telephone must include the number, a usage code of '03' - contact, and a service line code of '01' - Mobile or '02' - Landline or fixed line. An two digit area code should be provided for a landline.",
                    Location = "/xbrli:xbrl/tns:ElectronicContactTelephone/tns:ElectronicContact.Telephone.Minimal.Number",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.432016" },
                        new ProcessMessageParameter { Name = "IEE190", Value = report.IEE190 },
                        new ProcessMessageParameter { Name = "IEE187", Value = report.IEE187 },
                        new ProcessMessageParameter { Name = "IEE188", Value = report.IEE188 }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.432016

        #region VR.ATO.IEE.410083

        /*  VR.ATO.IEE.410083
                    Full period indicator or dates for central management and control of trust was outside Australia have been entered when the answer to the question was central management and control of the trust outside Australia is 'No'.

                Legacy Rule Format:
                    IF ([IEE17] = FALSE) AND (([IEE18] <> NULL) OR ([IEE19] <> NULL) OR ([IEE20] <> NULL)) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE17] = IEE:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustralia.Indicator
                    [IEE18] = IEE:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
                    [IEE19] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
                    [IEE20] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

            Technical Business Rule Format:
                    (^IEE17 = FALSE) AND ((^IEE18 <> NULL) OR (^IEE19 <> NULL) OR (^IEE20 <> NULL))

        Data Elements:
            
                    RP:^IEE17 = tns:Residency.CentralManagementAndControlOutsideAustralia.Indicator
            
                    RP:^IEE18 = tns:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
            
                    RP:^IEE19 = tns:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            
                    RP:^IEE20 = tns:Period.End.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
                    */
        protected void VRATOIEE410083(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            IEE2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP periodDetailsViolator = null;
            if (report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection != null && report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection.Count > 0)
            {
                List<IEE2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP> periodDetailsViolators =
                    report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection.FindAll(p => p.IEE19 != null || p.IEE20 != null);
                periodDetailsViolator = periodDetailsViolators[periodDetailsViolators.Count - 1];
            }

            bool assertion = report.IEE17 == false && (report.IEE18 != null || periodDetailsViolator != null);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410083",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "If central mgmnt and control indicator is 'false', full period or dates must not be present",
                    LongDescription = "Full period indicator or dates for central management and control of trust was outside Australia have been entered when the answer to the question was central management and control of the trust outside Australia is 'No'",
                    Location = "/xbrli:xbrl/tns:Residency.CentralManagementAndControlOutsideAustralia.Indicator",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410083" },
                        new ProcessMessageParameter { Name = "IEE17", Value = GetValueOrEmpty(report.IEE17) },
                        new ProcessMessageParameter { Name = "IEE18", Value = GetValueOrEmpty(report.IEE18) },
                        new ProcessMessageParameter { Name = "IEE19", Value = GetValueOrEmptyDate(periodDetailsViolator?.IEE19) },
                        new ProcessMessageParameter { Name = "IEE20", Value = GetValueOrEmptyDate(periodDetailsViolator?.IEE20) }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410083

        #region VR.ATO.IEE.410090

        /*  VR.ATO.IEE.410090
                    The answer to the company a non resident for tax purposes question is 'No', therefore full period indicator or dates company was not resident are not required.

                Legacy Rule Format:
                    IF ([IEE21] = "Resident") AND (([IEE22] <> NULL) OR ([IEE23] <> NULL) OR ([IEE24] <> NULL))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE21] = IEE:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
                    [IEE22] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator
                    [IEE23] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
                    [IEE24] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

            Technical Business Rule Format:
                    (^IEE21 = 'Resident') AND ((^IEE22 <> NULL) OR (^IEE23 <> NULL) OR (^IEE24 <> NULL))

        Data Elements:
            
                    RP:^IEE21 = tns:Residency.TaxPurposesOrganisationStatus.Code
            
                    RP:^IEE22 = tns:Residency.NonResidentFullPeriod.Indicator
            
                    RP:^IEE23 = tns:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
            
                    RP:^IEE24 = tns:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                    */
        protected void VRATOIEE410090(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            IEE2018.PeriodDetailsNonResidentPartPeriodRP periodDetailsViolator = null;
            if (report.PeriodDetailsNonResidentPartPeriodRPCollection != null && report.PeriodDetailsNonResidentPartPeriodRPCollection.Count > 0)
            {
                List<IEE2018.PeriodDetailsNonResidentPartPeriodRP> periodDetailsViolators =
                    report.PeriodDetailsNonResidentPartPeriodRPCollection.FindAll(p => p.IEE23 != null || p.IEE24 != null);

                periodDetailsViolator = periodDetailsViolators[periodDetailsViolators.Count - 1];
            }

            bool assertion = report.IEE21 == @"Resident" && (report.IEE22 != null || periodDetailsViolator != null);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410090",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "If company is not a non resident, Full period indicator and dates must not be present",
                    LongDescription = "The answer to the company a non resident for tax purposes question is 'No', therefore full period indicator or dates company was not resident are not required",
                    Location = "/xbrli:xbrl/tns:Residency.TaxPurposesOrganisationStatus.Code",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410090" },
                        new ProcessMessageParameter { Name = "IEE21", Value = report.IEE21 },
                        new ProcessMessageParameter { Name = "IEE22", Value = GetValueOrEmpty(report.IEE22) },
                        new ProcessMessageParameter { Name = "IEE23", Value = GetValueOrEmptyDate(periodDetailsViolator?.IEE23) },
                        new ProcessMessageParameter { Name = "IEE24", Value = GetValueOrEmptyDate(periodDetailsViolator?.IEE24) }
                    }
                };
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410090

        #region VR.ATO.IEE.410092

        /*  VR.ATO.IEE.410092
                    The answer to the question was trustee or partner a non-resident for tax purposes is 'Yes' and full period indicator or dates company was non-resident have not been supplied

                Legacy Rule Format:
                    IF ([IEE41] = TRUE) AND ([IEE42] <> TRUE) AND ([IEE43] = NULL) AND ([IEE44] = NULL) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE41] = IEE:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE42] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE43] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE44] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)

            Technical Business Rule Format:
                    (^IEE41 = TRUE) AND (^IEE42 <> TRUE) AND AnyOccurrence(^IEE43, ^IEE43 = NULL) AND AnyOccurrence(^IEE44, ^IEE44 = NULL)

        Data Elements:
            
                    RP:^IEE41 = tns:Residency.TaxPurposesPersonStatus.Indicator
            
                    RP:^IEE42 = tns:Residency.NonResidentFullPeriod.Indicator
            
                    RP:^IEE43 = tns:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
            
                    RP:^IEE44 = tns:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                    */
        protected void VRATOIEE410092(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            if (report.TrusteeOrPartnerCollectionExists)
            {
                foreach (IEE2018.TrusteeOrPartner trusteeOrPartner in report.TrusteeOrPartnerCollection)
                {
                    bool assertion = trusteeOrPartner.IEE41 == true && trusteeOrPartner.IEE42 != true &&
                                     (trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection == null ||
                                      trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Count < 1);
                    if (assertion)
                    {
                        var processMessage = new ProcessMessageDocument
                        {
                            Code = "CMN.ATO.IEE.410092",
                            Severity = ProcessMessageSeverity.Error,
                            Description = "Full period indicator or dates must be present if trustee or partner is a non-resident",
                            LongDescription = "The answer to the question was trustee or partner a non-resident for tax purposes is 'Yes' and full period indicator or dates trustee or partner was non-resident have not been supplied",
                            Location = $"/xbrli:xbrl/tns:TrusteeOrPartner{OccurrenceIndex(trusteeOrPartner.OccurrenceIndex)}/tns:Residency.TaxPurposesPersonStatus.Indicator",
                            Parameters = new ProcessMessageParameters
                            {
                                new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410092" },
                                new ProcessMessageParameter { Name = "IEE41", Value = GetValueOrEmpty(trusteeOrPartner.IEE41) },
                                new ProcessMessageParameter { Name = "IEE42", Value = GetValueOrEmpty(trusteeOrPartner.IEE42) },
                                new ProcessMessageParameter { Name = "IEE43", Value = null },
                                new ProcessMessageParameter { Name = "IEE44", Value = null }
                            }
                        };

                        response.Add(processMessage);
                    }
                }
            }
        }
        #endregion // VR.ATO.IEE.410092

        #region VR.ATO.IEE.410093

        /*  VR.ATO.IEE.410093
                    The answer to the question was trustee or partner a non-resident for tax purposes is 'No' and full period indicator or dates company was non-resident have been supplied

                Legacy Rule Format:
                    IF ([IEE41] = FALSE) AND (([IEE42] = TRUE) OR ([IEE43] <> NULL) OR ([IEE44] <> NULL)) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE41] = IEE:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE42] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE43] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE44] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)

            Technical Business Rule Format:
                    (^IEE41 = FALSE) AND ((^IEE42 = TRUE) OR AnyOccurrence(^IEE43, ^IEE43 <> NULL) OR AnyOccurrence(^IEE44, ^IEE44 <> NULL))

        Data Elements:
            
                    RP:^IEE41 = tns:Residency.TaxPurposesPersonStatus.Indicator
            
                    RP:^IEE42 = tns:Residency.NonResidentFullPeriod.Indicator
            
                    RP:^IEE43 = tns:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
            
                    RP:^IEE44 = tns:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                    */
        protected void VRATOIEE410093(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            if (report.TrusteeOrPartnerCollectionExists)
            {
                foreach (IEE2018.TrusteeOrPartner trusteeOrPartner in report.TrusteeOrPartnerCollection)
                {
                    bool assertion = trusteeOrPartner.IEE41 == false && trusteeOrPartner.IEE42 == true &&
                                     trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection != null
                                      && trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Count > 0;
                    if (assertion)
                    {
                        var processMessage = new ProcessMessageDocument
                        {
                            Code = "CMN.ATO.IEE.410093",
                            Severity = ProcessMessageSeverity.Error,
                            Description = "Full period indicator and dates must not be present if trustee or partner was not a non-resident",
                            LongDescription = "The answer to the question was trustee or partner a non-resident for tax purposes is 'No' and full period indicator or dates company was non-resident have been supplied",
                            Location = $"/xbrli:xbrl/tns:TrusteeOrPartner{OccurrenceIndex(trusteeOrPartner.OccurrenceIndex)}/tns:Residency.TaxPurposesPersonStatus.Indicator",
                            Parameters = new ProcessMessageParameters
                            {
                                new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410093" },
                                new ProcessMessageParameter{ Name = "IEE41", Value = GetValueOrEmpty(trusteeOrPartner.IEE41) },
                                new ProcessMessageParameter { Name = "IEE42", Value = GetValueOrEmpty(trusteeOrPartner.IEE42) },
                            },
                        };

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE43", Value = GetValueOrEmptyDate(trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection != null && trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Count > 0 ? trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection[trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Count - 1].IEE43 : null) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE44", Value = GetValueOrEmptyDate(trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection != null && trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Count > 0 ? trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection[trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Count - 1].IEE44 : null) });

                        response.Add(processMessage);
                    }
                }
            }

        }
        #endregion // VR.ATO.IEE.410093

        #region VR.ATO.IEE.410243

        /*  VR.ATO.IEE.410243
                    Person Unstructured Name not required

                Legacy Rule Format:
                    IF COUNT [IEE147] > 0 and COUNT (TUPLE(prsnstrcnm1.xx.xx:PersonUnstructuredName) > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE147] = IEE:RP.TrueAndCorrect:pyin.02.03:Declaration.StatementType.Code IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

            Technical Business Rule Format:
                    Count(^IEE147) > 0 and Count(^PersonUnstructuredName) > 0

        Data Elements:
            
                    RP:^IEE147 = tns:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']
            
                    ^PersonUnstructuredName = tns:PersonUnstructuredName
                    */
        protected void VRATOIEE410243(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            bool assertion = Count(report.DeclarationCollectionCount) > 0 &&
                             Count(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCount) > 0;
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410243",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Person Unstructured Name must not be present",
                    Location = $"/xbrli:xbrl/tns:Declaration{OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPLastOccurrenceIndex)}/tns:PersonUnstructuredName/tns:PersonUnstructuredName.Usage.Code",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410243" } }
                };
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410243

        #region VR.ATO.IEE.410261

        /*  VR.ATO.IEE.410261
                    Intermediary is present then intermediary declaration must be present

                Legacy Rule Format:
                    WHERE PARENT RETURN DOES NOT EXIST
                    IF INT<> NULLORBLANK AND COUNT (iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner) > 0 OR COUNT [IEE147] > 0
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE147] = IEE:RP.TrueAndCorrect:pyin.02.03:Declaration.StatementType.Code IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

            Technical Business Rule Format:
                    ^INT <> BLANK AND Count(^DeclarationOfTrusteeOrPartner) > 0 OR Count(^IEE147) > 0

        Data Elements:
            
                    RP:^IEE147 = tns:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']
            
                    ^DeclarationOfTrusteeOrPartner = tns:DeclarationOfTrusteeOrPartner
            
                    ^INT = INT
                    */
        protected void VRATOIEE410261(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            if (isStandalone)
            {
                bool assertion = report.INTExists && (Count(report.DeclarationOfTrusteeOrPartnerCollectionCount) > 0 ||
                                                 Count(report.DeclarationCollection_TrueAndCorrect_RPCount) > 0);
                if (assertion)
                {
                    var processMessage = new ProcessMessageDocument
                    {
                        Code = "CMN.ATO.IEE.410261",
                        Severity = ProcessMessageSeverity.Error,
                        Description = "If intermediary is present, then reporting party declaration or trustee or partner declaration must not be present",
                        Location = $"/xbrli:xbrl/tns:Declaration{OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPLastOccurrenceIndex)}/tns:Declaration.StatementType.Code",
                        Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410261" } }
                    };
                    response.Add(processMessage);
                }
            }
        }
        #endregion // VR.ATO.IEE.410261

        #region VR.ATO.IEE.410262
        /*  VR.ATO.IEE.410262
                    Parent return declaration must be present

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF INT <> NULLORBLANK AND COUNT [IEE159] > 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE159] = IEE:INT:pyin.02.03:Declaration.StatementType.Code WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

            Technical Business Rule Format:
                    ^INT <> BLANK AND Count(^IEE159) > 0

        Data Elements:
            
                    INT:^IEE159 = tns:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code='TrueAndCorrect']
            
                    INT:^INT = INT
                    */
        protected void VRATOIEE410262(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            bool assertion = !isStandalone && report.INTExists && Count(report.DeclarationCollection_TrueAndCorrect_INTCount) > 0;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410262",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If IEE is lodged as a schedule intermediary declaration must not be present for IEE",
                    Location = $"/xbrli:xbrl/tns:Declaration{OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTLastOccurrenceIndex)}/tns:Declaration.StatementType.Code",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410262" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410262

        #region VR.ATO.IEE.410265

        /*  VR.ATO.IEE.410265
                    Declaration Statement Type Code must be 'TrueAndCorrect'

                Legacy Rule Format:
                    IF [IEE147] <> NULLORBLANK AND
                    [IEE147] <> "TrueAndCorrect"
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE147] = IEE:RP.TrueAndCorrect:pyin.02.03:Declaration.StatementType.Code IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

            Technical Business Rule Format:
                    ^IEE147 <> BLANK AND ^IEE147 <> 'TrueAndCorrect'

        Data Elements:
            
                    RP:^IEE147 = tns:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']
                    */
        protected void VRATOIEE410265(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            // implemented in RequestConsumerValidateUnsupportedRules
        }
        #endregion // VR.ATO.IEE.410265

        #region VR.ATO.IEE.410006
        protected void VRATOIEE410006(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            ProcessMessageDocument processMessage;

            /*  VR.ATO.IEE.410006
                Interposed entity election  is incomplete.

            Legacy Rule Format:
                IF ([IEE116] = "E") AND (([IEE16] = NULLORBLANK) OR ([IEE45] = NULLORBLANK) OR ([IEE46] = NULL) OR ([IEE52] = NULLORBLANK) OR ([IEE53] = NULLORBLANK) OR ([IEE54] = NULLORBLANK) OR ([IEE58] = NULLORBLANK) OR ([IEE59] = NULL) OR ([IEE64] = NULL) OR ([IEE66] = NULLORBLANK) OR ([IEE70] = NULLORBLANK) OR ([IEE71] = NULLORBLANK) OR  ([IEE119] = NULLORBLANK) OR ([IEE170] = NULLORBLANK))
                   RETURN VALIDATION MESSAGE
                ENDIF

                [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                [IEE45] = IEE:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year
                [IEE46] = IEE:RP:rvctc3.02.00:Elections.DateSpecified.Date
                [IEE52] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
                [IEE53] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
                [IEE54] = IEE:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
                [IEE58] = IEE:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
                [IEE59] = IEE:RP:rvctc3.02.00:Elections.Commencement.Date
                [IEE64] = IEE:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
                [IEE66] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
                [IEE70] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
                [IEE71] = IEE:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
                [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
                [IEE119] = IEE:RP:pyde.02.18:OrganisationNameDetails.OrganisationalNameType.Code WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
                [IEE170] = IEE:RP:pyde.02.00:OrganisationNameDetails.Currency.Code WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)

            Technical Business Rule Format:
                (^IEE116 = 'E') AND ((^IEE16 = BLANK) OR (^IEE45 = BLANK) OR (^IEE46 = NULL) OR (^IEE52 = BLANK) OR (^IEE53 = BLANK) OR (^IEE54 = BLANK) OR (^IEE58 = BLANK) OR (^IEE59 = NULL) OR (^IEE64 = NULL) OR (^IEE66 = BLANK) OR (^IEE70 = BLANK) OR (^IEE71 = BLANK) OR  (^IEE119 = BLANK) OR (^IEE170 = BLANK))

            Data Elements:

                RP:^IEE116 = rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

                RP:^IEE16 = pyde.02.21:Party.Type.Code

                RP:^IEE45 = rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year

                RP:^IEE46 = rvctc3.02.00:Elections.DateSpecified.Date

                RP:^IEE52 = pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                RP:^IEE53 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']

                RP:^IEE54 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']

                RP:^IEE58 = rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year

                RP:^IEE59 = rvctc3.02.00:Elections.Commencement.Date

                RP:^IEE64 = pyde.02.00:PersonDemographicDetails.Birth.Date

                RP:^IEE66 = pyde.02.00:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']

                RP:^IEE70 = pyde.02.00:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']

                RP:^IEE71 = pyde.02.00:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']

                RP:^IEE119 = pyde.02.18:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                RP:^IEE170 = pyde.02.00:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                */

            bool assertion = report.IEE116 == @"E" && (string.IsNullOrWhiteSpace(report.IEE16) ||
                                                       report.IEE45 == null || report.IEE46 == null ||
                                                       string.IsNullOrWhiteSpace(report.IEE52) ||
                                                       string.IsNullOrWhiteSpace(report.IEE53) ||
                                                       string.IsNullOrWhiteSpace(report.IEE54) ||
                                                       report.IEE58 == null || report.IEE59 == null || report.IEE64 == null ||
                                                       string.IsNullOrWhiteSpace(report.IEE66) ||
                                                       string.IsNullOrWhiteSpace(report.IEE70) ||
                                                       string.IsNullOrWhiteSpace(report.IEE71) ||
                                                       string.IsNullOrWhiteSpace(report.IEE119) ||
                                                       string.IsNullOrWhiteSpace(report.IEE170));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410006",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Interposed entity election schedule must be complete",
                    LongDescription = "The full name and address of the family trust in respect of which the election is made, the income year, day specified, election commencement time, and/or date of birth, name and address of the specified individual must all be completed to make a valid Interposed entity election",
                    Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionOrRevocation.Code",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410006" },
                        new ProcessMessageParameter { Name = "IEE116", Value = report.IEE116 },
                        new ProcessMessageParameter { Name = "IEE16", Value = report.IEE16 },
                        new ProcessMessageParameter { Name = "IEE45", Value = report.IEE45.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "IEE46", Value = GetValueOrEmptyDate(report.IEE46) },
                        new ProcessMessageParameter { Name = "IEE52", Value = report.IEE52 },
                        new ProcessMessageParameter { Name = "IEE53", Value = report.IEE53 },
                        new ProcessMessageParameter { Name = "IEE54", Value = report.IEE54 },
                        new ProcessMessageParameter { Name = "IEE58", Value = report.IEE58.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "IEE59", Value = GetValueOrEmptyDate(report.IEE59) },
                        new ProcessMessageParameter { Name = "IEE64", Value = GetValueOrEmptyDate(report.IEE64) },
                        new ProcessMessageParameter { Name = "IEE66", Value = report.IEE66 },
                        new ProcessMessageParameter { Name = "IEE70", Value = report.IEE70 },
                        new ProcessMessageParameter { Name = "IEE71", Value = report.IEE71 },
                        new ProcessMessageParameter { Name = "IEE119", Value = report.IEE119 },
                        new ProcessMessageParameter { Name = "IEE170", Value = report.IEE170 }

                    },
                };

                response.Add(processMessage);
            }

        }
        #endregion

        #region VR.ATO.IEE.410248

        protected void VRATOIEE410248(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {

            /*  VR.ATO.IEE.410248
                Trustee declaration Name Type Code must be 'LGL', Usage Code  'DeclarationSignatory' and Currency Code 'C'.

            Legacy Rule Format:
                WHERE IN TUPLE (iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner\prsnstrcnm3.xx.xx:PersonNameDetails) 
                IF (PersonNameDetails.PersonNameType.Code <> "LGL") OR (PersonNameDetails.Usage.Code <> "DeclarationSignatory") OR (PersonNameDetails.Currency.Code <> "C")
                  RETURN VALIDATION MESSAGE 
                ENDIF

            Technical Business Rule Format:
                ^IEE199 = BLANK

            Data Elements:

                RP:^IEE199 = pyde.02.37:PersonNameDetails.Usage.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                */

            IEE2018.DeclarationOfTrusteeOrPartner violator = null;
            List<IEE2018.DeclarationOfTrusteeOrPartner> violators =
                report.DeclarationOfTrusteeOrPartnerCollection?.FindAll(d =>
                    d.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCount !=
                    d.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollectionCount);

            if (violators != null && violators.Count > 0)
                violator = violators[violators.Count - 1];

            if (violator != null)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410248",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Declaration name must have Type -'LGL', Usage - 'DeclarationSignatory' and Currency - 'C'",
                    LongDescription = "Each individual name for a trustee or partner declaration must include a Name Type Code of 'LGL', a Usage Code of  'DeclarationSignatory' and a Currency Code of 'C'",
                    Location = $"/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner{OccurrenceIndex(violator.OccurrenceIndex)}/tns:PersonNameDetails/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410248" },
                        new ProcessMessageParameter { Name = "IEE199", Value = "" }
                    }
                };

                response.Add(processMessage);
            }

        }
        #endregion

        #region VR.ATO.IEE.410089
        protected void VRATOIEE410089(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            /*  VR.ATO.IEE.410089
            The answer to the company being a non resident for tax purposes is 'Yes' and the full period indicator and the dates the company was a non resident have not been supplied.

            Legacy Rule Format:
            IF ([IEE21] = SET("NonResidentNoPermEstab","NonResidentPermEstab")) AND ([IEE22] = NULL AND ([IEE23] = NULL) AND ([IEE24] = NULL))
               RETURN VALIDATION MESSAGE
            ENDIF

            [IEE21] = IEE:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
            [IEE22] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator
            [IEE23] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
            [IEE24] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

            Technical Business Rule Format:
            (InSet(^IEE21, '"NonResidentNoPermEstab","NonResidentPermEstab"')) AND (^IEE22 = NULL) AND (^IEE23 = NULL) AND (^IEE24 = NULL)

            Data Elements:

            RP:^IEE21 = tns:Residency.TaxPurposesOrganisationStatus.Code

            RP:^IEE22 = tns:Residency.NonResidentFullPeriod.Indicator

            RP:^IEE23 = tns:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']

            RP:^IEE24 = tns:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
            */
            bool assertion = IsMatch(report.IEE21, @"^(NonResidentNoPermEstab|NonResidentPermEstab)$") &&
                             report.IEE22 == null && (report.PeriodDetailsNonResidentPartPeriodRPCollection == null ||
                                                      report.PeriodDetailsNonResidentPartPeriodRPCollection.Count < 1);
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410089",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "If Company is non resident for tax purposes, full period indicator or dates must be present",
                    LongDescription = "The answer to the company being a non resident for tax purposes is 'Yes' and the full period indicator and the dates the company was a non resident have not been supplied",
                    Location = "/xbrli:xbrl/tns:Residency.TaxPurposesOrganisationStatus.Code",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410089" },
                        new ProcessMessageParameter { Name = "IEE21", Value = report.IEE21 },
                        new ProcessMessageParameter { Name = "IEE22", Value = GetValueOrEmpty(report.IEE22) }
                    }
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE23", Value = GetValueOrEmptyDate((report.PeriodDetailsNonResidentPartPeriodRPCollection == null || report.PeriodDetailsNonResidentPartPeriodRPCollection.Count < 1) ? null : report.PeriodDetailsNonResidentPartPeriodRPCollection[report.PeriodDetailsNonResidentPartPeriodRPCollection.Count - 1].IEE23) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE24", Value = GetValueOrEmptyDate((report.PeriodDetailsNonResidentPartPeriodRPCollection == null || report.PeriodDetailsNonResidentPartPeriodRPCollection.Count < 1) ? null : report.PeriodDetailsNonResidentPartPeriodRPCollection[report.PeriodDetailsNonResidentPartPeriodRPCollection.Count - 1].IEE24) });

                response.Add(processMessage);
            }
        }
        #endregion

        #region VR.ATO.IEE.410120
        protected void VRATOIEE410120(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            /*  VR.ATO.IEE.410120
            Up to one Current address and up to one previous address for Interposed Entity only to be provided
    
            Legacy Rule Format:
            IF COUNT([IEE5]) >1 OR COUNT([IEE11]) >1
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE5] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails)
            [IEE11] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails)

            Technical Business Rule Format:
            Count(^IEE5) >1 OR Count(^IEE11) >1
    
            Data Elements:
    
            RP:^IEE5 = tns:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            RP:^IEE11 = tns:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            bool assertion = Count(report.AddressDetailsCollection_POS_C_RPCount) > 1 ||
                             Count(report.AddressDetailsCollection_POS_P_RPCount) > 1;
            if (assertion)
            {
                // get highest occurrence index of offending tuple
                int occurrenceIndex = Count(report.AddressDetailsCollection_POS_C_RPCount) > 1 && Count(report.AddressDetailsCollection_POS_P_RPCount) > 1 ? (report.AddressDetailsCollection_POS_C_RPLastOccurrenceIndex > report.AddressDetailsCollection_POS_P_RPLastOccurrenceIndex ? report.AddressDetailsCollection_POS_C_RPLastOccurrenceIndex : report.AddressDetailsCollection_POS_P_RPLastOccurrenceIndex) : (report.AddressDetailsCollection_POS_C_RPCount > 1 ? report.AddressDetailsCollection_POS_C_RPLastOccurrenceIndex : report.AddressDetailsCollection_POS_P_RPLastOccurrenceIndex);
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410120",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "More than one current address for the Interposed Entity must not be present",
                    Location = $"/xbrli:xbrl/tns:AddressDetails{OccurrenceIndex(occurrenceIndex)}/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410120" } }
                };
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410120

        #region VR.ATO.IEE.410150
        protected void VRATOIEE410150(List<ProcessMessageDocument> response, IEE2018 report, bool isStandalone)
        {
            /*  VR.ATO.IEE.410150
            Up to one Current Name and up to one Previous Name only to be provided
    
            Legacy Rule Format:
            IF COUNT([IEE4]) >1 OR COUNT([IEE10]) >1
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE4] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails)
            [IEE10] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "P") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails)

            Technical Business Rule Format:
            Count(^IEE4) >1 OR Count(^IEE10) >1
    
            Data Elements:
    
            RP:^IEE10 = tns:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
    
            RP:^IEE4 = tns:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            bool assertion = (Count(report.OrganisationNameDetailsCollection_MN_C_RPCount) > 1 || Count(report.OrganisationNameDetailsCollection_MN_P_RPCount) > 1);
            if (assertion)
            {
                // get highest occurrence index of offending tuple
                int occurrenceIndex = Count(report.OrganisationNameDetailsCollection_MN_C_RPCount) > 1 && Count(report.OrganisationNameDetailsCollection_MN_P_RPCount) > 1 ? (report.OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex > report.OrganisationNameDetailsCollection_MN_P_RPLastOccurrenceIndex ? report.OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex : report.OrganisationNameDetailsCollection_MN_P_RPLastOccurrenceIndex) : (report.OrganisationNameDetailsCollection_MN_C_RPCount > 1 ? report.OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex : report.OrganisationNameDetailsCollection_MN_P_RPLastOccurrenceIndex);
                var processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410150",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "There must be no more than one current and one previous Interposed Entity Name",
                    Location = $"/xbrli:xbrl/tns:OrganisationNameDetails{OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPLastOccurrenceIndex)}/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410150" } }
                };
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410150
        #endregion Manual Rules - Rules marked for manual coding
    }
}