﻿using System;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationFTER;
using Ato.EN.IntegrationServices.CodeGenerationSMSFAR;
using DataContracts;

namespace Ato.CD.Inbound.SMSFAR201802
{
    internal class CrossFormValidatorFTER : ICrossFormValidator
    {
        private SMSFAR2018 ParentDocument { get; }

        private FTER2018 ChildDocument { get; }

        internal CrossFormValidatorFTER(SMSFAR2018 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (FTER2018)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN402010(response);
            VRATOFTER408001(response);
            VRATOFTER408029(response);
            VRATOFTER408033(response);
            VRATOFTER408112(response);
            VRATOFTER408144(response);
            VRATOFTER408162(response);

            return response;
        }

        #region VR.ATO.GEN.402010
        //IF (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK)
        //    AND (PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK)
        //    AND (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier)
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOGEN402010(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ParentDocument.SMSFAR5) &&
                             !string.IsNullOrWhiteSpace(ChildDocument.FTER170) &&
                             !ParentDocument.SMSFAR5.Equals(ChildDocument.FTER170);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Your supplied ABN does not match the ABN supplied on the form it was submitted with",
                    LongDescription = "Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402010" } }
                });
            }
        }
        #endregion
        #region VR.ATO.FTER.408001
        //WHERE PARENT RETURN EXISTS 
        //IF(FTER:RP:entity.identifier.TFN<> NULLORBLANK) AND(FTER:RP:entity.identifier.TFN<> PARENT RETURN:RP:entity.identifier.TFN)
        //  RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOFTER408001(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.RPIdentifierTFN) &&
                             !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.410115",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "TFN on Schedule must match TFN on main form",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408001" } }
                });
            }
        }
        #endregion
        #region VR.ATO.FTER.408029
        //WHERE PARENT RETURN EXISTS
        //IF[FTER95] = "V" AND[FTER40] <> NULL AND[FTER40] <> PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        //[FTER95] = FTER:RP:Elections.FamilyTrustElectionRevocation.Code
        //[FTER40] = FTER:RP:Elections.FamilyTrustElectionStatus.Year
        private void VRATOFTER408029(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.FTER95) &&
                             ChildDocument.FTER95.ToUpper().Equals("V") && ChildDocument.FTER40.HasValue &&
                             ChildDocument.FTER40 != ParentDocument.SMSFAR1;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Income year for variation must match the year of return on the main form",
                    LongDescription = "Income year specified for the purpose of the FTE variation must match the year of return on the main form",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionStatus.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408029" },
                        new ProcessMessageParameter { Name = "FTER40", Value = ChildDocument.FTER40.ToString() }
                    }
                });
            }
        }
        #endregion
        #region VR.ATO.FTER.408033
        //IF[FTER95] = "R" AND[FTER96] <> NULL AND[FTER96] <> PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        //[FTER95] = FTER:RP:Elections.FamilyTrustElectionRevocation.Code
        //[FTER96] = FTER:RP:Elections.FamilyTrustRevocationStatusEffective.Year
        private void VRATOFTER408033(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.FTER95) &&
                             ChildDocument.FTER95.ToUpper().Equals("R") && ChildDocument.FTER96.HasValue &&
                             ChildDocument.FTER96 != ParentDocument.SMSFAR1;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Revocation income year must be the same as the year of the return",
                    LongDescription = "For a revocation, the income year from which the revocation is to be effective must be the same as the year of the parent return",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustRevocationStatusEffective.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408033" },
                        new ProcessMessageParameter { Name = "FTER96", Value = ChildDocument.FTER96.ToString() }
                    }
                });
            }
        }
        #endregion
        #region VR.ATO.FTER.408112
        //WHERE PARENT RETURN EXISTS
        //IF[FTER95] = "E" AND[FTER40] <>  PARENT RETURN:RP:Elections.FamilyTrustElectionStatus.Year
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        //[FTER95] = FTER:RP:Elections.FamilyTrustElectionRevocation.Code
        //[FTER40] = FTER:RP:Elections.FamilyTrustElectionStatus.Year
        private void VRATOFTER408112(List<ProcessMessageDocument> response)
        {
           
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.FTER95) &&
                              ChildDocument.FTER95.ToUpper().Equals("E") && ChildDocument.FTER40.HasValue &&
                              !ChildDocument.FTER40.GetValueOrDefault().Equals(ParentDocument.SMSFAR162.GetValueOrDefault());
            if (assertion)
                if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408112",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Income year specified on main return form must equal the income year specified on the FTE schedule.",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionStatus.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408112" },
                        new ProcessMessageParameter { Name = "FTER40", Value = ChildDocument.FTER40.ToString() }
                    }
                });
            }
        }
        #endregion
        #region VR.ATO.FTER.408144
        //WHERE PARENT RETURN EXISTS
        //IF[FTER4] <> NULLORBLANK AND[FTER4] <> (PARENT RETURN:xbrli\organisationname2.xx.xx.OrganisationName.Text
        //    WHERE TUPLE ELEMENT EXPLICIT OrganisationNameDetails.Currency.Code = "C"
        //AND TUPLE ELEMENT EXPLICIT OrganisationNameDetails.OrganisationalNameType.Code = "MN")
        //RETURN VALIDATION  MESSAGE
        //    ENDIF
        //        [FTER4] = FTER:RP:OrganisationNameDetails.OrganisationalName.Text
        //    WHERE((TUPLE ELEMENT EXPLICIT pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = "MN")
        //AND(TUPLE ELEMENT EXPLICIT pyde.xx.xx:OrganisationNameDetails.Currency.Code = "C"))
        //IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
        private void VRATOFTER408144(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.FTER4) &&
                             !string.IsNullOrWhiteSpace(ParentDocument.SMSFAR4) && !ChildDocument.FTER4.ToUpper()
                                 .Equals(ParentDocument.SMSFAR4.ToUpper());
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408144",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Current full name of the trust must match parent form",
                    LongDescription = "When used as a schedule, the name of the trust must be the same as the name of the trust on the parent form",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408144" },
                        new ProcessMessageParameter { Name = "FTER4", Value = ChildDocument.FTER4 }
                    }
                });
            }
        }
        #endregion
        #region VR.ATO.FTER.408162
        //WHERE PARENT RETURN EXISTS
        //IF[FTER57] > ((PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year)&"-06-30")
        //RETURN VALIDATION MESSAGE
        //    ENDIF
        //[FTER57] = FTER:RP:Elections.Revocation.Date
        private void VRATOFTER408162(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.FTER57.HasValue && DateTime.Compare(ChildDocument.FTER57.Value,
                                 Convert.ToDateTime(string.Concat(ParentDocument.SMSFAR1.ToString(), "-06-30"))) > 0;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408110",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Revocation date must not be after 30 June of the year of the return",
                    Location = "/xbrli:xbrl/tns:Elections.Revocation.Date",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408162" },
                        new ProcessMessageParameter { Name = "FTER57", Value = ChildDocument.FTER57.Value.ToString("yyyy-MM-dd") }
                    }
                });
            }
        }
        #endregion
    }
}