﻿using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using Ato.EN.IntegrationServices.Contrib.VaTS;
using Ato.EN.IntegrationServices.CodeGenerationCGTS;
using Ato.EN.IntegrationServices.Document.DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRT;

namespace Ato.CD.Inbound.TRT201802
{
    internal class CrossFormValidatorCGTS : ICrossFormValidator
    {
        private TRT2018 ParentDocument { get; }

        private CGTS2018 ChildDocument { get; }

        private List<Context> Contexts { get; }

        internal CrossFormValidatorCGTS(TRT2018 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (CGTS2018)childDocument.ConsumedReport;
            Contexts = ChildDocument.GetContexts();
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOTRT432213(response);
            VRATOTRT432189(response);
            VRATOGEN438022(response);
            VRATOGEN438023(response);
            VRATOGEN438024(response);
            VRATOGEN438025(response);
            VRATOGEN438026(response);
            VRATOGEN438027(response);
            VRATOGEN438028(response);
            VRATOGEN402009(response);
            VRATOGEN438000(response);
            VRATOCGTS438001(response);
            VRATOCGTS402165(response);
            VRATOCGTS402174(response);
            VRATOCGTS402230(response);            
            VRATOCGTS402171(response);
            VRATOCGTS402226(response);
            VRATOCGTS402240(response);

            return response;
        }

        #region VR.ATO.TRT.432213

        /*  VR.ATO.TRT.432213
                    Unapplied net capital losses carried forward on the Capital Gains Tax schedule (if lodged) must equal Net capital losses carried forward on the main income tax return.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([CGTS124] + [CGTS125]) <> [TRT475]  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS124] = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [CGTS125] = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [TRT475] = TRT:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('CGTS') = 1) AND (^CGTS124 + ^CGTS125) <> ^TRT475

        Data Elements:
            
                    RP.Closing(Instant):^TRT475 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Closing(Instant):^CGTS124 = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Closing(Instant):^CGTS125 = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        private void VRATOTRT432213(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.TRT475.GetValueOrDefault().Equals(ChildDocument.CGTS124.GetValueOrDefault() + ChildDocument.CGTS125.GetValueOrDefault());

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434088",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in CGT schedule",
                    LongDescription = @"The total of the 'Unapplied net capital losses carried forward'  in the Capital Gains Tax schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432213" },
                        new ProcessMessageParameter { Name = "TRT475", Value = ParentDocument .TRT475.GetValueOrDefault().ToString() }
                    }
                }; 
 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432213

        #region VR.ATO.TRT.432189

        /*  VR.ATO.TRT.432189
                    The Net Capital gain amount on the main return form must equal the Total net capital gain amount on the Capital Gains Tax schedule.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([TRT133] <> [CGTS131])
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS131] = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    (CountDocument('CGTS') = 1) AND (^TRT133 <> ^CGTS131)

        Data Elements:
            
                    RP:^TRT133 = tns:Income.CapitalGainsNet.Amount
            
                    RP:^CGTS131 = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    */
        private void VRATOTRT432189(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.TRT133.GetValueOrDefault().Equals(ChildDocument.CGTS131.GetValueOrDefault());

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434036",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gains in return must equal total net capital gain in CGT schedule.",
                    Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432189" },
                        new ProcessMessageParameter { Name = "TRT133", Value = ParentDocument .TRT133.GetValueOrDefault().ToString() }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432189

        #region VR.ATO.GEN.438022

        /*  VR.ATO.GEN.438022
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "A - Small business active asset reduction") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS127] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS127] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'A - Small business active asset reduction') AND (CountDocument('CGTS') = 1) AND ^CGTS127 = NULL

        Data Elements:
            
                    RP:^TRT448 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS127 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount
                    */
        private void VRATOGEN438022(List<ProcessMessageDocument> response)
        {
            bool assertion = (ParentDocument.TRT448Collection != null &&  ParentDocument.TRT448Collection.Any(c => c == @"A - Small business active asset reduction")) && !ChildDocument.CGTS127.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438022",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business active asset reduction amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",//[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438022" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438022
        
        #region VR.ATO.GEN.438023

        /*  VR.ATO.GEN.438023
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business retirement exemption (Subdivision152-D)", then Small business retirement exemption amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "B - Small business retirement exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS128] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS128] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'B - Small business retirement exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS128 = NULL

        Data Elements:
            
                    RP:^TRT448 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS128 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount
                    */
        private void VRATOGEN438023(List<ProcessMessageDocument> response )
        {
            bool assertion = (ParentDocument.TRT448Collection != null && ParentDocument.TRT448Collection.Any(c => c == @"B - Small business retirement exemption")) && !ChildDocument.CGTS128.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business retirement exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business retirement exemption (Subdivision152-D)"", then Small business retirement exemption amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",//[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438023" } }
                }; 

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438023

        #region VR.ATO.GEN.438024

        /*  VR.ATO.GEN.438024
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "C - Small business roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS129] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS129] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'C - Small business roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS129 = NULL

        Data Elements:
            
                    RP:^TRT448 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS129 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount
                    */
        private void VRATOGEN438024(List<ProcessMessageDocument> response )
        {
            bool assertion = (ParentDocument.TRT448Collection != null &&  ParentDocument.TRT448Collection.Any(c => c == @"C - Small business roll-over")) && !ChildDocument.CGTS129.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438024",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",//[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438024" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438024

        #region VR.ATO.GEN.438025

        /*  VR.ATO.GEN.438025
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "D - Small business 15 year exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS138] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS138] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'D - Small business 15 year exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS138 = NULL

        Data Elements:
            
                    RP:^TRT448 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS138 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount
                    */
        private void VRATOGEN438025(List<ProcessMessageDocument> response)
        {
            bool assertion = (ParentDocument.TRT448Collection != null && ParentDocument.TRT448Collection.Any(c => c == @"D - Small business 15 year exemption") )&& !ChildDocument.CGTS138.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business 15 year exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",//[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438025" } }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438025

        #region VR.ATO.GEN.438026

        /*  VR.ATO.GEN.438026
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Foreign resident CGT exemption (Division 855)", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "E - Foreign resident CGT exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS140] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS140] = CGTS:RP:rvctc3.02.11:Capital.Gains.ForeignResidentExemption.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'E - Foreign resident CGT exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS140 = NULL

        Data Elements:
            
                    RP:^TRT448 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS140 = CGTS:RP:rvctc3.02.11:Capital.Gains.ForeignResidentExemption.Amount
                    */
        private void VRATOGEN438026(List<ProcessMessageDocument> response  )
        {
            bool assertion = (ParentDocument.TRT448Collection != null &&  ParentDocument.TRT448Collection.Any(c => c == @"E - Foreign resident CGT exemption") ) && !ChildDocument.CGTS140.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438026",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a foreign resident amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Foreign resident CGT exemption (Division 855)"", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",//[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438026" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438026

        #region VR.ATO.GEN.438027

        /*  VR.ATO.GEN.438027
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "F - Scrip for scrip roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS45] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS45] = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'F - Scrip for scrip roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS45 = NULL

        Data Elements:
            
                    RP:^TRT448 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS45 = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount
                    */
        private void VRATOGEN438027(List<ProcessMessageDocument> response )
        {
            bool assertion = (ParentDocument.TRT448Collection != null &&  ParentDocument.TRT448Collection.Any(c => c == @"F - Scrip for scrip roll-over")) && !ChildDocument.CGTS45.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438027",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",//[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438027" } }
                }; 

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438027

        #region VR.ATO.GEN.438028

        /*  VR.ATO.GEN.438028
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Demerger exemption (Subdivision 125-C)", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "H - Demerger exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS47] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS47] = CGTS:RP:rvctc3.02.11:Capital.Gains.DemergingEntityExemption.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'H - Demerger exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS47 = NULL

        Data Elements:
            
                    RP:^TRT448 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS47 = CGTS:RP:rvctc3.02.11:Capital.Gains.DemergingEntityExemption.Amount
                    */
        private void VRATOGEN438028(List<ProcessMessageDocument> response )
        {
            bool assertion = (ParentDocument.TRT448Collection != null &&  ParentDocument.TRT448Collection.Any(c => c == @"H - Demerger exemption")) && !ChildDocument.CGTS47.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.438028",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a demerging entity amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Demerger exemption (Subdivision 125-C)"", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",//[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438028" } }
                };
 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438028

        #region VR.ATO.GEN.402009
        /* VR.ATO.GEN.402009
            IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                RETURN VALIDATION MESSAGE
            ENDIF 
        */
        private void VRATOGEN402009(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009
                
        #region VR.ATO.CGTS.402165

        /*  VR.ATO.CGTS.402165
                    Total losses transferred in applied is only valid for a company

                Legacy Rule Format:
                    IF (PARENT RETURN <> "CTR") AND ([CGTS122] <> NULL)  
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (Not(ParentDocument('CTR'))) AND (^CGTS122 <> NULL)

        Data Elements:
            
                    RP.Applied:^CGTS122 = tns:Capital.Losses.TransferredTotal.Amount
                    */
        private void VRATOCGTS402165(List<ProcessMessageDocument> response )
        {
            if (ChildDocument.CGTS122.HasValue)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402165",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total losses transferred in applied is only valid for a company",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.TransferredTotal.Amount",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402165" },
                        new ProcessMessageParameter { Name = "CGTS122", Value = ChildDocument .CGTS122.GetValueOrDefault().ToString() }
                    }
                };
                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402165

        #region VR.ATO.CGTS.402174

        /*  VR.ATO.CGTS.402174
                    The CGT discount rate available to trusts and companies is 50%.

                Legacy Rule Format:
                    IF PARENT RETURN = SET("TRT", "CTR","TRTAMI") AND  [CGTS126] > ((([CGTS118] - [CGTS123]) *0.5) +1)
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    PARENT = SET("TRT", "CTR","TRTAMI") AND  ^CGTS126 > (((^CGTS118 - ^CGTS123) *0.5) +1)

        Data Elements:
            
                    RP.Applied:^CGTS126 = tns:TaxConcession.CapitalGains.DiscountTotal.Amount
            
                    RP:^CGTS118 = tns:Capital.Gains.Total.Amount
            
                    RP.Applied:^CGTS123 = tns:Capital.Losses.Total.Amount
                    */
        private void VRATOCGTS402174(List<ProcessMessageDocument> response )
        {
            bool assertion = ChildDocument.CGTS126.GetValueOrDefault() > (ChildDocument.CGTS118.GetValueOrDefault() - ChildDocument.CGTS123.GetValueOrDefault()) * (decimal)0.5 + 1;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402174",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The CGT discount rate applied to capital gains by a trust or company must not be greater than 50%",
                    Location = "/xbrli:xbrl/tns:TaxConcession.CapitalGains.DiscountTotal.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402174" } }
                };

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402174
        
        #region VR.ATO.CGTS.402230

        /*  VR.ATO.CGTS.402230
                    A multi-class treatment can only be reported with a Trust Tax Return for Attribution Managed Investments (TRTAMI) when a multi-class election has been made

                Legacy Rule Format:
                    IF PARENT RETURN  <> "TRTAMI" AND
                    ([CGTS153] <> NULL OR [CGTS154] <> NULL OR [CGTS155] <> NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    PARENT  <> 'TRTAMI' AND (^CGTS153 <> NULL OR ^CGTS154 <> NULL OR ^CGTS155 <> NULL)

        Data Elements:
            
                    RP:^CGTS153 = tns:IncomeTax.AttributionManagedInvestmentTrustClass.Description
            
                    RP.TransfersToOtherClasses:^CGTS154 = tns:Capital.Gains.Total.Amount
            
                    RP.TransfersToOtherClasses:^CGTS155 = tns:Capital.Losses.Total.Amount
                    */
        private void VRATOCGTS402230(List<ProcessMessageDocument> response )
        {
            bool assertion = ! string.IsNullOrEmpty(ChildDocument.CGTS153) ||  ChildDocument.CGTS154.HasValue  || ChildDocument.CGTS155.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402230",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Multi-class treatment fields incorrectly supplied",
                    Location = "/xbrli:xbrl/tns:IncomeTax.AttributionManagedInvestmentTrustClass.Description",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402230" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402230
        
        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        private void VRATOGEN438000(List<ProcessMessageDocument> response )
        {
            response.AddRange(from context in Contexts
                              where context.StartDate .GetValueOrDefault() != ParentDocument.RPStartDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438000",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = @"The context period start date is incorrect.",
                                  LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:startDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } }
                              });
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.CGTS.438001
        /*  VR.ATO.CGTS.438001
            IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOCGTS438001(List<ProcessMessageDocument> response)
        {
            response.AddRange(from context in Contexts
                              where context.EndDate.GetValueOrDefault() != ParentDocument.RPEndDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438001",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = @"The context period end date is incorrect.",
                                  LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:endDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.438001" } }
                              });
        }
        #endregion // VR.ATO.CGTS.438001

        #region VR.ATO.CGTS.402171

        /*  VR.ATO.CGTS.402171
                    A capital loss must not be carried forward when a positive net capital gain amount has been provided

                Legacy Rule Format:
                    IF PARENT RETURN = SET("SMSFAR","FITR","TRT","TRTAMI") AND ([CGTS131] > 0) AND ([CGTS125]  > 0) 
                       RETURN VALIDATION MESSAGE 
                    ENDIF

            Technical Business Rule Format:
                    PARENT = SET("SMSFAR","FITR","TRT","TRTAMI") AND (^CGTS131 > 0) AND (^CGTS125  > 0)

        Data Elements:
            
                    RP.CGTOther.Unapplied(Instant):^CGTS125 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP:^CGTS131 = tns:Income.CapitalGainsNet.Amount
                    */
        private void VRATOCGTS402171(List<ProcessMessageDocument> response )
        {
            bool assertion = ChildDocument .CGTS131.GetValueOrDefault() > 0 && ChildDocument.CGTS125.GetValueOrDefault() > 0;

            if (assertion)
            {
               var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402171",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"A capital loss must not be carried forward when a positive net capital gain amount has been provided",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount[@contextRef='" + ChildDocument.RPCGTOtherUnappliedInstantId + "']",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402171" } }
                };

                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402171

        #region VR.ATO.CGTS.402226

        /*  VR.ATO.CGTS.402226
                    If the Amendment indicator on a parent return is set to TRUE, then the Income year earnout right created and Amount to be amended cannot be supplied.

                Legacy Rule Format:
                    IF (PARENT RETURN:RP:pyin.xx.xx:Report.Amendment.Indicator = TRUE) AND ([CGTS150] <> NULL AND [CGTS151] <> NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (PARENT:pyin.66.66:Report.Amendment.Indicator = TRUE) AND (^CGTS150 <> NULL AND ^CGTS151 <> NULL)

        Data Elements:
            
                    RP:^CGTS150 = tns:Report.TargetFinancial.Year
            
                    RP:^CGTS151 = tns:Miscellaneous.RequestedAmendment.Amount
                    */
        private void VRATOCGTS402226(List<ProcessMessageDocument> response )
        {
            bool assertion =  ParentDocument.TRT414.GetValueOrDefault()  && ChildDocument.CGTS150.HasValue && ChildDocument.CGTS151.HasValue ;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402226",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year earnout right created and Amount to be amended cannot be supplied when Parent return Amendment indicator is present.",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402226" } }
                }; 
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402226

        #region VR.ATO.CGTS.402240

        /*  VR.ATO.CGTS.402240
                    The year entered at Income year earnout right created cannot be the current year.

                Legacy Rule Format:
                    IF ([CGTS150] <> NULL) AND ([CGTS150] = (PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year))
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^CGTS150 <> NULL) AND (^CGTS150 = ^ParentTargetFinancialYear)

        Data Elements:
            
                    RP:^CGTS150 = tns:Report.TargetFinancial.Year
            
                    RP:^ParentTargetFinancialYear = ParentTargetFinancialYear
                    */
        private void VRATOCGTS402240(List<ProcessMessageDocument> response )
        {
            bool assertion =  ChildDocument.CGTS150.HasValue && ChildDocument.CGTS150.GetValueOrDefault().Equals( ParentDocument .TRT1.GetValueOrDefault());

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.CGTS.402240",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year earnout right created incorrect",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402240" },
                        new ProcessMessageParameter { Name = "CGTS150", Value = ChildDocument .CGTS150.GetValueOrDefault().ToString() }
                    }
                }; 

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CGTS.402240
    }
}