﻿using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRTAMI;
using Ato.EN.IntegrationServices.CodeGenerationTRTAMIS;

namespace Ato.CD.Inbound.TRTAMI201802
{
    internal class CrossFormValidatorTRTAMIS : ICrossFormValidator
    {

        private TRTAMI2018 ParentDocument { get; }
        private TRTAMIS2018 ChildDocument { get; }
        private List<Context> Contexts { get; }

        internal CrossFormValidatorTRTAMIS(TRTAMI2018 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (TRTAMIS2018)childDocument.ConsumedReport;
            Contexts = ChildDocument.GetContexts();
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();
            
            VRATOTRTAMI000237(response);
            VRATOGEN402009(response);
            VRATOGEN438000(response);
            VRATOGEN438001(response);
            VRATOTRTAMIS000001(response);
            VRATOGEN402010(response);
            
            return response;
        }

        #region VR.ATO.TRTAMI.000237
        //Rule ID: VR.ATO.TRTAMI.000237

        //    IF[TRTAMI102] = TRUE AND ANY OCCURRENCE OF([TRTAMIS313]) = NULLORBLANK
        //     RETURN VALIDATION MESSAGE
        //    ENDIF
        private void VRATOTRTAMI000237(List<ProcessMessageDocument> response)
        {
            bool assertion = string.IsNullOrWhiteSpace(ChildDocument.TRTAMIS313) && ParentDocument.TRTAMI102.GetValueOrDefault();

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMI.000237",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Name of AMIT Class must be provided for each TRTAMIS schedule",
                    LongDescription = "When the response to 'Have you chosen to treat separate classes in the trust as a separate AMIT?' is 'True' (yes), the Name of AMIT Class must be provided for each Trust Attribution Managed Investments Schedule (TRTAMIS)",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MultiClassTreatment.Indicator",
                    Parameters = new ProcessMessageParameters {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMI.000237" },
                        new ProcessMessageParameter { Name = "TRTAMI102", Value = ParentDocument.TRTAMI102.ToString() }
                    }
                };
                response.Add(processMessage);
            }

        }
        #endregion

        #region VR.ATO.GEN.402009
        /* VR.ATO.GEN.402009
            IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                RETURN VALIDATION MESSAGE
            ENDIF 
        */
        private void VRATOGEN402009(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = "Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                });
            }
        }
        #endregion // VR.ATO.GEN.402009

        #region VR.ATO.GEN.438000
        /*  VR.ATO.GEN.438000
            IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOGEN438000(List<ProcessMessageDocument> response)
        {
           
            response.AddRange(from context in Contexts
                              where context.StartDate.GetValueOrDefault() != ParentDocument.RPStartDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438000",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = "The context period start date is incorrect.",
                                  LongDescription = "Period start date within context on the schedule must match period start date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:startDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } }
                              });
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438001
        /*  VR.ATO.GEN.438001
            IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOGEN438001(List<ProcessMessageDocument> response)
        { 
            response.AddRange(from context in Contexts
                              where context.EndDate.GetValueOrDefault() != ParentDocument.RPEndDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438001",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = "The context period end date is incorrect.",
                                  LongDescription = "Period end date within context on the schedule must match period end date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:endDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } }
                              });
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.TRTAMIS.000001
        //Rule ID: VR.ATO.TRTAMIS.000001
        //    IF[TRTAMIS4] <> NULL AND[TRTAMIS4] <> PARENT RETURN:pyin.xx.xx:Report.TargetFinancial.Year
        //    RETURN VALIDATION MESSAGE
        //    ENDIF
        private void VRATOTRTAMIS000001(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.TRTAMIS4 != null && ChildDocument.TRTAMIS4 != ParentDocument.TRTAMI1;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRTAMIS.000001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "The Target Financial Year must match the Year shown on the parent form.",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRTAMIS.000001" } }
                });
            }
        }
        #endregion

        #region VR.ATO.GEN.402010
        /* VR.ATO.GEN.402009
            IF (RP:tns:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK)
                AND (PARENT RETURN:RP:tns:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK)
                AND (RP:tns:Identifiers.AustralianBusinessNumber.Identifier <> PARENT RETURN:RP:tns:Identifiers.AustralianBusinessNumber.Identifier)
                RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOGEN402010(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ParentDocument.TRTAMI5) &&
                             !string.IsNullOrWhiteSpace(ChildDocument.TRTAMIS5) &&
                             !ParentDocument.TRTAMI5.Equals(ChildDocument.TRTAMIS5);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Your supplied ABN does not match the ABN supplied on the form it was submitted with",
                    LongDescription = "Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402010" } }
                });
            }
        }
        #endregion // VR.ATO.GEN.402010
    }
}