using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Text.RegularExpressions;
using Au.Gov.Sbr.Xbrl.Document;
using DataContracts;
using Au.Gov.Sbr.Xbrl.Metadata;

namespace Ato.EN.IntegrationServices.CodeGenerationTRTAMI
{
    public class TRTAMI2018Consumer
    {

        private static Regex StripWhitespace = new Regex(@"\s*");

        #region Helper/Validation Methods
        private static bool? ToBoolean(string str)
        {
            bool returnValue;
            string value = (str ?? "").Trim();
            if (value == "0" || value == "1")
                return Convert.ToBoolean(Convert.ToInt32(value));

            if (Boolean.TryParse(value, out returnValue))
                return returnValue;
            return null;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(DateTime? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Derives the SBR element location.
        /// </summary>
        /// <param name="sbrElement">The SBR element.</param>
        /// <param name="addOccurrenceIndex">if set to <c>true</c> [add occurrence the occurrence index to the path].</param>
        /// <returns></returns>
        private static string DeriveSbrElementLocation(SBRElement sbrElement, bool addOccurrenceIndex = false)
        {
            if (sbrElement == null) return "/xbrli:xbrl";
            int xpathOccurrence = sbrElement.OccurrenceIndex + 1;
            string location;

            if (addOccurrenceIndex)
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name + "[" +
                           xpathOccurrence + "]";
            }
            else
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name;
            }

            if (sbrElement.IsInTuple)
            {
                location = DeriveSbrElementLocation(sbrElement.ParentTuple, true) + location;
            }
            else
            {
                location = "/xbrli:xbrl" + location;
            }

            return location;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

        #endregion

        public TRTAMI2018 Consume(SBRDocument sbrDocument)
        {
            List<ProcessMessageDocument> errors;
            return Consume(sbrDocument, false, out errors);
        }

        /// <summary>
        /// Allows for validation of rules that can not be generated and must run at a lower level.
        /// A sub-class will need to be created with an override for this method - which does nothing in its base implementation.
        /// </summary>
        /// <param name="sbrDocument">The SBR document.</param>
        /// <param name="populatedReportElementsMap">The populated report elements map can be used to find data elements and tuples.</param>
        /// <param name="contextArray">The context array can be used to find context values.</param>
        /// <param name="errors">The errors collection is to be populated with validation results.</param>
        protected virtual void ValidateUnsupportedRules(SBRDocument sbrDocument, IDictionary<string, SBRElement> populatedReportElementsMap, SBRContext[] contextArray, List<ProcessMessageDocument> errors)
        {

        }

        public TRTAMI2018 Consume(SBRDocument sbrDocument, bool validationMode, out List<ProcessMessageDocument> errors)
        {
            TRTAMI2018 report;
            report = new TRTAMI2018();
            errors = new List<ProcessMessageDocument>();

            SBRContext[] contextArray;
            IDictionary<string, SBRElement> populatedReportElementsMap;

            contextArray = sbrDocument.ContextsMap.Values.ToArray();
            populatedReportElementsMap = sbrDocument.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

            if (validationMode) ValidateUnsupportedRules(sbrDocument, populatedReportElementsMap, contextArray, errors);

            /// <summary>
            /// Current element being extracted
            /// </summary>
            SBRElement currentValue = null;
            /// <summary>
            /// Current elements being extracted
            /// </summary>
            IList<SBRElement> currentValues = null;
            /// <summary>
            /// Current occurrences being extracted
            /// </summary>
            IList<SBRElement> currentOccurrences = null;
            /// <summary>
            /// Working Variable for date time variables
            /// </summary>
            DateTime currentDateTimeValue;
            /// <summary>
            /// Working Variable for boolean variables
            /// </summary>
            bool currentBooleanValue;
            /// <summary>
            /// Working Variable for decimal variables
            /// </summary>
            decimal currentDecimalValue;
            /// <summary>
            /// Working Variable for integer variables
            /// </summary>
            int currentIntValue;
            /// <summary>
            /// Working Variable for long variables
            /// </summary>
            long currentLongValue;
            XmlDocumentFragment currentSegment;
            XmlNamespaceManager currentNamespaceManager;
            XmlNode currentNode;

            #region sbrRPContext
            IEnumerable<SBRContext> sbrRPContexts = null;
            SBRContext sbrRPContext = null;

            sbrRPContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPContexts != null && sbrRPContexts.Count() > 0)
            {
                sbrRPContext = sbrRPContexts.First();

                report.RPId = sbrRPContext.Id;
                report.RPIdentifier = sbrRPContext.EntityIdentifier;
                report.RPIdentifierScheme = sbrRPContext.EntityScheme;

                report.RPCount = sbrRPContexts.Count();
                report.RPExists = true;
                report.RPOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContext) + 1;
                report.RPLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContexts.Last()) + 1;


                if (report.RPIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPIdentifierTFN = report.RPIdentifier;

                report.RPStartDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationStartAsDateObject;
                report.RPEndDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPContexts.Count(); i++)
                {
                    sbrRPContext = sbrRPContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }
                    report.TRTAMI2 = report.RPStartDate;
                    report.TRTAMI3 = report.RPEndDate;
                    report.TRTAMI4 = report.RPIdentifier;

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Report.TargetFinancial.Year", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI1 = int.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Identifiers.AustralianBusinessNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI5 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Report.Amendment.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI6 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Report.AmendmentType.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI7 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Report.AmendmentReason.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI8 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Report.AmendmentSequence.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI9 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}OrganisationNameDetails.OrganisationalName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI10 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.TrustComponentDeficitOffset.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI35 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.TrustShortfallIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI36 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.TrustExcessOffset.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI39 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.TrustComponentNotReflected.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI40 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}FinancialInstitutionAccount.BankStateBranch.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI41 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}FinancialInstitutionAccount.FinancialInstitutionAccount.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI42 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}FinancialInstitutionAccount.FinancialInstitutionAccountName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI43 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}OrganisationDetails.MainIncomeActivity.Description", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI44 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}OrganisationDetails.OrganisationIndustry2006Extended.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI45 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Income.Interest.EarlyPaymentCredit.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI46 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI47 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI48 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Expense.Interest.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI49 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Expense.Royalties.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI50 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}InternationalDealings.TransactionsWithTaxHavenCountries.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI137 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}InternationalDealings.TransactionControlTaxHavenCountries.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI138 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Income.TrustEstateIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI73 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.MultiClassTreatment.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI102 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.MultiClassTreatment.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI103 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.MultiClassTransfer.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI104 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Elections.ManagedInvestmentTrusts.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI32 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.AttributionManagedInvestmentTrustStapledStructure.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI112 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Assets.PropertyPlantAndEquipment.DepreciatingAssets.FirstDeductedSelfAssessedEffectiveLife.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI63 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Assets.PropertyPlantAndEquipment.DepreciatingAssets.EffectiveLifeRecalculation.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI64 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Expense.DepreciationAllowableDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI118 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI71 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Expense.CapitalWorksDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI140 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Expense.DividendInterestRoyalty.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI121 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Income.TrustDistributions.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI108 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Expense.ReturnsPaidDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI111 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.Deduction.EligibleInvestmentBusiness.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI113 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.Deduction.CreditsEligibleTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI114 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.RentalSafeHarbour.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI116 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.EligibleInvestmentBusinessSafeHarbour.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI115 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.Deduction.NonEligibleInvestmentBusinessPercentage.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI141 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.AttributionManagedInvestmentTrustRelatedEntitiesPayment.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI117 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.RelatedEntitiesPaymentsReceived.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI142 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Report.Statement.Summary.PaymentRefundOrNil.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI33 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Lodgment.FinalReturn.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI34 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI101 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Report.CompletionHours.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI74 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPContext

            #region sbrRPInstantContext
            IEnumerable<SBRContext> sbrRPInstantContexts = null;
            SBRContext sbrRPInstantContext = null;

            sbrRPInstantContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.INSTANT &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPInstantElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPInstantContexts != null && sbrRPInstantContexts.Count() > 0)
            {
                sbrRPInstantContext = sbrRPInstantContexts.First();

                report.RPInstantId = sbrRPInstantContext.Id;
                report.RPInstantIdentifier = sbrRPInstantContext.EntityIdentifier;
                report.RPInstantIdentifierScheme = sbrRPInstantContext.EntityScheme;

                report.RPInstantCount = sbrRPInstantContexts.Count();
                report.RPInstantExists = true;
                report.RPInstantOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInstantContext) + 1;
                report.RPInstantLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInstantContexts.Last()) + 1;


                if (report.RPInstantIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPInstantIdentifierTFN = report.RPInstantIdentifier;

                report.RPInstantInstantDate = ((SBRInstantPeriod)sbrRPInstantContext.Period).InstantAsDateObject;

                sbrPopulatedRPInstantElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPInstantContexts.Count(); i++)
                {
                    sbrRPInstantContext = sbrRPInstantContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPInstantContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPInstantElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPInstantElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPInstantElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPInstantElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Assets.CurrentTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI57 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPInstantElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Assets.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI58 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPInstantElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Liabilities.CurrentTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI59 = decimal.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPInstantElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Liabilities.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI60 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPInstantContext

            #region sbrRPPOSContext
            IEnumerable<SBRContext> sbrRPPOSContexts = null;
            SBRContext sbrRPPOSContext = null;

            sbrRPPOSContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}AddressUsageDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}POS")))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPPOSElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPPOSContexts != null && sbrRPPOSContexts.Count() > 0)
            {
                sbrRPPOSContext = sbrRPPOSContexts.First();

                report.RPPOSId = sbrRPPOSContext.Id;
                report.RPPOSIdentifier = sbrRPPOSContext.EntityIdentifier;
                report.RPPOSIdentifierScheme = sbrRPPOSContext.EntityScheme;

                report.RPPOSCount = sbrRPPOSContexts.Count();
                report.RPPOSExists = true;
                report.RPPOSOccurrenceIndex = Array.IndexOf(contextArray, sbrRPPOSContext) + 1;
                report.RPPOSLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPPOSContexts.Last()) + 1;


                if (report.RPPOSIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPPOSIdentifierTFN = report.RPPOSIdentifier;

                report.RPPOSStartDate = ((SBRDurationPeriod)sbrRPPOSContext.Period).DurationStartAsDateObject;
                report.RPPOSEndDate = ((SBRDurationPeriod)sbrRPPOSContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPPOSElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPPOSContexts.Count(); i++)
                {
                    sbrRPPOSContext = sbrRPPOSContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPPOSContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPPOSElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPPOSElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPPOSElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPPOSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Line1.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI12 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Line2.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI13 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.LocalityName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI14 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.StateOrTerritory.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI15 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Postcode.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI16 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Country.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI17 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrRPPOSContext

            #region sbrRPPOSPrevContext
            IEnumerable<SBRContext> sbrRPPOSPrevContexts = null;
            SBRContext sbrRPPOSPrevContext = null;

            sbrRPPOSPrevContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}AddressUsageDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}POS")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}DataCurrencyDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Previous"))))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPPOSPrevElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPPOSPrevContexts != null && sbrRPPOSPrevContexts.Count() > 0)
            {
                sbrRPPOSPrevContext = sbrRPPOSPrevContexts.First();

                report.RPPOSPrevId = sbrRPPOSPrevContext.Id;
                report.RPPOSPrevIdentifier = sbrRPPOSPrevContext.EntityIdentifier;
                report.RPPOSPrevIdentifierScheme = sbrRPPOSPrevContext.EntityScheme;

                report.RPPOSPrevCount = sbrRPPOSPrevContexts.Count();
                report.RPPOSPrevExists = true;
                report.RPPOSPrevOccurrenceIndex = Array.IndexOf(contextArray, sbrRPPOSPrevContext) + 1;
                report.RPPOSPrevLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPPOSPrevContexts.Last()) + 1;


                if (report.RPPOSPrevIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPPOSPrevIdentifierTFN = report.RPPOSPrevIdentifier;

                report.RPPOSPrevStartDate = ((SBRDurationPeriod)sbrRPPOSPrevContext.Period).DurationStartAsDateObject;
                report.RPPOSPrevEndDate = ((SBRDurationPeriod)sbrRPPOSPrevContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPPOSPrevElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPPOSPrevContexts.Count(); i++)
                {
                    sbrRPPOSPrevContext = sbrRPPOSPrevContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPPOSPrevContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPPOSPrevElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPPOSPrevElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPPOSPrevElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPPOSPrevElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Line1.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI18 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSPrevElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Line2.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI19 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSPrevElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.LocalityName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI20 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSPrevElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.StateOrTerritory.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI21 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSPrevElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Postcode.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI22 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPPOSPrevElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Country.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI23 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrRPPOSPrevContext

            #region sbrRPPrevContext
            IEnumerable<SBRContext> sbrRPPrevContexts = null;
            SBRContext sbrRPPrevContext = null;

            sbrRPPrevContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}DataCurrencyDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Previous")))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPPrevElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPPrevContexts != null && sbrRPPrevContexts.Count() > 0)
            {
                sbrRPPrevContext = sbrRPPrevContexts.First();

                report.RPPrevId = sbrRPPrevContext.Id;
                report.RPPrevIdentifier = sbrRPPrevContext.EntityIdentifier;
                report.RPPrevIdentifierScheme = sbrRPPrevContext.EntityScheme;

                report.RPPrevCount = sbrRPPrevContexts.Count();
                report.RPPrevExists = true;
                report.RPPrevOccurrenceIndex = Array.IndexOf(contextArray, sbrRPPrevContext) + 1;
                report.RPPrevLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPPrevContexts.Last()) + 1;


                if (report.RPPrevIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPPrevIdentifierTFN = report.RPPrevIdentifier;

                report.RPPrevStartDate = ((SBRDurationPeriod)sbrRPPrevContext.Period).DurationStartAsDateObject;
                report.RPPrevEndDate = ((SBRDurationPeriod)sbrRPPrevContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPPrevElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPPrevContexts.Count(); i++)
                {
                    sbrRPPrevContext = sbrRPPrevContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPPrevContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPPrevElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPPrevElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPPrevElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPPrevElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}OrganisationNameDetails.OrganisationalName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI11 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrRPPrevContext

            #region sbrRPTrusteeContext
            IEnumerable<SBRContext> sbrRPTrusteeContexts = null;
            SBRContext sbrRPTrusteeContext = null;

            sbrRPTrusteeContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}SecondaryPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Trustee")))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTrusteeElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTrusteeContexts != null && sbrRPTrusteeContexts.Count() > 0)
            {
                sbrRPTrusteeContext = sbrRPTrusteeContexts.First();

                report.RPTrusteeId = sbrRPTrusteeContext.Id;
                report.RPTrusteeIdentifier = sbrRPTrusteeContext.EntityIdentifier;
                report.RPTrusteeIdentifierScheme = sbrRPTrusteeContext.EntityScheme;

                report.RPTrusteeCount = sbrRPTrusteeContexts.Count();
                report.RPTrusteeExists = true;
                report.RPTrusteeOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTrusteeContext) + 1;
                report.RPTrusteeLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTrusteeContexts.Last()) + 1;


                if (report.RPTrusteeIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTrusteeIdentifierTFN = report.RPTrusteeIdentifier;

                report.RPTrusteeStartDate = ((SBRDurationPeriod)sbrRPTrusteeContext.Period).DurationStartAsDateObject;
                report.RPTrusteeEndDate = ((SBRDurationPeriod)sbrRPTrusteeContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPTrusteeElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTrusteeContexts.Count(); i++)
                {
                    sbrRPTrusteeContext = sbrRPTrusteeContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTrusteeContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTrusteeElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTrusteeElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTrusteeElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPTrusteeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.Title.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI24 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrusteeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.FamilyName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI25 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrusteeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.GivenName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI26 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrusteeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.OtherGivenName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI27 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrusteeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}OrganisationNameDetails.OrganisationalName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI28 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrusteeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Identifiers.AustralianBusinessNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI29 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrusteeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}ElectronicContact.Telephone.Area.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI30 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrusteeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}ElectronicContact.Telephone.Minimal.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI31 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrRPTrusteeContext

            #region sbrRPStapledEntityContext
            IEnumerable<SBRContext> sbrRPStapledEntityContexts = null;
            SBRContext sbrRPStapledEntityContext = null;

            sbrRPStapledEntityContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}SecondaryPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}StapledEntity")))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPStapledEntityElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPStapledEntityContexts != null && sbrRPStapledEntityContexts.Count() > 0)
            {
                sbrRPStapledEntityContext = sbrRPStapledEntityContexts.First();

                report.RPStapledEntityId = sbrRPStapledEntityContext.Id;
                report.RPStapledEntityIdentifier = sbrRPStapledEntityContext.EntityIdentifier;
                report.RPStapledEntityIdentifierScheme = sbrRPStapledEntityContext.EntityScheme;

                report.RPStapledEntityCount = sbrRPStapledEntityContexts.Count();
                report.RPStapledEntityExists = true;
                report.RPStapledEntityOccurrenceIndex = Array.IndexOf(contextArray, sbrRPStapledEntityContext) + 1;
                report.RPStapledEntityLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPStapledEntityContexts.Last()) + 1;


                if (report.RPStapledEntityIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPStapledEntityIdentifierTFN = report.RPStapledEntityIdentifier;

                report.RPStapledEntityStartDate = ((SBRDurationPeriod)sbrRPStapledEntityContext.Period).DurationStartAsDateObject;
                report.RPStapledEntityEndDate = ((SBRDurationPeriod)sbrRPStapledEntityContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPStapledEntityElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPStapledEntityContexts.Count(); i++)
                {
                    sbrRPStapledEntityContext = sbrRPStapledEntityContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPStapledEntityContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPStapledEntityElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPStapledEntityElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPStapledEntityElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPStapledEntityElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Identifiers.AustralianBusinessNumber.Identifier", out currentValues))
                    {
                        report.TRTAMI139Collection = new List<string>();
                        try
                        {
                            foreach (SBRElement occurrence in currentValues)
                            {
                                if (!occurrence.IsNil)
                                    report.TRTAMI139Collection.Add(occurrence.Value);
                            }
                        }
                        catch (Exception)
                        {
                            // The above code is known to throw exceptions in some cases but not others.
                            // We don't really know why but think that catching them like this will result in correct behaviour
                        }
                    }
                }
            }
            #endregion End of Context sbrRPStapledEntityContext

            #region sbrRPSoASeqNumContext
            IEnumerable<SBRContext> sbrRPSoASeqNumContexts = null;
            SBRContext sbrRPSoASeqNumContext = null;

            sbrRPSoASeqNumContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.TYPED && dd.Dimension.EndsWith("}SequenceNumberDimension")))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPSoASeqNumElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPSoASeqNumContexts != null && sbrRPSoASeqNumContexts.Count() > 0)
            {

                report.RPSoASeqNumContextCollection = new List<TRTAMI2018.RPSoASeqNumContext>();
                for (int i = 0; i < sbrRPSoASeqNumContexts.Count(); i++)
                {
                    sbrRPSoASeqNumContext = sbrRPSoASeqNumContexts.ElementAt(i);
                    TRTAMI2018.RPSoASeqNumContext rpSoASeqNumContext = new TRTAMI2018.RPSoASeqNumContext();
                    report.RPSoASeqNumContextCollection.Add(rpSoASeqNumContext);

                    rpSoASeqNumContext.Id = sbrRPSoASeqNumContext.Id;
                    rpSoASeqNumContext.Identifier = sbrRPSoASeqNumContext.EntityIdentifier;
                    rpSoASeqNumContext.IdentifierScheme = sbrRPSoASeqNumContext.EntityScheme;

                    rpSoASeqNumContext.Count = sbrRPSoASeqNumContexts.Count();
                    rpSoASeqNumContext.Exists = true;
                    rpSoASeqNumContext.OccurrenceIndex = Array.IndexOf(contextArray, sbrRPSoASeqNumContext) + 1;
                    rpSoASeqNumContext.LastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPSoASeqNumContexts.Last()) + 1;


                    if (rpSoASeqNumContext.IdentifierScheme == "http://www.ato.gov.au/tfn") rpSoASeqNumContext.IdentifierTFN = rpSoASeqNumContext.Identifier;

                    rpSoASeqNumContext.StartDate = ((SBRDurationPeriod)sbrRPSoASeqNumContext.Period).DurationStartAsDateObject;
                    rpSoASeqNumContext.EndDate = ((SBRDurationPeriod)sbrRPSoASeqNumContext.Period).DurationEndAsDateObject;

                    currentSegment = sbrRPSoASeqNumContext.Segment.Segment;

                    currentNamespaceManager = new XmlNamespaceManager(currentSegment.OwnerDocument.NameTable);
                    currentNamespaceManager.AddNamespace("xbrldi", "http://xbrl.org/2006/xbrldi");

                    currentNode = currentSegment.SelectSingleNode("xbrldi:typedMember[contains(@dimension, 'SequenceNumberDimension')]", currentNamespaceManager);

                    if (currentNode != null)
                    {
                        rpSoASeqNumContext.SequenceNumberDimensionValue = currentNode.InnerText.Trim();
                    }

                    sbrPopulatedRPSoASeqNumElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPSoASeqNumContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPSoASeqNumElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPSoASeqNumElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPSoASeqNumElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPSoASeqNumElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Identifiers.TaxFileNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumContext.TRTAMI75 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.Title.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumContext.TRTAMI76 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.FamilyName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumContext.TRTAMI77 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.GivenName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumContext.TRTAMI78 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.OtherGivenName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumContext.TRTAMI79 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonDemographicDetails.Birth.Date", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumContext.TRTAMI124 = DateTime.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPSoASeqNumElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}OrganisationNameDetails.OrganisationalName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumContext.TRTAMI80 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}IncomeTax.AssessmentCalculation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumContext.TRTAMI81 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}InternationalDealings.AssessableIncomeNonResidentBeneficiary.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumContext.TRTAMI82 = decimal.Parse(currentValues[0].Value);
                    }
                }
            }
            #endregion End of Context sbrRPSoASeqNumContext

            #region sbrRPSoASeqNumBUSContext
            IEnumerable<SBRContext> sbrRPSoASeqNumBUSContexts = null;
            SBRContext sbrRPSoASeqNumBUSContext = null;

            sbrRPSoASeqNumBUSContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.TYPED && dd.Dimension.EndsWith("}SequenceNumberDimension")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}AddressUsageDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}BUS"))))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPSoASeqNumBUSElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPSoASeqNumBUSContexts != null && sbrRPSoASeqNumBUSContexts.Count() > 0)
            {

                report.RPSoASeqNumBUSContextCollection = new List<TRTAMI2018.RPSoASeqNumBUSContext>();
                for (int i = 0; i < sbrRPSoASeqNumBUSContexts.Count(); i++)
                {
                    sbrRPSoASeqNumBUSContext = sbrRPSoASeqNumBUSContexts.ElementAt(i);
                    TRTAMI2018.RPSoASeqNumBUSContext rpSoASeqNumBUSContext = new TRTAMI2018.RPSoASeqNumBUSContext();
                    report.RPSoASeqNumBUSContextCollection.Add(rpSoASeqNumBUSContext);

                    rpSoASeqNumBUSContext.Id = sbrRPSoASeqNumBUSContext.Id;
                    rpSoASeqNumBUSContext.Identifier = sbrRPSoASeqNumBUSContext.EntityIdentifier;
                    rpSoASeqNumBUSContext.IdentifierScheme = sbrRPSoASeqNumBUSContext.EntityScheme;

                    rpSoASeqNumBUSContext.Count = sbrRPSoASeqNumBUSContexts.Count();
                    rpSoASeqNumBUSContext.Exists = true;
                    rpSoASeqNumBUSContext.OccurrenceIndex = Array.IndexOf(contextArray, sbrRPSoASeqNumBUSContext) + 1;
                    rpSoASeqNumBUSContext.LastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPSoASeqNumBUSContexts.Last()) + 1;


                    if (rpSoASeqNumBUSContext.IdentifierScheme == "http://www.ato.gov.au/tfn") rpSoASeqNumBUSContext.IdentifierTFN = rpSoASeqNumBUSContext.Identifier;

                    rpSoASeqNumBUSContext.StartDate = ((SBRDurationPeriod)sbrRPSoASeqNumBUSContext.Period).DurationStartAsDateObject;
                    rpSoASeqNumBUSContext.EndDate = ((SBRDurationPeriod)sbrRPSoASeqNumBUSContext.Period).DurationEndAsDateObject;

                    currentSegment = sbrRPSoASeqNumBUSContext.Segment.Segment;

                    currentNamespaceManager = new XmlNamespaceManager(currentSegment.OwnerDocument.NameTable);
                    currentNamespaceManager.AddNamespace("xbrldi", "http://xbrl.org/2006/xbrldi");

                    currentNode = currentSegment.SelectSingleNode("xbrldi:typedMember[contains(@dimension, 'SequenceNumberDimension')]", currentNamespaceManager);

                    if (currentNode != null)
                    {
                        rpSoASeqNumBUSContext.SequenceNumberDimensionValue = currentNode.InnerText.Trim();
                    }

                    sbrPopulatedRPSoASeqNumBUSElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPSoASeqNumBUSContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPSoASeqNumBUSElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPSoASeqNumBUSElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPSoASeqNumBUSElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPSoASeqNumBUSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Line1.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumBUSContext.TRTAMI131 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumBUSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Line2.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumBUSContext.TRTAMI132 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumBUSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.LocalityName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumBUSContext.TRTAMI133 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumBUSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.PostalCode.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumBUSContext.TRTAMI144 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumBUSElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Country.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumBUSContext.TRTAMI136 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrRPSoASeqNumBUSContext

            #region sbrRPSoASeqNumRESContext
            IEnumerable<SBRContext> sbrRPSoASeqNumRESContexts = null;
            SBRContext sbrRPSoASeqNumRESContext = null;

            sbrRPSoASeqNumRESContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.TYPED && dd.Dimension.EndsWith("}SequenceNumberDimension")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}AddressUsageDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RES"))))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPSoASeqNumRESElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPSoASeqNumRESContexts != null && sbrRPSoASeqNumRESContexts.Count() > 0)
            {

                report.RPSoASeqNumRESContextCollection = new List<TRTAMI2018.RPSoASeqNumRESContext>();
                for (int i = 0; i < sbrRPSoASeqNumRESContexts.Count(); i++)
                {
                    sbrRPSoASeqNumRESContext = sbrRPSoASeqNumRESContexts.ElementAt(i);
                    TRTAMI2018.RPSoASeqNumRESContext rpSoASeqNumRESContext = new TRTAMI2018.RPSoASeqNumRESContext();
                    report.RPSoASeqNumRESContextCollection.Add(rpSoASeqNumRESContext);

                    rpSoASeqNumRESContext.Id = sbrRPSoASeqNumRESContext.Id;
                    rpSoASeqNumRESContext.Identifier = sbrRPSoASeqNumRESContext.EntityIdentifier;
                    rpSoASeqNumRESContext.IdentifierScheme = sbrRPSoASeqNumRESContext.EntityScheme;

                    rpSoASeqNumRESContext.Count = sbrRPSoASeqNumRESContexts.Count();
                    rpSoASeqNumRESContext.Exists = true;
                    rpSoASeqNumRESContext.OccurrenceIndex = Array.IndexOf(contextArray, sbrRPSoASeqNumRESContext) + 1;
                    rpSoASeqNumRESContext.LastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPSoASeqNumRESContexts.Last()) + 1;


                    if (rpSoASeqNumRESContext.IdentifierScheme == "http://www.ato.gov.au/tfn") rpSoASeqNumRESContext.IdentifierTFN = rpSoASeqNumRESContext.Identifier;

                    rpSoASeqNumRESContext.StartDate = ((SBRDurationPeriod)sbrRPSoASeqNumRESContext.Period).DurationStartAsDateObject;
                    rpSoASeqNumRESContext.EndDate = ((SBRDurationPeriod)sbrRPSoASeqNumRESContext.Period).DurationEndAsDateObject;

                    currentSegment = sbrRPSoASeqNumRESContext.Segment.Segment;

                    currentNamespaceManager = new XmlNamespaceManager(currentSegment.OwnerDocument.NameTable);
                    currentNamespaceManager.AddNamespace("xbrldi", "http://xbrl.org/2006/xbrldi");

                    currentNode = currentSegment.SelectSingleNode("xbrldi:typedMember[contains(@dimension, 'SequenceNumberDimension')]", currentNamespaceManager);

                    if (currentNode != null)
                    {
                        rpSoASeqNumRESContext.SequenceNumberDimensionValue = currentNode.InnerText.Trim();
                    }

                    sbrPopulatedRPSoASeqNumRESElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPSoASeqNumRESContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPSoASeqNumRESElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPSoASeqNumRESElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPSoASeqNumRESElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }

                    if (sbrPopulatedRPSoASeqNumRESElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Line1.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumRESContext.TRTAMI125 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumRESElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Line2.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumRESContext.TRTAMI126 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumRESElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.LocalityName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumRESContext.TRTAMI127 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumRESElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.PostalCode.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumRESContext.TRTAMI143 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPSoASeqNumRESElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}AddressDetails.Country.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpSoASeqNumRESContext.TRTAMI130 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrRPSoASeqNumRESContext

            #region sbrRPTrueAndCorrectContext
            IEnumerable<SBRContext> sbrRPTrueAndCorrectContexts = null;
            SBRContext sbrRPTrueAndCorrectContext = null;

            sbrRPTrueAndCorrectContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}DeclarationTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}TrueAndCorrect")))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTrueAndCorrectElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTrueAndCorrectContexts != null && sbrRPTrueAndCorrectContexts.Count() > 0)
            {
                sbrRPTrueAndCorrectContext = sbrRPTrueAndCorrectContexts.First();

                report.RPTrueAndCorrectId = sbrRPTrueAndCorrectContext.Id;
                report.RPTrueAndCorrectIdentifier = sbrRPTrueAndCorrectContext.EntityIdentifier;
                report.RPTrueAndCorrectIdentifierScheme = sbrRPTrueAndCorrectContext.EntityScheme;

                report.RPTrueAndCorrectCount = sbrRPTrueAndCorrectContexts.Count();
                report.RPTrueAndCorrectExists = true;
                report.RPTrueAndCorrectOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTrueAndCorrectContext) + 1;
                report.RPTrueAndCorrectLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTrueAndCorrectContexts.Last()) + 1;


                if (report.RPTrueAndCorrectIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTrueAndCorrectIdentifierTFN = report.RPTrueAndCorrectIdentifier;

                report.RPTrueAndCorrectStartDate = ((SBRDurationPeriod)sbrRPTrueAndCorrectContext.Period).DurationStartAsDateObject;
                report.RPTrueAndCorrectEndDate = ((SBRDurationPeriod)sbrRPTrueAndCorrectContext.Period).DurationEndAsDateObject;

                sbrPopulatedRPTrueAndCorrectElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTrueAndCorrectContexts.Count(); i++)
                {
                    sbrRPTrueAndCorrectContext = sbrRPTrueAndCorrectContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTrueAndCorrectContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTrueAndCorrectElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTrueAndCorrectElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTrueAndCorrectElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }
                    report.TRTAMI93 = report.RPTrueAndCorrectIdentifier;

                    if (sbrPopulatedRPTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonUnstructuredName.FullName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI94 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.Position.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI95 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}ElectronicContact.Telephone.Area.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI96 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}ElectronicContact.Telephone.Minimal.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI97 = currentValues[0].Value;
                    }

                    if (sbrPopulatedRPTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Declaration.Signature.Date", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI98 = DateTime.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Declaration.StatementAccepted.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI99 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedRPTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Declaration.SignatoryIdentifier.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI100 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrRPTrueAndCorrectContext

            #region sbrINTTrueAndCorrectContext
            IEnumerable<SBRContext> sbrINTTrueAndCorrectContexts = null;
            SBRContext sbrINTTrueAndCorrectContext = null;

            sbrINTTrueAndCorrectContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Intermediary")) &&
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}DeclarationTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}TrueAndCorrect")))));
            Dictionary<string, IList<SBRElement>> sbrPopulatedINTTrueAndCorrectElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrINTTrueAndCorrectContexts != null && sbrINTTrueAndCorrectContexts.Count() > 0)
            {
                sbrINTTrueAndCorrectContext = sbrINTTrueAndCorrectContexts.First();

                report.INTTrueAndCorrectId = sbrINTTrueAndCorrectContext.Id;
                report.INTTrueAndCorrectIdentifier = sbrINTTrueAndCorrectContext.EntityIdentifier;
                report.INTTrueAndCorrectIdentifierScheme = sbrINTTrueAndCorrectContext.EntityScheme;

                report.INTTrueAndCorrectCount = sbrINTTrueAndCorrectContexts.Count();
                report.INTTrueAndCorrectExists = true;
                report.INTTrueAndCorrectOccurrenceIndex = Array.IndexOf(contextArray, sbrINTTrueAndCorrectContext) + 1;
                report.INTTrueAndCorrectLastOccurrenceIndex = Array.IndexOf(contextArray, sbrINTTrueAndCorrectContexts.Last()) + 1;


                if (report.INTTrueAndCorrectIdentifierScheme == "http://www.abr.gov.au/abn") report.INTTrueAndCorrectIdentifierABN = report.INTTrueAndCorrectIdentifier;

                report.INTTrueAndCorrectStartDate = ((SBRDurationPeriod)sbrINTTrueAndCorrectContext.Period).DurationStartAsDateObject;
                report.INTTrueAndCorrectEndDate = ((SBRDurationPeriod)sbrINTTrueAndCorrectContext.Period).DurationEndAsDateObject;

                sbrPopulatedINTTrueAndCorrectElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrINTTrueAndCorrectContexts.Count(); i++)
                {
                    sbrINTTrueAndCorrectContext = sbrINTTrueAndCorrectContexts.ElementAt(i);

                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrINTTrueAndCorrectContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedINTTrueAndCorrectElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedINTTrueAndCorrectElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedINTTrueAndCorrectElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement });
                            }
                        }
                    }
                    report.TRTAMI92 = report.INTTrueAndCorrectIdentifier;

                    if (sbrPopulatedINTTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonUnstructuredName.FullName.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI83 = currentValues[0].Value;
                    }

                    if (sbrPopulatedINTTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}PersonNameDetails.Position.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI84 = currentValues[0].Value;
                    }

                    if (sbrPopulatedINTTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}ElectronicContact.Telephone.Area.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI85 = currentValues[0].Value;
                    }

                    if (sbrPopulatedINTTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}ElectronicContact.Telephone.Minimal.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI86 = currentValues[0].Value;
                    }

                    if (sbrPopulatedINTTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Declaration.Signature.Date", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI87 = DateTime.Parse(currentValues[0].Value);
                    }

                    if (sbrPopulatedINTTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Declaration.StatementAccepted.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI88 = ToBoolean(currentValues[0].Value);
                    }

                    if (sbrPopulatedINTTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Declaration.SignatoryIdentifier.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI89 = currentValues[0].Value;
                    }

                    if (sbrPopulatedINTTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Identifiers.TaxAgentClientReference.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI90 = currentValues[0].Value;
                    }

                    if (sbrPopulatedINTTrueAndCorrectElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/trtami}Identifiers.TaxAgentNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.TRTAMI91 = currentValues[0].Value;
                    }
                }
            }
            #endregion End of Context sbrINTTrueAndCorrectContext
            return report;

        } // Of Consume Method
    } // Of Class
} // Of Namespace

