using System;
using System.Runtime.CompilerServices;
using System.Linq;
using System.Collections.Generic;
using DataContracts;
using System.Text.RegularExpressions;

namespace Ato.EN.IntegrationServices.CodeGenerationFTER
{

    public partial class FTER2018Validator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="FTER2018Validator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public FTER2018Validator(int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }


        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString().ToLower() : string.Empty;
        }

        private static string GetValueOrEmptyDate(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-dd") : string.Empty;
        }

        private static string GetValueOrEmptyDate(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-dd");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-ddTHH:mm:ssZ");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-ddTHH:mm:ssZ") : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

         /// <summary>
         /// Get string value between [first] a and [last] b.
         /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

         /// <summary>
         /// Get string value after [first] a.
         /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

         /// <summary>
         /// Get string value after [last] a.
         /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
            return 0;
            else
            return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return (from x in nodes select x).Distinct().Count() != 1;
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
           if (nodes == null)
                return false;

            nodes = nodes.Where(x => x != null).ToArray();
            return !((from x in nodes select x).Distinct().Count() == nodes.Count());
        
        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

        public DateTime Date(DateTime? datetime)
        {                   
            return datetime.GetValueOrDefault().Date;
        }

				//The logic in After function expects "---" for day and "--" for month. 
				//Since hyphen was missing the date always returned null
        public DateTime? ConvertToDate(int day, int month, int year)
        {
					return ConvertToDate(day == 0 ? null : "---" + day.ToString(), month == 0 ? null : "--" + month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }

        private static DateTime? AddMonthsToDate(DateTime? dateTime, int months)
        {
            return dateTime == null ? null : (DateTime?)dateTime.Value.AddMonths(months);
        }

        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response = true;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length == 14)
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 3), out numeric))
                        response = false;

                }
                else if (usi.Length == 9)
                {

                    if (Regex.IsMatch(usi, @"^([a-zA-Z]{3}\d{4}[a-zA-Z]{2})$"))
                        response = false;
                }
            }
            return response;
        }

        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
             return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
             decimal numberValue;
             decimal.TryParse(value, out numberValue);
             return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags="")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags="")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':                        
                        options = options | RegexOptions.IgnoreCase;                       
                        break;
                    case 'm':                        
                        options = options | RegexOptions.Multiline;                       
                        break;
                    case 's':                        
                            options = options | RegexOptions.Singleline;                       
                        break;
                    case 'n':                        
                            options = options | RegexOptions.ExplicitCapture;                       
                        break;
                    case 'x':                       
                            options = options | RegexOptions.IgnorePatternWhitespace;                       
                        break;
                }
            }

            return options;
        }

        private static string Substring(object field, int start, int length)
        {
            return field.ToString().Substring(start, length);
        }
        
        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public FTER2018 ConsumedReport { get; private set; }

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string,ProcessMessageDocument>();

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(FTER2018 report, bool isStandalone)
        {

            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            bool assertion;

            this.ConsumedReport = report;

            // ------------------------------------------------------------------------------
            // Validations are now done in code.
            // This version supports full validation of a report - for those rules that have successfully parsed and could be generated.
            // Generated Validations include:
            //   - Validation logic in C#.
            //   - Production of errors in EBMS, SBR and SWS format.
            // 
            // The generation at this stage does not support the following - and has to be completed manually (later versions of the generation will do this for you)
            //   - Business Rules that did not parse.  The ESR team will help support you where this happens as an effort is being made to rectify these
            //   - You will get TODO tasks for those that the parser could not cope with
            // ------------------------------------------------------------------------------

            #region Manual Rules - Rules marked for manual coding
            
                #region VR.ATO.FTER.408031
        
                /*  VR.ATO.FTER.408031
                Variation must not include central management or trustee details

                Legacy Rule Format:
                IF [FTER95] = "V" AND ( [FTER16] <> NULL OR [FTER17] <> NULL OR COUNT (TUPLE(xbrli\perioddetails1.xx.xx.PeriodDetails
                WHERE TUPLE ELEMENT EXPLICIT pyin.xx.xx:Period.Type.Code = "ControlOutsideAustraliaPartPeriod")) > 0 OR COUNT (TUPLE(fter.0002.lodge.req.xx.xx:Trustee)) > 0   )
                   RETURN VALIDATION MESSAGE
                ENDIF
                
                [FTER16] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustralia.Indicator
                [FTER17] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
                [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code
    
                Technical Business Rule Format:
                ^FTER95 = 'V' AND (^FTER16 <> NULL OR ^FTER17 <> NULL OR Count(TUPLE(xbrli\perioddetails1.66.66.PeriodDetailsWHERE  pyin.66.66:Period.Type.Code = 'ControlOutsideAustraliaPartPeriod')) > 0 OR Count(^Trustee) > 0  )
        
                Data Elements:
        
                ^FTER16 = RP:Residency.CentralManagementAndControlOutsideAustralia.Indicator
        
                ^FTER17 = RP:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
        
                ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
        
                ^Trustee = RP:Trustee
                */
    VRATOFTER408031(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408031

    #region VR.ATO.FTER.408051

/*  VR.ATO.FTER.408051
            Dates central management and control of trust was outside Australia must be present

        Legacy Rule Format:
            IF [FTER16] = TRUE AND [FTER17] <> TRUE AND COUNT (TUPLE(xbrli\perioddetails1.xx.xx.PeriodDetails WHERE TUPLE ELEMENT EXPLICIT pyin.xx.xx:Period.Type.Code = "ControlOutsideAustraliaPartPeriod")) = 0
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER16] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustralia.Indicator
            [FTER17] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator

    Technical Business Rule Format:
            ^FTER16 = TRUE AND ^FTER17 <> TRUE AND Count(TUPLE(xbrli\perioddetails1.66.66.PeriodDetails WHERE  pyin.66.66:Period.Type.Code = 'ControlOutsideAustraliaPartPeriod')) = 0

Data Elements:
    
    ^FTER16 = RP:Residency.CentralManagementAndControlOutsideAustralia.Indicator
    
    ^FTER17 = RP:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
    */
    VRATOFTER408051(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408051

    #region VR.ATO.FTER.408052

/*  VR.ATO.FTER.408052
            Time central management and control of trust was outside Australia must NOT be present

        Legacy Rule Format:
            IF [FTER16] = FALSE AND ([FTER17] <> NULL OR COUNT (TUPLE(xbrli\perioddetails1.xx.xx.PeriodDetails WHERE TUPLE ELEMENT EXPLICIT pyin.xx.xx:Period.Type.Code = "ControlOutsideAustraliaPartPeriod"))) > 0
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER16] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustralia.Indicator
            [FTER17] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator

    Technical Business Rule Format:
            ^FTER16 = FALSE AND (^FTER17 <> NULL OR Count(TUPLE(xbrli\perioddetails1.66.66.PeriodDetails WHERE  pyin.66.66:Period.Type.Code = 'ControlOutsideAustraliaPartPeriod'))) > 0

Data Elements:
    
    ^FTER16 = RP:Residency.CentralManagementAndControlOutsideAustralia.Indicator
    
    ^FTER17 = RP:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
    */
    VRATOFTER408052(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408052

    #region VR.ATO.FTER.408055

/*  VR.ATO.FTER.408055
            Central management and control - full period indicator and dates must not both be present

        Legacy Rule Format:
            IF [FTER17] = TRUE AND COUNT (TUPLE(xbrli\perioddetails1.xx.xx.PeriodDetails WHERE TUPLE ELEMENT EXPLICIT pyin.xx.xx:Period.Type.Code = "ControlOutsideAustraliaPartPeriod")) > 0
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER17] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator

    Technical Business Rule Format:
            ^FTER17 = TRUE AND Count(TUPLE(xbrli\perioddetails1.66.66.PeriodDetails WHERE  pyin.66.66:Period.Type.Code = 'ControlOutsideAustraliaPartPeriod')) > 0

Data Elements:
    
    ^FTER17 = RP:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
    */
    VRATOFTER408055(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408055

    #region VR.ATO.FTER.408056

/*  VR.ATO.FTER.408056
            Number of Central Management and Control Period details supplied cannot exceed 12

        Legacy Rule Format:
            IF COUNT (TUPLE(xbrli\perioddetails1.xx.xx.PeriodDetails WHERE TUPLE ELEMENT EXPLICIT pyin.xx.xx:Period.Type.Code = "ControlOutsideAustraliaPartPeriod")) > 12
               RETURN VALIDATION  MESSAGE
            ENDIF

    Technical Business Rule Format:
            Count(^FTER88) > 12

Data Elements:
    
    ^FTER88 = RP:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
    */
    VRATOFTER408056(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408056

    #region VR.ATO.FTER.408077

/*  VR.ATO.FTER.408077
            If the answer to the question 'was trustee a non resident for tax purposes' is 'No' (false), then the Full period indicator must not be present and a period of non residency must NOT be present

        Legacy Rule Format:
            WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:Trustee)
            IF [FTER36] = FALSE AND ([FTER37] <> NULL OR COUNT (TUPLE(perioddetails1.xx.xx:PeriodDetails)) > 0)
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER36] = FTER:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
            [FTER37] = FTER:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator IN TUPLE(fter.0002.lodge.req.02.00:Trustee)

    Technical Business Rule Format:
            ^Trustee <> NULL AND ^FTER36 = FALSE AND (^FTER37 <> NULL OR Count(^PeriodDetails) > 0)

Data Elements:
    
    ^FTER36 = RP:Trustee:Residency.TaxPurposesPersonStatus.Indicator
    
    ^FTER37 = RP:Trustee:Residency.NonResidentFullPeriod.Indicator
    
    ^PeriodDetails = RP:PeriodDetails
    
    ^Trustee = RP:Trustee
    */
    VRATOFTER408077(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408077

    #region VR.ATO.FTER.408078

/*  VR.ATO.FTER.408078
            Trustee full period indicator or period of non-residency dates must be present

        Legacy Rule Format:
            WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:Trustee)
            IF [FTER36] = TRUE AND ( ([FTER37] = NULL OR [FTER37] = FALSE) AND COUNT (TUPLE(perioddetails1.xx.xx:PeriodDetails)) = 0)
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER36] = FTER:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
            [FTER37] = FTER:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator IN TUPLE(fter.0002.lodge.req.02.00:Trustee)

    Technical Business Rule Format:
            ^Trustee <> NULL AND ^FTER36 = TRUE AND ((^FTER37 = NULL OR ^FTER37 = FALSE) AND Count(^PeriodDetails) = 0)

Data Elements:
    
    ^FTER37 = RP:Trustee:Residency.NonResidentFullPeriod.Indicator
    
    ^FTER36 = RP:Trustee:Residency.TaxPurposesPersonStatus.Indicator
    
    ^PeriodDetails = RP:PeriodDetails
    
    ^Trustee = RP:Trustee
    */
    VRATOFTER408078(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408078

    #region VR.ATO.FTER.408079

/*  VR.ATO.FTER.408079
            Full period indicator and dates must not both be present

        Legacy Rule Format:
            WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:Trustee)
            IF [FTER37] = TRUE AND COUNT (TUPLE(perioddetails1.xx.xx:PeriodDetails)) > 0
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER37] = FTER:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator IN TUPLE(fter.0002.lodge.req.02.00:Trustee)

    Technical Business Rule Format:
            ^Trustee <> NULL AND ^FTER37 = TRUE AND Count(^PeriodDetails) > 0

Data Elements:
    
    ^FTER37 = RP:Trustee:Residency.NonResidentFullPeriod.Indicator
    
    ^PeriodDetails = RP:PeriodDetails
    
    ^Trustee = RP:Trustee
    */
    VRATOFTER408079(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408079

    #region VR.ATO.FTER.408081

/*  VR.ATO.FTER.408081
            Trustee non-resident Period Type Code must be 'NonResidentPartPeriod'

        Legacy Rule Format:
            WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:Trustee\perioddetails1.xx.xx.PeriodDetails)
            IF pyid.xx.xx:Period.Type.Code <> "NonResidentPartPeriod"
               RETURN VALIDATION MESSAGE
            ENDIF

    Technical Business Rule Format:
            ^FTER90 <> 'NonResidentPartPeriod'

Data Elements:
    
    ^FTER90 = RP:Trustee:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='NonResidentPartPeriod']
    */
    VRATOFTER408081(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408081

    #region VR.ATO.FTER.408084

/*  VR.ATO.FTER.408084
            Trustee non-resident period must not be before the substituted accounting period

        Legacy Rule Format:
            IF [FTER41] <> NULL AND (ANY OCCURRENCE OF [FTER38] <> NULL AND [FTER38] < [FTER41])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER38] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails) IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
            [FTER41] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails)

    Technical Business Rule Format:
            ^FTER41 <> NULL AND AnyOccurrence(^FTER38, ^FTER38 <> NULL AND ^FTER38 < ^FTER41)

Data Elements:
    
    ^FTER38 = RP:Trustee:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
    
    ^FTER41 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    */
    VRATOFTER408084(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408084

    #region VR.ATO.FTER.408085

/*  VR.ATO.FTER.408085
            Trustee non-resident period must not be before the election commencement time

        Legacy Rule Format:
            IF [FTER43] <> NULL AND (ANY OCCURRENCE OF [FTER38] <> NULL AND [FTER38] < [FTER43])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER38] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails) IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
            [FTER43] = FTER:RP:rvctc3.02.00:Elections.Commencement.Date

    Technical Business Rule Format:
            ^FTER43 <> NULL AND AnyOccurrence(^FTER38, ^FTER38 <> NULL AND ^FTER38 < ^FTER43)

Data Elements:
    
    ^FTER38 = RP:Trustee:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
    
    ^FTER43 = RP:Elections.Commencement.Date
    */
    VRATOFTER408085(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408085

    #region VR.ATO.FTER.408086

/*  VR.ATO.FTER.408086
            Trustee non-resident period must not be before the start of income year specified

        Legacy Rule Format:
            IF [FTER40] <> NULL AND [FTER41] = NULL AND [FTER43] = NULL AND (ANY OCCURRENCE OF [FTER38] <> NULL) AND ([FTER38] < ([FTER40] - 1)&"-07-01")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER38] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails) IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
            [FTER40] = FTER:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
            [FTER41] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails)
            [FTER43] = FTER:RP:rvctc3.02.00:Elections.Commencement.Date

    Technical Business Rule Format:
            ^FTER40 <> NULL AND ^FTER41 = NULL AND ^FTER43 = NULL AND (AnyOccurrence(^FTER38, ^FTER38 <> NULL AND ^FTER38 < AsDate(Concat(^FTER40 - 1, '-07-01'))))

Data Elements:
    
    ^FTER38 = RP:Trustee:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
    
    ^FTER40 = RP:Elections.FamilyTrustElectionStatus.Year
    
    ^FTER41 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
    ^FTER43 = RP:Elections.Commencement.Date
    */
    VRATOFTER408086(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408086

    #region VR.ATO.FTER.408164

/*  VR.ATO.FTER.408164
            One Declaration Statement Type Code must be 'TrueAndCorrect'

        Legacy Rule Format:
            IF COUNT(INT) = 0 AND 
                COUNT (pyin.xx.xx:Declaration.StatementType.Code ="TrueAndCorrect" IN TUPLE(fter.0002.lodge.req.xx.xx:DeclarationOfTrustee\declaration2.xx.xx:Declaration)) <> 1
              RETURN VALIDATION MESSAGE
            ENDIF

    Technical Business Rule Format:
            Count(^Context) = 0 AND Count(^FTER99 = 'TrueAndCorrect') <> 1

Data Elements:
    
    ^FTER99 = RP:DeclarationOfTrustee:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
    ^Context = INT
    */
    VRATOFTER408164(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408164

    #region VR.ATO.FTER.408166

/*  VR.ATO.FTER.408166

        Legacy Rule Format:
            WHERE IN TUPLE(Declaration2.xx.xx:Declaration)
            IF pyin.xx.xx:Declaration.StatementAccepted.Indicator <> TRUE OR Declaration.SignatoryIdentifier.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

    Technical Business Rule Format:
            ^FTER160 <> TRUE OR ^FTER163 = BLANK

Data Elements:
    
    ^FTER160 = INT:Declaration:Declaration.StatementAccepted.Indicator WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
    ^FTER163 = INT:Declaration:Declaration.SignatoryIdentifier.Text WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    */
    VRATOFTER408166(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408166

    #region VR.ATO.FTER.408166

/*  VR.ATO.FTER.408166

        Legacy Rule Format:
            WHERE IN TUPLE(Declaration2.xx.xx:Declaration)
            IF pyin.xx.xx:Declaration.StatementAccepted.Indicator <> TRUE OR Declaration.SignatoryIdentifier.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

    Technical Business Rule Format:
            ^FTER169 <> TRUE OR ^FTER100 = BLANK

Data Elements:
    
    ^FTER169 = RP:DeclarationOfTrustee:Declaration:Declaration.StatementAccepted.Indicator WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
    ^FTER100 = RP:DeclarationOfTrustee:Declaration:Declaration.SignatoryIdentifier.Text WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    */
    VRATOFTER408166_(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408166

    #region VR.ATO.FTER.408169

/*  VR.ATO.FTER.408169
            Trust Address Details Usage Code must be 'POS' and Currency Code must be 'C' or 'P'

        Legacy Rule Format:
            WHERE IN TUPLE (xbrli\address2.xx.xx.AddressDetails)
            IF (AddressDetails.Usage.Code <> "POS") OR (AddressDetails.Currency.Code <> "C" AND AddressDetails.Currency.Code <> "P")
               RETURN VALIDATION MESSAGE
            ENDIF

    Technical Business Rule Format:
            WHERE IN TUPLE (xbrli\address2.66.66.AddressDetails)IF (AddressDetails.Usage.Code <> 'POS') OR (AddressDetails.Currency.Code <> 'C' AND AddressDetails.Currency.Code <> 'P')

Data Elements:
*/
    VRATOFTER408169(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408169

    #region VR.ATO.FTER.408169

/*  VR.ATO.FTER.408169
            Trust Address Details Usage Code must be 'POS' and Currency Code must be 'C' or 'P'

        Legacy Rule Format:
            WHERE IN TUPLE (xbrli\address2.xx.xx.AddressDetails)
            IF (AddressDetails.Usage.Code <> "POS") OR (AddressDetails.Currency.Code <> "C" AND AddressDetails.Currency.Code <> "P")
               RETURN VALIDATION MESSAGE
            ENDIF

    Technical Business Rule Format:
            WHERE IN TUPLE (xbrli\address2.66.66.AddressDetails)IF (AddressDetails.Usage.Code <> 'POS') OR (AddressDetails.Currency.Code <> 'C' AND AddressDetails.Currency.Code <> 'P')

Data Elements:
*/
    //VRATOFTER408169_(response, report, isStandalone);
    #endregion // VR.ATO.FTER.408169

    #region VR.ATO.GEN.408058

/*  VR.ATO.GEN.408058

        Legacy Rule Format:
            WHERE IN TUPLE (xbrli\perioddetails1.xx.xx.PeriodDetails)
            IF pyin.xx.xx:Period.Type.Code <> NULLORBLANK AND pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod", "SubstitutedAccountingPeriod")
               RETURN VALIDATION MESSAGE
            ENDIF

    Technical Business Rule Format:
            ((^FTER88 <> BLANK) AND (^FTER88 <> 'ControlOutsideAustraliaPartPeriod'))

Data Elements:
    
    ^FTER88 = RP:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
    */
    VRATOGEN408058(response, report, isStandalone);
            #endregion // VR.ATO.GEN.408058

            #region VR.ATO.GEN.408058
            // VRATOGEN408058_  Removed manually as not required - covered by previous method
            #endregion // VR.ATO.GEN.408058
            #endregion Manual Rules - Rules marked for manual coding

            #region Repeating report.TrusteeCollection
            if (report.TrusteeCollection != null)
            {    
                int itemIndex1 = 0;
                foreach (FTER2018.Trustee trustee in report.TrusteeCollection)
                {
                    itemIndex1++;

                    #region VR.ATO.FTER.408003

                    /*  VR.ATO.FTER.408003
                    Trustee individual name or non-individual name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:Trustee)
                    IF (COUNT (TUPLE(personstructuredname3.xx.xx:PersonNameDetails)) = 0 AND COUNT (TUPLE(organisationname2.xx.xx:OrganisationNameDetails)) = 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (Count(^PersonNameDetails) = 0 AND Count(^OrganisationNameDetails) = 0)
            
                    Data Elements:
            
                    ^PersonNameDetails = RP:Trustee:PersonNameDetails
            
                    ^OrganisationNameDetails = RP:Trustee:OrganisationNameDetails
                    */
                    assertion = (Count(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPCount) == 0 && Count(trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPCount) == 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408003", Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee individual name or non-individual name must be present",
                            LongDescription = @"For an election, details of each trustee's family name or non-individual name must be present",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPLastOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408003"} },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408003
            
                    #region VR.ATO.FTER.408004
            
                    /*  VR.ATO.FTER.408004
                    For each trustee one address must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:Trustee)
                    IF COUNT (TUPLE(address2.xx.xx.AddressDetails)) <> 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^Trustee <> NULL AND Count(^AddressDetails) <> 1
            
                    Data Elements:
            
                    ^Trustee = RP:Trustee
            
                    ^AddressDetails = RP:Trustee:AddressDetails
                    */
                    assertion = (report.TrusteeCollectionExists != false && Count(trustee.Trustee_AddressDetailsCollection_POS_C_RPCount) != 1);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408004", Severity = ProcessMessageSeverity.Error,
                            Description = @"For each trustee one address must be present",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPLastOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408004"} },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408004
            
                    #region VR.ATO.FTER.408005
            
                    /*  VR.ATO.FTER.408005
                    Question regarding non-residency of trustee must be answered for each trustee
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:Trustee)
                    IF [FTER36] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER36] = FTER:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
        
                    Technical Business Rule Format:
                    ^FTER36 = NULL
            
                    Data Elements:
            
                    ^FTER36 = RP:Trustee:Residency.TaxPurposesPersonStatus.Indicator
                    */
                    assertion = (trustee.FTER36 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408005", Severity = ProcessMessageSeverity.Error,
                            Description = @"Question regarding non-residency of trustee must be answered for each trustee",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:Residency.TaxPurposesPersonStatus.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408005"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER36", Value = GetValueOrEmpty(trustee.FTER36) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408005
            
                    #region VR.ATO.FTER.408065
            
                    /*  VR.ATO.FTER.408065
                    Trustee does not have a TFN indicator must not be 'yes' (true)
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:Trustee)
                    IF [FTER21] = TRUE AND [FTER20] <> NULLORBLANK
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER20] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
                    [FTER21] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
        
                    Technical Business Rule Format:
                    ^FTER21 = TRUE AND ^FTER20 <> BLANK
            
                    Data Elements:
            
                    ^FTER21 = RP:Trustee:Identifiers.TaxFileNumberInexistent.Indicator
            
                    ^FTER20 = RP:Trustee:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (trustee.FTER21 == true && string.IsNullOrWhiteSpace(trustee.FTER20) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408065", Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee does not have a TFN indicator must not be 'yes' (true)",
                            LongDescription = @"If the TFN of trustee is present, then the 'trustee does not have a TFN must not be 'yes' (true)",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumberInexistent.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408065"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER21", Value = GetValueOrEmpty(trustee.FTER21) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER20", Value = GetValueOrEmpty(trustee.FTER20) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408065
            
                    #region VR.ATO.FTER.408066
            
                    /*  VR.ATO.FTER.408066
                    TFN of trustee must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:Trustee)
                    IF ([FTER21] = NULL OR [FTER21] = FALSE) AND [FTER20] = NULLORBLANK
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER20] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
                    [FTER21] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
        
                    Technical Business Rule Format:
                    (^FTER21 = NULL OR ^FTER21 = FALSE) AND ^FTER20 = BLANK
            
                    Data Elements:
            
                    ^FTER20 = RP:Trustee:Identifiers.TaxFileNumber.Identifier
            
                    ^FTER21 = RP:Trustee:Identifiers.TaxFileNumberInexistent.Indicator
                    */
                    assertion = ((trustee.FTER21 == null || trustee.FTER21 == false) && string.IsNullOrWhiteSpace(trustee.FTER20) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408066", Severity = ProcessMessageSeverity.Error,
                            Description = @"TFN of trustee must be present",
                            LongDescription = @"Either the TFN of trustee must be present or the 'trustee does not have a TFN' indicator must be 'yes' (true)",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408066"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER21", Value = GetValueOrEmpty(trustee.FTER21) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER20", Value = GetValueOrEmpty(trustee.FTER20) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408066
            
                    #region VR.ATO.FTER.408071
            
                    /*  VR.ATO.FTER.408071
                    A trustee must not include both an individual and a non-individual name
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:Trustee)
                    IF COUNT (TUPLE(personstructuredname3.xx.xx:PersonNameDetails)) > 0 AND COUNT (TUPLE(organisationname2.xx.xx:OrganisationNameDetails)) > 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Count(^PersonNameDetails) > 0 AND Count(^OrganisationNameDetails) > 0
            
                    Data Elements:
            
                    ^PersonNameDetails = RP:Trustee:PersonNameDetails
            
                    ^OrganisationNameDetails = RP:Trustee:OrganisationNameDetails
                    */
                    assertion = (Count(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPCount) > 0 && Count(trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPCount) > 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408071", Severity = ProcessMessageSeverity.Error,
                            Description = @"A trustee must not include both an individual and a non-individual name",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPLastOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408071"} },
                        };
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408071
            
                    #region VR.ATO.FTER.408072
            
                    /*  VR.ATO.FTER.408072
                    First name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:Trustee) IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails) 
                    IF pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK AND pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK
                        RETURN VALIDATION MESSAGE 
                     ENDIF
        
                    Technical Business Rule Format:
                    ^FTER23 <> BLANK AND ^FTER25 = BLANK
            
                    Data Elements:
            
                    ^FTER25 = RP:Trustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            
                    ^FTER23 = RP:Trustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER23) != true && string.IsNullOrWhiteSpace(trustee.FTER25) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408072", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name must be present",
                            LongDescription = @"Trustee family name is present without trustee first name.",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408072"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER23", Value = trustee.FTER23 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER25", Value = GetValueOrEmpty(trustee.FTER25) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408072
            
                    #region VR.ATO.FTER.408121
            
                    /*  VR.ATO.FTER.408121
                    ACN and ARBN must not both be present
    
                    Legacy Rule Format:
                    IF [FTER86] <> NULLORBLANK AND [FTER29] <> NULLORBLANK
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER29] = FTER:RP:pyid.02.00:Identifiers.AustralianRegisteredBodyNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
                    [FTER86] = FTER:RP:pyid.02.00:Identifiers.AustralianCompanyNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:Trustee)
        
                    Technical Business Rule Format:
                    ^FTER86 <> BLANK AND ^FTER29 <> BLANK
            
                    Data Elements:
            
                    ^FTER86 = RP:Trustee:Identifiers.AustralianCompanyNumber.Identifier
            
                    ^FTER29 = RP:Trustee:Identifiers.AustralianRegisteredBodyNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER86) != true && string.IsNullOrWhiteSpace(trustee.FTER29) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408122", Severity = ProcessMessageSeverity.Error,
                            Description = @"ACN and ARBN must not both be present",
                            LongDescription = @"Australian Company Number and Australian Registered Body Number for the company which has an interposed entity election in force must not both be supplied",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:Identifiers.AustralianCompanyNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408121"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER86", Value = GetValueOrEmpty(trustee.FTER86) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER29", Value = GetValueOrEmpty(trustee.FTER29) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408121
            
                    #region VR.ATO.GEN.000406
            
                    /*  VR.ATO.GEN.000406
                    Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER30 <> BLANK) AND (StartsWithSet(^FTER30,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^FTER30, '" C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^FTER30 = RP:Trustee:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER30) != true && IsMatch(trustee.FTER30, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(trustee.FTER30, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER30", Value = trustee.FTER30 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000406
            
                    #region VR.ATO.GEN.000408
            
                    /*  VR.ATO.GEN.000408
                    Care of' is not valid for address line 2
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER31 <> BLANK) AND (StartsWithSet(^FTER31,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^FTER31, '" C/- "," C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^FTER31 = RP:Trustee:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER31) != true && (IsMatch(trustee.FTER31, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(trustee.FTER31, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                            Description = @"Care of' is not valid for address line 2",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER31", Value = GetValueOrEmpty(trustee.FTER31) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000408
            
                    #region VR.ATO.GEN.000458
            
                    /*  VR.ATO.GEN.000458
                    Suffix is invalid
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER24 <> BLANK) AND (NotInDomain(^FTER24, 'SUFFIX CODES'))
            
                    Data Elements:
            
                    ^FTER24 = RP:Trustee:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER24) != true && !(IsMatch(trustee.FTER24, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000458", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suffix is invalid",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.NameSuffix.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER24", Value = GetValueOrEmpty(trustee.FTER24) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000458
            
                    #region VR.ATO.GEN.000459
            
                    /*  VR.ATO.GEN.000459
                    Title is invalid
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER22 <> BLANK) AND (NotInDomain(^FTER22, 'TITLE CODES'))
            
                    Data Elements:
            
                    ^FTER22 = RP:Trustee:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER22) != true && !(IsMatch(trustee.FTER22, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000459", Severity = ProcessMessageSeverity.Error,
                            Description = @"Title is invalid",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.Title.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER22", Value = GetValueOrEmpty(trustee.FTER22) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000459
            
                    #region VR.ATO.GEN.410002
            
                    /*  VR.ATO.GEN.410002
                    Address Line 2 must be present if Address Line 1 contains 'C/-'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ((Contains(^FTER30, 'C/-')) AND (^FTER31 = BLANK))
            
                    Data Elements:
            
                    ^FTER30 = RP:Trustee:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER31 = RP:Trustee:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((trustee.FTER30 == null ? false : trustee.FTER30.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(trustee.FTER31) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER30", Value = trustee.FTER30 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER31", Value = GetValueOrEmpty(trustee.FTER31) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410002
            
                    #region VR.ATO.GEN.410008
            
                    /*  VR.ATO.GEN.410008
                    Address Line 3 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^FTER123 <> BLANK
            
                    Data Elements:
            
                    ^FTER123 = RP:Trustee:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER123) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 3 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER123", Value = GetValueOrEmpty(trustee.FTER123) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410008
            
                    #region VR.ATO.GEN.410013
            
                    /*  VR.ATO.GEN.410013
                    Address Line 4 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^FTER124 <> BLANK
            
                    Data Elements:
            
                    ^FTER124 = RP:Trustee:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER124) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 4 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER124", Value = GetValueOrEmpty(trustee.FTER124) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410013
            
                    #region VR.ATO.GEN.410031
            
                    /*  VR.ATO.GEN.410031
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier <> NULLORBLANK) AND (TFNALGORITHM (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier) = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    (^FTER20 <> BLANK) AND (FailsTFNAlgorithm(^FTER20))
            
                    Data Elements:
            
                    ^FTER20 = RP:Trustee:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER20) != true && FailsTFNAlgorithm(trustee.FTER20));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410031", Severity = ProcessMessageSeverity.Error,
                            Description = @"TFN must be a valid TFN",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410031"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER20", Value = GetValueOrEmpty(trustee.FTER20) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410031
            
                    #region VR.ATO.GEN.410038
            
                    /*  VR.ATO.GEN.410038
                    Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (StartsWith(^FTER27, 'T/A ')) AND (EndsWithSet(^FTER27, '" Pship"," P'ship"," P/ship"'))
            
                    Data Elements:
            
                    ^FTER27 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (IsMatch(trustee.FTER27, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(trustee.FTER27, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER27", Value = trustee.FTER27 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410038
            
                    #region VR.ATO.GEN.410039
            
                    /*  VR.ATO.GEN.410039
                    Organisation name must contain at least one alpha or numeric character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER27  <> BLANK) AND (NotContainsSet(^FTER27, '"a-z", "A-Z", "0-9"'))
            
                    Data Elements:
            
                    ^FTER27 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER27) != true && !(IsMatch(trustee.FTER27, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must contain at least one alpha or numeric character",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER27", Value = trustee.FTER27 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410039
            
                    #region VR.ATO.GEN.410063
            
                    /*  VR.ATO.GEN.410063
                    Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER25 <> BLANK) AND (NotContainsSet(^FTER25, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^FTER25 = RP:Trustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER25) != true && !(IsMatch(trustee.FTER25, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410063", Severity = ProcessMessageSeverity.Error,
                            Description = @"Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410063"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER25", Value = GetValueOrEmpty(trustee.FTER25) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410063
            
                    #region VR.ATO.GEN.410105
            
                    /*  VR.ATO.GEN.410105
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianRegisteredBodyNumber.Identifier <> NULLORBLANK) AND (ARBNALGORITHM (pyid.xx.xx:Identifiers.AustralianRegisteredBodyNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    (^FTER29 <> BLANK) AND (FailsARBNAlgorithm(^FTER29))
            
                    Data Elements:
            
                    ^FTER29 = RP:Trustee:Identifiers.AustralianRegisteredBodyNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER29) != true && FailsACNAlgorithm(trustee.FTER29));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410105", Severity = ProcessMessageSeverity.Error,
                            Description = @"ARBN must be a valid number that passes the algorithm",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:Identifiers.AustralianRegisteredBodyNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410105"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER29", Value = GetValueOrEmpty(trustee.FTER29) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410105
            
                    #region VR.ATO.GEN.410131
            
                    /*  VR.ATO.GEN.410131
                    Other Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER26 <> BLANK) AND (NotContainsSet(^FTER26, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^FTER26 = RP:Trustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER26) != true && !(IsMatch(trustee.FTER26, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410131", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER26", Value = GetValueOrEmpty(trustee.FTER26) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410131
            
                    #region VR.ATO.GEN.410191
            
                    /*  VR.ATO.GEN.410191
                    Country code must be present for overseas address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER126 = TRUE) AND (^FTER125 = NULL)
            
                    Data Elements:
            
                    ^FTER125 = RP:Trustee:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER126 = RP:Trustee:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER126 == true && trustee.FTER125 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                            Description = @"Country code must be present for overseas address",
                            LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER126", Value = GetValueOrEmpty(trustee.FTER126) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER125", Value = GetValueOrEmpty(trustee.FTER125) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410191
            
                    #region VR.ATO.GEN.410194
            
                    /*  VR.ATO.GEN.410194
                    Address Line 1 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                         RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^FTER30) > 38
            
                    Data Elements:
            
                    ^FTER30 = RP:Trustee:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(trustee.FTER30) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER30", Value = trustee.FTER30 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410194
            
                    #region VR.ATO.GEN.410195
            
                    /*  VR.ATO.GEN.410195
                    Address Line 2 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                         RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^FTER31) > 38
            
                    Data Elements:
            
                    ^FTER31 = RP:Trustee:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(trustee.FTER31) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER31", Value = GetValueOrEmpty(trustee.FTER31) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410195
            
                    #region VR.ATO.GEN.410200
            
                    /*  VR.ATO.GEN.410200
                    Family name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER23, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^FTER23 = RP:Trustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER23, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000424", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER23", Value = trustee.FTER23 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410200
            
                    #region VR.ATO.GEN.410201
            
                    /*  VR.ATO.GEN.410201
                    Family name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER23, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^FTER23 = RP:Trustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER23, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000426", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER23", Value = trustee.FTER23 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410201
            
                    #region VR.ATO.GEN.410203
            
                    /*  VR.ATO.GEN.410203
                    First name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER25, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^FTER25 = RP:Trustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER25, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000437", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER25", Value = GetValueOrEmpty(trustee.FTER25) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410203
            
                    #region VR.ATO.GEN.410204
            
                    /*  VR.ATO.GEN.410204
                    First name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER25, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^FTER25 = RP:Trustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER25, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000438", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER25", Value = GetValueOrEmpty(trustee.FTER25) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410204
            
                    #region VR.ATO.GEN.410205
            
                    /*  VR.ATO.GEN.410205
                    Address Line 1 must not contain 'AS ABOVE'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER30, '"AS ABOVE"'))
            
                    Data Elements:
            
                    ^FTER30 = RP:Trustee:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER30, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not contain 'AS ABOVE'",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER30", Value = trustee.FTER30 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410205
            
                    #region VR.ATO.GEN.410208
            
                    /*  VR.ATO.GEN.410208
                    Other given name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER26, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^FTER26 = RP:Trustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER26, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000449", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER26", Value = GetValueOrEmpty(trustee.FTER26) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410208
            
                    #region VR.ATO.GEN.410209
            
                    /*  VR.ATO.GEN.410209
                    Other given name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER26, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^FTER26 = RP:Trustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER26, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000450", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER26", Value = GetValueOrEmpty(trustee.FTER26) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410209
            
                    #region VR.ATO.GEN.410211
            
                    /*  VR.ATO.GEN.410211
                    Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER125 = 'au') AND  (^FTER126 = TRUE)
            
                    Data Elements:
            
                    ^FTER125 = RP:Trustee:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER126 = RP:Trustee:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER125 == @"au" && trustee.FTER126 == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                            Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                            LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER125", Value = GetValueOrEmpty(trustee.FTER125) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER126", Value = GetValueOrEmpty(trustee.FTER126) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410211
            
                    #region VR.ATO.GEN.410212
            
                    /*  VR.ATO.GEN.410212
                    If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER125 <> BLANK) AND (^FTER125 <> 'au') AND  (^FTER126 = FALSE)
            
                    Data Elements:
            
                    ^FTER125 = RP:Trustee:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER126 = RP:Trustee:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER125) != true && trustee.FTER125 != @"au" && trustee.FTER126 == false);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                            Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                            LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER125", Value = GetValueOrEmpty(trustee.FTER125) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER126", Value = GetValueOrEmpty(trustee.FTER126) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410212
            
                    #region VR.ATO.GEN.410213
            
                    /*  VR.ATO.GEN.410213
                    Country Name must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^FTER35 <> BLANK
            
                    Data Elements:
            
                    ^FTER35 = RP:Trustee:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER35) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410213", Severity = ProcessMessageSeverity.Error,
                            Description = @"Country Name must be left blank",
                            LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER35", Value = GetValueOrEmpty(trustee.FTER35) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410213
            
                    #region VR.ATO.GEN.410218
            
                    /*  VR.ATO.GEN.410218
                    FirstName must be entered if OtherGivenNames is entered
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                        RETURN VALIDATION MESSAGE 
                     ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND ((^FTER26 <> BLANK) AND (^FTER25 = BLANK))
            
                    Data Elements:
            
                    ^FTER25 = RP:Trustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            
                    ^FTER26 = RP:Trustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:Trustee:PersonNameDetails
                    */
                    assertion = (trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPExists != false && (string.IsNullOrWhiteSpace(trustee.FTER26) != true && string.IsNullOrWhiteSpace(trustee.FTER25) == true));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410218", Severity = ProcessMessageSeverity.Error,
                            Description = @"FirstName must be entered if OtherGivenNames is entered",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER26", Value = GetValueOrEmpty(trustee.FTER26) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER25", Value = GetValueOrEmpty(trustee.FTER25) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410218
            
                    #region VR.ATO.GEN.428021
            
                    /*  VR.ATO.GEN.428021
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianCompanyNumber.Identifier <> NULLORBLANK) AND (ACNALGORITHM(pyid.xx.xx:Identifiers.AustralianCompanyNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    (^FTER86 <> BLANK) AND (FailsACNAlgorithm(^FTER86))
            
                    Data Elements:
            
                    ^FTER86 = RP:Trustee:Identifiers.AustralianCompanyNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER86) != true && FailsACNAlgorithm(trustee.FTER86));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.428021", Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid ACN",
                            LongDescription = @"The Australian Company Number (ACN) has failed the algorithm check",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:Identifiers.AustralianCompanyNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428021"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER86", Value = GetValueOrEmpty(trustee.FTER86) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428021
            
                    #region VR.ATO.GEN.428230
            
                    /*  VR.ATO.GEN.428230
                    Suburb/Town/City cannot contain state code
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER32, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
            
                    Data Elements:
            
                    ^FTER32 = RP:Trustee:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER32, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain state code",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER32", Value = trustee.FTER32 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428230
            
                    #region VR.ATO.GEN.428254
            
                    /*  VR.ATO.GEN.428254
                    Address line 1 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
                    RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER30, 'UNKNOWN')
            
                    Data Elements:
            
                    ^FTER30 = RP:Trustee:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER30 == null ? false : trustee.FTER30.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER30", Value = trustee.FTER30 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428254
            
                    #region VR.ATO.GEN.428255
            
                    /*  VR.ATO.GEN.428255
                    Address line 2 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER31, 'UNKNOWN')
            
                    Data Elements:
            
                    ^FTER31 = RP:Trustee:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER31 == null ? false : trustee.FTER31.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 2 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER31", Value = GetValueOrEmpty(trustee.FTER31) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428255
            
                    #region VR.ATO.GEN.428256
            
                    /*  VR.ATO.GEN.428256
                    Suburb/Town/City cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER32, 'UNKNOWN')
            
                    Data Elements:
            
                    ^FTER32 = RP:Trustee:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER32 == null ? false : trustee.FTER32.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER32", Value = trustee.FTER32 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428256
            
                    #region VR.ATO.GEN.428258
            
                    /*  VR.ATO.GEN.428258
                    Non Individual Name cannot contain space hyphen space combinations
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER27, ' - ')
            
                    Data Elements:
            
                    ^FTER27 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER27 == null ? false : trustee.FTER27.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain space hyphen space combinations",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER27", Value = trustee.FTER27 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428258
            
                    #region VR.ATO.GEN.428259
            
                    /*  VR.ATO.GEN.428259
                    Non Individual Name cannot contain "P/L"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER27, 'P/L')
            
                    Data Elements:
            
                    ^FTER27 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER27 == null ? false : trustee.FTER27.ToUpperInvariant().Contains(@"P/L"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""P/L""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER27", Value = trustee.FTER27 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428259
            
                    #region VR.ATO.GEN.428260
            
                    /*  VR.ATO.GEN.428260
                    Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^FTER27, '"--","''","  "'))
            
                    Data Elements:
            
                    ^FTER27 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER27, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER27", Value = trustee.FTER27 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428260
            
                    #region VR.ATO.GEN.428261
            
                    /*  VR.ATO.GEN.428261
                    Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (EndsWithSet(^FTER27, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
            
                    Data Elements:
            
                    ^FTER27 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER27, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER27", Value = trustee.FTER27 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428261
            
                    #region VR.ATO.GEN.428262
            
                    /*  VR.ATO.GEN.428262
                    Family Name cannot contain "space hyphen space"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER23, ' - ')
            
                    Data Elements:
            
                    ^FTER23 = RP:Trustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER23 == null ? false : trustee.FTER23.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000423", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name cannot contain ""space hyphen space""",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428262"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER23", Value = trustee.FTER23 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428262
            
                    #region VR.ATO.GEN.428263
            
                    /*  VR.ATO.GEN.428263
                    Family name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^FTER23, '"--","''","  "'))
            
                    Data Elements:
            
                    ^FTER23 = RP:Trustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER23, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000427", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428263"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER23", Value = trustee.FTER23 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428263
            
                    #region VR.ATO.GEN.428264
            
                    /*  VR.ATO.GEN.428264
                    First name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER25, ' - ')
            
                    Data Elements:
            
                    ^FTER25 = RP:Trustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER25 == null ? false : trustee.FTER25.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000434", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428264"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER25", Value = GetValueOrEmpty(trustee.FTER25) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428264
            
                    #region VR.ATO.GEN.428265
            
                    /*  VR.ATO.GEN.428265
                    First name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^FTER25, '"--","''","  "'))
            
                    Data Elements:
            
                    ^FTER25 = RP:Trustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER25, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000439", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428265"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER25", Value = GetValueOrEmpty(trustee.FTER25) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428265
            
                    #region VR.ATO.GEN.428266
            
                    /*  VR.ATO.GEN.428266
                    Other given name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER26, ' - ')
            
                    Data Elements:
            
                    ^FTER26 = RP:Trustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER26 == null ? false : trustee.FTER26.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000446", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428266"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER26", Value = GetValueOrEmpty(trustee.FTER26) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428266
            
                    #region VR.ATO.GEN.428267
            
                    /*  VR.ATO.GEN.428267
                    Other given name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^FTER26, '"--","''","  "'))
            
                    Data Elements:
            
                    ^FTER26 = RP:Trustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER26, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000451", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428267"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER26", Value = GetValueOrEmpty(trustee.FTER26) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428267
            
                    #region VR.ATO.GEN.430246
            
                    /*  VR.ATO.GEN.430246
                    Suburb/Town/City cannot contain numeric characters
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER126 = FALSE) AND (ContainsSet(^FTER32, '0-9'))
            
                    Data Elements:
            
                    ^FTER32 = RP:Trustee:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER126 = RP:Trustee:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (trustee.FTER126 == false && IsMatch(trustee.FTER32, @"[0-9]",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain numeric characters",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER126", Value = GetValueOrEmpty(trustee.FTER126) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER32", Value = trustee.FTER32 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430246
            
                    #region VR.ATO.GEN.434147
            
                    /*  VR.ATO.GEN.434147
                    State and post code must be present for an Australian address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^FTER126 = FALSE AND (^FTER33 = NULL OR ^FTER34 = NULL))
            
                    Data Elements:
            
                    ^FTER126 = RP:Trustee:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER33 = RP:Trustee:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER34 = RP:Trustee:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:Trustee:AddressDetails
                    */
                    assertion = (trustee.Trustee_AddressDetailsCollection_POS_C_RPExists != false && (trustee.FTER126 == false && (trustee.FTER33 == null || trustee.FTER34 == null)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                            Description = @"State and post code must be present for an Australian address",
                            LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER126", Value = GetValueOrEmpty(trustee.FTER126) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER33", Value = GetValueOrEmpty(trustee.FTER33) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER34", Value = GetValueOrEmpty(trustee.FTER34) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.434147
            
                    #region VR.ATO.GEN.500132
            
                    /*  VR.ATO.GEN.500132
                    Organisation name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^OrganisationNameDetails <> NULL) AND (^FTER27 = BLANK)
            
                    Data Elements:
            
                    ^FTER27 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^OrganisationNameDetails = RP:Trustee:OrganisationNameDetails
                    */
                    assertion = (trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(trustee.FTER27) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must be present.",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER27", Value = trustee.FTER27 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500132
            
                    #region VR.ATO.GEN.500133
            
                    /*  VR.ATO.GEN.500133
                    Address Line 1 must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^FTER30 = BLANK)
            
                    Data Elements:
            
                    ^FTER30 = RP:Trustee:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:Trustee:AddressDetails
                    */
                    assertion = (trustee.Trustee_AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(trustee.FTER30) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must be present.",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER30", Value = trustee.FTER30 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500133
            
                    #region VR.ATO.GEN.500134
            
                    /*  VR.ATO.GEN.500134
                    Suburb/Town/City must be present in an address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^FTER32 = BLANK)
            
                    Data Elements:
            
                    ^FTER32 = RP:Trustee:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:Trustee:AddressDetails
                    */
                    assertion = (trustee.Trustee_AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(trustee.FTER32) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City must be present in an address.",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER32", Value = trustee.FTER32 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500134
            
                    #region VR.ATO.GEN.500135
            
                    /*  VR.ATO.GEN.500135
                    Family Name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF pyde.xx.xx:PersonNameDetails.FamilyName.Text = BLANK
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND (^FTER23 = BLANK)
            
                    Data Elements:
            
                    ^FTER23 = RP:Trustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:Trustee:PersonNameDetails
                    */
                    assertion = (trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPExists != false && string.IsNullOrWhiteSpace(trustee.FTER23) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500135", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name must be present.",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500135"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER23", Value = trustee.FTER23 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500135
            
                    #region VR.ATO.GEN.500141
            
                    /*  VR.ATO.GEN.500141
                    For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
                    AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER33 <> BLANK) AND (NotInSet(^FTER33, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
            
                    Data Elements:
            
                    ^FTER33 = RP:Trustee:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trustee.FTER33) != true && !(IsMatch(trustee.FTER33, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                            Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER33", Value = GetValueOrEmpty(trustee.FTER33) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500141
            
                    #region VR.ATO.GEN.500142
            
                    /*  VR.ATO.GEN.500142
                    Family name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF pyde.xx.xx:PersonNameDetails.FamilyName.Text DOES NOT CONTAIN SET("A-Z","a-z")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND (NotContainsSet(^FTER23, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^FTER23 = RP:Trustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:Trustee:PersonNameDetails
                    */
                    assertion = (trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPExists != false && !(IsMatch(trustee.FTER23, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500142", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name must contain at least one alphabetical character.",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500142"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER23", Value = trustee.FTER23 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500142
            
                    #region VR.ATO.GEN.500151
            
                    /*  VR.ATO.GEN.500151
                    First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER25, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^FTER25 = RP:Trustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER25, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010436", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500151"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER25", Value = GetValueOrEmpty(trustee.FTER25) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500151
            
                    #region VR.ATO.GEN.500152
            
                    /*  VR.ATO.GEN.500152
                    Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER26, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^FTER26 = RP:Trustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER26, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010448", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500152"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER26", Value = GetValueOrEmpty(trustee.FTER26) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500152
            
                    #region VR.ATO.GEN.500153
            
                    /*  VR.ATO.GEN.500153
                    Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER23, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^FTER23 = RP:Trustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trustee.FTER23, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500153", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500153"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER23", Value = trustee.FTER23 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500153
                    }
                }
        
                #endregion // Foreach loop
    
            #region VR.ATO.FTER.408006
    
            /*  VR.ATO.FTER.408006
            A schedule must not have its own intermediary as the intermediary on the main form will submit and declare any schedules attached to the main form.
    
            Legacy Rule Format:
            WHERE PARENT RETURN EXISTS
            IF COUNT (INT) > 0
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 0
    
            Data Elements:
    
            ^Context = INT
            */
            if (!isStandalone)
            {
                assertion = (Count(report.INTCount) > 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.FTER.408006", Severity = ProcessMessageSeverity.Error,
                        Description = @"Schedule must not have an intermediary",
                        LongDescription = @"Any intermediary on a parent return also applies to its schedules therefore a schedule cannot have an intermediary.",
                        Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTLastOccurrenceIndex),
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408006"} },
                    };
                    response.Add(processMessage);
                }
                }
            #endregion // VR.ATO.FTER.408006
    
            #region VR.ATO.FTER.408022
    
            /*  VR.ATO.FTER.408022
            Answer to Election, revocation or election question must be present
    
            Legacy Rule Format:
            IF [FTER95] = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            ^FTER95 = BLANK
    
            Data Elements:
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER95) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408022", Severity = ProcessMessageSeverity.Error,
                    Description = @"Answer to election, revocation or election question must be present",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408022"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408022
    
            #region VR.ATO.FTER.408023
    
            /*  VR.ATO.FTER.408023
            Year of return must be present when FTER is a stand-alone form
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF (RP:pyin.xx.xx.Lodgment.PeriodYear.Year = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER1 = NULL
    
            Data Elements:
    
            ^FTER1 = RP:Lodgment.PeriodYear.Year
            */
            if (isStandalone)
            {
                assertion = (report.FTER1 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.FTER.408023", Severity = ProcessMessageSeverity.Error,
                        Description = @"Year of return must be present when FTER is a stand-alone form",
                        Location = "/xbrli:xbrl/tns:Lodgment.PeriodYear.Year",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408023"} },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER1", Value = GetValueOrEmpty(report.FTER1) });
        
                    response.Add(processMessage);
                }
                }
            #endregion // VR.ATO.FTER.408023
    
            #region VR.ATO.FTER.408027
    
            /*  VR.ATO.FTER.408027
            Family trust revocation details must be left blank
    
            Legacy Rule Format:
            IF [FTER95] = SET("E", "V") AND ([FTER57] <> NULL OR [FTER96] <> NULL OR [FTER58] <> NULL OR [FTER60] <> NULLORBLANK OR [FTER61] <> NULL OR (COUNT TUPLE(fter.0002.lodge.req.xx.xx:InterposedEntity) > 0))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER57] = FTER:RP:rvctc3.02.00:Elections.Revocation.Date
            [FTER58] = FTER:RP:rvctc3.02.00:Elections.NotificationOfElectionBeingRevoked.Year
            [FTER60] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier
            [FTER61] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code
            [FTER96] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustRevocationStatusEffective.Year

            Technical Business Rule Format:
            InSet(^FTER95, '"E", "V"') AND (^FTER57 <> NULL OR ^FTER96 <> NULL OR ^FTER58 <> NULL OR ^FTER60 <> BLANK OR ^FTER61 <> NULL OR (Count(^InterposedEntity) > 0))
    
            Data Elements:
    
            ^FTER58 = RP:Elections.NotificationOfElectionBeingRevoked.Year
    
            ^FTER57 = RP:Elections.Revocation.Date
    
            ^FTER60 = RP:Identifiers.TaxFileNumber.Identifier
    
            ^FTER61 = RP:Identifiers.TaxFileNumberInexistent.Indicator
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
    
            ^FTER96 = RP:Elections.FamilyTrustRevocationStatusEffective.Year
    
            ^InterposedEntity = RP:InterposedEntity
            */
            assertion = (IsMatch(report.FTER95, @"^(E|V)$") && (report.FTER57 != null || report.FTER96 != null || report.FTER58 != null || string.IsNullOrWhiteSpace(report.FTER60) != true || report.FTER61 != null || Count(report.InterposedEntityCollectionCount) > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408027", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family trust revocation details must be left blank",
                    LongDescription = @"If the election-revocation code is  'E' - election or 'V' - variation, then revocation details must not be completed",
                    Location = "/xbrli:xbrl/tns:Elections.NotificationOfElectionBeingRevoked.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408027"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER57", Value = GetValueOrEmptyDate(report.FTER57) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER96", Value = GetValueOrEmpty(report.FTER96) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER58", Value = GetValueOrEmpty(report.FTER58) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER60", Value = GetValueOrEmpty(report.FTER60) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER61", Value = GetValueOrEmpty(report.FTER61) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408027
    
            #region VR.ATO.FTER.408030
    
            /*  VR.ATO.FTER.408030
            Details required for a variation must be completed
    
            Legacy Rule Format:
            IF [FTER95] = "V" AND (([FTER40] = NULL AND [FTER43] = NULL) OR COUNT (TUPLE(fter.0002.lodge.req.xx.xx:SpecifiedIndividual)) = 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER40] = FTER:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
            [FTER43] = FTER:RP:rvctc3.02.00:Elections.Commencement.Date
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            ^FTER95 = 'V' AND ((^FTER40 = NULL AND ^FTER43 = NULL) OR Count(^SpecifiedIndividual) = 0)
    
            Data Elements:
    
            ^FTER40 = RP:Elections.FamilyTrustElectionStatus.Year
    
            ^FTER43 = RP:Elections.Commencement.Date
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
    
            ^SpecifiedIndividual = RP:SpecifiedIndividual
            */
            assertion = (report.FTER95 == @"V" && (report.FTER40 == null && report.FTER43 == null || Count(report.SpecifiedIndividualCollectionCount) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408030", Severity = ProcessMessageSeverity.Error,
                    Description = @"Details required for a variation must be completed",
                    LongDescription = @"When used for a variation the 'Income Year Specified' or 'Commencement Year' must be completed; and the TFN, name, address and date of birth of the Specified Individual must be completed",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408030"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER40", Value = GetValueOrEmpty(report.FTER40) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER43", Value = GetValueOrEmptyDate(report.FTER43) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408030
    
            #region VR.ATO.FTER.408032
    
            /*  VR.ATO.FTER.408032
            Revocation indicated and date FTE ceases to be in force or income year of revocation is missing
    
            Legacy Rule Format:
            IF [FTER95] = "R" AND [FTER57] = NULL AND [FTER96] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER57] = FTER:RP:rvctc3.02.00:Elections.Revocation.Date
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code
            [FTER96] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustRevocationStatusEffective.Year

            Technical Business Rule Format:
            ^FTER95 = 'R' AND ^FTER57 = NULL AND ^FTER96 = NULL
    
            Data Elements:
    
            ^FTER57 = RP:Elections.Revocation.Date
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
    
            ^FTER96 = RP:Elections.FamilyTrustRevocationStatusEffective.Year
            */
            assertion = (report.FTER95 == @"R" && report.FTER57 == null && report.FTER96 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408032", Severity = ProcessMessageSeverity.Error,
                    Description = @"Either the date the election ceases to be in force, or the income year of revocation must be present",
                    LongDescription = @"For a revocation, either the date the election ceases to be in force must be present or the income year from which the revocation is to be effective must be present",
                    Location = "/xbrli:xbrl/tns:Elections.Revocation.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408032"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER57", Value = GetValueOrEmptyDate(report.FTER57) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER96", Value = GetValueOrEmpty(report.FTER96) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408032
    
            #region VR.ATO.FTER.408053
    
            /*  VR.ATO.FTER.408053
            Election details must be completed
    
            Legacy Rule Format:
            IF [FTER95] = "E" AND ([FTER16] = NULL OR [FTER40] = NULL OR COUNT (TUPLE(fter.0002.lodge.req.xx.xx:SpecifiedIndividual)) <> 1 OR COUNT (TUPLE(fter.0002.lodge.req.xx.xx:Trustee)) = 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER16] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustralia.Indicator
            [FTER40] = FTER:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            ^FTER95 = 'E' AND (^FTER16 = NULL OR ^FTER40 = NULL OR Count(^SpecifiedIndividual) <> 1 OR Count(^Trustee) = 0)
    
            Data Elements:
    
            ^FTER16 = RP:Residency.CentralManagementAndControlOutsideAustralia.Indicator
    
            ^FTER40 = RP:Elections.FamilyTrustElectionStatus.Year
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
    
            ^SpecifiedIndividual = RP:SpecifiedIndividual
    
            ^Trustee = RP:Trustee
            */
            assertion = (report.FTER95 == @"E" && (report.FTER16 == null || report.FTER40 == null || Count(report.SpecifiedIndividualCollectionCount) != 1 || Count(report.TrusteeCollectionCount) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Election details must be completed",
                    LongDescription = @"For an election, the central management question, details of at least one trustee, income year and details of the specified individual must all be present",
                    Location = "/xbrli:xbrl/tns:Residency.CentralManagementAndControlOutsideAustralia.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408053"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER16", Value = GetValueOrEmpty(report.FTER16) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER40", Value = GetValueOrEmpty(report.FTER40) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408053
    
            #region VR.ATO.FTER.408054
    
            /*  VR.ATO.FTER.408054
            Revocation details must not be provided with election or variation details
    
            Legacy Rule Format:
            IF ([FTER16] <> NULL OR [FTER17] <> NULL OR COUNT (TUPLE(xbrli\perioddetails1.xx.xx.PeriodDetails) WHERE TUPLE ELEMENT EXPLICIT pyin.xx.xx:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") > 0 OR COUNT (TUPLE(xbrli\perioddetails1.xx.xx.PeriodDetails) WHERE TUPLE ELEMENT EXPLICIT pyin.xx.xx:Period.Type.Code = "SubstitutedAccountingPeriod") > 0 OR COUNT (TUPLE(fter.0002.lodge.req.xx.xx:Trustee)) > 0 OR [FTER43] <> NULL OR COUNT (TUPLE(fter.0002.lodge.req.xx.xx:SpecifiedIndividual)) > 0) AND ([FTER57] <> NULL OR [FTER96] <> NULL OR [FTER58] <> NULL OR [FTER60] <> NULLORBLANK OR [FTER61] <> NULL OR COUNT (TUPLE(fter.0002.lodge.req.xx.xx:InterposedEntity)) > 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER16] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustralia.Indicator
            [FTER17] = FTER:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
            [FTER43] = FTER:RP:rvctc3.02.00:Elections.Commencement.Date
            [FTER57] = FTER:RP:rvctc3.02.00:Elections.Revocation.Date
            [FTER58] = FTER:RP:rvctc3.02.00:Elections.NotificationOfElectionBeingRevoked.Year
            [FTER60] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier
            [FTER61] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator
            [FTER96] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustRevocationStatusEffective.Year

            Technical Business Rule Format:
            (^FTER16 <> NULL OR ^FTER17 <> NULL OR Count(^FTER88) > 0 OR Count(^FTER89) > 0 OR Count(^Trustee) > 0 OR ^FTER43 <> NULL OR Count(^SpecifiedIndividual) > 0) AND (^FTER57 <> NULL OR ^FTER96 <> NULL OR ^FTER58 <> NULL OR ^FTER60 <> BLANK OR ^FTER61 <> NULL OR Count(^InterposedEntity) > 0)
    
            Data Elements:
    
            ^FTER16 = RP:Residency.CentralManagementAndControlOutsideAustralia.Indicator
    
            ^FTER17 = RP:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
    
            ^FTER43 = RP:Elections.Commencement.Date
    
            ^FTER57 = RP:Elections.Revocation.Date
    
            ^FTER58 = RP:Elections.NotificationOfElectionBeingRevoked.Year
    
            ^FTER60 = RP:Identifiers.TaxFileNumber.Identifier
    
            ^FTER61 = RP:Identifiers.TaxFileNumberInexistent.Indicator
    
            ^FTER96 = RP:Elections.FamilyTrustRevocationStatusEffective.Year
    
            ^Trustee = RP:Trustee
    
            ^SpecifiedIndividual = RP:SpecifiedIndividual
    
            ^InterposedEntity = RP:InterposedEntity
    
            ^FTER88 = RP:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
    
            ^FTER89 = RP:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
            */
            assertion = ((report.FTER16 != null || report.FTER17 != null || Count(report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection == null ? new List<string>() : report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection.Select(f => f.FTER88)) > 0 || Count(report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCount) > 0 || Count(report.TrusteeCollectionCount) > 0 || report.FTER43 != null || Count(report.SpecifiedIndividualCollectionCount) > 0) && (report.FTER57 != null || report.FTER96 != null || report.FTER58 != null || string.IsNullOrWhiteSpace(report.FTER60) != true || report.FTER61 != null || Count(report.InterposedEntityCollectionCount) > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Revocation details must not be provided with election or variation details",
                    LongDescription = @"A single FTER schedule must not contain details of a revocation in addition to details related to an election or a variation",
                    Location = "/xbrli:xbrl/tns:Residency.CentralManagementAndControlOutsideAustralia.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408054"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER16", Value = GetValueOrEmpty(report.FTER16) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER17", Value = GetValueOrEmpty(report.FTER17) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER43", Value = GetValueOrEmptyDate(report.FTER43) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER57", Value = GetValueOrEmptyDate(report.FTER57) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER96", Value = GetValueOrEmpty(report.FTER96) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER58", Value = GetValueOrEmpty(report.FTER58) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER60", Value = GetValueOrEmpty(report.FTER60) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER61", Value = GetValueOrEmpty(report.FTER61) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408054
    
            #region VR.ATO.FTER.408064
    
            /*  VR.ATO.FTER.408064
            Number of Trustees must not exceed 10
    
            Legacy Rule Format:
            IF COUNT (TUPLE(fter.0002.lodge.req.xx.xx:Trustee)) > 10
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Trustee) > 10
    
            Data Elements:
    
            ^Trustee = RP:Trustee
            */
            assertion = (Count(report.TrusteeCollectionCount) > 10);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408064", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of Trustees must not exceed 10",
                    Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(report.TrusteeCollection == null ? 0 : (report.TrusteeCollection.Count() == 0 ? 0 : report.TrusteeCollection.Last().OccurrenceIndex)) + "/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408064"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408064
    
            #region VR.ATO.FTER.408087
    
            /*  VR.ATO.FTER.408087
            Income year specified must not be less than 2005
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF [FTER95] = "E" AND [FTER40] <> NULL AND [FTER40] < 2005
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER40] = FTER:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            ^FTER95 = 'E' AND ^FTER40 <> NULL AND ^FTER40 < 2005
    
            Data Elements:
    
            ^FTER40 = RP:Elections.FamilyTrustElectionStatus.Year
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
            */
            if (isStandalone)
            {
                assertion = (report.FTER95 == @"E" && report.FTER40 != null && report.FTER40.GetValueOrDefault() < 2005);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.FTER.408087", Severity = ProcessMessageSeverity.Error,
                        Description = @"Income year specified must not be less than 2005",
                        Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionStatus.Year",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408087"} },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER95", Value = report.FTER95 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER40", Value = GetValueOrEmpty(report.FTER40) });
        
                    response.Add(processMessage);
                }
                }
            #endregion // VR.ATO.FTER.408087
    
            #region VR.ATO.FTER.408088
    
            /*  VR.ATO.FTER.408088
            Election commencement time must not be before the start of specified income year
    
            Legacy Rule Format:
            IF [FTER40] <> NULL AND [FTER43] <> NULL AND [FTER41] = NULL AND [FTER43] < (([FTER40] - 1)&"-07-01")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER40] = FTER:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
            [FTER41] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails)
            [FTER43] = FTER:RP:rvctc3.02.00:Elections.Commencement.Date

            Technical Business Rule Format:
            ^FTER40 <> NULL AND ^FTER43 <> NULL AND ^FTER41 = NULL AND ^FTER43 < AsDate(Concat(^FTER40 - 1, '-07-01'))
    
            Data Elements:
    
            ^FTER40 = RP:Elections.FamilyTrustElectionStatus.Year
    
            ^FTER41 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^FTER43 = RP:Elections.Commencement.Date
            */
            assertion = (report.FTER40 != null && report.FTER43 != null && report.FTER41 == null && report.FTER43.GetValueOrDefault() < AsDate(string.Concat(report.FTER40.GetValueOrDefault() - 1, @"-07-01")));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408088", Severity = ProcessMessageSeverity.Error,
                    Description = @"Election commencement time must not be before the start of specified income year",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408088"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER40", Value = GetValueOrEmpty(report.FTER40) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER43", Value = GetValueOrEmptyDate(report.FTER43) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER41", Value = GetValueOrEmptyDate(report.FTER41) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408088
    
            #region VR.ATO.FTER.408096
    
            /*  VR.ATO.FTER.408096
            Election commencement time must not be before the start of the substituted accounting period
    
            Legacy Rule Format:
            IF [FTER43] <> NULL AND [FTER41] <> NULL AND [FTER43] < [FTER41]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER41] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails)
            [FTER43] = FTER:RP:rvctc3.02.00:Elections.Commencement.Date

            Technical Business Rule Format:
            ^FTER43 <> NULL AND ^FTER41 <> NULL AND ^FTER43 < ^FTER41
    
            Data Elements:
    
            ^FTER41 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^FTER43 = RP:Elections.Commencement.Date
            */
            assertion = (report.FTER43 != null && report.FTER41 != null && report.FTER43.GetValueOrDefault() < report.FTER41.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408096", Severity = ProcessMessageSeverity.Error,
                    Description = @"Election commencement time must not be before the start of the substituted accounting period",
                    Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408096"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER43", Value = GetValueOrEmptyDate(report.FTER43) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER41", Value = GetValueOrEmptyDate(report.FTER41) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408096
    
            #region VR.ATO.FTER.408098
    
            /*  VR.ATO.FTER.408098
            Specified individual date of birth cannot be later than the FTE commencement date
    
            Legacy Rule Format:
            IF ([FTER43] <> NULL AND [FTER46] <> NULL AND [FTER46] > [FTER43]) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER43] = FTER:RP:rvctc3.02.00:Elections.Commencement.Date
            [FTER46] = FTER:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(fter.0002.lodge.req.02.00:SpecifiedIndividual)

            Technical Business Rule Format:
            ^FTER43 <> NULL AND ^FTER46 <> NULL AND ^FTER46 > ^FTER43
    
            Data Elements:
    
            ^FTER46 = RP:SpecifiedIndividual:PersonDemographicDetails.Birth.Date
    
            ^FTER43 = RP:Elections.Commencement.Date
            */
            assertion = (report.FTER43 != null && report.FTER46 != null && report.FTER46.GetValueOrDefault() > report.FTER43.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408098", Severity = ProcessMessageSeverity.Error,
                    Description = @"Specified individual date of birth cannot be later than the FTE commencement date",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonDemographicDetails.Birth.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408098"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER43", Value = GetValueOrEmptyDate(report.FTER43) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER46", Value = GetValueOrEmptyDate(report.FTER46) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408098
    
            #region VR.ATO.FTER.408099
    
            /*  VR.ATO.FTER.408099
            Date of Birth for the Specified Individual must be present (one only)
    
            Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:SpecifiedIndividual)
            IF PersonDemographicDetails.Birth.Date = NULL
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^SpecifiedIndividual <> NULL) AND (^FTER46 = NULL)
    
            Data Elements:
    
            ^FTER46 = RP:SpecifiedIndividual:PersonDemographicDetails.Birth.Date
    
            ^SpecifiedIndividual = RP:SpecifiedIndividual
            */
            assertion = (report.SpecifiedIndividualCollectionExists != false && report.FTER46 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408099", Severity = ProcessMessageSeverity.Error,
                    Description = @"Date of Birth for the Specified Individual must be present (one only)",
                    LongDescription = @"Date of Birth for the Specified Individual must be present (one only)",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonDemographicDetails.Birth.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408099"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER46", Value = GetValueOrEmptyDate(report.FTER46) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408099
    
            #region VR.ATO.FTER.408100
    
            /*  VR.ATO.FTER.408100
            No TFN indicator must not be 'yes' (true)
    
            Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:SpecifiedIndividual)
            IF [FTER45] = TRUE AND [FTER44] <> NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER44] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:SpecifiedIndividual)
            [FTER45] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(fter.0002.lodge.req.02.00:SpecifiedIndividual)

            Technical Business Rule Format:
            ^FTER45 = TRUE AND ^FTER44 <> BLANK
    
            Data Elements:
    
            ^FTER45 = RP:SpecifiedIndividual:Identifiers.TaxFileNumberInexistent.Indicator
    
            ^FTER44 = RP:SpecifiedIndividual:Identifiers.TaxFileNumber.Identifier
            */
            assertion = (report.FTER45 == true && string.IsNullOrWhiteSpace(report.FTER44) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408100", Severity = ProcessMessageSeverity.Error,
                    Description = @"No TFN indicator must not be 'yes' (true)",
                    LongDescription = @"If the TFN of the Specified individual is present, then the 'does not have a TFN indicator' must be 'no' (false) or left blank",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:Identifiers.TaxFileNumberInexistent.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408100"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER45", Value = GetValueOrEmpty(report.FTER45) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER44", Value = GetValueOrEmpty(report.FTER44) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408100
    
            #region VR.ATO.FTER.408101
    
            /*  VR.ATO.FTER.408101
            No TFN indicator must not be 'yes' (true)
    
            Legacy Rule Format:
            IF [FTER61] = TRUE AND [FTER60] <> NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER60] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier
            [FTER61] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator

            Technical Business Rule Format:
            ^FTER61 = TRUE AND ^FTER60 <> BLANK
    
            Data Elements:
    
            ^FTER61 = RP:Identifiers.TaxFileNumberInexistent.Indicator
    
            ^FTER60 = RP:Identifiers.TaxFileNumber.Identifier
            */
            assertion = (report.FTER61 == true && string.IsNullOrWhiteSpace(report.FTER60) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408101", Severity = ProcessMessageSeverity.Error,
                    Description = @"No TFN indicator must not be 'yes' (true)",
                    LongDescription = @"If the TFN of the Specified individual is present, then the 'does not have a TFN indicator' must be 'no' (false) or left blank",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumberInexistent.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408101"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER61", Value = GetValueOrEmpty(report.FTER61) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER60", Value = GetValueOrEmpty(report.FTER60) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408101
    
            #region VR.ATO.FTER.408102
    
            /*  VR.ATO.FTER.408102
            Income Year must be present
    
            Legacy Rule Format:
            IF [FTER95] = "R" AND [FTER58] = NULL AND [FTER40] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER40] = FTER:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
            [FTER58] = FTER:RP:rvctc3.02.00:Elections.NotificationOfElectionBeingRevoked.Year
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            ^FTER95 = 'R' AND ^FTER58 = NULL AND ^FTER40 = NULL
    
            Data Elements:
    
            ^FTER58 = RP:Elections.NotificationOfElectionBeingRevoked.Year
    
            ^FTER40 = RP:Elections.FamilyTrustElectionStatus.Year
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
            */
            assertion = (report.FTER95 == @"R" && report.FTER58 == null && report.FTER40 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408102", Severity = ProcessMessageSeverity.Error,
                    Description = @"Income Year must be present",
                    LongDescription = @"For a revocation, if the TFN or 'No TFN indicator' of the individual specified in the election is present, then the Income Year for the tax return that the election details were provided, and/or the  Income year specified in the election being revoked must be present",
                    Location = "/xbrli:xbrl/tns:Elections.NotificationOfElectionBeingRevoked.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408102"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER58", Value = GetValueOrEmpty(report.FTER58) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER40", Value = GetValueOrEmpty(report.FTER40) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408102
    
            #region VR.ATO.FTER.408107
    
            /*  VR.ATO.FTER.408107
            TFN of specified individual or 'no TFN indicator' must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:SpecifiedIndividual)
            IF [FTER44] = NULL AND [FTER45] <> TRUE
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER44] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:SpecifiedIndividual)
            [FTER45] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(fter.0002.lodge.req.02.00:SpecifiedIndividual)

            Technical Business Rule Format:
            (^SpecifiedIndividual <> NULL) AND (^FTER44 = NULL AND ^FTER45 <> TRUE)
    
            Data Elements:
    
            ^FTER44 = RP:SpecifiedIndividual:Identifiers.TaxFileNumber.Identifier
    
            ^FTER45 = RP:SpecifiedIndividual:Identifiers.TaxFileNumberInexistent.Indicator
    
            ^SpecifiedIndividual = RP:SpecifiedIndividual
            */
            assertion = (report.SpecifiedIndividualCollectionExists != false && (report.FTER44 == null && report.FTER45 != true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408107", Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN of specified individual or 'no TFN indicator' must be present",
                    LongDescription = @"The TFN of the specified individual must be present or the 'specified individual does not have a TFN indicator' must be 'yes' (true)",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408107"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER44", Value = GetValueOrEmpty(report.FTER44) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER45", Value = GetValueOrEmpty(report.FTER45) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408107
    
            #region VR.ATO.FTER.408113
    
            /*  VR.ATO.FTER.408113
            For a revocation, the Income year for which details of the election being revoked were included in the tax return must not be less than 1998
    
            Legacy Rule Format:
            IF [FTER58] <> NULL AND [FTER58] < 1998
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER58] = FTER:RP:rvctc3.02.00:Elections.NotificationOfElectionBeingRevoked.Year

            Technical Business Rule Format:
            ^FTER58 <> NULL AND ^FTER58 < 1998
    
            Data Elements:
    
            ^FTER58 = RP:Elections.NotificationOfElectionBeingRevoked.Year
            */
            assertion = (report.FTER58 != null && report.FTER58.GetValueOrDefault() < 1998);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408113", Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year must not be less than 1998",
                    LongDescription = @"For a revocation, the Income year for which details of the election being revoked were included in the tax return must not be less than 1998",
                    Location = "/xbrli:xbrl/tns:Elections.NotificationOfElectionBeingRevoked.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408113"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER58", Value = GetValueOrEmpty(report.FTER58) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408113
    
            #region VR.ATO.FTER.408114
    
            /*  VR.ATO.FTER.408114
            Specified individual TFN or 'No TFN indicator' must be present for revocation
    
            Legacy Rule Format:
            IF [FTER95] = "R" AND [FTER60] = NULLORBLANK AND [FTER61] <> TRUE
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER60] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier
            [FTER61] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            ^FTER95 = 'R' AND ^FTER60 = BLANK AND ^FTER61 <> TRUE
    
            Data Elements:
    
            ^FTER60 = RP:Identifiers.TaxFileNumber.Identifier
    
            ^FTER61 = RP:Identifiers.TaxFileNumberInexistent.Indicator
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
            */
            assertion = (report.FTER95 == @"R" && string.IsNullOrWhiteSpace(report.FTER60) == true && report.FTER61 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408114", Severity = ProcessMessageSeverity.Error,
                    Description = @"Specified individual TFN or 'No TFN indicator' must be present for revocation",
                    LongDescription = @"For a revocation, the TFN of the specified individual must be present or the 'No TFN indicator' must be 'yes' (true)",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408114"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER60", Value = GetValueOrEmpty(report.FTER60) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER61", Value = GetValueOrEmpty(report.FTER61) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408114
    
            #region VR.ATO.FTER.408115
    
            /*  VR.ATO.FTER.408115
            For a revocation, the Income year as it appeared in the family trust election must not be less than 1995
    
            Legacy Rule Format:
            IF [FTER95] = "R" AND [FTER40] <> NULL AND [FTER40] < 1995
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER40] = FTER:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            ^FTER95 = 'R' AND ^FTER40 <> NULL AND ^FTER40 < 1995
    
            Data Elements:
    
            ^FTER40 = RP:Elections.FamilyTrustElectionStatus.Year
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
            */
            assertion = (report.FTER95 == @"R" && report.FTER40 != null && report.FTER40.GetValueOrDefault() < 1995);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408115", Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year must not be less than 1995",
                    LongDescription = @"For a revocation, the Income year as it appeared in the family trust election must not be less than 1995",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408115"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER40", Value = GetValueOrEmpty(report.FTER40) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408115
    
            #region VR.ATO.FTER.408116
    
            /*  VR.ATO.FTER.408116
            Year of election must not be later than return year
    
            Legacy Rule Format:
            IF [FTER95] = "R" AND [FTER58] <> NULL AND [FTER40] > [FTER58]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER40] = FTER:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
            [FTER58] = FTER:RP:rvctc3.02.00:Elections.NotificationOfElectionBeingRevoked.Year
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            ^FTER95 = 'R' AND ^FTER58 <> NULL AND ^FTER40 > ^FTER58
    
            Data Elements:
    
            ^FTER40 = RP:Elections.FamilyTrustElectionStatus.Year
    
            ^FTER58 = RP:Elections.NotificationOfElectionBeingRevoked.Year
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
            */
            assertion = (report.FTER95 == @"R" && report.FTER58 != null && report.FTER40.GetValueOrDefault() > report.FTER58.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408116", Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of election must not be later than return year",
                    LongDescription = @"For a revocation, the income year as it appeared in the family trust election included in the return for the trust, must not be later than the income year of the tax return in which the election was made",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408116"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER58", Value = GetValueOrEmpty(report.FTER58) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER40", Value = GetValueOrEmpty(report.FTER40) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408116
    
            #region VR.ATO.FTER.408145
    
            /*  VR.ATO.FTER.408145
            Current full name of the trust must be present (once only)
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF COUNT (TUPLE(xbrli\organisationname2.xx.xx:OrganisationNameDetails) WHERE TUPLE ELEMENT EXPLICIT OrganisationNameDetails.Currency.Code = "C" AND TUPLE ELEMENT EXPLICIT OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND TUPLE ELEMENT EXPLICIT OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) <> 1
               RETURN VALIDATION  MESSAGE
            ENDIF

            Technical Business Rule Format:
            Not(Count(^OrganisationNameDetails) = 1 AND ^FTER4 <> BLANK)
    
            Data Elements:
    
            ^FTER4 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^OrganisationNameDetails = RP:OrganisationNameDetails
            */
            if (isStandalone)
            {
                assertion =  !((Count(report.OrganisationNameDetailsCollection_MN_C_RPCount) == 1 && string.IsNullOrWhiteSpace(report.FTER4) != true) );
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.FTER.408145", Severity = ProcessMessageSeverity.Error,
                        Description = @"Current full name of the trust must be present (once only)",
                        LongDescription = @"The name of the trust must be present with a Currency Code of 'C' and a Name Type code of 'MN'. No more than once instance may be present",
                        Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408145"} },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER4", Value = report.FTER4 });
        
                    response.Add(processMessage);
                }
                }
            #endregion // VR.ATO.FTER.408145
    
            #region VR.ATO.FTER.408146
    
            /*  VR.ATO.FTER.408146
            Current postal address of the trust must be present (once only)
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF COUNT (TUPLE(xbrli\address2.xx.xx.AddressDetails) WHERE TUPLE ELEMENT EXPLICIT pyde.xx.xx:AddressDetails.Currency.Code = "C" AND TUPLE ELEMENT EXPLICIT pyde.xx.xx:AddressDetails.Usage.Code = "POS" AND TUPLE ELEMENT EXPLICIT pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) <> 1 
               RETURN VALIDATION  MESSAGE
            ENDIF

            Technical Business Rule Format:
            Not(Count(^AddressDetails) = 1 AND ^FTER5 <> BLANK)
    
            Data Elements:
    
            ^FTER5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            if (isStandalone)
            {
                assertion =  !((Count(report.AddressDetailsCollection_POS_C_RPCount) == 1 && string.IsNullOrWhiteSpace(report.FTER5) != true) );
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.FTER.408146", Severity = ProcessMessageSeverity.Error,
                        Description = @"Current postal address of the trust must be present (once only)",
                        Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPLastOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408146"} },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER5", Value = report.FTER5 });
        
                    response.Add(processMessage);
                }
                }
            #endregion // VR.ATO.FTER.408146
    
            #region VR.ATO.FTER.408150
    
            /*  VR.ATO.FTER.408150
            An address for the Specified individual must be present (one only)
    
            Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:SpecifiedIndividual)
            IF COUNT (TUPLE(address2.xx.xx.AddressDetails)) <> 1
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^SpecifiedIndividual <> NULL AND Count(^AddressDetails) <> 1
    
            Data Elements:
    
            ^SpecifiedIndividual = RP:SpecifiedIndividual
    
            ^AddressDetails = RP:SpecifiedIndividual:AddressDetails
            */
            assertion = (report.SpecifiedIndividualCollectionExists != false && Count(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPCount) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408150", Severity = ProcessMessageSeverity.Error,
                    Description = @"An address for the Specified individual must be present (one only)",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPLastOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408150"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408150
    
            #region VR.ATO.FTER.408151
    
            /*  VR.ATO.FTER.408151
            Number of interposed entities supplied must not exceed 120
    
            Legacy Rule Format:
            IF COUNT (TUPLE(fter.0002.lodge.req.xx.xx:InterposedEntity)) > 120
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^InterposedEntity) > 120
    
            Data Elements:
    
            ^InterposedEntity = RP:InterposedEntity
            */
            assertion = (Count(report.InterposedEntityCollectionCount) > 120);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408151", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of interposed entities supplied must not exceed 120",
                    LongDescription = @"SBR limits the submission within a single FTER schedule of the number of entities which have an interposed entity election in force in respect of a trust to 120",
                    Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(report.InterposedEntityCollection == null ? 0 : (report.InterposedEntityCollection.Count() == 0 ? 0 : report.InterposedEntityCollection.Last().OccurrenceIndex)) + "/tns:Party.Type.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408151"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408151
    
            #region VR.ATO.FTER.408154
    
            /*  VR.ATO.FTER.408154
            Number of Trustee Declarations supplied must exceed 10
    
            Legacy Rule Format:
            IF COUNT(fter.0002.lodge.req.xx.xx:DeclarationOfTrustee) > 10
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^DeclarationOfTrustee) > 10
    
            Data Elements:
    
            ^DeclarationOfTrustee = RP:DeclarationOfTrustee
            */
            assertion = (Count(report.DeclarationOfTrusteeCollectionCount) > 10);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408154", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of Trustee Declarations supplied must not exceed 10",
                    Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(report.DeclarationOfTrusteeCollection == null ? 0 : (report.DeclarationOfTrusteeCollection.Count() == 0 ? 0 : report.DeclarationOfTrusteeCollection.Last().OccurrenceIndex)) + "/tns:Declaration/tns:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408154"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408154
    
            #region VR.ATO.FTER.408155
    
            /*  VR.ATO.FTER.408155
            Number of Trustee Declarations supplied must equal number of trustees
    
            Legacy Rule Format:
            IF COUNT(INT) = 0 AND [FTER95] = "E" AND COUNT(fter.0002.lodge.req.xx.xx:DeclarationOfTrustee) <> COUNT(fter.0002.lodge.req.xx.xx:Trustee)
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            Count(^Context) = 0 AND ^FTER95 = 'E' AND Count(^DeclarationOfTrustee) <> Count(^Trustee)
    
            Data Elements:
    
            ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
    
            ^DeclarationOfTrustee = RP:DeclarationOfTrustee
    
            ^Trustee = RP:Trustee
    
            ^Context = INT
            */
            assertion = (Count(report.INTCount) == 0 && report.FTER95 == @"E" && Count(report.DeclarationOfTrusteeCollectionCount) != Count(report.TrusteeCollectionCount));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408155", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of Trustee Declarations supplied must equal the number of trustees",
                    LongDescription = @"All of the trustees identified in an election must supply a Trustee Declaration",
                    Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(report.DeclarationOfTrusteeCollection == null ? 0 : (report.DeclarationOfTrusteeCollection.Count() == 0 ? 0 : report.DeclarationOfTrusteeCollection.Last().OccurrenceIndex)) + "/tns:Declaration/tns:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408155"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER95", Value = report.FTER95 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408155
    
            #region VR.ATO.FTER.408161
    
            /*  VR.ATO.FTER.408161
            Year of return must not be less than 2011
    
            Legacy Rule Format:
            IF (RP:pyin.xx.xx.Lodgment.PeriodYear.Year <> NULL) AND (RP:pyin.xx.xx:Lodgment.PeriodYear.Year < 2011)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER1 <> NULL) AND (^FTER1 < 2011)
    
            Data Elements:
    
            ^FTER1 = RP:Lodgment.PeriodYear.Year
            */
            assertion = (report.FTER1 != null && report.FTER1.GetValueOrDefault() < 2011);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FTER.408161", Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of return must not be less than 2011",
                    Location = "/xbrli:xbrl/tns:Lodgment.PeriodYear.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408161"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER1", Value = GetValueOrEmpty(report.FTER1) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408161
    
            #region VR.ATO.FTER.408165
    
            /*  VR.ATO.FTER.408165
            The ABN of the Reporting Party must be supplied on the standalone form if self-reporting (no intermediary) or a business intermediary
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXISTS
            IF COUNT(INT) = 0 AND RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) = 0 AND ^FTER170 = BLANK
    
            Data Elements:
    
            ^FTER170 = RP:Identifiers.AustralianBusinessNumber.Identifier
    
            ^Context = INT
            */
            if (isStandalone)
            {
                assertion = (Count(report.INTCount) == 0 && string.IsNullOrWhiteSpace(report.FTER170) == true);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.FTER.408165", Severity = ProcessMessageSeverity.Error,
                        Description = @"The ABN of the Reporting Party must be supplied on the standalone form if no intermediary.",
                        LongDescription = @"The ABN of the Reporting Party is required for authentication on the standalone form unless the from is submitted by a tax agent intermediary.",
                        Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408165"} },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER170", Value = GetValueOrEmpty(report.FTER170) });
        
                    response.Add(processMessage);
                }
                }
            #endregion // VR.ATO.FTER.408165
    
            #region VR.ATO.FTER.408168
    
            /*  VR.ATO.FTER.408168

        Legacy Rule Format:
            IF INT <> NULLORBLANK AND INT:pyin.xx.xx:Declaration.StatementAccepted.Indicator = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT <> BLANK AND ^FTER160 = NULL
    
            Data Elements:
    
            ^FTER160 = INT:Declaration:Declaration.StatementAccepted.Indicator WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            ^INT = INT
            */
            assertion = (report.INTExists != false && report.FTER160 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430249", Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration by the person submitting the return must be present",
                    LongDescription = @"If an Intermediary is present, there must be a Declaration by the Intermediary. If an Intermediary is not present, there must be a Declaration by the Reporting Party",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:Declaration.StatementType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408168"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER160", Value = GetValueOrEmpty(report.FTER160) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.FTER.408168
    
            #region VR.ATO.GEN.000201
    
            /*  VR.ATO.GEN.000201

        Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = INT
    
            ^EndDate = INT
            */
            assertion = ((report.INTExists == true) && (report.INTStartDate >= report.INTEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201
    
            #region VR.ATO.GEN.000201
    
            /*  VR.ATO.GEN.000201

        Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^EndDate = RP
            */
            assertion = ((report.RPExists == true) && (report.RPStartDate >= report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201
    
            #region VR.ATO.GEN.000209
    
            /*  VR.ATO.GEN.000209

        Legacy Rule Format:
            IF COUNT(RP) <> 1
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) <> 1
    
            Data Elements:
    
            ^Context = RP
            */
            assertion = (Count(report.RPCount) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430296", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party contexts must equal 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000209"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000209
    
            #region VR.ATO.GEN.000227
    
            /*  VR.ATO.GEN.000227

        Legacy Rule Format:
            IF COUNT(INT) > 1
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = INT
            */
            assertion = (Count(report.INTCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.436280", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Intermediary contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000227"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000227
    
            #region VR.ATO.GEN.000406
    
            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER52 <> BLANK) AND (StartsWithSet(^FTER52,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^FTER52, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^FTER52 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER52) != true && IsMatch(report.FTER52, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.FTER52, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER52", Value = report.FTER52 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406
    
            #region VR.ATO.GEN.000406
    
            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER5 <> BLANK) AND (StartsWithSet(^FTER5,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^FTER5, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^FTER5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER5) != true && IsMatch(report.FTER5, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.FTER5, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER5", Value = report.FTER5 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406
    
            #region VR.ATO.GEN.000406
    
            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER11 <> BLANK) AND (StartsWithSet(^FTER11,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^FTER11, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^FTER11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER11) != true && IsMatch(report.FTER11, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.FTER11, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER11", Value = report.FTER11 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406
    
            #region VR.ATO.GEN.000408
    
            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER134 <> BLANK) AND (StartsWithSet(^FTER134,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^FTER134, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^FTER134 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER134) != true && (IsMatch(report.FTER134, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.FTER134, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER134", Value = GetValueOrEmpty(report.FTER134) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408
    
            #region VR.ATO.GEN.000408
    
            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER110 <> BLANK) AND (StartsWithSet(^FTER110,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^FTER110, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^FTER110 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER110) != true && (IsMatch(report.FTER110, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.FTER110, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER110", Value = GetValueOrEmpty(report.FTER110) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408
    
            #region VR.ATO.GEN.000408
    
            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER115 <> BLANK) AND (StartsWithSet(^FTER115,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^FTER115, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^FTER115 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER115) != true && (IsMatch(report.FTER115, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.FTER115, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER115", Value = GetValueOrEmpty(report.FTER115) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408
    
            #region VR.ATO.GEN.000458
    
            /*  VR.ATO.GEN.000458
            Suffix is invalid
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER51 <> BLANK) AND (NotInDomain(^FTER51, 'SUFFIX CODES'))
    
            Data Elements:
    
            ^FTER51 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER51) != true && !(IsMatch(report.FTER51, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000458", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suffix is invalid",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.NameSuffix.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER51", Value = GetValueOrEmpty(report.FTER51) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000458
    
            #region VR.ATO.GEN.000459
    
            /*  VR.ATO.GEN.000459
            Title is invalid
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER47 <> BLANK) AND (NotInDomain(^FTER47, 'TITLE CODES'))
    
            Data Elements:
    
            ^FTER47 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER47) != true && !(IsMatch(report.FTER47, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000459", Severity = ProcessMessageSeverity.Error,
                    Description = @"Title is invalid",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.Title.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER47", Value = GetValueOrEmpty(report.FTER47) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000459
    
            #region VR.ATO.GEN.408060
    
            /*  VR.ATO.GEN.408060
            Start date must be before end date
    
            Legacy Rule Format:
            WHERE IN TUPLE (perioddetails1.xx.xx:PeriodDetails)
            IF pyin.xx.xx:Period.End.Date <= pyin.xx.xx:Period.Start.Date
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PeriodDetails <> NULL) AND (^FTER42 <= ^FTER41)
    
            Data Elements:
    
            ^FTER42 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^FTER41 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^PeriodDetails = RP:PeriodDetails
            */
            assertion = (report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPExists != false && report.FTER42.GetValueOrDefault() <= report.FTER41.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.408060", Severity = ProcessMessageSeverity.Error,
                    Description = @"Start date must be before end date",
                    LongDescription = @"The start date must be earlier than the end date in a Period.",
                    Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPOccurrenceIndex) + "/tns:Period.End.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.408060"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER42", Value = GetValueOrEmptyDate(report.FTER42) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER41", Value = GetValueOrEmptyDate(report.FTER41) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.408060
    
            #region VR.ATO.GEN.410002
    
            /*  VR.ATO.GEN.410002
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^FTER52, 'C/-')) AND (^FTER134 = BLANK))
    
            Data Elements:
    
            ^FTER52 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^FTER134 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.FTER52 == null ? false : report.FTER52.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.FTER134) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER52", Value = report.FTER52 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER134", Value = GetValueOrEmpty(report.FTER134) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002
    
            #region VR.ATO.GEN.410002
    
            /*  VR.ATO.GEN.410002
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^FTER5, 'C/-')) AND (^FTER110 = BLANK))
    
            Data Elements:
    
            ^FTER5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^FTER110 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.FTER5 == null ? false : report.FTER5.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.FTER110) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER5", Value = report.FTER5 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER110", Value = GetValueOrEmpty(report.FTER110) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002
    
            #region VR.ATO.GEN.410002
    
            /*  VR.ATO.GEN.410002
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^FTER11, 'C/-')) AND (^FTER115 = BLANK))
    
            Data Elements:
    
            ^FTER11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^FTER115 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = ((report.FTER11 == null ? false : report.FTER11.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.FTER115) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER11", Value = report.FTER11 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER115", Value = GetValueOrEmpty(report.FTER115) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002
    
            #region VR.ATO.GEN.410008
    
            /*  VR.ATO.GEN.410008
            Address Line 3 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER135 <> BLANK
    
            Data Elements:
    
            ^FTER135 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER135) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER135", Value = GetValueOrEmpty(report.FTER135) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008
    
            #region VR.ATO.GEN.410008
    
            /*  VR.ATO.GEN.410008
            Address Line 3 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER111 <> BLANK
    
            Data Elements:
    
            ^FTER111 = RP:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER111) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER111", Value = GetValueOrEmpty(report.FTER111) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008
    
            #region VR.ATO.GEN.410008
    
            /*  VR.ATO.GEN.410008
            Address Line 3 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER116 <> BLANK
    
            Data Elements:
    
            ^FTER116 = RP:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER116) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER116", Value = GetValueOrEmpty(report.FTER116) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008
    
            #region VR.ATO.GEN.410013
    
            /*  VR.ATO.GEN.410013
            Address Line 4 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER136 <> BLANK
    
            Data Elements:
    
            ^FTER136 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER136) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER136", Value = GetValueOrEmpty(report.FTER136) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013
    
            #region VR.ATO.GEN.410013
    
            /*  VR.ATO.GEN.410013
            Address Line 4 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER112 <> BLANK
    
            Data Elements:
    
            ^FTER112 = RP:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER112) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER112", Value = GetValueOrEmpty(report.FTER112) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013
    
            #region VR.ATO.GEN.410013
    
            /*  VR.ATO.GEN.410013
            Address Line 4 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER117 <> BLANK
    
            Data Elements:
    
            ^FTER117 = RP:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER117) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER117", Value = GetValueOrEmpty(report.FTER117) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013
    
            #region VR.ATO.GEN.410031
    
            /*  VR.ATO.GEN.410031

        Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier <> NULLORBLANK) AND (TFNALGORITHM (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier) = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER44 <> BLANK) AND (FailsTFNAlgorithm(^FTER44))
    
            Data Elements:
    
            ^FTER44 = RP:SpecifiedIndividual:Identifiers.TaxFileNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER44) != true && FailsTFNAlgorithm(report.FTER44));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410031", Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN must be a valid TFN",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410031"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER44", Value = GetValueOrEmpty(report.FTER44) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410031
    
            #region VR.ATO.GEN.410038
    
            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^FTER4, 'T/A ')) AND (EndsWithSet(^FTER4, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^FTER4 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (IsMatch(report.FTER4, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(report.FTER4, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER4", Value = report.FTER4 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410038
    
            #region VR.ATO.GEN.410038
    
            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^FTER10, 'T/A ')) AND (EndsWithSet(^FTER10, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^FTER10 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = (IsMatch(report.FTER10, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(report.FTER10, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER10", Value = report.FTER10 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410038
    
            #region VR.ATO.GEN.410039
    
            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER4  <> BLANK) AND (NotContainsSet(^FTER4, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^FTER4 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER4) != true && !(IsMatch(report.FTER4, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER4", Value = report.FTER4 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410039
    
            #region VR.ATO.GEN.410039
    
            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER10  <> BLANK) AND (NotContainsSet(^FTER10, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^FTER10 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER10) != true && !(IsMatch(report.FTER10, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER10", Value = report.FTER10 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410039
    
            #region VR.ATO.GEN.410063
    
            /*  VR.ATO.GEN.410063
            Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER48 <> BLANK) AND (NotContainsSet(^FTER48, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^FTER48 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER48) != true && !(IsMatch(report.FTER48, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410063", Severity = ProcessMessageSeverity.Error,
                    Description = @"Given Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410063"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER48", Value = GetValueOrEmpty(report.FTER48) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410063
    
            #region VR.ATO.GEN.410131
    
            /*  VR.ATO.GEN.410131
            Other Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER49 <> BLANK) AND (NotContainsSet(^FTER49, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^FTER49 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER49) != true && !(IsMatch(report.FTER49, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410131", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other Given Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER49", Value = GetValueOrEmpty(report.FTER49) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410131
    
            #region VR.ATO.GEN.410191
    
            /*  VR.ATO.GEN.410191
            Country code must be present for overseas address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER138 = TRUE) AND (^FTER137 = NULL)
    
            Data Elements:
    
            ^FTER137 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^FTER138 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER138 == true && report.FTER137 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be present for overseas address",
                    LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER138", Value = GetValueOrEmpty(report.FTER138) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER137", Value = GetValueOrEmpty(report.FTER137) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410191
    
            #region VR.ATO.GEN.410191
    
            /*  VR.ATO.GEN.410191
            Country code must be present for overseas address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER114 = TRUE) AND (^FTER113 = NULL)
    
            Data Elements:
    
            ^FTER113 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^FTER114 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER114 == true && report.FTER113 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be present for overseas address",
                    LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER114", Value = GetValueOrEmpty(report.FTER114) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER113", Value = GetValueOrEmpty(report.FTER113) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410191
    
            #region VR.ATO.GEN.410191
    
            /*  VR.ATO.GEN.410191
            Country code must be present for overseas address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER119 = TRUE) AND (^FTER118 = NULL)
    
            Data Elements:
    
            ^FTER118 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^FTER119 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.FTER119 == true && report.FTER118 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be present for overseas address",
                    LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER119", Value = GetValueOrEmpty(report.FTER119) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER118", Value = GetValueOrEmpty(report.FTER118) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410191
    
            #region VR.ATO.GEN.410194
    
            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^FTER52) > 38
    
            Data Elements:
    
            ^FTER52 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.FTER52) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER52", Value = report.FTER52 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194
    
            #region VR.ATO.GEN.410194
    
            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^FTER5) > 38
    
            Data Elements:
    
            ^FTER5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.FTER5) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER5", Value = report.FTER5 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194
    
            #region VR.ATO.GEN.410194
    
            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^FTER11) > 38
    
            Data Elements:
    
            ^FTER11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (Length(report.FTER11) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER11", Value = report.FTER11 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194
    
            #region VR.ATO.GEN.410195
    
            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^FTER134) > 38
    
            Data Elements:
    
            ^FTER134 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.FTER134) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER134", Value = GetValueOrEmpty(report.FTER134) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195
    
            #region VR.ATO.GEN.410195
    
            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^FTER110) > 38
    
            Data Elements:
    
            ^FTER110 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.FTER110) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER110", Value = GetValueOrEmpty(report.FTER110) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195
    
            #region VR.ATO.GEN.410195
    
            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^FTER115) > 38
    
            Data Elements:
    
            ^FTER115 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (Length(report.FTER115) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER115", Value = GetValueOrEmpty(report.FTER115) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195
    
            #region VR.ATO.GEN.410200
    
            /*  VR.ATO.GEN.410200
            Family name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER50, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^FTER50 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER50, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000424", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER50", Value = report.FTER50 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410200
    
            #region VR.ATO.GEN.410201
    
            /*  VR.ATO.GEN.410201
            Family name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER50, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^FTER50 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER50, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000426", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER50", Value = report.FTER50 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410201
    
            #region VR.ATO.GEN.410203
    
            /*  VR.ATO.GEN.410203
            First name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER48, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^FTER48 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER48, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000437", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER48", Value = GetValueOrEmpty(report.FTER48) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410203
    
            #region VR.ATO.GEN.410204
    
            /*  VR.ATO.GEN.410204
            First name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER48, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^FTER48 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER48, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000438", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER48", Value = GetValueOrEmpty(report.FTER48) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410204
    
            #region VR.ATO.GEN.410205
    
            /*  VR.ATO.GEN.410205
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER52, '"AS ABOVE"'))
    
            Data Elements:
    
            ^FTER52 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER52, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER52", Value = report.FTER52 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205
    
            #region VR.ATO.GEN.410205
    
            /*  VR.ATO.GEN.410205
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER5, '"AS ABOVE"'))
    
            Data Elements:
    
            ^FTER5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER5, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER5", Value = report.FTER5 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205
    
            #region VR.ATO.GEN.410205
    
            /*  VR.ATO.GEN.410205
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER11, '"AS ABOVE"'))
    
            Data Elements:
    
            ^FTER11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.FTER11, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER11", Value = report.FTER11 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205
    
            #region VR.ATO.GEN.410208
    
            /*  VR.ATO.GEN.410208
            Other given name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER49, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^FTER49 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER49, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000449", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER49", Value = GetValueOrEmpty(report.FTER49) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410208
    
            #region VR.ATO.GEN.410209
    
            /*  VR.ATO.GEN.410209
            Other given name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER49, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^FTER49 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER49, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000450", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER49", Value = GetValueOrEmpty(report.FTER49) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410209
    
            #region VR.ATO.GEN.410211
    
            /*  VR.ATO.GEN.410211
            Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER137 = 'au') AND  (^FTER138 = TRUE)
    
            Data Elements:
    
            ^FTER137 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^FTER138 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER137 == @"au" && report.FTER138 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                    LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER137", Value = GetValueOrEmpty(report.FTER137) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER138", Value = GetValueOrEmpty(report.FTER138) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410211
    
            #region VR.ATO.GEN.410211
    
            /*  VR.ATO.GEN.410211
            Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER113 = 'au') AND  (^FTER114 = TRUE)
    
            Data Elements:
    
            ^FTER113 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^FTER114 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER113 == @"au" && report.FTER114 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                    LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER113", Value = GetValueOrEmpty(report.FTER113) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER114", Value = GetValueOrEmpty(report.FTER114) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410211
    
            #region VR.ATO.GEN.410211
    
            /*  VR.ATO.GEN.410211
            Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER118 = 'au') AND  (^FTER119 = TRUE)
    
            Data Elements:
    
            ^FTER118 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^FTER119 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.FTER118 == @"au" && report.FTER119 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                    LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER118", Value = GetValueOrEmpty(report.FTER118) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER119", Value = GetValueOrEmpty(report.FTER119) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410211
    
            #region VR.ATO.GEN.410212
    
            /*  VR.ATO.GEN.410212
            If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER137 <> BLANK) AND (^FTER137 <> 'au') AND  (^FTER138 = FALSE)
    
            Data Elements:
    
            ^FTER137 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^FTER138 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER137) != true && report.FTER137 != @"au" && report.FTER138 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                    Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                    LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER137", Value = GetValueOrEmpty(report.FTER137) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER138", Value = GetValueOrEmpty(report.FTER138) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410212
    
            #region VR.ATO.GEN.410212
    
            /*  VR.ATO.GEN.410212
            If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER113 <> BLANK) AND (^FTER113 <> 'au') AND  (^FTER114 = FALSE)
    
            Data Elements:
    
            ^FTER113 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^FTER114 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER113) != true && report.FTER113 != @"au" && report.FTER114 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                    Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                    LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER113", Value = GetValueOrEmpty(report.FTER113) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER114", Value = GetValueOrEmpty(report.FTER114) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410212
    
            #region VR.ATO.GEN.410212
    
            /*  VR.ATO.GEN.410212
            If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER118 <> BLANK) AND (^FTER118 <> 'au') AND  (^FTER119 = FALSE)
    
            Data Elements:
    
            ^FTER118 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^FTER119 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER118) != true && report.FTER118 != @"au" && report.FTER119 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                    Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                    LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER118", Value = GetValueOrEmpty(report.FTER118) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER119", Value = GetValueOrEmpty(report.FTER119) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410212
    
            #region VR.ATO.GEN.410213
    
            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER56 <> BLANK
    
            Data Elements:
    
            ^FTER56 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER56) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213", Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER56", Value = GetValueOrEmpty(report.FTER56) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213
    
            #region VR.ATO.GEN.410213
    
            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER9 <> BLANK
    
            Data Elements:
    
            ^FTER9 = RP:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER9) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213", Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER9", Value = GetValueOrEmpty(report.FTER9) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213
    
            #region VR.ATO.GEN.410213
    
            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER15 <> BLANK
    
            Data Elements:
    
            ^FTER15 = RP:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER15) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213", Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER15", Value = GetValueOrEmpty(report.FTER15) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213
    
            #region VR.ATO.GEN.410218
    
            /*  VR.ATO.GEN.410218
            FirstName must be entered if OtherGivenNames is entered
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                RETURN VALIDATION MESSAGE 
             ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND ((^FTER49 <> BLANK) AND (^FTER48 = BLANK))
    
            Data Elements:
    
            ^FTER48 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
    
            ^FTER49 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
    
            ^PersonNameDetails = RP:SpecifiedIndividual:PersonNameDetails
            */
            assertion = (report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPExists != false && (string.IsNullOrWhiteSpace(report.FTER49) != true && string.IsNullOrWhiteSpace(report.FTER48) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410218", Severity = ProcessMessageSeverity.Error,
                    Description = @"FirstName must be entered if OtherGivenNames is entered",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER49", Value = GetValueOrEmpty(report.FTER49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER48", Value = GetValueOrEmpty(report.FTER48) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410218
    
            #region VR.ATO.GEN.428016
    
            /*  VR.ATO.GEN.428016

        Legacy Rule Format:
            IF (RP:entity.identifier.TFN <> NULLORBLANK) AND (TFNALGORITHM (RP:entity.identifier.TFN) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TFN <> BLANK) AND (FailsTFNAlgorithm(^TFN))
    
            Data Elements:
    
            ^TFN = RP
            */
            assertion = (string.IsNullOrWhiteSpace(report.RPIdentifierTFN) != true && FailsTFNAlgorithm(report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428016", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax File Number has failed the algorithm check",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428016"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428016
    
            #region VR.ATO.GEN.428040
    
            /*  VR.ATO.GEN.428040

        Legacy Rule Format:
            IF (pyid.xx.xx:IdentifiersAustralianBusinessNumberIdentifier <> NULLORBLANK) and (ABNALGORITHM(pyid.xx.xx:IdentifiersAustralianBusinessNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER170 <> BLANK) and (FailsABNAlgorithm(^FTER170))
    
            Data Elements:
    
            ^FTER170 = RP:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER170) != true && FailsABNAlgorithm(report.FTER170));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000477", Severity = ProcessMessageSeverity.Error,
                    Description = @"ABN is invalid",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428040"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER170", Value = GetValueOrEmpty(report.FTER170) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428040
    
            #region VR.ATO.GEN.428230
    
            /*  VR.ATO.GEN.428230
            Suburb/Town/City cannot contain state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER53, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^FTER53 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER53, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER53", Value = report.FTER53 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230
    
            #region VR.ATO.GEN.428230
    
            /*  VR.ATO.GEN.428230
            Suburb/Town/City cannot contain state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER6, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^FTER6 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER6, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER6", Value = report.FTER6 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230
    
            #region VR.ATO.GEN.428230
    
            /*  VR.ATO.GEN.428230
            Suburb/Town/City cannot contain state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER12, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^FTER12 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.FTER12, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER12", Value = report.FTER12 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230
    
            #region VR.ATO.GEN.428247
    
            /*  VR.ATO.GEN.428247

        Legacy Rule Format:
            IF ((pyid.xx.xx:Identifiers.TaxAgentNumber.Identifier) <> NULLORBLANK) AND (TANALGORITHM(pyid.xx.xx:Identifiers.TaxAgentNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER2 <> BLANK) AND (FailsTANAlgorithm(^FTER2))
    
            Data Elements:
    
            ^FTER2 = INT:Identifiers.TaxAgentNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER2) != true && FailsTANAlgorithm(report.FTER2));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410009", Severity = ProcessMessageSeverity.Error,
                    Description = @"The Tax Agent number has failed the Tax Agent number algorithm check",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxAgentNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428247"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER2", Value = GetValueOrEmpty(report.FTER2) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428247
    
            #region VR.ATO.GEN.428253
    
            /*  VR.ATO.GEN.428253

        Legacy Rule Format:
            IF (INT:entity.identifier.ABN <> NULLORBLANK) AND (ABNALGORITHM (INT:entity.identifier.ABN) = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^ABN <> BLANK) AND (FailsABNAlgorithm(^ABN))
    
            Data Elements:
    
            ^ABN = INT
            */
            assertion = (string.IsNullOrWhiteSpace(report.INTIdentifierABN) != true && FailsABNAlgorithm(report.INTIdentifierABN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid ABN",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428253"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428253
    
            #region VR.ATO.GEN.428254
    
            /*  VR.ATO.GEN.428254
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER52, 'UNKNOWN')
    
            Data Elements:
    
            ^FTER52 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER52 == null ? false : report.FTER52.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER52", Value = report.FTER52 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428254
    
            #region VR.ATO.GEN.428254
    
            /*  VR.ATO.GEN.428254
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER5, 'UNKNOWN')
    
            Data Elements:
    
            ^FTER5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER5 == null ? false : report.FTER5.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER5", Value = report.FTER5 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428254
    
            #region VR.ATO.GEN.428254
    
            /*  VR.ATO.GEN.428254
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER11, 'UNKNOWN')
    
            Data Elements:
    
            ^FTER11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.FTER11 == null ? false : report.FTER11.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER11", Value = report.FTER11 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428254
    
            #region VR.ATO.GEN.428255
    
            /*  VR.ATO.GEN.428255
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER134, 'UNKNOWN')
    
            Data Elements:
    
            ^FTER134 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER134 == null ? false : report.FTER134.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER134", Value = GetValueOrEmpty(report.FTER134) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428255
    
            #region VR.ATO.GEN.428255
    
            /*  VR.ATO.GEN.428255
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER110, 'UNKNOWN')
    
            Data Elements:
    
            ^FTER110 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER110 == null ? false : report.FTER110.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER110", Value = GetValueOrEmpty(report.FTER110) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428255
    
            #region VR.ATO.GEN.428255
    
            /*  VR.ATO.GEN.428255
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER115, 'UNKNOWN')
    
            Data Elements:
    
            ^FTER115 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.FTER115 == null ? false : report.FTER115.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER115", Value = GetValueOrEmpty(report.FTER115) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428255
    
            #region VR.ATO.GEN.428256
    
            /*  VR.ATO.GEN.428256
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER53, 'UNKNOWN')
    
            Data Elements:
    
            ^FTER53 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER53 == null ? false : report.FTER53.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER53", Value = report.FTER53 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428256
    
            #region VR.ATO.GEN.428256
    
            /*  VR.ATO.GEN.428256
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER6, 'UNKNOWN')
    
            Data Elements:
    
            ^FTER6 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER6 == null ? false : report.FTER6.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER6", Value = report.FTER6 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428256
    
            #region VR.ATO.GEN.428256
    
            /*  VR.ATO.GEN.428256
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER12, 'UNKNOWN')
    
            Data Elements:
    
            ^FTER12 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.FTER12 == null ? false : report.FTER12.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER12", Value = report.FTER12 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428256
    
            #region VR.ATO.GEN.428258
    
            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER4, ' - ')
    
            Data Elements:
    
            ^FTER4 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.FTER4 == null ? false : report.FTER4.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER4", Value = report.FTER4 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428258
    
            #region VR.ATO.GEN.428258
    
            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER10, ' - ')
    
            Data Elements:
    
            ^FTER10 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = (report.FTER10 == null ? false : report.FTER10.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER10", Value = report.FTER10 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428258
    
            #region VR.ATO.GEN.428259
    
            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER4, 'P/L')
    
            Data Elements:
    
            ^FTER4 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.FTER4 == null ? false : report.FTER4.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER4", Value = report.FTER4 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428259
    
            #region VR.ATO.GEN.428259
    
            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER10, 'P/L')
    
            Data Elements:
    
            ^FTER10 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = (report.FTER10 == null ? false : report.FTER10.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER10", Value = report.FTER10 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428259
    
            #region VR.ATO.GEN.428260
    
            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^FTER4, '"--","''","  "'))
    
            Data Elements:
    
            ^FTER4 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER4, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER4", Value = report.FTER4 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428260
    
            #region VR.ATO.GEN.428260
    
            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^FTER10, '"--","''","  "'))
    
            Data Elements:
    
            ^FTER10 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.FTER10, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER10", Value = report.FTER10 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428260
    
            #region VR.ATO.GEN.428261
    
            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^FTER4, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            ^FTER4 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER4, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER4", Value = report.FTER4 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428261
    
            #region VR.ATO.GEN.428261
    
            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^FTER10, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            ^FTER10 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.FTER10, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER10", Value = report.FTER10 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428261
    
            #region VR.ATO.GEN.428262
    
            /*  VR.ATO.GEN.428262
            Family Name cannot contain "space hyphen space"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER50, ' - ')
    
            Data Elements:
    
            ^FTER50 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (report.FTER50 == null ? false : report.FTER50.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000423", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Name cannot contain ""space hyphen space""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428262"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER50", Value = report.FTER50 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428262
    
            #region VR.ATO.GEN.428263
    
            /*  VR.ATO.GEN.428263
            Family name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^FTER50, '"--","''","  "'))
    
            Data Elements:
    
            ^FTER50 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER50, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000427", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428263"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER50", Value = report.FTER50 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428263
    
            #region VR.ATO.GEN.428264
    
            /*  VR.ATO.GEN.428264
            First name cannot contain space hyphen space
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER48, ' - ')
    
            Data Elements:
    
            ^FTER48 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (report.FTER48 == null ? false : report.FTER48.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000434", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428264"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER48", Value = GetValueOrEmpty(report.FTER48) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428264
    
            #region VR.ATO.GEN.428265
    
            /*  VR.ATO.GEN.428265
            First name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^FTER48, '"--","''","  "'))
    
            Data Elements:
    
            ^FTER48 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER48, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000439", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428265"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER48", Value = GetValueOrEmpty(report.FTER48) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428265
    
            #region VR.ATO.GEN.428266
    
            /*  VR.ATO.GEN.428266
            Other given name cannot contain space hyphen space
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^FTER49, ' - ')
    
            Data Elements:
    
            ^FTER49 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (report.FTER49 == null ? false : report.FTER49.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000446", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428266"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER49", Value = GetValueOrEmpty(report.FTER49) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428266
    
            #region VR.ATO.GEN.428267
    
            /*  VR.ATO.GEN.428267
            Other given name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^FTER49, '"--","''","  "'))
    
            Data Elements:
    
            ^FTER49 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER49, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000451", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428267"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER49", Value = GetValueOrEmpty(report.FTER49) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428267
    
            #region VR.ATO.GEN.430206
    
            /*  VR.ATO.GEN.430206
            Hours taken to prepare and complete form must be 1 to 999
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Report.CompletionHours.Number <> NULL) AND (pyin.xx.xx:Report.CompletionHours.Number <> SET (1-999))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER85 <> NULL) AND ((^FTER85 < 1 OR ^FTER85 > 999))
    
            Data Elements:
    
            ^FTER85 = RP:Report.CompletionHours.Number
            */
            assertion = (report.FTER85 != null && (report.FTER85.GetValueOrDefault() < 1 || report.FTER85.GetValueOrDefault() > 999));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430206", Severity = ProcessMessageSeverity.Error,
                    Description = @"Hours taken to prepare and complete form must be 1 to 999",
                    Location = "/xbrli:xbrl/tns:Report.CompletionHours.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430206"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER85", Value = report.FTER85.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430206
    
            #region VR.ATO.GEN.430246
    
            /*  VR.ATO.GEN.430246
            Suburb/Town/City cannot contain numeric characters
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER138 = FALSE) AND (ContainsSet(^FTER53, '0-9'))
    
            Data Elements:
    
            ^FTER53 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^FTER138 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER138 == false && IsMatch(report.FTER53, @"[0-9]",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER138", Value = GetValueOrEmpty(report.FTER138) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER53", Value = report.FTER53 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430246
    
            #region VR.ATO.GEN.430246
    
            /*  VR.ATO.GEN.430246
            Suburb/Town/City cannot contain numeric characters
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER114 = FALSE) AND (ContainsSet(^FTER6, '0-9'))
    
            Data Elements:
    
            ^FTER6 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^FTER114 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.FTER114 == false && IsMatch(report.FTER6, @"[0-9]",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER114", Value = GetValueOrEmpty(report.FTER114) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER6", Value = report.FTER6 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430246
    
            #region VR.ATO.GEN.430246
    
            /*  VR.ATO.GEN.430246
            Suburb/Town/City cannot contain numeric characters
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER119 = FALSE) AND (ContainsSet(^FTER12, '0-9'))
    
            Data Elements:
    
            ^FTER12 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^FTER119 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.FTER119 == false && IsMatch(report.FTER12, @"[0-9]",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER119", Value = GetValueOrEmpty(report.FTER119) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER12", Value = report.FTER12 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430246
    
            #region VR.ATO.GEN.430252
    
            /*  VR.ATO.GEN.430252

        Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^FTER82) > 200
    
            Data Elements:
    
            ^FTER82 = INT:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = (Length(report.FTER82) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER82", Value = report.FTER82 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430252
    
            #region VR.ATO.GEN.430252
    
            /*  VR.ATO.GEN.430252

        Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^FTER165) > 200
    
            Data Elements:
    
            ^FTER165 = INT:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (Length(report.FTER165) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER165", Value = report.FTER165 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430252
    
            #region VR.ATO.GEN.430255
    
            /*  VR.ATO.GEN.430255

        Legacy Rule Format:
            IF pyin.xx.xx:Declaration.Signature.Date > DATE(TODAY)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^FTER162 > Today()
    
            Data Elements:
    
            ^FTER162 = INT:Declaration:Declaration.Signature.Date WHERE [Declaration.StatementType.Code='TrueAndCorrect']
            */
            assertion = (report.FTER162.GetValueOrDefault() > DateTime.Now.Date);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430255", Severity = ProcessMessageSeverity.Error,
                    Description = @"Signature date must not be later than today",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:Declaration.StatementType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER162", Value = GetValueOrEmptyDate(report.FTER162) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430255
    
            #region VR.ATO.GEN.430316
    
            /*  VR.ATO.GEN.430316

        Legacy Rule Format:
            IF entity.identifier.ABN = NULLORBLANK WHERE CONTEXT = "INT"
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^ABN = BLANK
    
            Data Elements:
    
            ^ABN = INT
    
            ^Context = INT
            */
            assertion = (report.INTExists != false && string.IsNullOrWhiteSpace(report.INTIdentifierABN) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430316", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. ABN must be present in the Intermediary context",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430316"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430316
    
            #region VR.ATO.GEN.430317
    
            /*  VR.ATO.GEN.430317

        Legacy Rule Format:
            IF entity.identifier.TFN = NULLORBLANK WHERE CONTEXT <> "INT"
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TFN = BLANK
    
            Data Elements:
    
            ^TFN = RP
            */
            assertion = (string.IsNullOrWhiteSpace(report.RPIdentifierTFN) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430317", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. TFN must be present in the context",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430317"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430317
    
            #region VR.ATO.GEN.434147
    
            /*  VR.ATO.GEN.434147
            State and post code must be present for an Australian address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^FTER138 = FALSE AND (^FTER54 = NULL OR ^FTER55 = NULL))
    
            Data Elements:
    
            ^FTER138 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^FTER54 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^FTER55 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:SpecifiedIndividual:AddressDetails
            */
            assertion = (report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPExists != false && (report.FTER138 == false && (report.FTER54 == null || report.FTER55 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                    Description = @"State and post code must be present for an Australian address",
                    LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER138", Value = GetValueOrEmpty(report.FTER138) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER54", Value = GetValueOrEmpty(report.FTER54) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER55", Value = GetValueOrEmpty(report.FTER55) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.434147
    
            #region VR.ATO.GEN.434147
    
            /*  VR.ATO.GEN.434147
            State and post code must be present for an Australian address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^FTER114 = FALSE AND (^FTER7 = NULL OR ^FTER8 = NULL))
    
            Data Elements:
    
            ^FTER114 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^FTER7 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^FTER8 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && (report.FTER114 == false && (report.FTER7 == null || report.FTER8 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                    Description = @"State and post code must be present for an Australian address",
                    LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER114", Value = GetValueOrEmpty(report.FTER114) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER7", Value = GetValueOrEmpty(report.FTER7) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER8", Value = GetValueOrEmpty(report.FTER8) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.434147
    
            #region VR.ATO.GEN.434147
    
            /*  VR.ATO.GEN.434147
            State and post code must be present for an Australian address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^FTER119 = FALSE AND (^FTER13 = NULL OR ^FTER14 = NULL))
    
            Data Elements:
    
            ^FTER119 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^FTER13 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^FTER14 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && (report.FTER119 == false && (report.FTER13 == null || report.FTER14 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                    Description = @"State and post code must be present for an Australian address",
                    LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER119", Value = GetValueOrEmpty(report.FTER119) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER13", Value = GetValueOrEmpty(report.FTER13) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER14", Value = GetValueOrEmpty(report.FTER14) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.434147
    
            #region VR.ATO.GEN.438029
    
            /*  VR.ATO.GEN.438029

        Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP
    
            ^Context = RP
            */
            assertion = (report.RPExists != false && report.RPIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029
    
            #region VR.ATO.GEN.438030
    
            /*  VR.ATO.GEN.438030

        Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:Intermediary”) AND (entity.identifier.scheme <> “http://www.ato.gov.au/abn”)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^IdentifierScheme <> 'http://www.ato.gov.au/abn'
    
            Data Elements:
    
            ^IdentifierScheme = INT
    
            ^Context = INT
            */
            assertion = (report.INTExists != false && report.INTIdentifierScheme != @"http://www.ato.gov.au/abn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438030", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for Intermediary must be ""http://www.ato.gov.au/abn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438030"} },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438030
    
            #region VR.ATO.GEN.500132
    
            /*  VR.ATO.GEN.500132
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^OrganisationNameDetails <> NULL) AND (^FTER4 = BLANK)
    
            Data Elements:
    
            ^FTER4 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^OrganisationNameDetails = RP:OrganisationNameDetails
            */
            assertion = (report.OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(report.FTER4) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER4", Value = report.FTER4 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500132
    
            #region VR.ATO.GEN.500132
    
            /*  VR.ATO.GEN.500132
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^OrganisationNameDetails <> NULL) AND (^FTER10 = BLANK)
    
            Data Elements:
    
            ^FTER10 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
    
            ^OrganisationNameDetails = RP:OrganisationNameDetails
            */
            assertion = (report.OrganisationNameDetailsCollection_MN_P_RPExists != false && string.IsNullOrWhiteSpace(report.FTER10) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER10", Value = report.FTER10 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500132
    
            #region VR.ATO.GEN.500133
    
            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^FTER52 = BLANK)
    
            Data Elements:
    
            ^FTER52 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:SpecifiedIndividual:AddressDetails
            */
            assertion = (report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPExists != false && string.IsNullOrWhiteSpace(report.FTER52) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER52", Value = report.FTER52 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500133
    
            #region VR.ATO.GEN.500133
    
            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^FTER5 = BLANK)
    
            Data Elements:
    
            ^FTER5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(report.FTER5) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER5", Value = report.FTER5 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500133
    
            #region VR.ATO.GEN.500133
    
            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^FTER11 = BLANK)
    
            Data Elements:
    
            ^FTER11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && string.IsNullOrWhiteSpace(report.FTER11) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER11", Value = report.FTER11 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500133
    
            #region VR.ATO.GEN.500134
    
            /*  VR.ATO.GEN.500134
            Suburb/Town/City must be present in an address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^FTER53 = BLANK)
    
            Data Elements:
    
            ^FTER53 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:SpecifiedIndividual:AddressDetails
            */
            assertion = (report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPExists != false && string.IsNullOrWhiteSpace(report.FTER53) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City must be present in an address.",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER53", Value = report.FTER53 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500134
    
            #region VR.ATO.GEN.500134
    
            /*  VR.ATO.GEN.500134
            Suburb/Town/City must be present in an address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^FTER6 = BLANK)
    
            Data Elements:
    
            ^FTER6 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(report.FTER6) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City must be present in an address.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER6", Value = report.FTER6 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500134
    
            #region VR.ATO.GEN.500134
    
            /*  VR.ATO.GEN.500134
            Suburb/Town/City must be present in an address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^FTER12 = BLANK)
    
            Data Elements:
    
            ^FTER12 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && string.IsNullOrWhiteSpace(report.FTER12) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City must be present in an address.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER12", Value = report.FTER12 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500134
    
            #region VR.ATO.GEN.500135
    
            /*  VR.ATO.GEN.500135
            Family Name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
            IF pyde.xx.xx:PersonNameDetails.FamilyName.Text = BLANK
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND (^FTER50 = BLANK)
    
            Data Elements:
    
            ^FTER50 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
    
            ^PersonNameDetails = RP:SpecifiedIndividual:PersonNameDetails
            */
            assertion = (report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPExists != false && string.IsNullOrWhiteSpace(report.FTER50) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500135", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Name must be present.",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500135"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER50", Value = report.FTER50 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500135
    
            #region VR.ATO.GEN.500141
    
            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER54 <> BLANK) AND (NotInSet(^FTER54, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^FTER54 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER54) != true && !(IsMatch(report.FTER54, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER54", Value = GetValueOrEmpty(report.FTER54) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500141
    
            #region VR.ATO.GEN.500141
    
            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER7 <> BLANK) AND (NotInSet(^FTER7, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^FTER7 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER7) != true && !(IsMatch(report.FTER7, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER7", Value = GetValueOrEmpty(report.FTER7) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500141
    
            #region VR.ATO.GEN.500141
    
            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FTER13 <> BLANK) AND (NotInSet(^FTER13, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^FTER13 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.FTER13) != true && !(IsMatch(report.FTER13, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER13", Value = GetValueOrEmpty(report.FTER13) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500141
    
            #region VR.ATO.GEN.500142
    
            /*  VR.ATO.GEN.500142
            Family name must contain at least one alphabetical character
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
            IF pyde.xx.xx:PersonNameDetails.FamilyName.Text DOES NOT CONTAIN SET("A-Z","a-z")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND (NotContainsSet(^FTER50, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^FTER50 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
    
            ^PersonNameDetails = RP:SpecifiedIndividual:PersonNameDetails
            */
            assertion = (report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPExists != false && !(IsMatch(report.FTER50, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500142", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name must contain at least one alphabetical character.",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500142"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER50", Value = report.FTER50 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500142
    
            #region VR.ATO.GEN.500151
    
            /*  VR.ATO.GEN.500151
            First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER48, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^FTER48 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER48, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010436", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500151"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER48", Value = GetValueOrEmpty(report.FTER48) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500151
    
            #region VR.ATO.GEN.500152
    
            /*  VR.ATO.GEN.500152
            Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER49, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^FTER49 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER49, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010448", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500152"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER49", Value = GetValueOrEmpty(report.FTER49) });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500152
    
            #region VR.ATO.GEN.500153
    
            /*  VR.ATO.GEN.500153
            Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^FTER50, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^FTER50 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.FTER50, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500153", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500153"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "FTER50", Value = report.FTER50 });
    
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500153

            #region Repeating report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection
            if (report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection != null)
            {    
                int itemIndex1 = 0;
                foreach (FTER2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP periodDetailsControlOutsideAustraliaPartPeriod in report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection)
                {
                    itemIndex1++;
            
                    #region VR.ATO.FTER.408061
            
                    /*  VR.ATO.FTER.408061
                    Control of the trust outside of Australia must not be before start of substituted accounting period
    
                    Legacy Rule Format:
                    IF [FTER18] <> NULL AND [FTER41] <> NULL AND (ANY OCCURRENCE OF [FTER18]) < [FTER41]
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER18] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails)
                    [FTER41] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails)
        
                    Technical Business Rule Format:
                    ^FTER18 <> NULL AND ^FTER41 <> NULL AND ^FTER18 < ^FTER41
            
                    Data Elements:
            
                    ^FTER18 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            
                    ^FTER41 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
                    */
                    assertion = (periodDetailsControlOutsideAustraliaPartPeriod.FTER18 != null && report.FTER41 != null && periodDetailsControlOutsideAustraliaPartPeriod.FTER18.GetValueOrDefault() < report.FTER41.GetValueOrDefault());
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408061", Severity = ProcessMessageSeverity.Error,
                            Description = @"Central mgmnt and control outside Australia period must not be before start of the SAP",
                            LongDescription = @"The 'from' date of a period of Central management and control of the trust outside of Australia must not be before the start of the substituted accounting period",
                            Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(periodDetailsControlOutsideAustraliaPartPeriod.PeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408061"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER18", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.FTER18) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER41", Value = GetValueOrEmptyDate(report.FTER41) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408061
            
                    #region VR.ATO.FTER.408062
            
                    /*  VR.ATO.FTER.408062
                    Control of the trust outside of Australia must not be before start of election commencement time
    
                    Legacy Rule Format:
                    IF [FTER18] <> NULL AND [FTER43] <> NULL AND (ANY OCCURRENCE OF [FTER18]) < [FTER43]
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER18] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails)
                    [FTER43] = FTER:RP:rvctc3.02.00:Elections.Commencement.Date
        
                    Technical Business Rule Format:
                    ^FTER18 <> NULL AND ^FTER43 <> NULL AND (AnyOccurrence(^FTER18, ^FTER18 < ^FTER43))
            
                    Data Elements:
            
                    ^FTER18 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            
                    ^FTER43 = RP:Elections.Commencement.Date
                    */
                    assertion = (periodDetailsControlOutsideAustraliaPartPeriod.FTER18 != null && report.FTER43 != null && (report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection == null ? false : report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection.Any(FTER18Repeat => FTER18Repeat.FTER18.GetValueOrDefault() < report.FTER43.GetValueOrDefault())));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408062", Severity = ProcessMessageSeverity.Error,
                            Description = @"Central mgmnt and control outside Australia period must not be before election commencement date",
                            LongDescription = @"The 'from' date of a period of Central management and control of the trust outside of Australia must not be before the date of the commencement time for the family trust election",
                            Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(periodDetailsControlOutsideAustraliaPartPeriod.PeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408062"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER18", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.FTER18) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER43", Value = GetValueOrEmptyDate(report.FTER43) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408062
            
                    #region VR.ATO.FTER.408063
            
                    /*  VR.ATO.FTER.408063
                    Central management period must not be before Income year
    
                    Legacy Rule Format:
                    IF [FTER95] = SET("E", "V") AND [FTER40] <> NULL  AND [FTER41] = NULL AND [FTER18] <> NULL AND (ANY OCCURRENCE OF [FTER18] < (([FTER40] - 1)&"-07-01"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER18] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails)
                    [FTER40] = FTER:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year
                    [FTER41] = FTER:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(fter.0002.lodge.req.02.00:PeriodDetails)
                    [FTER95] = FTER:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code
        
                    Technical Business Rule Format:
                    InSet(^FTER95, '"E", "V"') AND ^FTER40 <> NULL  AND ^FTER41 = NULL AND ^FTER18 <> NULL AND (AnyOccurrence(^FTER18, ^FTER18 < AsDate(Concat(^FTER40 - 1, '-07-01'))))
            
                    Data Elements:
            
                    ^FTER18 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            
                    ^FTER40 = RP:Elections.FamilyTrustElectionStatus.Year
            
                    ^FTER41 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
            
                    ^FTER95 = RP:Elections.FamilyTrustElectionRevocation.Code
                    */
                    assertion = (IsMatch(report.FTER95, @"^(E|V)$") && report.FTER40 != null && report.FTER41 == null && periodDetailsControlOutsideAustraliaPartPeriod.FTER18 != null && (report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection == null ? false : report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection.Any(FTER18Repeat => FTER18Repeat.FTER18.GetValueOrDefault() < AsDate(string.Concat(report.FTER40.GetValueOrDefault() - 1, @"-07-01")))));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408063", Severity = ProcessMessageSeverity.Error,
                            Description = @"Central mgmnt period must not be before Income year",
                            LongDescription = @"The 'from' date of a period of Central management and control of the trust outside of Australia must not be before 1 July of the Income year specified",
                            Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(periodDetailsControlOutsideAustraliaPartPeriod.PeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408063"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER95", Value = report.FTER95 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER40", Value = GetValueOrEmpty(report.FTER40) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER41", Value = GetValueOrEmptyDate(report.FTER41) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER18", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.FTER18) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408063
            
                    #region VR.ATO.GEN.408060
            
                    /*  VR.ATO.GEN.408060
                    Start date must be before end date
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (perioddetails1.xx.xx:PeriodDetails)
                    IF pyin.xx.xx:Period.End.Date <= pyin.xx.xx:Period.Start.Date
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^FTER19 <= ^FTER18
            
                    Data Elements:
            
                    ^FTER19 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            
                    ^FTER18 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
                    */
                    assertion = (periodDetailsControlOutsideAustraliaPartPeriod.FTER19.GetValueOrDefault() <= periodDetailsControlOutsideAustraliaPartPeriod.FTER18.GetValueOrDefault());
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.408060", Severity = ProcessMessageSeverity.Error,
                            Description = @"Start date must be before end date",
                            LongDescription = @"The start date must be earlier than the end date in a Period.",
                            Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(periodDetailsControlOutsideAustraliaPartPeriod.PeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPOccurrenceIndex) + "/tns:Period.End.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.408060"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER19", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.FTER19) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER18", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.FTER18) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.408060
                    }
                }
        
                #endregion // Foreach loop

            #region Repeating report.InterposedEntityCollection
            if (report.InterposedEntityCollection != null)
            {    
                int itemIndex1 = 0;
                foreach (FTER2018.InterposedEntity interposedEntity in report.InterposedEntityCollection)
                {
                    itemIndex1++;
            
                    #region VR.ATO.FTER.408117
            
                    /*  VR.ATO.FTER.408117
                    Interposed Entity Party Type Code must be either '051', '161', '212' or '217'
    
                    Legacy Rule Format:
                    IF [FTER62] <> NULLORBLANK AND [FTER62] <> SET("051","161","212","217")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER62] = FTER:RP:pyde.02.21:Party.Type.Code IN TUPLE(fter.0002.lodge.req.02.00:InterposedEntity)
        
                    Technical Business Rule Format:
                    ^FTER62 <> BLANK AND NotInSet(^FTER62, '"051","161","212","217"')
            
                    Data Elements:
            
                    ^FTER62 = RP:InterposedEntity:Party.Type.Code
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER62) != true && !(IsMatch(interposedEntity.FTER62, @"^(051|161|212|217)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408117", Severity = ProcessMessageSeverity.Error,
                            Description = @"Interposed Entity Party Type Code must be either '051', '161', '212' or '217'",
                            LongDescription = @"Interposed entity type must be '051' - Company, '161' - Partnership, '212' - Trust, '217' - Fund",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:Party.Type.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408117"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER62", Value = GetValueOrEmpty(interposedEntity.FTER62) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408117
            
                    #region VR.ATO.FTER.408118
            
                    /*  VR.ATO.FTER.408118
                    If the TFN for an interposed entity is present, then the 'entity does not have a TFN' indicator must be 'no' (false) or blank
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:InterposedEntity)
                    IF [FTER63] = TRUE AND [FTER64] <> NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER63] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(fter.0002.lodge.req.02.00:InterposedEntity)
                    [FTER64] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:InterposedEntity)
        
                    Technical Business Rule Format:
                    ^FTER63 = TRUE AND ^FTER64 <> BLANK
            
                    Data Elements:
            
                    ^FTER63 = RP:InterposedEntity:Identifiers.TaxFileNumberInexistent.Indicator
            
                    ^FTER64 = RP:InterposedEntity:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (interposedEntity.FTER63 == true && string.IsNullOrWhiteSpace(interposedEntity.FTER64) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408118", Severity = ProcessMessageSeverity.Error,
                            Description = @"No TFN indicator  must NOT be 'yes' (true)",
                            LongDescription = @"If the TFN for an interposed entity is present, then the 'entity does not have a TFN' indicator must be 'no' (false) or blank",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumberInexistent.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408118"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER63", Value = GetValueOrEmpty(interposedEntity.FTER63) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER64", Value = GetValueOrEmpty(interposedEntity.FTER64) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408118
            
                    #region VR.ATO.FTER.408119
            
                    /*  VR.ATO.FTER.408119
                    Entity TFN must be present or 'No TFN indicator' must be 'yes' (true)
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:InterposedEntity)
                    IF [FTER64] = NULLORBLANK AND [FTER63] <> TRUE
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER63] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(fter.0002.lodge.req.02.00:InterposedEntity)
                    [FTER64] = FTER:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:InterposedEntity)
        
                    Technical Business Rule Format:
                    ^FTER64 = BLANK AND ^FTER63 <> TRUE
            
                    Data Elements:
            
                    ^FTER64 = RP:InterposedEntity:Identifiers.TaxFileNumber.Identifier
            
                    ^FTER63 = RP:InterposedEntity:Identifiers.TaxFileNumberInexistent.Indicator
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER64) == true && interposedEntity.FTER63 != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408119", Severity = ProcessMessageSeverity.Error,
                            Description = @"Entity TFN must be present or 'No TFN indicator' must be 'yes' (true)",
                            LongDescription = @"For a revocation, the TFN of the company/partnership/trust which has an interposed entity election in force must be present, or the 'No TFN indicator' must be 'yes' (true)",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408119"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER64", Value = GetValueOrEmpty(interposedEntity.FTER64) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER63", Value = GetValueOrEmpty(interposedEntity.FTER63) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408119
            
                    #region VR.ATO.FTER.408122
            
                    /*  VR.ATO.FTER.408122
                    ACN and ARBN must not both be present
    
                    Legacy Rule Format:
                    IF [FTER87] <> NULLORBLANK AND [FTER67] <> NULLORBLANK
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER67] = FTER:RP:pyid.02.00:Identifiers.AustralianRegisteredBodyNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:InterposedEntity)
                    [FTER87] = FTER:RP:pyid.02.00:Identifiers.AustralianCompanyNumber.Identifier IN TUPLE(fter.0002.lodge.req.02.00:InterposedEntity)
        
                    Technical Business Rule Format:
                    ^FTER87 <> BLANK AND ^FTER67 <> BLANK
            
                    Data Elements:
            
                    ^FTER87 = RP:InterposedEntity:Identifiers.AustralianCompanyNumber.Identifier
            
                    ^FTER67 = RP:InterposedEntity:Identifiers.AustralianRegisteredBodyNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER87) != true && string.IsNullOrWhiteSpace(interposedEntity.FTER67) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408122", Severity = ProcessMessageSeverity.Error,
                            Description = @"ACN and ARBN must not both be present",
                            LongDescription = @"Australian Company Number and Australian Registered Body Number for the company which has an interposed entity election in force must not both be supplied",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:Identifiers.AustralianCompanyNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408122"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER87", Value = GetValueOrEmpty(interposedEntity.FTER87) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER67", Value = GetValueOrEmpty(interposedEntity.FTER67) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408122
            
                    #region VR.ATO.GEN.000406
            
                    /*  VR.ATO.GEN.000406
                    Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER68 <> BLANK) AND (StartsWithSet(^FTER68,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^FTER68, '" C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^FTER68 = RP:InterposedEntity:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER68) != true && IsMatch(interposedEntity.FTER68, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(interposedEntity.FTER68, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER68", Value = interposedEntity.FTER68 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000406
            
                    #region VR.ATO.GEN.000408
            
                    /*  VR.ATO.GEN.000408
                    Care of' is not valid for address line 2
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER69 <> BLANK) AND (StartsWithSet(^FTER69,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^FTER69, '" C/- "," C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^FTER69 = RP:InterposedEntity:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER69) != true && (IsMatch(interposedEntity.FTER69, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(interposedEntity.FTER69, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                            Description = @"Care of' is not valid for address line 2",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER69", Value = GetValueOrEmpty(interposedEntity.FTER69) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000408
            
                    #region VR.ATO.GEN.410002
            
                    /*  VR.ATO.GEN.410002
                    Address Line 2 must be present if Address Line 1 contains 'C/-'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ((Contains(^FTER68, 'C/-')) AND (^FTER69 = BLANK))
            
                    Data Elements:
            
                    ^FTER68 = RP:InterposedEntity:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER69 = RP:InterposedEntity:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((interposedEntity.FTER68 == null ? false : interposedEntity.FTER68.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(interposedEntity.FTER69) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER68", Value = interposedEntity.FTER68 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER69", Value = GetValueOrEmpty(interposedEntity.FTER69) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410002
            
                    #region VR.ATO.GEN.410008
            
                    /*  VR.ATO.GEN.410008
                    Address Line 3 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^FTER143 <> BLANK
            
                    Data Elements:
            
                    ^FTER143 = RP:InterposedEntity:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER143) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 3 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER143", Value = GetValueOrEmpty(interposedEntity.FTER143) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410008
            
                    #region VR.ATO.GEN.410013
            
                    /*  VR.ATO.GEN.410013
                    Address Line 4 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^FTER144 <> BLANK
            
                    Data Elements:
            
                    ^FTER144 = RP:InterposedEntity:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER144) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 4 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER144", Value = GetValueOrEmpty(interposedEntity.FTER144) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410013
            
                    #region VR.ATO.GEN.410031
            
                    /*  VR.ATO.GEN.410031
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier <> NULLORBLANK) AND (TFNALGORITHM (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier) = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    (^FTER64 <> BLANK) AND (FailsTFNAlgorithm(^FTER64))
            
                    Data Elements:
            
                    ^FTER64 = RP:InterposedEntity:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER64) != true && FailsTFNAlgorithm(interposedEntity.FTER64));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410031", Severity = ProcessMessageSeverity.Error,
                            Description = @"TFN must be a valid TFN",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410031"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER64", Value = GetValueOrEmpty(interposedEntity.FTER64) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410031
            
                    #region VR.ATO.GEN.410038
            
                    /*  VR.ATO.GEN.410038
                    Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (StartsWith(^FTER65, 'T/A ')) AND (EndsWithSet(^FTER65, '" Pship"," P'ship"," P/ship"'))
            
                    Data Elements:
            
                    ^FTER65 = RP:InterposedEntity:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (IsMatch(interposedEntity.FTER65, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(interposedEntity.FTER65, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER65", Value = interposedEntity.FTER65 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410038
            
                    #region VR.ATO.GEN.410039
            
                    /*  VR.ATO.GEN.410039
                    Organisation name must contain at least one alpha or numeric character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER65  <> BLANK) AND (NotContainsSet(^FTER65, '"a-z", "A-Z", "0-9"'))
            
                    Data Elements:
            
                    ^FTER65 = RP:InterposedEntity:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER65) != true && !(IsMatch(interposedEntity.FTER65, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must contain at least one alpha or numeric character",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER65", Value = interposedEntity.FTER65 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410039
            
                    #region VR.ATO.GEN.410191
            
                    /*  VR.ATO.GEN.410191
                    Country code must be present for overseas address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER146 = TRUE) AND (^FTER145 = NULL)
            
                    Data Elements:
            
                    ^FTER145 = RP:InterposedEntity:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER146 = RP:InterposedEntity:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (interposedEntity.FTER146 == true && interposedEntity.FTER145 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                            Description = @"Country code must be present for overseas address",
                            LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER146", Value = GetValueOrEmpty(interposedEntity.FTER146) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER145", Value = GetValueOrEmpty(interposedEntity.FTER145) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410191
            
                    #region VR.ATO.GEN.410194
            
                    /*  VR.ATO.GEN.410194
                    Address Line 1 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                         RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^FTER68) > 38
            
                    Data Elements:
            
                    ^FTER68 = RP:InterposedEntity:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(interposedEntity.FTER68) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER68", Value = interposedEntity.FTER68 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410194
            
                    #region VR.ATO.GEN.410195
            
                    /*  VR.ATO.GEN.410195
                    Address Line 2 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                         RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^FTER69) > 38
            
                    Data Elements:
            
                    ^FTER69 = RP:InterposedEntity:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(interposedEntity.FTER69) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER69", Value = GetValueOrEmpty(interposedEntity.FTER69) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410195
            
                    #region VR.ATO.GEN.410205
            
                    /*  VR.ATO.GEN.410205
                    Address Line 1 must not contain 'AS ABOVE'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER68, '"AS ABOVE"'))
            
                    Data Elements:
            
                    ^FTER68 = RP:InterposedEntity:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(interposedEntity.FTER68, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not contain 'AS ABOVE'",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER68", Value = interposedEntity.FTER68 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410205
            
                    #region VR.ATO.GEN.410211
            
                    /*  VR.ATO.GEN.410211
                    Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER145 = 'au') AND  (^FTER146 = TRUE)
            
                    Data Elements:
            
                    ^FTER145 = RP:InterposedEntity:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER146 = RP:InterposedEntity:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (interposedEntity.FTER145 == @"au" && interposedEntity.FTER146 == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                            Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                            LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER145", Value = GetValueOrEmpty(interposedEntity.FTER145) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER146", Value = GetValueOrEmpty(interposedEntity.FTER146) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410211
            
                    #region VR.ATO.GEN.410212
            
                    /*  VR.ATO.GEN.410212
                    If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER145 <> BLANK) AND (^FTER145 <> 'au') AND  (^FTER146 = FALSE)
            
                    Data Elements:
            
                    ^FTER145 = RP:InterposedEntity:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER146 = RP:InterposedEntity:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER145) != true && interposedEntity.FTER145 != @"au" && interposedEntity.FTER146 == false);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                            Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                            LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER145", Value = GetValueOrEmpty(interposedEntity.FTER145) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER146", Value = GetValueOrEmpty(interposedEntity.FTER146) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410212
            
                    #region VR.ATO.GEN.410213
            
                    /*  VR.ATO.GEN.410213
                    Country Name must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^FTER73 <> BLANK
            
                    Data Elements:
            
                    ^FTER73 = RP:InterposedEntity:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER73) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410213", Severity = ProcessMessageSeverity.Error,
                            Description = @"Country Name must be left blank",
                            LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER73", Value = GetValueOrEmpty(interposedEntity.FTER73) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410213
            
                    #region VR.ATO.GEN.428230
            
                    /*  VR.ATO.GEN.428230
                    Suburb/Town/City cannot contain state code
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER70, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
            
                    Data Elements:
            
                    ^FTER70 = RP:InterposedEntity:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(interposedEntity.FTER70, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain state code",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER70", Value = interposedEntity.FTER70 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428230
            
                    #region VR.ATO.GEN.428254
            
                    /*  VR.ATO.GEN.428254
                    Address line 1 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
                    RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER68, 'UNKNOWN')
            
                    Data Elements:
            
                    ^FTER68 = RP:InterposedEntity:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (interposedEntity.FTER68 == null ? false : interposedEntity.FTER68.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER68", Value = interposedEntity.FTER68 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428254
            
                    #region VR.ATO.GEN.428255
            
                    /*  VR.ATO.GEN.428255
                    Address line 2 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER69, 'UNKNOWN')
            
                    Data Elements:
            
                    ^FTER69 = RP:InterposedEntity:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (interposedEntity.FTER69 == null ? false : interposedEntity.FTER69.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 2 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER69", Value = GetValueOrEmpty(interposedEntity.FTER69) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428255
            
                    #region VR.ATO.GEN.428256
            
                    /*  VR.ATO.GEN.428256
                    Suburb/Town/City cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER70, 'UNKNOWN')
            
                    Data Elements:
            
                    ^FTER70 = RP:InterposedEntity:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (interposedEntity.FTER70 == null ? false : interposedEntity.FTER70.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER70", Value = interposedEntity.FTER70 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428256
            
                    #region VR.ATO.GEN.428258
            
                    /*  VR.ATO.GEN.428258
                    Non Individual Name cannot contain space hyphen space combinations
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER65, ' - ')
            
                    Data Elements:
            
                    ^FTER65 = RP:InterposedEntity:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (interposedEntity.FTER65 == null ? false : interposedEntity.FTER65.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain space hyphen space combinations",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER65", Value = interposedEntity.FTER65 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428258
            
                    #region VR.ATO.GEN.428259
            
                    /*  VR.ATO.GEN.428259
                    Non Individual Name cannot contain "P/L"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER65, 'P/L')
            
                    Data Elements:
            
                    ^FTER65 = RP:InterposedEntity:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (interposedEntity.FTER65 == null ? false : interposedEntity.FTER65.ToUpperInvariant().Contains(@"P/L"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""P/L""",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER65", Value = interposedEntity.FTER65 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428259
            
                    #region VR.ATO.GEN.428260
            
                    /*  VR.ATO.GEN.428260
                    Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^FTER65, '"--","''","  "'))
            
                    Data Elements:
            
                    ^FTER65 = RP:InterposedEntity:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(interposedEntity.FTER65, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER65", Value = interposedEntity.FTER65 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428260
            
                    #region VR.ATO.GEN.428261
            
                    /*  VR.ATO.GEN.428261
                    Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (EndsWithSet(^FTER65, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
            
                    Data Elements:
            
                    ^FTER65 = RP:InterposedEntity:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(interposedEntity.FTER65, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER65", Value = interposedEntity.FTER65 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428261
            
                    #region VR.ATO.GEN.430246
            
                    /*  VR.ATO.GEN.430246
                    Suburb/Town/City cannot contain numeric characters
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER146 = FALSE) AND (ContainsSet(^FTER70, '0-9'))
            
                    Data Elements:
            
                    ^FTER70 = RP:InterposedEntity:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER146 = RP:InterposedEntity:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (interposedEntity.FTER146 == false && IsMatch(interposedEntity.FTER70, @"[0-9]",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain numeric characters",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER146", Value = GetValueOrEmpty(interposedEntity.FTER146) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER70", Value = interposedEntity.FTER70 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430246
            
                    #region VR.ATO.GEN.434147
            
                    /*  VR.ATO.GEN.434147
                    State and post code must be present for an Australian address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^FTER146 = FALSE AND (^FTER71 = NULL OR ^FTER72 = NULL))
            
                    Data Elements:
            
                    ^FTER146 = RP:InterposedEntity:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER71 = RP:InterposedEntity:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^FTER72 = RP:InterposedEntity:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:InterposedEntity:AddressDetails
                    */
                    assertion = (interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPExists != false && (interposedEntity.FTER146 == false && (interposedEntity.FTER71 == null || interposedEntity.FTER72 == null)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                            Description = @"State and post code must be present for an Australian address",
                            LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER146", Value = GetValueOrEmpty(interposedEntity.FTER146) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER71", Value = GetValueOrEmpty(interposedEntity.FTER71) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER72", Value = GetValueOrEmpty(interposedEntity.FTER72) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.434147
            
                    #region VR.ATO.GEN.500132
            
                    /*  VR.ATO.GEN.500132
                    Organisation name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^OrganisationNameDetails <> NULL) AND (^FTER65 = BLANK)
            
                    Data Elements:
            
                    ^FTER65 = RP:InterposedEntity:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^OrganisationNameDetails = RP:InterposedEntity:OrganisationNameDetails
                    */
                    assertion = (interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(interposedEntity.FTER65) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must be present.",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER65", Value = interposedEntity.FTER65 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500132
            
                    #region VR.ATO.GEN.500133
            
                    /*  VR.ATO.GEN.500133
                    Address Line 1 must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^FTER68 = BLANK)
            
                    Data Elements:
            
                    ^FTER68 = RP:InterposedEntity:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:InterposedEntity:AddressDetails
                    */
                    assertion = (interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(interposedEntity.FTER68) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must be present.",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER68", Value = interposedEntity.FTER68 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500133
            
                    #region VR.ATO.GEN.500134
            
                    /*  VR.ATO.GEN.500134
                    Suburb/Town/City must be present in an address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^FTER70 = BLANK)
            
                    Data Elements:
            
                    ^FTER70 = RP:InterposedEntity:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:InterposedEntity:AddressDetails
                    */
                    assertion = (interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(interposedEntity.FTER70) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City must be present in an address.",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER70", Value = interposedEntity.FTER70 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500134
            
                    #region VR.ATO.GEN.500141
            
                    /*  VR.ATO.GEN.500141
                    For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
                    AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER71 <> BLANK) AND (NotInSet(^FTER71, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
            
                    Data Elements:
            
                    ^FTER71 = RP:InterposedEntity:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(interposedEntity.FTER71) != true && !(IsMatch(interposedEntity.FTER71, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                            Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                            Location = "/xbrli:xbrl/tns:InterposedEntity" + OccurrenceIndex(interposedEntity.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER71", Value = GetValueOrEmpty(interposedEntity.FTER71) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500141
                    }
                }
        
                #endregion // Foreach loop

            #region Repeating report.DeclarationOfTrusteeCollection
            if (report.DeclarationOfTrusteeCollection != null)
            {    
                int itemIndex1 = 0;
                foreach (FTER2018.DeclarationOfTrustee declarationOfTrustee in report.DeclarationOfTrusteeCollection)
                {
                    itemIndex1++;
            
                    #region VR.ATO.FTER.408130
            
                    /*  VR.ATO.FTER.408130
                    Trustee declaration must contain either an individual name or a non-individual name
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:DeclarationOfTrustee) 
                    IF [FTER75] = NULLORBLANK AND [FTER79] = NULLORBLANK 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FTER75] = FTER:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(fter.0002.lodge.req.02.00:PersonNameDetails) IN TUPLE(fter.0002.lodge.req.02.00:DeclarationOfTrustee)
                    [FTER79] = FTER:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(fter.0002.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(fter.0002.lodge.req.02.00:DeclarationOfTrustee)
        
                    Technical Business Rule Format:
                    ^FTER75 = BLANK AND ^FTER79 = BLANK
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^FTER79 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrustee.FTER75) == true && string.IsNullOrWhiteSpace(declarationOfTrustee.FTER79) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408130", Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee declaration must contain either an individual name or a non-individual name",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408130"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER79", Value = declarationOfTrustee.FTER79 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408130
            
                    #region VR.ATO.FTER.408135
            
                    /*  VR.ATO.FTER.408135
                    Individual name and non-individual name of a trustee declaration must not both be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:DeclarationOfTrustee)
                    IF ([FTER79] <> NULL) AND ([FTER75] <> NULL)
                      RETURN VALIDATION MESSAGE 
                    ENDIF
                    
                    [FTER75] = FTER:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(fter.0002.lodge.req.02.00:PersonNameDetails) IN TUPLE(fter.0002.lodge.req.02.00:DeclarationOfTrustee)
                    [FTER79] = FTER:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(fter.0002.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(fter.0002.lodge.req.02.00:DeclarationOfTrustee)
        
                    Technical Business Rule Format:
                    (^FTER79 <> NULL) AND (^FTER75 <> NULL)
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^FTER79 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrustee.FTER79 != null && declarationOfTrustee.FTER75 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408135", Severity = ProcessMessageSeverity.Error,
                            Description = @"Individual name and non-individual name of a trustee declaration must not both be present",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408135"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER79", Value = declarationOfTrustee.FTER79 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408135
            
                    #region VR.ATO.FTER.408136
            
                    /*  VR.ATO.FTER.408136
                    Individual trustee declaration First name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:DeclarationOfTrustee)
                    IF [FTER75] <> NULL AND [FTER77] = NULL
                      RETURN VALIDATION MESSAGE 
                    ENDIF
                    
                    [FTER75] = FTER:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(fter.0002.lodge.req.02.00:PersonNameDetails) IN TUPLE(fter.0002.lodge.req.02.00:DeclarationOfTrustee)
                    [FTER77] = FTER:RP:pyde.02.00:PersonNameDetails.GivenName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(fter.0002.lodge.req.02.00:PersonNameDetails) IN TUPLE(fter.0002.lodge.req.02.00:DeclarationOfTrustee)
        
                    Technical Business Rule Format:
                    ^FTER75 <> NULL AND ^FTER77 = NULL
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^FTER77 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrustee.FTER75 != null && declarationOfTrustee.FTER77 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408158", Severity = ProcessMessageSeverity.Error,
                            Description = @"Individual trustee declaration First name must be present",
                            LongDescription = @"If a trustee's family name or other given name is present, first given name must be present",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408136"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER77", Value = GetValueOrEmpty(declarationOfTrustee.FTER77) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.FTER.408136
            
                    #region VR.ATO.GEN.000458
            
                    /*  VR.ATO.GEN.000458
                    Suffix is invalid
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER76 <> BLANK) AND (NotInDomain(^FTER76, 'SUFFIX CODES'))
            
                    Data Elements:
            
                    ^FTER76 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrustee.FTER76) != true && !(IsMatch(declarationOfTrustee.FTER76, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000458", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suffix is invalid",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.NameSuffix.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER76", Value = GetValueOrEmpty(declarationOfTrustee.FTER76) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000458
            
                    #region VR.ATO.GEN.000459
            
                    /*  VR.ATO.GEN.000459
                    Title is invalid
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER74 <> BLANK) AND (NotInDomain(^FTER74, 'TITLE CODES'))
            
                    Data Elements:
            
                    ^FTER74 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrustee.FTER74) != true && !(IsMatch(declarationOfTrustee.FTER74, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000459", Severity = ProcessMessageSeverity.Error,
                            Description = @"Title is invalid",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.Title.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER74", Value = GetValueOrEmpty(declarationOfTrustee.FTER74) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000459
            
                    #region VR.ATO.GEN.410038
            
                    /*  VR.ATO.GEN.410038
                    Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (StartsWith(^FTER79, 'T/A ')) AND (EndsWithSet(^FTER79, '" Pship"," P'ship"," P/ship"'))
            
                    Data Elements:
            
                    ^FTER79 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (IsMatch(declarationOfTrustee.FTER79, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(declarationOfTrustee.FTER79, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER79", Value = declarationOfTrustee.FTER79 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410038
            
                    #region VR.ATO.GEN.410039
            
                    /*  VR.ATO.GEN.410039
                    Organisation name must contain at least one alpha or numeric character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER79  <> BLANK) AND (NotContainsSet(^FTER79, '"a-z", "A-Z", "0-9"'))
            
                    Data Elements:
            
                    ^FTER79 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrustee.FTER79) != true && !(IsMatch(declarationOfTrustee.FTER79, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must contain at least one alpha or numeric character",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER79", Value = declarationOfTrustee.FTER79 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410039
            
                    #region VR.ATO.GEN.410063
            
                    /*  VR.ATO.GEN.410063
                    Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER77 <> BLANK) AND (NotContainsSet(^FTER77, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^FTER77 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrustee.FTER77) != true && !(IsMatch(declarationOfTrustee.FTER77, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410063", Severity = ProcessMessageSeverity.Error,
                            Description = @"Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410063"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER77", Value = GetValueOrEmpty(declarationOfTrustee.FTER77) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410063
            
                    #region VR.ATO.GEN.410131
            
                    /*  VR.ATO.GEN.410131
                    Other Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^FTER78 <> BLANK) AND (NotContainsSet(^FTER78, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^FTER78 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrustee.FTER78) != true && !(IsMatch(declarationOfTrustee.FTER78, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410131", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER78", Value = GetValueOrEmpty(declarationOfTrustee.FTER78) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410131
            
                    #region VR.ATO.GEN.410200
            
                    /*  VR.ATO.GEN.410200
                    Family name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER75, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER75, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000424", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410200
            
                    #region VR.ATO.GEN.410201
            
                    /*  VR.ATO.GEN.410201
                    Family name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER75, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER75, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000426", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410201
            
                    #region VR.ATO.GEN.410203
            
                    /*  VR.ATO.GEN.410203
                    First name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER77, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^FTER77 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER77, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000437", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER77", Value = GetValueOrEmpty(declarationOfTrustee.FTER77) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410203
            
                    #region VR.ATO.GEN.410204
            
                    /*  VR.ATO.GEN.410204
                    First name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER77, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^FTER77 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER77, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000438", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER77", Value = GetValueOrEmpty(declarationOfTrustee.FTER77) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410204
            
                    #region VR.ATO.GEN.410208
            
                    /*  VR.ATO.GEN.410208
                    Other given name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER78, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^FTER78 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER78, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000449", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER78", Value = GetValueOrEmpty(declarationOfTrustee.FTER78) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410208
            
                    #region VR.ATO.GEN.410209
            
                    /*  VR.ATO.GEN.410209
                    Other given name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER78, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^FTER78 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER78, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000450", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER78", Value = GetValueOrEmpty(declarationOfTrustee.FTER78) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410209
            
                    #region VR.ATO.GEN.410218
            
                    /*  VR.ATO.GEN.410218
                    FirstName must be entered if OtherGivenNames is entered
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                        RETURN VALIDATION MESSAGE 
                     ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND ((^FTER78 <> BLANK) AND (^FTER77 = BLANK))
            
                    Data Elements:
            
                    ^FTER77 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^FTER78 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:DeclarationOfTrustee:PersonNameDetails
                    */
                    assertion = (declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists != false && (string.IsNullOrWhiteSpace(declarationOfTrustee.FTER78) != true && string.IsNullOrWhiteSpace(declarationOfTrustee.FTER77) == true));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410218", Severity = ProcessMessageSeverity.Error,
                            Description = @"FirstName must be entered if OtherGivenNames is entered",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER78", Value = GetValueOrEmpty(declarationOfTrustee.FTER78) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER77", Value = GetValueOrEmpty(declarationOfTrustee.FTER77) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410218
            
                    #region VR.ATO.GEN.428258
            
                    /*  VR.ATO.GEN.428258
                    Non Individual Name cannot contain space hyphen space combinations
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER79, ' - ')
            
                    Data Elements:
            
                    ^FTER79 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrustee.FTER79 == null ? false : declarationOfTrustee.FTER79.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain space hyphen space combinations",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER79", Value = declarationOfTrustee.FTER79 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428258
            
                    #region VR.ATO.GEN.428259
            
                    /*  VR.ATO.GEN.428259
                    Non Individual Name cannot contain "P/L"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER79, 'P/L')
            
                    Data Elements:
            
                    ^FTER79 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrustee.FTER79 == null ? false : declarationOfTrustee.FTER79.ToUpperInvariant().Contains(@"P/L"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""P/L""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER79", Value = declarationOfTrustee.FTER79 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428259
            
                    #region VR.ATO.GEN.428260
            
                    /*  VR.ATO.GEN.428260
                    Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^FTER79, '"--","''","  "'))
            
                    Data Elements:
            
                    ^FTER79 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER79, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER79", Value = declarationOfTrustee.FTER79 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428260
            
                    #region VR.ATO.GEN.428261
            
                    /*  VR.ATO.GEN.428261
                    Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (EndsWithSet(^FTER79, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
            
                    Data Elements:
            
                    ^FTER79 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER79, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER79", Value = declarationOfTrustee.FTER79 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428261
            
                    #region VR.ATO.GEN.428262
            
                    /*  VR.ATO.GEN.428262
                    Family Name cannot contain "space hyphen space"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER75, ' - ')
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrustee.FTER75 == null ? false : declarationOfTrustee.FTER75.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000423", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name cannot contain ""space hyphen space""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428262"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428262
            
                    #region VR.ATO.GEN.428263
            
                    /*  VR.ATO.GEN.428263
                    Family name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^FTER75, '"--","''","  "'))
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER75, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000427", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428263"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428263
            
                    #region VR.ATO.GEN.428264
            
                    /*  VR.ATO.GEN.428264
                    First name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER77, ' - ')
            
                    Data Elements:
            
                    ^FTER77 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrustee.FTER77 == null ? false : declarationOfTrustee.FTER77.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000434", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428264"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER77", Value = GetValueOrEmpty(declarationOfTrustee.FTER77) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428264
            
                    #region VR.ATO.GEN.428265
            
                    /*  VR.ATO.GEN.428265
                    First name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^FTER77, '"--","''","  "'))
            
                    Data Elements:
            
                    ^FTER77 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER77, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000439", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428265"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER77", Value = GetValueOrEmpty(declarationOfTrustee.FTER77) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428265
            
                    #region VR.ATO.GEN.428266
            
                    /*  VR.ATO.GEN.428266
                    Other given name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^FTER78, ' - ')
            
                    Data Elements:
            
                    ^FTER78 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrustee.FTER78 == null ? false : declarationOfTrustee.FTER78.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000446", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428266"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER78", Value = GetValueOrEmpty(declarationOfTrustee.FTER78) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428266
            
                    #region VR.ATO.GEN.428267
            
                    /*  VR.ATO.GEN.428267
                    Other given name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^FTER78, '"--","''","  "'))
            
                    Data Elements:
            
                    ^FTER78 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER78, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000451", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428267"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER78", Value = GetValueOrEmpty(declarationOfTrustee.FTER78) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428267
            
                    #region VR.ATO.GEN.430252
            
                    /*  VR.ATO.GEN.430252
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    Length(^FTER166) > 200
            
                    Data Elements:
            
                    ^FTER166 = RP:DeclarationOfTrustee:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
                    */
                    assertion = (Length(declarationOfTrustee.FTER166) > 200);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430252", Severity = ProcessMessageSeverity.Error,
                            Description = @"Unstructured name must not be more than 200 characters",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:Declaration" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER166", Value = declarationOfTrustee.FTER166 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430252
            
                    #region VR.ATO.GEN.430255
            
                    /*  VR.ATO.GEN.430255
    
            Legacy Rule Format:
            IF pyin.xx.xx:Declaration.Signature.Date > DATE(TODAY)
               RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    ^FTER168 > Today()
            
                    Data Elements:
            
                    ^FTER168 = RP:DeclarationOfTrustee:Declaration:Declaration.Signature.Date WHERE [Declaration.StatementType.Code='TrueAndCorrect']
                    */
                    assertion = (declarationOfTrustee.FTER168.GetValueOrDefault() > DateTime.Now.Date);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430255", Severity = ProcessMessageSeverity.Error,
                            Description = @"Signature date must not be later than today",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:Declaration" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:Declaration.StatementType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430255"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER168", Value = GetValueOrEmptyDate(declarationOfTrustee.FTER168) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430255
            
                    #region VR.ATO.GEN.500132
            
                    /*  VR.ATO.GEN.500132
                    Organisation name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^OrganisationNameDetails <> NULL) AND (^FTER79 = BLANK)
            
                    Data Elements:
            
                    ^FTER79 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^OrganisationNameDetails = RP:DeclarationOfTrustee:OrganisationNameDetails
                    */
                    assertion = (declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(declarationOfTrustee.FTER79) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must be present.",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER79", Value = declarationOfTrustee.FTER79 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500132
            
                    #region VR.ATO.GEN.500135
            
                    /*  VR.ATO.GEN.500135
                    Family Name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF pyde.xx.xx:PersonNameDetails.FamilyName.Text = BLANK
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND (^FTER75 = BLANK)
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:DeclarationOfTrustee:PersonNameDetails
                    */
                    assertion = (declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists != false && string.IsNullOrWhiteSpace(declarationOfTrustee.FTER75) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500135", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name must be present.",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500135"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500135
            
                    #region VR.ATO.GEN.500142
            
                    /*  VR.ATO.GEN.500142
                    Family name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF pyde.xx.xx:PersonNameDetails.FamilyName.Text DOES NOT CONTAIN SET("A-Z","a-z")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND (NotContainsSet(^FTER75, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:DeclarationOfTrustee:PersonNameDetails
                    */
                    assertion = (declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists != false && !(IsMatch(declarationOfTrustee.FTER75, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500142", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name must contain at least one alphabetical character.",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500142"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500142
            
                    #region VR.ATO.GEN.500151
            
                    /*  VR.ATO.GEN.500151
                    First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER77, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^FTER77 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER77, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010436", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500151"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER77", Value = GetValueOrEmpty(declarationOfTrustee.FTER77) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500151
            
                    #region VR.ATO.GEN.500152
            
                    /*  VR.ATO.GEN.500152
                    Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER78, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^FTER78 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER78, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010448", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500152"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER78", Value = GetValueOrEmpty(declarationOfTrustee.FTER78) });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500152
            
                    #region VR.ATO.GEN.500153
            
                    /*  VR.ATO.GEN.500153
                    Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^FTER75, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^FTER75 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrustee.FTER75, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500153", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrustee" + OccurrenceIndex(declarationOfTrustee.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500153"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "FTER75", Value = declarationOfTrustee.FTER75 });
            
                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500153
                    }
                }
        
                #endregion // Foreach loop

            #region Repeating report.TrusteeCollection
            if (report.TrusteeCollection != null)
            {    
                int itemIndex1 = 0;
                foreach (FTER2018.Trustee trustee in report.TrusteeCollection)
                {
                    itemIndex1++;
        
                    #region Repeating trustee.Trustee_PeriodDetailsNonResidentPartPeriodRPCollection
                    if (trustee.Trustee_PeriodDetailsNonResidentPartPeriodRPCollection != null)
                    {    
                        int itemIndex2 = 0;
                        foreach (FTER2018.Trustee.Trustee_PeriodDetailsNonResidentPartPeriodRP periodDetailsNonResidentPartPeriod in trustee.Trustee_PeriodDetailsNonResidentPartPeriodRPCollection)
                        {
                            itemIndex2++;
                    
                            #region VR.ATO.GEN.408060
                    
                            /*  VR.ATO.GEN.408060
                            Start date must be before end date
            
                            Legacy Rule Format:
                            WHERE IN TUPLE (perioddetails1.xx.xx:PeriodDetails)
                            IF pyin.xx.xx:Period.End.Date <= pyin.xx.xx:Period.Start.Date
                               RETURN VALIDATION MESSAGE
                            ENDIF
                
                            Technical Business Rule Format:
                            ^FTER39 <= ^FTER38
                    
                            Data Elements:
                    
                            ^FTER39 = RP:Trustee:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                    
                            ^FTER38 = RP:Trustee:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                            */
                            assertion = (periodDetailsNonResidentPartPeriod.FTER39.GetValueOrDefault() <= periodDetailsNonResidentPartPeriod.FTER38.GetValueOrDefault());
                            if (assertion)
                            {
                                processMessage = new ProcessMessageDocument()
                                {
                                    Code = "CMN.ATO.GEN.408060", Severity = ProcessMessageSeverity.Error,
                                    Description = @"Start date must be before end date",
                                    LongDescription = @"The start date must be earlier than the end date in a Period.",
                                    Location = "/xbrli:xbrl/tns:Trustee" + OccurrenceIndex(trustee.OccurrenceIndex) + "/tns:PeriodDetails" + OccurrenceIndex(periodDetailsNonResidentPartPeriod.Trustee_PeriodDetailsCollection_NonResidentPartPeriod_RPOccurrenceIndex) + "/tns:Period.End.Date",
                                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.408060"} },
                                };
                                processMessage.Parameters.Add(new ProcessMessageParameter
                                    { Name = "FTER39", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.FTER39) });
                    
                                processMessage.Parameters.Add(new ProcessMessageParameter
                                    { Name = "FTER38", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.FTER38) });
                    
                                response.Add(processMessage);
                            }
                            #endregion // VR.ATO.GEN.408060
                            }
                        }
                
                        #endregion // Foreach loop
                        }
                    }
            
                    #endregion // Foreach loop

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Value = _emptyParameterValue;
                        }

                        if (currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }


    }
} 
