using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using DataContracts;
using System.Runtime.CompilerServices;

namespace Ato.EN.IntegrationServices.CodeGenerationIEE
{

    public partial class IEE2018Validator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="IEE2018Validator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public IEE2018Validator(int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }


        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString().ToLower() : string.Empty;
        }

        private static string GetValueOrEmptyDate(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-dd") : string.Empty;
        }

        private static string GetValueOrEmptyDate(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-dd");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-ddTHH:mm:ssZ");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-ddTHH:mm:ssZ") : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Get string value between [first] a and [last] b.
        /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

        /// <summary>
        /// Get string value after [first] a.
        /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

        /// <summary>
        /// Get string value after [last] a.
        /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
                return 0;
            else
                return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return (from x in nodes select x).Distinct().Count() != 1;
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            nodes = nodes.Where(x => x != null).ToArray();
            return !((from x in nodes select x).Distinct().Count() == nodes.Count());

        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

        public DateTime Date(DateTime? datetime)
        {
            return datetime.GetValueOrDefault().Date;
        }

        //The logic in After function expects "---" for day and "--" for month. 
        //Since hyphen was missing the date always returned null
        public DateTime? ConvertToDate(int day, int month, int year)
        {
            return ConvertToDate(day == 0 ? null : "---" + day.ToString(), month == 0 ? null : "--" + month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }

        private static DateTime? AddMonthsToDate(DateTime? dateTime, int months)
        {
            return dateTime == null ? null : (DateTime?)dateTime.Value.AddMonths(months);
        }

        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response = true;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length == 14)
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 3), out numeric))
                        response = false;

                }
                else if (usi.Length == 9)
                {

                    if (Regex.IsMatch(usi, @"^([a-zA-Z]{3}\d{4}[a-zA-Z]{2})$"))
                        response = false;
                }
            }
            return response;
        }

        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
            return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
            decimal numberValue;
            decimal.TryParse(value, out numberValue);
            return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':
                        options = options | RegexOptions.IgnoreCase;
                        break;
                    case 'm':
                        options = options | RegexOptions.Multiline;
                        break;
                    case 's':
                        options = options | RegexOptions.Singleline;
                        break;
                    case 'n':
                        options = options | RegexOptions.ExplicitCapture;
                        break;
                    case 'x':
                        options = options | RegexOptions.IgnorePatternWhitespace;
                        break;
                }
            }

            return options;
        }

        private static string Substring(object field, int start, int length)
        {
            return field.ToString().Substring(start, length);
        }

        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public IEE2018 ConsumedReport { get; private set; }

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string, ProcessMessageDocument>();

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(IEE2018 report, bool isStandalone)
        {

            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            bool assertion;

            this.ConsumedReport = report;

            // ------------------------------------------------------------------------------
            // Validations are now done in code.
            // This version supports full validation of a report - for those rules that have successfully parsed and could be generated.
            // Generated Validations include:
            //   - Validation logic in C#.
            //   - Production of errors in EBMS, SBR and SWS format.
            // 
            // The generation at this stage does not support the following - and has to be completed manually (later versions of the generation will do this for you)
            //   - Business Rules that did not parse.  The ESR team will help support you where this happens as an effort is being made to rectify these
            //   - You will get TODO tasks for those that the parser could not cope with
            // ------------------------------------------------------------------------------

            #region Manual Rules - Rules marked for manual coding

            #region VR.ATO.GEN.410116

            /*  VR.ATO.GEN.410116
                Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

            Legacy Rule Format:
                IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                    (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                   RETURN VALIDATION MESSAGE
                ENDIF

            Technical Business Rule Format:
                (^IEE97 <> BLANK) AND     (NotInSet(^IEE97, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

            Data Elements:

            ^IEE97 = RP:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            */
            VRATOGEN410116(response, report, isStandalone);
            #endregion // VR.ATO.GEN.410116

            #region VR.ATO.GEN.410116

            /*  VR.ATO.GEN.410116
                        Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

                    Legacy Rule Format:
                        IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                            (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                Technical Business Rule Format:
                        (^IEE98 <> BLANK) AND     (NotInSet(^IEE98, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

            Data Elements:

                ^IEE98 = RP:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='NonResidentPartPeriod']
                */
            VRATOGEN410116_(response, report, isStandalone);
            #endregion // VR.ATO.GEN.410116

            #region VR.ATO.GEN.410116

            /*  VR.ATO.GEN.410116
                        Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

                    Legacy Rule Format:
                        IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                            (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                Technical Business Rule Format:
                        (^IEE103 <> BLANK) AND     (NotInSet(^IEE103, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

            Data Elements:

                ^IEE103 = RP:FamilyTrust:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
                */
            VRATOGEN410116__(response, report, isStandalone);
            #endregion // VR.ATO.GEN.410116

            #region VR.ATO.GEN.410116

            /*  VR.ATO.GEN.410116
                        Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

                    Legacy Rule Format:
                        IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                            (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                Technical Business Rule Format:
                        (^IEE99 <> BLANK) AND     (NotInSet(^IEE99, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

            Data Elements:

                ^IEE99 = RP:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
                */
            VRATOGEN410116___(response, report, isStandalone);
            #endregion // VR.ATO.GEN.410116

            #region VR.ATO.GEN.410116

            /*  VR.ATO.GEN.410116
                        Period code must be 'ControlOutsideAustraliaPartPeriod', 'NonResidentPartPeriod' or 'SubstitutedAccountingPeriod'

                    Legacy Rule Format:
                        IF (pyin.xx.xx:Period.Type.Code <> NULLORBLANK) AND 
                            (pyin.xx.xx:Period.Type.Code <> SET("ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                Technical Business Rule Format:
                        (^IEE102 <> BLANK) AND     (NotInSet(^IEE102, '"ControlOutsideAustraliaPartPeriod","NonResidentPartPeriod","SubstitutedAccountingPeriod"'))

            Data Elements:

                ^IEE102 = RP:TrusteeOrPartner:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='NonResidentPartPeriod']
                */
            VRATOGEN410116____(response, report, isStandalone);
            #endregion // VR.ATO.GEN.410116

            #region VR.ATO.GEN.432016

            /*  VR.ATO.GEN.432016

                    Legacy Rule Format:
                        WHERE IN TUPLE(electroniccontacttelephone1.xx.xx:ElectronicContactTelephone)
                        IF pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number = NULL OR pyde.xx.xx:ElectronicContact.Telephone.Usage.Code <> "03" OR pyde.xx.xx:Electronic Contact.Telephone.ServiceLine.Code <> SET ("01", "02")
                           RETURN VALIDATION MESSAGE
                        ENDIF

                Technical Business Rule Format:
                        (^ElectronicContactTelephone <> NULL) AND (^IEE190 = NULL OR ^IEE187 <> '03' OR NotInSet(^IEE188, '"01", "02"'))

            Data Elements:

                ^IEE190 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.Minimal.Number WHERE [ElectronicContact.Telephone.Usage.Code = '03']

                ^IEE187 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.Usage.Code WHERE [ElectronicContact.Telephone.Usage.Code = '03']

                ^IEE188 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.ServiceLine.Code WHERE [ElectronicContact.Telephone.Usage.Code = '03']

                ^ElectronicContactTelephone = INT:ElectronicContactTelephone
                */
            VRATOGEN432016(response, report, isStandalone);
            #endregion // VR.ATO.GEN.432016

            #region VR.ATO.IEE.410083

            /*  VR.ATO.IEE.410083
                        Full period indicator or dates for central management and control of trust was outside Australia have been entered when the answer to the question was central management and control of the trust outside Australia is 'No'.

                    Legacy Rule Format:
                        IF ([IEE17] = FALSE) AND (([IEE18] <> NULL) OR ([IEE19] <> NULL) OR ([IEE20] <> NULL)) 
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE17] = IEE:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustralia.Indicator
                        [IEE18] = IEE:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
                        [IEE19] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
                        [IEE20] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

                Technical Business Rule Format:
                        (^IEE17 = FALSE) AND ((^IEE18 <> NULL) OR (^IEE19 <> NULL) OR (^IEE20 <> NULL))

            Data Elements:

                ^IEE17 = RP:Residency.CentralManagementAndControlOutsideAustralia.Indicator

                ^IEE18 = RP:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator

                ^IEE19 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']

                ^IEE20 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
                */
            VRATOIEE410083(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410083

            #region VR.ATO.IEE.410089

            /*  VR.ATO.IEE.410089
                        The answer to the company being a non resident for tax purposes is 'Yes' and the full period indicator and the dates the company was a non resident have not been supplied.

                    Legacy Rule Format:
                        IF ([IEE21] = SET("NonResidentNoPermEstab","NonResidentPermEstab")) AND ([IEE22] = NULL AND ([IEE23] = NULL) AND ([IEE24] = NULL))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE21] = IEE:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
                        [IEE22] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator
                        [IEE23] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
                        [IEE24] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

                Technical Business Rule Format:
                        (InSet(^IEE21, '"NonResidentNoPermEstab","NonResidentPermEstab"')) AND (^IEE22 = NULL) AND (^IEE23 = NULL) AND (^IEE24 = NULL)

            Data Elements:

                ^IEE21 = RP:Residency.TaxPurposesOrganisationStatus.Code

                ^IEE22 = RP:Residency.NonResidentFullPeriod.Indicator

                ^IEE23 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']

                ^IEE24 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                */
            VRATOIEE410089(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410089

            #region VR.ATO.IEE.410090

            /*  VR.ATO.IEE.410090
                        The answer to the company a non resident for tax purposes question is 'No', therefore full period indicator or dates company was not resident are not required.

                    Legacy Rule Format:
                        IF ([IEE21] = "Resident") AND (([IEE22] <> NULL) OR ([IEE23] <> NULL) OR ([IEE24] <> NULL))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE21] = IEE:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
                        [IEE22] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator
                        [IEE23] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
                        [IEE24] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

                Technical Business Rule Format:
                        (^IEE21 = 'Resident') AND ((^IEE22 <> NULL) OR (^IEE23 <> NULL) OR (^IEE24 <> NULL))

            Data Elements:

                ^IEE21 = RP:Residency.TaxPurposesOrganisationStatus.Code

                ^IEE22 = RP:Residency.NonResidentFullPeriod.Indicator

                ^IEE23 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']

                ^IEE24 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                */
            VRATOIEE410090(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410090

            #region VR.ATO.IEE.410092

            /*  VR.ATO.IEE.410092
                        The answer to the question was trustee or partner a non-resident for tax purposes is 'Yes' and full period indicator or dates company was non-resident have not been supplied

                    Legacy Rule Format:
                        IF ([IEE41] = TRUE) AND ([IEE42] <> TRUE) AND ([IEE43] = NULL) AND ([IEE44] = NULL) 
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE41] = IEE:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                        [IEE42] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                        [IEE43] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                        [IEE44] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)

                Technical Business Rule Format:
                        (^IEE41 = TRUE) AND (^IEE42 <> TRUE) AND AnyOccurrence(^IEE43, ^IEE43 = NULL) AND AnyOccurrence(^IEE44, ^IEE44 = NULL)

            Data Elements:

                ^IEE41 = RP:TrusteeOrPartner:Residency.TaxPurposesPersonStatus.Indicator

                ^IEE42 = RP:TrusteeOrPartner:Residency.NonResidentFullPeriod.Indicator

                ^IEE43 = RP:TrusteeOrPartner:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']

                ^IEE44 = RP:TrusteeOrPartner:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                */
            VRATOIEE410092(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410092

            #region VR.ATO.IEE.410093

            /*  VR.ATO.IEE.410093
                        The answer to the question was trustee or partner a non-resident for tax purposes is 'No' and full period indicator or dates company was non-resident have been supplied

                    Legacy Rule Format:
                        IF ([IEE41] = FALSE) AND (([IEE42] = TRUE) OR ([IEE43] <> NULL) OR ([IEE44] <> NULL)) 
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE41] = IEE:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                        [IEE42] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                        [IEE43] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                        [IEE44] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)

                Technical Business Rule Format:
                        (^IEE41 = FALSE) AND ((^IEE42 = TRUE) OR AnyOccurrence(^IEE43, ^IEE43 <> NULL) OR AnyOccurrence(^IEE44, ^IEE44 <> NULL))

            Data Elements:

                ^IEE41 = RP:TrusteeOrPartner:Residency.TaxPurposesPersonStatus.Indicator

                ^IEE42 = RP:TrusteeOrPartner:Residency.NonResidentFullPeriod.Indicator

                ^IEE43 = RP:TrusteeOrPartner:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']

                ^IEE44 = RP:TrusteeOrPartner:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                */
            VRATOIEE410093(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410093

            #region VR.ATO.IEE.410120

            /*  VR.ATO.IEE.410120
                        Up to one Current address and up to one previous address for Interposed Entity only to be provided

                    Legacy Rule Format:
                        IF COUNT([IEE5]) >1 OR COUNT([IEE11]) >1
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE5] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails)
                        [IEE11] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails)

                Technical Business Rule Format:
                        Count(^IEE5) >1 OR Count(^IEE11) >1

            Data Elements:

                ^IEE5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']

                ^IEE11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
                */
            VRATOIEE410120(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410120

            #region VR.ATO.IEE.410139

            /*  VR.ATO.IEE.410139
                        Address Currency Code must be 'C'

                    Legacy Rule Format:
                        IN TUPLE((iee.0003.lodge.request.xx.xx:TrusteeOrPartner) OR (iee.0003.lodge.request.xx.xx:FamilyTrust) OR (iee.0003.lodge.request.xx.xx:SpecifiedIndividual)) 
                        IN TUPLE(address2.xx.xx:AddressDetails) 
                        IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Currency.Code <> "C")
                            RETURN VALIDATION MESSAGE 
                        ENDIF

                Technical Business Rule Format:
                        (^IEE130 <> BLANK) AND (^IEE130 <> 'C')

            Data Elements:

                ^IEE130 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                */
            VRATOIEE410139(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410139

            #region VR.ATO.IEE.410139

            /*  VR.ATO.IEE.410139
                        Address Currency Code must be 'C'

                    Legacy Rule Format:
                        IN TUPLE((iee.0003.lodge.request.xx.xx:TrusteeOrPartner) OR (iee.0003.lodge.request.xx.xx:FamilyTrust) OR (iee.0003.lodge.request.xx.xx:SpecifiedIndividual)) 
                        IN TUPLE(address2.xx.xx:AddressDetails) 
                        IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Currency.Code <> "C")
                            RETURN VALIDATION MESSAGE 
                        ENDIF

                Technical Business Rule Format:
                        (^IEE130 <> BLANK) AND (^IEE130 <> 'C')

            Data Elements:

                ^IEE130 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                */
            VRATOIEE410139_(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410139

            #region VR.ATO.IEE.410139

            /*  VR.ATO.IEE.410139
                        Address Currency Code must be 'C'

                    Legacy Rule Format:
                        IN TUPLE((iee.0003.lodge.request.xx.xx:TrusteeOrPartner) OR (iee.0003.lodge.request.xx.xx:FamilyTrust) OR (iee.0003.lodge.request.xx.xx:SpecifiedIndividual)) 
                        IN TUPLE(address2.xx.xx:AddressDetails) 
                        IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Currency.Code <> "C")
                            RETURN VALIDATION MESSAGE 
                        ENDIF

                Technical Business Rule Format:
                        (^IEE130 <> BLANK) AND (^IEE130 <> 'C')

            Data Elements:

                ^IEE130 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                */
            VRATOIEE410139__(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410139

            #region VR.ATO.IEE.410150

            /*  VR.ATO.IEE.410150
                        Up to one Current Name and up to one Previous Name only to be provided

                    Legacy Rule Format:
                        IF COUNT([IEE4]) >1 OR COUNT([IEE10]) >1
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE4] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails)
                        [IEE10] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "P") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails)

                Technical Business Rule Format:
                        Count(^IEE4) >1 OR Count(^IEE10) >1

            Data Elements:

                ^IEE10 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']

                ^IEE4 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                */
            VRATOIEE410150(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410150

            #region VR.ATO.IEE.410189

            /*  VR.ATO.IEE.410189
                        More than four periods must not be specified for a trustee or partner

                    Legacy Rule Format:
                        IF COUNT([IEE43]) > 4
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE43] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)

                Technical Business Rule Format:
                        Count(^IEE43) > 4

            Data Elements:

                ^IEE43 = RP:TrusteeOrPartner:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                */
            VRATOIEE410189(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410189

            #region VR.ATO.IEE.410243

            /*  VR.ATO.IEE.410243
                        Person Unstructured Name not required

                    Legacy Rule Format:
                        IF COUNT [IEE147] > 0 and COUNT (TUPLE(prsnstrcnm1.xx.xx:PersonUnstructuredName) > 0)
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE147] = IEE:RP:pyin.02.03:Declaration.StatementType.Code WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

                Technical Business Rule Format:
                        Count(^IEE147) > 0 and Count(^PersonUnstructuredName) > 0

            Data Elements:

                ^IEE147 = RP:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']

                ^PersonUnstructuredName = INT:Declaration:PersonUnstructuredName
                */
            VRATOIEE410243(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410243

            #region VR.ATO.IEE.410248

            /*  VR.ATO.IEE.410248
                        Trustee declaration Name Type Code must be 'LGL', Usage Code  'DeclarationSignatory' and Currency Code 'C'.

                    Legacy Rule Format:
                        WHERE IN TUPLE (iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner\prsnstrcnm3.xx.xx:PersonNameDetails) 
                        IF (PersonNameDetails.PersonNameType.Code <> "LGL") OR (PersonNameDetails.Usage.Code <> "DeclarationSignatory") OR (PersonNameDetails.Currency.Code <> "C")
                          RETURN VALIDATION MESSAGE 
                        ENDIF

                Technical Business Rule Format:
                        (^IEE198 <> 'LGL' OR ^IEE199 <> 'DeclarationSignatory' OR ^IEE200 <> 'C')

            Data Elements:

                ^IEE198 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.PersonNameType.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']

                ^IEE199 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.Usage.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']

                ^IEE200 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                */
            VRATOIEE410248(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410248

            #region VR.ATO.IEE.410249

            /*  VR.ATO.IEE.410249
                        Non-individual declaration Name Type Code must be 'MN' and Currency Code must be 'C'

                    Legacy Rule Format:
                        WHERE IN TUPLE (iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner\orgname2.xx.xx:OrganisationNameDetails) 
                        IF (OrganisationNameDetails.OrganisationalNameType.Code <> "MN") OR (OrganisationNameDetails.Currency.Code <> "C")
                          RETURN VALIDATION MESSAGE 
                        ENDIF

                Technical Business Rule Format:
                        ^IEE88 = BLANK

            Data Elements:

                ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                */
            VRATOIEE410249(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410249

            #region VR.ATO.IEE.410258

            /*  VR.ATO.IEE.410258

                    Legacy Rule Format:
                        WHERE IN TUPLE(iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner\declaration2.xx.xx:Declaration)
                        IF COUNT (TUPLE(prsnstrcnm1.xx.xx:PersonUnstructuredName) > 0)
                          RETURN VALIDATION MESSAGE
                        ENDIF

                Technical Business Rule Format:
                        ^DeclarationOfTrusteeOrPartner <> NULL AND ^Declaration <> NULL AND Count(^PersonUnstructuredName) > 0

            Data Elements:

                ^DeclarationOfTrusteeOrPartner = RP:DeclarationOfTrusteeOrPartner

                ^Declaration = RP:DeclarationOfTrusteeOrPartner:Declaration

                ^PersonUnstructuredName = RP:DeclarationOfTrusteeOrPartner:Declaration:PersonUnstructuredName
                */
            VRATOIEE410258(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410258

            #region VR.ATO.IEE.410261

            /*  VR.ATO.IEE.410261
                        Intermediary is present then intermediary declaration must be present

                    Legacy Rule Format:
                        WHERE PARENT RETURN DOES NOT EXIST
                        IF INT<> NULLORBLANK AND COUNT (iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner) > 0 OR COUNT [IEE147] > 0
                             RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE147] = IEE:RP:pyin.02.03:Declaration.StatementType.Code WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

                Technical Business Rule Format:
                        ^INT <> BLANK AND Count(^DeclarationOfTrusteeOrPartner) > 0 OR Count(^IEE147) > 0

            Data Elements:

                ^IEE147 = RP:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']

                ^DeclarationOfTrusteeOrPartner = RP:DeclarationOfTrusteeOrPartner

                ^INT = INT
                */
            VRATOIEE410261(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410261

            #region VR.ATO.IEE.410262

            /*  VR.ATO.IEE.410262
                        Parent return declaration must be present

                    Legacy Rule Format:
                        WHERE PARENT RETURN EXISTS
                        IF INT <> NULLORBLANK AND COUNT [IEE159] > 0
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE159] = IEE:INT:pyin.02.03:Declaration.StatementType.Code WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

                Technical Business Rule Format:
                        ^INT <> BLANK AND Count(^IEE159) > 0

            Data Elements:

                ^IEE159 = INT:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code='TrueAndCorrect']

                ^INT = INT
                */
            VRATOIEE410262(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410262

            #region VR.ATO.IEE.410265

            /*  VR.ATO.IEE.410265
                        Declaration Statement Type Code must be 'TrueAndCorrect'

                    Legacy Rule Format:
                        IF [IEE147] <> NULLORBLANK AND
                        [IEE147] <> "TrueAndCorrect"
                          RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE147] = IEE:RP:pyin.02.03:Declaration.StatementType.Code WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

                Technical Business Rule Format:
                        ^IEE147 <> BLANK AND ^IEE147 <> 'TrueAndCorrect'

            Data Elements:

                ^IEE147 = RP:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']
                */
            VRATOIEE410265(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410265

            #region VR.ATO.IEE.410269

            /*  VR.ATO.IEE.410269
                        Trustee/partner Declaration Family name and Given name or non individual name must match at least one Trustee or partner Family name and Given name or non individual name.

                    Legacy Rule Format:
                        WHERE IN TUPLE(iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner)
                        IF ([IEE203] <> NULL AND ([IEE203] <> ANY OCCURRENCE OF([IEE28]) OR [IEE204] <> (ANY OCCURRENCE OF([IEE30]) WHERE [IEE28] = [IEE203])))
                         OR ([IEE88] <> NULL AND [IEE88] <> ANY OCCURRENCE OF([IEE32]))
                          RETURN VALIDATION MESSAGE
                        ENDIF

                        [IEE28] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                        [IEE30] = IEE:RP:pyde.02.00:PersonNameDetails.GivenName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                        [IEE32] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                        [IEE88] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
                        [IEE203] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
                        [IEE204] = IEE:RP:pyde.02.00:PersonNameDetails.GivenName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)

                Technical Business Rule Format:
                        (^IEE203 <> NULL AND (AnyOccurrence(^IEE28, ^IEE28 <> ^IEE203)) OR AnyOccurrence(^IEE30, ^IEE30 <> ^IEE204) WHERE ^IEE28 = ^IEE203))) OR (^IEE88 <> NULL AND AnyOccurrence(^IEE32, ^IEE32 <> ^IEE88))

            Data Elements:

                ^IEE28 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']

                ^IEE30 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']

                ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

                ^IEE203 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']

                ^IEE204 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                */
            VRATOIEE410269(response, report, isStandalone);
            #endregion // VR.ATO.IEE.410269
            #endregion Manual Rules - Rules marked for manual coding

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201

        Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = INT
    
            ^EndDate = INT
            */
            assertion = ((report.INTExists == true) && (report.INTStartDate >= report.INTEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000201

            /*  VR.ATO.GEN.000201

        Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT(ALL)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^EndDate = RP
            */
            assertion = ((report.RPExists == true) && (report.RPStartDate >= report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000201" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000201

            #region VR.ATO.GEN.000208

            /*  VR.ATO.GEN.000208

        Legacy Rule Format:
            IF COUNT(RP) <> 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) <> 1
    
            Data Elements:
    
            ^Context = RP
            */
            assertion = (Count(report.RPCount) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200007",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid XBRL schema",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000208" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000208

            #region VR.ATO.GEN.000226

            /*  VR.ATO.GEN.000226

        Legacy Rule Format:
            IF COUNT(INT) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = INT
            */
            assertion = (Count(report.INTCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200007",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid XBRL schema",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000226" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000226

            #region VR.ATO.GEN.000405

            /*  VR.ATO.GEN.000405
            Address must not contain "UNKNOWN".
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE53, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE53 = RP:FamilyTrust:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.IEE53 == null ? false : report.IEE53.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000405" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE53", Value = report.IEE53 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000405

            #region VR.ATO.GEN.000405

            /*  VR.ATO.GEN.000405
            Address must not contain "UNKNOWN".
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE5, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.IEE5 == null ? false : report.IEE5.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000405" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE5", Value = report.IEE5 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000405

            #region VR.ATO.GEN.000405

            /*  VR.ATO.GEN.000405
            Address must not contain "UNKNOWN".
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE11, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.IEE11 == null ? false : report.IEE11.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000405" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE11", Value = report.IEE11 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000405

            #region VR.ATO.GEN.000405

            /*  VR.ATO.GEN.000405
            Address must not contain "UNKNOWN".
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE70, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE70 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.IEE70 == null ? false : report.IEE70.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000405" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE70", Value = report.IEE70 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000405

            #region VR.ATO.GEN.000406

            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE53 <> BLANK) AND (StartsWithSet(^IEE53,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^IEE53, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^IEE53 = RP:FamilyTrust:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE53) != true && IsMatch(report.IEE53, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.IEE53, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE53", Value = report.IEE53 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406

            #region VR.ATO.GEN.000406

            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE5 <> BLANK) AND (StartsWithSet(^IEE5,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^IEE5, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^IEE5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE5) != true && IsMatch(report.IEE5, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.IEE5, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE5", Value = report.IEE5 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406

            #region VR.ATO.GEN.000406

            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE11 <> BLANK) AND (StartsWithSet(^IEE11,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^IEE11, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^IEE11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE11) != true && IsMatch(report.IEE11, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.IEE11, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE11", Value = report.IEE11 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406

            #region VR.ATO.GEN.000406

            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE70 <> BLANK) AND (StartsWithSet(^IEE70,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^IEE70, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^IEE70 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE70) != true && IsMatch(report.IEE70, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.IEE70, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE70", Value = report.IEE70 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000406

            #region VR.ATO.GEN.000407

            /*  VR.ATO.GEN.000407
            Address Line 2 cannot contain 'UNKNOWN'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE134, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE134 = RP:FamilyTrust:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.IEE134 == null ? false : report.IEE134.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000407" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE134", Value = GetValueOrEmpty(report.IEE134) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000407

            #region VR.ATO.GEN.000407

            /*  VR.ATO.GEN.000407
            Address Line 2 cannot contain 'UNKNOWN'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE122, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE122 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.IEE122 == null ? false : report.IEE122.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000407" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE122", Value = GetValueOrEmpty(report.IEE122) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000407

            #region VR.ATO.GEN.000407

            /*  VR.ATO.GEN.000407
            Address Line 2 cannot contain 'UNKNOWN'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE126, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE126 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.IEE126 == null ? false : report.IEE126.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000407" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE126", Value = GetValueOrEmpty(report.IEE126) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000407

            #region VR.ATO.GEN.000407

            /*  VR.ATO.GEN.000407
            Address Line 2 cannot contain 'UNKNOWN'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE138, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE138 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.IEE138 == null ? false : report.IEE138.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000407" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE138", Value = GetValueOrEmpty(report.IEE138) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000407

            #region VR.ATO.GEN.000408

            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE134 <> BLANK) AND (StartsWithSet(^IEE134,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^IEE134, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^IEE134 = RP:FamilyTrust:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE134) != true && (IsMatch(report.IEE134, @"^(C/- |C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.IEE134, @" C/- | C/O | C/ | Care Of ", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE134", Value = GetValueOrEmpty(report.IEE134) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408

            #region VR.ATO.GEN.000408

            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE122 <> BLANK) AND (StartsWithSet(^IEE122,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^IEE122, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^IEE122 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE122) != true && (IsMatch(report.IEE122, @"^(C/- |C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.IEE122, @" C/- | C/O | C/ | Care Of ", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE122", Value = GetValueOrEmpty(report.IEE122) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408

            #region VR.ATO.GEN.000408

            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE126 <> BLANK) AND (StartsWithSet(^IEE126,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^IEE126, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^IEE126 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE126) != true && (IsMatch(report.IEE126, @"^(C/- |C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.IEE126, @" C/- | C/O | C/ | Care Of ", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE126", Value = GetValueOrEmpty(report.IEE126) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408

            #region VR.ATO.GEN.000408

            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE138 <> BLANK) AND (StartsWithSet(^IEE138,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^IEE138, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^IEE138 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE138) != true && (IsMatch(report.IEE138, @"^(C/- |C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(report.IEE138, @" C/- | C/O | C/ | Care Of ", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE138", Value = GetValueOrEmpty(report.IEE138) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000408

            #region VR.ATO.GEN.000409

            /*  VR.ATO.GEN.000409
            Address must not contain "UNKNOWN".
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE54, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE54 = RP:FamilyTrust:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.IEE54 == null ? false : report.IEE54.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000409" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE54", Value = report.IEE54 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000409

            #region VR.ATO.GEN.000409

            /*  VR.ATO.GEN.000409
            Address must not contain "UNKNOWN".
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE6, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE6 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.IEE6 == null ? false : report.IEE6.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000409" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE6", Value = report.IEE6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000409

            #region VR.ATO.GEN.000409

            /*  VR.ATO.GEN.000409
            Address must not contain "UNKNOWN".
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE12, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE12 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.IEE12 == null ? false : report.IEE12.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000409" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE12", Value = report.IEE12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000409

            #region VR.ATO.GEN.000409

            /*  VR.ATO.GEN.000409
            Address must not contain "UNKNOWN".
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE71, 'UNKNOWN')
    
            Data Elements:
    
            ^IEE71 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.IEE71 == null ? false : report.IEE71.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000409" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE71", Value = report.IEE71 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000409

            #region VR.ATO.GEN.000410

            /*  VR.ATO.GEN.000410
            Address - Locality must not contain numerics.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE54, '0-9')
    
            Data Elements:
    
            ^IEE54 = RP:FamilyTrust:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE54, @"[0-9]", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000410" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE54", Value = report.IEE54 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000410

            #region VR.ATO.GEN.000410

            /*  VR.ATO.GEN.000410
            Address - Locality must not contain numerics.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE6, '0-9')
    
            Data Elements:
    
            ^IEE6 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE6, @"[0-9]", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000410" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE6", Value = report.IEE6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000410

            #region VR.ATO.GEN.000410

            /*  VR.ATO.GEN.000410
            Address - Locality must not contain numerics.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE12, '0-9')
    
            Data Elements:
    
            ^IEE12 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.IEE12, @"[0-9]", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000410" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE12", Value = report.IEE12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000410

            #region VR.ATO.GEN.000410

            /*  VR.ATO.GEN.000410
            Address - Locality must not contain numerics.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE71, '0-9')
    
            Data Elements:
    
            ^IEE71 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE71, @"[0-9]", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000410" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE71", Value = report.IEE71 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000410

            #region VR.ATO.GEN.000413

            /*  VR.ATO.GEN.000413
            Orqanisation Name cannot contain 'space hyphen space' .
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE52, ' - ')
    
            Data Elements:
    
            ^IEE52 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.IEE52 == null ? false : report.IEE52.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:OrganisationNameDetails" + OccurrenceIndex(report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000413" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE52", Value = report.IEE52 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000413

            #region VR.ATO.GEN.000414

            /*  VR.ATO.GEN.000414
            Organisation Name must not contain 'P/L'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE52, 'P/L')
    
            Data Elements:
    
            ^IEE52 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.IEE52 == null ? false : report.IEE52.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:OrganisationNameDetails" + OccurrenceIndex(report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000414" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE52", Value = report.IEE52 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000414

            #region VR.ATO.GEN.000416

            /*  VR.ATO.GEN.000416
            Organisation Name must not contain repeated hyphen, apostrophe, or space characters.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE52, '"--","''","  "')
    
            Data Elements:
    
            ^IEE52 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE52, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:OrganisationNameDetails" + OccurrenceIndex(report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000416" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE52", Value = report.IEE52 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000416

            #region VR.ATO.GEN.000417

            /*  VR.ATO.GEN.000417
            Non-individualname must not end with a leading space followed by 'T/A', 'T/A P'ship', 'T/A Pship', 'T/A P/Ship' or 'T/A Partnership'.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            EndsWithSet(^IEE52, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"')
    
            Data Elements:
    
            ^IEE52 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE52, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:OrganisationNameDetails" + OccurrenceIndex(report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000417" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE52", Value = report.IEE52 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000417

            #region VR.ATO.GEN.000423

            /*  VR.ATO.GEN.000423
            Family Name must not contain 'space hyphen space' .
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE66, ' - ')
    
            Data Elements:
    
            ^IEE66 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (report.IEE66 == null ? false : report.IEE66.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000423",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Name cannot contain ""space hyphen space""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000423" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE66", Value = report.IEE66 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000423

            #region VR.ATO.GEN.000423

            /*  VR.ATO.GEN.000423
            Family Name must not contain 'space hyphen space' .
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE84, ' - ')
    
            Data Elements:
    
            ^IEE84 = RP:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (report.IEE84 == null ? false : report.IEE84.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000423",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Name cannot contain ""space hyphen space""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000423" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE84", Value = report.IEE84 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000423

            #region VR.ATO.GEN.000427

            /*  VR.ATO.GEN.000427
            Individualname must not contain repeated hyphen, apostrophe, or space characters.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE66, '"--","''","  "')
    
            Data Elements:
    
            ^IEE66 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE66, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000427",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000427" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE66", Value = report.IEE66 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000427

            #region VR.ATO.GEN.000427

            /*  VR.ATO.GEN.000427
            Individualname must not contain repeated hyphen, apostrophe, or space characters.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE84, '"--","''","  "')
    
            Data Elements:
    
            ^IEE84 = RP:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE84, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000427",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000427" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE84", Value = report.IEE84 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000427

            #region VR.ATO.GEN.000434

            /*  VR.ATO.GEN.000434
            Given Name must not contain 'space hyphen space' .
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE68, ' - ')
    
            Data Elements:
    
            ^IEE68 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (report.IEE68 == null ? false : report.IEE68.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000434",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000434" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE68", Value = GetValueOrEmpty(report.IEE68) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000434

            #region VR.ATO.GEN.000434

            /*  VR.ATO.GEN.000434
            Given Name must not contain 'space hyphen space' .
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE86, ' - ')
    
            Data Elements:
    
            ^IEE86 = RP:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (report.IEE86 == null ? false : report.IEE86.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000434",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000434" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE86", Value = GetValueOrEmpty(report.IEE86) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000434

            #region VR.ATO.GEN.000439

            /*  VR.ATO.GEN.000439
            Given name must not contain repeated hyphen, apostrophe, or space characters.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE68, '"--","''","  "')
    
            Data Elements:
    
            ^IEE68 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE68, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000439",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000439" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE68", Value = GetValueOrEmpty(report.IEE68) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000439

            #region VR.ATO.GEN.000439

            /*  VR.ATO.GEN.000439
            Given name must not contain repeated hyphen, apostrophe, or space characters.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE86, '"--","''","  "')
    
            Data Elements:
    
            ^IEE86 = RP:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE86, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000439",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000439" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE86", Value = GetValueOrEmpty(report.IEE86) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000439

            #region VR.ATO.GEN.000446

            /*  VR.ATO.GEN.000446
            Other Given Name must not contain 'space hyphen space' .
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE69, ' - ')
    
            Data Elements:
    
            ^IEE69 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (report.IEE69 == null ? false : report.IEE69.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000446",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000446" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE69", Value = GetValueOrEmpty(report.IEE69) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000446

            #region VR.ATO.GEN.000446

            /*  VR.ATO.GEN.000446
            Other Given Name must not contain 'space hyphen space' .
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE87, ' - ')
    
            Data Elements:
    
            ^IEE87 = RP:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (report.IEE87 == null ? false : report.IEE87.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000446",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000446" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE87", Value = GetValueOrEmpty(report.IEE87) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000446

            #region VR.ATO.GEN.000451

            /*  VR.ATO.GEN.000451
            Individual name must not contain repeated hyphen, apostrophe, or space characters.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE69, '"--","''","  "')
    
            Data Elements:
    
            ^IEE69 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE69, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000451",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000451" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE69", Value = GetValueOrEmpty(report.IEE69) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000451

            #region VR.ATO.GEN.000451

            /*  VR.ATO.GEN.000451
            Individual name must not contain repeated hyphen, apostrophe, or space characters.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^IEE87, '"--","''","  "')
    
            Data Elements:
    
            ^IEE87 = RP:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE87, @"--|''|  ", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000451",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000451" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE87", Value = GetValueOrEmpty(report.IEE87) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000451

            #region VR.ATO.GEN.000458

            /*  VR.ATO.GEN.000458
            Suffix must equal a code in the Individual Suffix Code table
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE67 <> BLANK) AND (NotInDomain(^IEE67, 'SUFFIX CODES'))
    
            Data Elements:
    
            ^IEE67 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE67) != true && !(IsMatch(report.IEE67, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000458",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suffix is invalid",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.NameSuffix.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE67", Value = GetValueOrEmpty(report.IEE67) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000458

            #region VR.ATO.GEN.000458

            /*  VR.ATO.GEN.000458
            Suffix must equal a code in the Individual Suffix Code table
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE85 <> BLANK) AND (NotInDomain(^IEE85, 'SUFFIX CODES'))
    
            Data Elements:
    
            ^IEE85 = RP:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE85) != true && !(IsMatch(report.IEE85, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000458",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suffix is invalid",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.NameSuffix.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE85", Value = GetValueOrEmpty(report.IEE85) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000458

            #region VR.ATO.GEN.000459

            /*  VR.ATO.GEN.000459
            Title must equal a code in the Individual Title Code table
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE65 <> BLANK) AND (NotInDomain(^IEE65, 'TITLE CODES'))
    
            Data Elements:
    
            ^IEE65 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE65) != true && !(IsMatch(report.IEE65, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000459",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Title is invalid",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.Title.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE65", Value = GetValueOrEmpty(report.IEE65) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000459

            #region VR.ATO.GEN.000459

            /*  VR.ATO.GEN.000459
            Title must equal a code in the Individual Title Code table
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE83 <> BLANK) AND (NotInDomain(^IEE83, 'TITLE CODES'))
    
            Data Elements:
    
            ^IEE83 = RP:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE83) != true && !(IsMatch(report.IEE83, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000459",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Title is invalid",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.Title.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE83", Value = GetValueOrEmpty(report.IEE83) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000459

            #region VR.ATO.GEN.000460

            /*  VR.ATO.GEN.000460
            Postcode must be a valid code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0200-0299)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0800-7499)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(7800-9799))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE56 <> BLANK AND (IsNumeric(^IEE56) = 'FALSE' OR ((AsNumeric(^IEE56) < 200 OR AsNumeric(^IEE56) > 299) AND (AsNumeric(^IEE56) < 800 OR AsNumeric(^IEE56) > 7499) AND (AsNumeric(^IEE56) < 7800 OR AsNumeric(^IEE56) > 9799)) )
    
            Data Elements:
    
            ^IEE56 = RP:FamilyTrust:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE56) != true && (IsNumeric(report.IEE56) == false || (AsNumeric(report.IEE56) < 200 || AsNumeric(report.IEE56) > 299) && (AsNumeric(report.IEE56) < 800 || AsNumeric(report.IEE56) > 7499) && (AsNumeric(report.IEE56) < 7800 || AsNumeric(report.IEE56) > 9799)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000460",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode is invalid",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000460" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE56", Value = GetValueOrEmpty(report.IEE56) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000460

            #region VR.ATO.GEN.000460

            /*  VR.ATO.GEN.000460
            Postcode must be a valid code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0200-0299)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0800-7499)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(7800-9799))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE8 <> BLANK AND (IsNumeric(^IEE8) = 'FALSE' OR ((AsNumeric(^IEE8) < 200 OR AsNumeric(^IEE8) > 299) AND (AsNumeric(^IEE8) < 800 OR AsNumeric(^IEE8) > 7499) AND (AsNumeric(^IEE8) < 7800 OR AsNumeric(^IEE8) > 9799)) )
    
            Data Elements:
    
            ^IEE8 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE8) != true && (IsNumeric(report.IEE8) == false || (AsNumeric(report.IEE8) < 200 || AsNumeric(report.IEE8) > 299) && (AsNumeric(report.IEE8) < 800 || AsNumeric(report.IEE8) > 7499) && (AsNumeric(report.IEE8) < 7800 || AsNumeric(report.IEE8) > 9799)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000460",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode is invalid",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000460" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE8", Value = GetValueOrEmpty(report.IEE8) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000460

            #region VR.ATO.GEN.000460

            /*  VR.ATO.GEN.000460
            Postcode must be a valid code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0200-0299)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0800-7499)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(7800-9799))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE14 <> BLANK AND (IsNumeric(^IEE14) = 'FALSE' OR ((AsNumeric(^IEE14) < 200 OR AsNumeric(^IEE14) > 299) AND (AsNumeric(^IEE14) < 800 OR AsNumeric(^IEE14) > 7499) AND (AsNumeric(^IEE14) < 7800 OR AsNumeric(^IEE14) > 9799)) )
    
            Data Elements:
    
            ^IEE14 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE14) != true && (IsNumeric(report.IEE14) == false || (AsNumeric(report.IEE14) < 200 || AsNumeric(report.IEE14) > 299) && (AsNumeric(report.IEE14) < 800 || AsNumeric(report.IEE14) > 7499) && (AsNumeric(report.IEE14) < 7800 || AsNumeric(report.IEE14) > 9799)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000460",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode is invalid",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000460" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE14", Value = GetValueOrEmpty(report.IEE14) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000460

            #region VR.ATO.GEN.000460

            /*  VR.ATO.GEN.000460
            Postcode must be a valid code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0200-0299)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0800-7499)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(7800-9799))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE73 <> BLANK AND (IsNumeric(^IEE73) = 'FALSE' OR ((AsNumeric(^IEE73) < 200 OR AsNumeric(^IEE73) > 299) AND (AsNumeric(^IEE73) < 800 OR AsNumeric(^IEE73) > 7499) AND (AsNumeric(^IEE73) < 7800 OR AsNumeric(^IEE73) > 9799)) )
    
            Data Elements:
    
            ^IEE73 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE73) != true && (IsNumeric(report.IEE73) == false || (AsNumeric(report.IEE73) < 200 || AsNumeric(report.IEE73) > 299) && (AsNumeric(report.IEE73) < 800 || AsNumeric(report.IEE73) > 7499) && (AsNumeric(report.IEE73) < 7800 || AsNumeric(report.IEE73) > 9799)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000460",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode is invalid",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000460" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE73", Value = GetValueOrEmpty(report.IEE73) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.000460

            #region VR.ATO.GEN.410002

            /*  VR.ATO.GEN.410002
            If Address Line 1 contains C/-,  Address line 2 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^IEE53, 'C/-')) AND (^IEE134 = BLANK))
    
            Data Elements:
    
            ^IEE53 = RP:FamilyTrust:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE134 = RP:FamilyTrust:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.IEE53 == null ? false : report.IEE53.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.IEE134) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE53", Value = report.IEE53 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE134", Value = GetValueOrEmpty(report.IEE134) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002

            #region VR.ATO.GEN.410002

            /*  VR.ATO.GEN.410002
            If Address Line 1 contains C/-,  Address line 2 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^IEE5, 'C/-')) AND (^IEE122 = BLANK))
    
            Data Elements:
    
            ^IEE5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE122 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.IEE5 == null ? false : report.IEE5.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.IEE122) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE5", Value = report.IEE5 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE122", Value = GetValueOrEmpty(report.IEE122) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002

            #region VR.ATO.GEN.410002

            /*  VR.ATO.GEN.410002
            If Address Line 1 contains C/-,  Address line 2 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^IEE11, 'C/-')) AND (^IEE126 = BLANK))
    
            Data Elements:
    
            ^IEE11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE126 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = ((report.IEE11 == null ? false : report.IEE11.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.IEE126) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE11", Value = report.IEE11 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE126", Value = GetValueOrEmpty(report.IEE126) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002

            #region VR.ATO.GEN.410002

            /*  VR.ATO.GEN.410002
            If Address Line 1 contains C/-,  Address line 2 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^IEE70, 'C/-')) AND (^IEE138 = BLANK))
    
            Data Elements:
    
            ^IEE70 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE138 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.IEE70 == null ? false : report.IEE70.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.IEE138) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE70", Value = report.IEE70 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE138", Value = GetValueOrEmpty(report.IEE138) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410002

            #region VR.ATO.GEN.410003

            /*  VR.ATO.GEN.410003
            If State or Postcode are present, Suburb/town must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) OR (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK)) AND (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((^IEE55 <> BLANK) OR (^IEE56 <> BLANK)) AND (^IEE54 = BLANK)
    
            Data Elements:
    
            ^IEE54 = RP:FamilyTrust:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE55 = RP:FamilyTrust:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE56 = RP:FamilyTrust:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((string.IsNullOrWhiteSpace(report.IEE55) != true || string.IsNullOrWhiteSpace(report.IEE56) != true) && string.IsNullOrWhiteSpace(report.IEE54) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If State or Postcode are present, Suburb/town must be present",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410003" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE55", Value = GetValueOrEmpty(report.IEE55) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE56", Value = GetValueOrEmpty(report.IEE56) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE54", Value = report.IEE54 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410003

            #region VR.ATO.GEN.410003

            /*  VR.ATO.GEN.410003
            If State or Postcode are present, Suburb/town must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) OR (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK)) AND (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((^IEE7 <> BLANK) OR (^IEE8 <> BLANK)) AND (^IEE6 = BLANK)
    
            Data Elements:
    
            ^IEE6 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE7 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE8 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((string.IsNullOrWhiteSpace(report.IEE7) != true || string.IsNullOrWhiteSpace(report.IEE8) != true) && string.IsNullOrWhiteSpace(report.IEE6) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If State or Postcode are present, Suburb/town must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410003" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE7", Value = GetValueOrEmpty(report.IEE7) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE8", Value = GetValueOrEmpty(report.IEE8) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE6", Value = report.IEE6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410003

            #region VR.ATO.GEN.410003

            /*  VR.ATO.GEN.410003
            If State or Postcode are present, Suburb/town must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) OR (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK)) AND (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((^IEE13 <> BLANK) OR (^IEE14 <> BLANK)) AND (^IEE12 = BLANK)
    
            Data Elements:
    
            ^IEE12 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE13 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE14 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = ((string.IsNullOrWhiteSpace(report.IEE13) != true || string.IsNullOrWhiteSpace(report.IEE14) != true) && string.IsNullOrWhiteSpace(report.IEE12) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If State or Postcode are present, Suburb/town must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410003" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE13", Value = GetValueOrEmpty(report.IEE13) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE14", Value = GetValueOrEmpty(report.IEE14) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE12", Value = report.IEE12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410003

            #region VR.ATO.GEN.410003

            /*  VR.ATO.GEN.410003
            If State or Postcode are present, Suburb/town must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) OR (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK)) AND (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((^IEE72 <> BLANK) OR (^IEE73 <> BLANK)) AND (^IEE71 = BLANK)
    
            Data Elements:
    
            ^IEE71 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE72 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE73 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((string.IsNullOrWhiteSpace(report.IEE72) != true || string.IsNullOrWhiteSpace(report.IEE73) != true) && string.IsNullOrWhiteSpace(report.IEE71) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If State or Postcode are present, Suburb/town must be present",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410003" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE72", Value = GetValueOrEmpty(report.IEE72) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE73", Value = GetValueOrEmpty(report.IEE73) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE71", Value = report.IEE71 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410003

            #region VR.ATO.GEN.410008

            /*  VR.ATO.GEN.410008
            Address Line 3 is not required
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE135 <> BLANK
    
            Data Elements:
    
            ^IEE135 = RP:FamilyTrust:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE135) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE135", Value = GetValueOrEmpty(report.IEE135) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008

            #region VR.ATO.GEN.410008

            /*  VR.ATO.GEN.410008
            Address Line 3 is not required
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE123 <> BLANK
    
            Data Elements:
    
            ^IEE123 = RP:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE123) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE123", Value = GetValueOrEmpty(report.IEE123) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008

            #region VR.ATO.GEN.410008

            /*  VR.ATO.GEN.410008
            Address Line 3 is not required
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE127 <> BLANK
    
            Data Elements:
    
            ^IEE127 = RP:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE127) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE127", Value = GetValueOrEmpty(report.IEE127) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008

            #region VR.ATO.GEN.410008

            /*  VR.ATO.GEN.410008
            Address Line 3 is not required
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE139 <> BLANK
    
            Data Elements:
    
            ^IEE139 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE139) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE139", Value = GetValueOrEmpty(report.IEE139) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410008

            #region VR.ATO.GEN.410009

            /*  VR.ATO.GEN.410009

        Legacy Rule Format:
            IF ((pyid.xx.xx:Identifiers.TaxAgentNumber.Identifier) <> NULLORBLANK) AND (TANALGORITHM (Tax Agent number) = FALSE)
            RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE2 <> BLANK) AND FailsTANAlgorithm(^IEE2)
    
            Data Elements:
    
            ^IEE2 = INT:Identifiers.TaxAgentNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE2) != true && FailsTANAlgorithm(report.IEE2));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Tax Agent number has failed the Tax Agent number algorithm check",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxAgentNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410009" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE2", Value = GetValueOrEmpty(report.IEE2) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410009

            #region VR.ATO.GEN.410013

            /*  VR.ATO.GEN.410013
            Address Line 4 is not required
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE136 <> BLANK
    
            Data Elements:
    
            ^IEE136 = RP:FamilyTrust:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE136) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE136", Value = GetValueOrEmpty(report.IEE136) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013

            #region VR.ATO.GEN.410013

            /*  VR.ATO.GEN.410013
            Address Line 4 is not required
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE124 <> BLANK
    
            Data Elements:
    
            ^IEE124 = RP:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE124) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE124", Value = GetValueOrEmpty(report.IEE124) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013

            #region VR.ATO.GEN.410013

            /*  VR.ATO.GEN.410013
            Address Line 4 is not required
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE128 <> BLANK
    
            Data Elements:
    
            ^IEE128 = RP:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE128) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE128", Value = GetValueOrEmpty(report.IEE128) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013

            #region VR.ATO.GEN.410013

            /*  VR.ATO.GEN.410013
            Address Line 4 is not required
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE140 <> BLANK
    
            Data Elements:
    
            ^IEE140 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE140) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE140", Value = GetValueOrEmpty(report.IEE140) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410013

            #region VR.ATO.GEN.410017

            /*  VR.ATO.GEN.410017
            If Postcode is present, State must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE56 <> BLANK) AND (^IEE55 = BLANK)
    
            Data Elements:
    
            ^IEE55 = RP:FamilyTrust:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE56 = RP:FamilyTrust:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE56) != true && string.IsNullOrWhiteSpace(report.IEE55) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"State must be present if Postcode is present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410017" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE56", Value = GetValueOrEmpty(report.IEE56) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE55", Value = GetValueOrEmpty(report.IEE55) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410017

            #region VR.ATO.GEN.410017

            /*  VR.ATO.GEN.410017
            If Postcode is present, State must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE8 <> BLANK) AND (^IEE7 = BLANK)
    
            Data Elements:
    
            ^IEE7 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE8 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE8) != true && string.IsNullOrWhiteSpace(report.IEE7) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"State must be present if Postcode is present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410017" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE8", Value = GetValueOrEmpty(report.IEE8) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE7", Value = GetValueOrEmpty(report.IEE7) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410017

            #region VR.ATO.GEN.410017

            /*  VR.ATO.GEN.410017
            If Postcode is present, State must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE14 <> BLANK) AND (^IEE13 = BLANK)
    
            Data Elements:
    
            ^IEE13 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE14 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE14) != true && string.IsNullOrWhiteSpace(report.IEE13) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"State must be present if Postcode is present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410017" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE14", Value = GetValueOrEmpty(report.IEE14) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE13", Value = GetValueOrEmpty(report.IEE13) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410017

            #region VR.ATO.GEN.410017

            /*  VR.ATO.GEN.410017
            If Postcode is present, State must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE73 <> BLANK) AND (^IEE72 = BLANK)
    
            Data Elements:
    
            ^IEE72 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE73 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE73) != true && string.IsNullOrWhiteSpace(report.IEE72) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"State must be present if Postcode is present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410017" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE73", Value = GetValueOrEmpty(report.IEE73) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE72", Value = GetValueOrEmpty(report.IEE72) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410017

            #region VR.ATO.GEN.410018

            /*  VR.ATO.GEN.410018
            State and postcode or country must be entered.
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^IEE55 = BLANK) AND (^IEE56 = BLANK) AND (^IEE155 = BLANK))
    
            Data Elements:
    
            ^IEE55 = RP:FamilyTrust:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE56 = RP:FamilyTrust:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE155 = RP:FamilyTrust:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:FamilyTrust:AddressDetails
            */
            assertion = (report.FamilyTrust_AddressDetailsCollection_POS_C_RPExists != false && (string.IsNullOrWhiteSpace(report.IEE55) == true && string.IsNullOrWhiteSpace(report.IEE56) == true && string.IsNullOrWhiteSpace(report.IEE155) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410018",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Either state and postcode, or country must be present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410018" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE55", Value = GetValueOrEmpty(report.IEE55) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE56", Value = GetValueOrEmpty(report.IEE56) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE155", Value = GetValueOrEmpty(report.IEE155) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410018

            #region VR.ATO.GEN.410018

            /*  VR.ATO.GEN.410018
            State and postcode or country must be entered.
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^IEE7 = BLANK) AND (^IEE8 = BLANK) AND (^IEE111 = BLANK))
    
            Data Elements:
    
            ^IEE7 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE8 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE111 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && (string.IsNullOrWhiteSpace(report.IEE7) == true && string.IsNullOrWhiteSpace(report.IEE8) == true && string.IsNullOrWhiteSpace(report.IEE111) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410018",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Either state and postcode, or country must be present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410018" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE7", Value = GetValueOrEmpty(report.IEE7) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE8", Value = GetValueOrEmpty(report.IEE8) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE111", Value = GetValueOrEmpty(report.IEE111) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410018

            #region VR.ATO.GEN.410018

            /*  VR.ATO.GEN.410018
            State and postcode or country must be entered.
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^IEE13 = BLANK) AND (^IEE14 = BLANK) AND (^IEE112 = BLANK))
    
            Data Elements:
    
            ^IEE13 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE14 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE112 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && (string.IsNullOrWhiteSpace(report.IEE13) == true && string.IsNullOrWhiteSpace(report.IEE14) == true && string.IsNullOrWhiteSpace(report.IEE112) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410018",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Either state and postcode, or country must be present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410018" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE13", Value = GetValueOrEmpty(report.IEE13) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE14", Value = GetValueOrEmpty(report.IEE14) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE112", Value = GetValueOrEmpty(report.IEE112) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410018

            #region VR.ATO.GEN.410018

            /*  VR.ATO.GEN.410018
            State and postcode or country must be entered.
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND ((^IEE72 = BLANK) AND (^IEE73 = BLANK) AND (^IEE114 = BLANK))
    
            Data Elements:
    
            ^IEE72 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE73 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE114 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:SpecifiedIndividual:AddressDetails
            */
            assertion = (report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPExists != false && (string.IsNullOrWhiteSpace(report.IEE72) == true && string.IsNullOrWhiteSpace(report.IEE73) == true && string.IsNullOrWhiteSpace(report.IEE114) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410018",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Either state and postcode, or country must be present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410018" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE72", Value = GetValueOrEmpty(report.IEE72) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE73", Value = GetValueOrEmpty(report.IEE73) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE114", Value = GetValueOrEmpty(report.IEE114) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410018

            #region VR.ATO.GEN.410031

            /*  VR.ATO.GEN.410031

        Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier <> NULLORBLANK) AND (TFNALGORITHM (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier) = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE50 <> BLANK) AND (FailsTFNAlgorithm(^IEE50))
    
            Data Elements:
    
            ^IEE50 = RP:FamilyTrust:Identifiers.TaxFileNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE50) != true && FailsTFNAlgorithm(report.IEE50));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410031",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN must be a valid TFN",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410031" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE50", Value = GetValueOrEmpty(report.IEE50) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410031

            #region VR.ATO.GEN.410031

            /*  VR.ATO.GEN.410031

        Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier <> NULLORBLANK) AND (TFNALGORITHM (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier) = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE62 <> BLANK) AND (FailsTFNAlgorithm(^IEE62))
    
            Data Elements:
    
            ^IEE62 = RP:SpecifiedIndividual:Identifiers.TaxFileNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE62) != true && FailsTFNAlgorithm(report.IEE62));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410031",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN must be a valid TFN",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410031" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE62", Value = GetValueOrEmpty(report.IEE62) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410031

            #region VR.ATO.GEN.410038

            /*  VR.ATO.GEN.410038
            Non-individual Name cannot start with 'T/A' followed by a space and end with a space followed by 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^IEE52, 'T/A ')) AND (EndsWithSet(^IEE52, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^IEE52 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (IsMatch(report.IEE52, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(report.IEE52, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:OrganisationNameDetails" + OccurrenceIndex(report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE52", Value = report.IEE52 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410038

            #region VR.ATO.GEN.410039

            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE52  <> BLANK) AND (NotContainsSet(^IEE52, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^IEE52 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE52) != true && !(IsMatch(report.IEE52, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:OrganisationNameDetails" + OccurrenceIndex(report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE52", Value = report.IEE52 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410039

            #region VR.ATO.GEN.410053

            /*  VR.ATO.GEN.410053
            Postcode, State and Country must not all be blank.
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE53 <> BLANK) AND ((^IEE55 = BLANK) AND (^IEE56 = BLANK) AND (^IEE155 = BLANK))
    
            Data Elements:
    
            ^IEE53 = RP:FamilyTrust:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE55 = RP:FamilyTrust:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE56 = RP:FamilyTrust:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE155 = RP:FamilyTrust:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE53) != true && (string.IsNullOrWhiteSpace(report.IEE55) == true && string.IsNullOrWhiteSpace(report.IEE56) == true && string.IsNullOrWhiteSpace(report.IEE155) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode and State, or Country code must be present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country code must be present",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410053" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE53", Value = report.IEE53 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE55", Value = GetValueOrEmpty(report.IEE55) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE56", Value = GetValueOrEmpty(report.IEE56) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE155", Value = GetValueOrEmpty(report.IEE155) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410053

            #region VR.ATO.GEN.410053

            /*  VR.ATO.GEN.410053
            Postcode, State and Country must not all be blank.
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE5 <> BLANK) AND ((^IEE7 = BLANK) AND (^IEE8 = BLANK) AND (^IEE111 = BLANK))
    
            Data Elements:
    
            ^IEE5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE7 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE8 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE111 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE5) != true && (string.IsNullOrWhiteSpace(report.IEE7) == true && string.IsNullOrWhiteSpace(report.IEE8) == true && string.IsNullOrWhiteSpace(report.IEE111) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode and State, or Country code must be present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country code must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410053" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE5", Value = report.IEE5 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE7", Value = GetValueOrEmpty(report.IEE7) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE8", Value = GetValueOrEmpty(report.IEE8) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE111", Value = GetValueOrEmpty(report.IEE111) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410053

            #region VR.ATO.GEN.410053

            /*  VR.ATO.GEN.410053
            Postcode, State and Country must not all be blank.
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE11 <> BLANK) AND ((^IEE13 = BLANK) AND (^IEE14 = BLANK) AND (^IEE112 = BLANK))
    
            Data Elements:
    
            ^IEE11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE13 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE14 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE112 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE11) != true && (string.IsNullOrWhiteSpace(report.IEE13) == true && string.IsNullOrWhiteSpace(report.IEE14) == true && string.IsNullOrWhiteSpace(report.IEE112) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode and State, or Country code must be present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country code must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410053" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE11", Value = report.IEE11 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE13", Value = GetValueOrEmpty(report.IEE13) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE14", Value = GetValueOrEmpty(report.IEE14) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE112", Value = GetValueOrEmpty(report.IEE112) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410053

            #region VR.ATO.GEN.410053

            /*  VR.ATO.GEN.410053
            Postcode, State and Country must not all be blank.
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE70 <> BLANK) AND ((^IEE72 = BLANK) AND (^IEE73 = BLANK) AND (^IEE114 = BLANK))
    
            Data Elements:
    
            ^IEE70 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE72 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE73 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE114 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE70) != true && (string.IsNullOrWhiteSpace(report.IEE72) == true && string.IsNullOrWhiteSpace(report.IEE73) == true && string.IsNullOrWhiteSpace(report.IEE114) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410053",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode and State, or Country code must be present",
                    LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country code must be present",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410053" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE70", Value = report.IEE70 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE72", Value = GetValueOrEmpty(report.IEE72) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE73", Value = GetValueOrEmpty(report.IEE73) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE114", Value = GetValueOrEmpty(report.IEE114) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410053

            #region VR.ATO.GEN.410055

            /*  VR.ATO.GEN.410055
            If State is present, Postcode must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE55 <> BLANK) AND (^IEE56 = BLANK)
    
            Data Elements:
    
            ^IEE56 = RP:FamilyTrust:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE55 = RP:FamilyTrust:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE55) != true && string.IsNullOrWhiteSpace(report.IEE56) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410055",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode must be present if State is present",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410055" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE55", Value = GetValueOrEmpty(report.IEE55) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE56", Value = GetValueOrEmpty(report.IEE56) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410055

            #region VR.ATO.GEN.410055

            /*  VR.ATO.GEN.410055
            If State is present, Postcode must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE7 <> BLANK) AND (^IEE8 = BLANK)
    
            Data Elements:
    
            ^IEE8 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE7 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE7) != true && string.IsNullOrWhiteSpace(report.IEE8) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410055",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode must be present if State is present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410055" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE7", Value = GetValueOrEmpty(report.IEE7) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE8", Value = GetValueOrEmpty(report.IEE8) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410055

            #region VR.ATO.GEN.410055

            /*  VR.ATO.GEN.410055
            If State is present, Postcode must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE13 <> BLANK) AND (^IEE14 = BLANK)
    
            Data Elements:
    
            ^IEE14 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^IEE13 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE13) != true && string.IsNullOrWhiteSpace(report.IEE14) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410055",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode must be present if State is present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410055" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE13", Value = GetValueOrEmpty(report.IEE13) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE14", Value = GetValueOrEmpty(report.IEE14) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410055

            #region VR.ATO.GEN.410055

            /*  VR.ATO.GEN.410055
            If State is present, Postcode must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE72 <> BLANK) AND (^IEE73 = BLANK)
    
            Data Elements:
    
            ^IEE73 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE72 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE72) != true && string.IsNullOrWhiteSpace(report.IEE73) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410055",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Postcode must be present if State is present",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410055" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE72", Value = GetValueOrEmpty(report.IEE72) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE73", Value = GetValueOrEmpty(report.IEE73) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410055

            #region VR.ATO.GEN.410105

            /*  VR.ATO.GEN.410105

        Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianRegisteredBodyNumber.Identifier <> NULLORBLANK) AND (ARBNALGORITHM (pyid.xx.xx:Identifiers.AustralianRegisteredBodyNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE101 <> BLANK) AND (FailsARBNAlgorithm(^IEE101))
    
            Data Elements:
    
            ^IEE101 = RP:Identifiers.AustralianRegisteredBodyNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE101) != true && FailsACNAlgorithm(report.IEE101));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410105",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"ARBN must be a valid number that passes the algorithm",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianRegisteredBodyNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410105" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE101", Value = GetValueOrEmpty(report.IEE101) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410105

            #region VR.ATO.GEN.410131

            /*  VR.ATO.GEN.410131
            Individual name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE69 <> BLANK) AND (NotContainsSet(^IEE69, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^IEE69 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE69) != true && !(IsMatch(report.IEE69, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410131",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other Given Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE69", Value = GetValueOrEmpty(report.IEE69) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410131

            #region VR.ATO.GEN.410131

            /*  VR.ATO.GEN.410131
            Individual name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE87 <> BLANK) AND (NotContainsSet(^IEE87, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^IEE87 = RP:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE87) != true && !(IsMatch(report.IEE87, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410131",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other Given Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE87", Value = GetValueOrEmpty(report.IEE87) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410131

            #region VR.ATO.GEN.410194

            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE53) > 38
    
            Data Elements:
    
            ^IEE53 = RP:FamilyTrust:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.IEE53) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE53", Value = report.IEE53 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194

            #region VR.ATO.GEN.410194

            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE5) > 38
    
            Data Elements:
    
            ^IEE5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.IEE5) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE5", Value = report.IEE5 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194

            #region VR.ATO.GEN.410194

            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE11) > 38
    
            Data Elements:
    
            ^IEE11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (Length(report.IEE11) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE11", Value = report.IEE11 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194

            #region VR.ATO.GEN.410194

            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE70) > 38
    
            Data Elements:
    
            ^IEE70 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.IEE70) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE70", Value = report.IEE70 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410194

            #region VR.ATO.GEN.410195

            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE134) > 38
    
            Data Elements:
    
            ^IEE134 = RP:FamilyTrust:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.IEE134) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE134", Value = GetValueOrEmpty(report.IEE134) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195

            #region VR.ATO.GEN.410195

            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE122) > 38
    
            Data Elements:
    
            ^IEE122 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.IEE122) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE122", Value = GetValueOrEmpty(report.IEE122) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195

            #region VR.ATO.GEN.410195

            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE126) > 38
    
            Data Elements:
    
            ^IEE126 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (Length(report.IEE126) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE126", Value = GetValueOrEmpty(report.IEE126) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195

            #region VR.ATO.GEN.410195

            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE138) > 38
    
            Data Elements:
    
            ^IEE138 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.IEE138) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE138", Value = GetValueOrEmpty(report.IEE138) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410195

            #region VR.ATO.GEN.410200

            /*  VR.ATO.GEN.410200
            Individualname must not contain 'Exec for', 'Rep for' or 'Trustee for'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE66, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^IEE66 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE66, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000424",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE66", Value = report.IEE66 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410200

            #region VR.ATO.GEN.410200

            /*  VR.ATO.GEN.410200
            Individualname must not contain 'Exec for', 'Rep for' or 'Trustee for'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE84, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^IEE84 = RP:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE84, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000424",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE84", Value = report.IEE84 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410200

            #region VR.ATO.GEN.410201

            /*  VR.ATO.GEN.410201
            IndividualNamefield must not contain MR, MRS, MISS, MS with space before and after.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE66, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^IEE66 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE66, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000426",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE66", Value = report.IEE66 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410201

            #region VR.ATO.GEN.410201

            /*  VR.ATO.GEN.410201
            IndividualNamefield must not contain MR, MRS, MISS, MS with space before and after.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE84, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^IEE84 = RP:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE84, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000426",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE84", Value = report.IEE84 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410201

            #region VR.ATO.GEN.410203

            /*  VR.ATO.GEN.410203
            Given Name must not contain trustee reference
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE68, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^IEE68 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE68, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000437",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE68", Value = GetValueOrEmpty(report.IEE68) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410203

            #region VR.ATO.GEN.410203

            /*  VR.ATO.GEN.410203
            Given Name must not contain trustee reference
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE86, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^IEE86 = RP:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE86, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000437",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE86", Value = GetValueOrEmpty(report.IEE86) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410203

            #region VR.ATO.GEN.410204

            /*  VR.ATO.GEN.410204
            Given Name must not contain a title
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE68, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^IEE68 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE68, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000438",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE68", Value = GetValueOrEmpty(report.IEE68) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410204

            #region VR.ATO.GEN.410204

            /*  VR.ATO.GEN.410204
            Given Name must not contain a title
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE86, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^IEE86 = RP:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE86, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000438",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE86", Value = GetValueOrEmpty(report.IEE86) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410204

            #region VR.ATO.GEN.410205

            /*  VR.ATO.GEN.410205
            Address Line 1 cannot contain 'as above'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE53, '"AS ABOVE"'))
    
            Data Elements:
    
            ^IEE53 = RP:FamilyTrust:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE53, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE53", Value = report.IEE53 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205

            #region VR.ATO.GEN.410205

            /*  VR.ATO.GEN.410205
            Address Line 1 cannot contain 'as above'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE5, '"AS ABOVE"'))
    
            Data Elements:
    
            ^IEE5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE5, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE5", Value = report.IEE5 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205

            #region VR.ATO.GEN.410205

            /*  VR.ATO.GEN.410205
            Address Line 1 cannot contain 'as above'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE11, '"AS ABOVE"'))
    
            Data Elements:
    
            ^IEE11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.IEE11, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE11", Value = report.IEE11 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205

            #region VR.ATO.GEN.410205

            /*  VR.ATO.GEN.410205
            Address Line 1 cannot contain 'as above'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE70, '"AS ABOVE"'))
    
            Data Elements:
    
            ^IEE70 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE70, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE70", Value = report.IEE70 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410205

            #region VR.ATO.GEN.410208

            /*  VR.ATO.GEN.410208
            Individualname must not contain 'Exec for', 'Rep for' or 'Trustee for'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE69, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^IEE69 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE69, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000449",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE69", Value = GetValueOrEmpty(report.IEE69) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410208

            #region VR.ATO.GEN.410208

            /*  VR.ATO.GEN.410208
            Individualname must not contain 'Exec for', 'Rep for' or 'Trustee for'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE87, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^IEE87 = RP:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE87, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000449",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE87", Value = GetValueOrEmpty(report.IEE87) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410208

            #region VR.ATO.GEN.410209

            /*  VR.ATO.GEN.410209
            Other Given Name must not contain a title
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE69, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^IEE69 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE69, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000450",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE69", Value = GetValueOrEmpty(report.IEE69) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410209

            #region VR.ATO.GEN.410209

            /*  VR.ATO.GEN.410209
            Other Given Name must not contain a title
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE87, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^IEE87 = RP:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE87, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000450",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE87", Value = GetValueOrEmpty(report.IEE87) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410209

            #region VR.ATO.GEN.410213

            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE57 <> BLANK
    
            Data Elements:
    
            ^IEE57 = RP:FamilyTrust:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE57) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE57", Value = GetValueOrEmpty(report.IEE57) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213

            #region VR.ATO.GEN.410213

            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE9 <> BLANK
    
            Data Elements:
    
            ^IEE9 = RP:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE9) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE9", Value = GetValueOrEmpty(report.IEE9) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213

            #region VR.ATO.GEN.410213

            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE15 <> BLANK
    
            Data Elements:
    
            ^IEE15 = RP:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE15) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE15", Value = GetValueOrEmpty(report.IEE15) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213

            #region VR.ATO.GEN.410213

            /*  VR.ATO.GEN.410213
            Country Name must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE74 <> BLANK
    
            Data Elements:
    
            ^IEE74 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE74) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410213",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Country Name must be left blank",
                    LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE74", Value = GetValueOrEmpty(report.IEE74) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410213

            #region VR.ATO.GEN.410218

            /*  VR.ATO.GEN.410218
            Firstname must be entered if OtherGivenNames is entered
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                RETURN VALIDATION MESSAGE 
             ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND ((^IEE69 <> BLANK) AND (^IEE68 = BLANK))
    
            Data Elements:
    
            ^IEE69 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
    
            ^IEE68 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
    
            ^PersonNameDetails = RP:SpecifiedIndividual:PersonNameDetails
            */
            assertion = (report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPExists != false && (string.IsNullOrWhiteSpace(report.IEE69) != true && string.IsNullOrWhiteSpace(report.IEE68) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410218",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"FirstName must be entered if OtherGivenNames is entered",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE69", Value = GetValueOrEmpty(report.IEE69) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE68", Value = GetValueOrEmpty(report.IEE68) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410218

            #region VR.ATO.GEN.410218

            /*  VR.ATO.GEN.410218
            Firstname must be entered if OtherGivenNames is entered
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                RETURN VALIDATION MESSAGE 
             ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND ((^IEE87 <> BLANK) AND (^IEE86 = BLANK))
    
            Data Elements:
    
            ^IEE87 = RP:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
    
            ^IEE86 = RP:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
    
            ^PersonNameDetails = RP:PersonNameDetails
            */
            assertion = (report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists != false && (string.IsNullOrWhiteSpace(report.IEE87) != true && string.IsNullOrWhiteSpace(report.IEE86) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410218",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"FirstName must be entered if OtherGivenNames is entered",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE87", Value = GetValueOrEmpty(report.IEE87) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE86", Value = GetValueOrEmpty(report.IEE86) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.410218

            #region VR.ATO.GEN.428016

            /*  VR.ATO.GEN.428016

        Legacy Rule Format:
            IF (RP:entity.identifier.TFN <> NULLORBLANK) AND (TFNALGORITHM (RP:entity.identifier.TFN) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TFN <> BLANK) AND (FailsTFNAlgorithm(^TFN))
    
            Data Elements:
    
            ^TFN = RP
            */
            assertion = (string.IsNullOrWhiteSpace(report.RPIdentifierTFN) != true && FailsTFNAlgorithm(report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428016",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax File Number has failed the algorithm check",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428016" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428016

            #region VR.ATO.GEN.428021

            /*  VR.ATO.GEN.428021

        Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianCompanyNumber.Identifier <> NULLORBLANK) AND (ACNALGORITHM(pyid.xx.xx:Identifiers.AustralianCompanyNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE100 <> BLANK) AND (FailsACNAlgorithm(^IEE100))
    
            Data Elements:
    
            ^IEE100 = RP:Identifiers.AustralianCompanyNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE100) != true && FailsACNAlgorithm(report.IEE100));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428021",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid ACN",
                    LongDescription = @"The Australian Company Number (ACN) has failed the algorithm check",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianCompanyNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428021" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE100", Value = GetValueOrEmpty(report.IEE100) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428021

            #region VR.ATO.GEN.428040

            /*  VR.ATO.GEN.428040

        Legacy Rule Format:
            IF (pyid.xx.xx:IdentifiersAustralianBusinessNumberIdentifier <> NULLORBLANK) and (ABNALGORITHM(pyid.xx.xx:IdentifiersAustralianBusinessNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE162 <> BLANK) and (FailsABNAlgorithm(^IEE162))
    
            Data Elements:
    
            ^IEE162 = RP:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE162) != true && FailsABNAlgorithm(report.IEE162));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000477",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"ABN is invalid",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428040" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE162", Value = GetValueOrEmpty(report.IEE162) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428040

            #region VR.ATO.GEN.428230

            /*  VR.ATO.GEN.428230
            Address - Locality must not contain a state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE54, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^IEE54 = RP:FamilyTrust:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE54, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE54", Value = report.IEE54 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230

            #region VR.ATO.GEN.428230

            /*  VR.ATO.GEN.428230
            Address - Locality must not contain a state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE6, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^IEE6 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE6, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE6", Value = report.IEE6 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230

            #region VR.ATO.GEN.428230

            /*  VR.ATO.GEN.428230
            Address - Locality must not contain a state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE12, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^IEE12 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.IEE12, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE12", Value = report.IEE12 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230

            #region VR.ATO.GEN.428230

            /*  VR.ATO.GEN.428230
            Address - Locality must not contain a state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE71, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^IEE71 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE71, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE71", Value = report.IEE71 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428230

            #region VR.ATO.GEN.428253

            /*  VR.ATO.GEN.428253
            ABN must pass the ABN algorithm check
    
            Legacy Rule Format:
            IF (INT:entity.identifier.ABN <> NULLORBLANK) AND (ABNALGORITHM (INT:entity.identifier.ABN) = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^ABN <> BLANK) AND (FailsABNAlgorithm(^ABN))
    
            Data Elements:
    
            ^ABN = INT
            */
            assertion = (string.IsNullOrWhiteSpace(report.INTIdentifierABN) != true && FailsABNAlgorithm(report.INTIdentifierABN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid ABN",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428253" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.428253

            #region VR.ATO.GEN.430206

            /*  VR.ATO.GEN.430206
            Hours taken to prepare and complete form must be 1 to 999
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Report.CompletionHours.Number <> NULL) AND (pyin.xx.xx:Report.CompletionHours.Number <> SET (1-999))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE94 <> NULL) AND ((^IEE94 < 1 OR ^IEE94 > 999))
    
            Data Elements:
    
            ^IEE94 = RP:Report.CompletionHours.Number
            */
            assertion = (report.IEE94 != null && (report.IEE94.GetValueOrDefault() < 1 || report.IEE94.GetValueOrDefault() > 999));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430206",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Hours taken to prepare and complete form must be 1 to 999",
                    Location = "/xbrli:xbrl/tns:Report.CompletionHours.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430206" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE94", Value = report.IEE94.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430206

            #region VR.ATO.GEN.430252

            /*  VR.ATO.GEN.430252

        Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE186) > 200
    
            Data Elements:
    
            ^IEE186 = INT:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = (Length(report.IEE186) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE186", Value = report.IEE186 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430252

            #region VR.ATO.GEN.430252

            /*  VR.ATO.GEN.430252

        Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^IEE184) > 200
    
            Data Elements:
    
            ^IEE184 = INT:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (Length(report.IEE184) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE184", Value = report.IEE184 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430252

            #region VR.ATO.GEN.430253

            /*  VR.ATO.GEN.430253

        Legacy Rule Format:
            IF pyde.xx.xx:PersonUnstructuredName.FullName.Text  <> NULLORBLANK AND pyde.xx.xx:PersonUnstructuredName.FullName.Text <> SET("a-z", "A-Z","0-9", "-", " ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE186  <> BLANK AND NotInSet(^IEE186, '"a-z", "A-Z","0-9", "-", " "')
    
            Data Elements:
    
            ^IEE186 = INT:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE186) != true && !(IsMatch(report.IEE186, @"^([a-z]|[A-Z]|[0-9]|-| )*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430253",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must only contain A to Z,  a to z, 0 to 9, hyphens and spaces",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430253" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE186", Value = report.IEE186 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430253

            #region VR.ATO.GEN.430253

            /*  VR.ATO.GEN.430253

        Legacy Rule Format:
            IF pyde.xx.xx:PersonUnstructuredName.FullName.Text  <> NULLORBLANK AND pyde.xx.xx:PersonUnstructuredName.FullName.Text <> SET("a-z", "A-Z","0-9", "-", " ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE184  <> BLANK AND NotInSet(^IEE184, '"a-z", "A-Z","0-9", "-", " "')
    
            Data Elements:
    
            ^IEE184 = INT:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE184) != true && !(IsMatch(report.IEE184, @"^([a-z]|[A-Z]|[0-9]|-| )*$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430253",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must only contain A to Z,  a to z, 0 to 9, hyphens and spaces",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430253" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE184", Value = report.IEE184 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430253

            #region VR.ATO.GEN.430255

            /*  VR.ATO.GEN.430255

        Legacy Rule Format:
            IF pyin.xx.xx:Declaration.Signature.Date > DATE(TODAY)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE181 > Today()
    
            Data Elements:
    
            ^IEE181 = INT:Declaration:Declaration.Signature.Date WHERE [Declaration.StatementType.Code='TrueAndCorrect']
            */
            assertion = (report.IEE181.GetValueOrDefault() > DateTime.Now.Date);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430255",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Signature date must not be later than today",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430255" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE181", Value = GetValueOrEmptyDate(report.IEE181) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430255

            #region VR.ATO.GEN.430255

            /*  VR.ATO.GEN.430255

        Legacy Rule Format:
            IF pyin.xx.xx:Declaration.Signature.Date > DATE(TODAY)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE191 > Today()
    
            Data Elements:
    
            ^IEE191 = RP:Declaration:Declaration.Signature.Date WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']
            */
            assertion = (report.IEE191.GetValueOrDefault() > DateTime.Now.Date);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430255",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Signature date must not be later than today",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430255" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE191", Value = GetValueOrEmptyDate(report.IEE191) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.430255

            #region VR.ATO.GEN.438029

            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP
    
            ^Context = RP
            */
            assertion = (report.RPExists != false && report.RPIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438029

            #region VR.ATO.GEN.438030

            /*  VR.ATO.GEN.438030

        Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:Intermediary”) AND (entity.identifier.scheme <> “http://www.ato.gov.au/abn”)
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^IdentifierScheme <> 'http://www.ato.gov.au/abn'
    
            Data Elements:
    
            ^IdentifierScheme = INT
    
            ^Context = INT
            */
            assertion = (report.INTExists != false && report.INTIdentifierScheme != @"http://www.ato.gov.au/abn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for Intermediary must be ""http://www.ato.gov.au/abn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438030" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.438030

            #region VR.ATO.GEN.500132

            /*  VR.ATO.GEN.500132
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^OrganisationNameDetails <> NULL) AND (^IEE52 = BLANK)
    
            Data Elements:
    
            ^IEE52 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^OrganisationNameDetails = RP:FamilyTrust:OrganisationNameDetails
            */
            assertion = (report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(report.IEE52) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500132",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present.",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:OrganisationNameDetails" + OccurrenceIndex(report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE52", Value = report.IEE52 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500132

            #region VR.ATO.GEN.500133

            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^IEE53 = BLANK)
    
            Data Elements:
    
            ^IEE53 = RP:FamilyTrust:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:FamilyTrust:AddressDetails
            */
            assertion = (report.FamilyTrust_AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(report.IEE53) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE53", Value = report.IEE53 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500133

            #region VR.ATO.GEN.500133

            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^IEE5 = BLANK)
    
            Data Elements:
    
            ^IEE5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(report.IEE5) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE5", Value = report.IEE5 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500133

            #region VR.ATO.GEN.500133

            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^IEE11 = BLANK)
    
            Data Elements:
    
            ^IEE11 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && string.IsNullOrWhiteSpace(report.IEE11) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE11", Value = report.IEE11 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500133

            #region VR.ATO.GEN.500133

            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^IEE70 = BLANK)
    
            Data Elements:
    
            ^IEE70 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:SpecifiedIndividual:AddressDetails
            */
            assertion = (report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPExists != false && string.IsNullOrWhiteSpace(report.IEE70) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE70", Value = report.IEE70 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500133

            #region VR.ATO.GEN.500141

            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE55 <> BLANK) AND (NotInSet(^IEE55, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^IEE55 = RP:FamilyTrust:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE55) != true && !(IsMatch(report.IEE55, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:AddressDetails" + OccurrenceIndex(report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE55", Value = GetValueOrEmpty(report.IEE55) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500141

            #region VR.ATO.GEN.500141

            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE7 <> BLANK) AND (NotInSet(^IEE7, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^IEE7 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE7) != true && !(IsMatch(report.IEE7, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE7", Value = GetValueOrEmpty(report.IEE7) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500141

            #region VR.ATO.GEN.500141

            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE13 <> BLANK) AND (NotInSet(^IEE13, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^IEE13 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE13) != true && !(IsMatch(report.IEE13, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE13", Value = GetValueOrEmpty(report.IEE13) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500141

            #region VR.ATO.GEN.500141

            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE72 <> BLANK) AND (NotInSet(^IEE72, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^IEE72 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE72) != true && !(IsMatch(report.IEE72, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:AddressDetails" + OccurrenceIndex(report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE72", Value = GetValueOrEmpty(report.IEE72) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500141

            #region VR.ATO.GEN.500143

            /*  VR.ATO.GEN.500143
            Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
            IF pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND (NotContainsSet(^IEE68, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^IEE68 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
    
            ^PersonNameDetails = RP:SpecifiedIndividual:PersonNameDetails
            */
            assertion = (report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPExists != false && !(IsMatch(report.IEE68, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500143",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Given Name must contain at least one alphabetical character.",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500143" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE68", Value = GetValueOrEmpty(report.IEE68) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500143

            #region VR.ATO.GEN.500143

            /*  VR.ATO.GEN.500143
            Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
            IF pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND (NotContainsSet(^IEE86, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^IEE86 = RP:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
    
            ^PersonNameDetails = RP:PersonNameDetails
            */
            assertion = (report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists != false && !(IsMatch(report.IEE86, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500143",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Given Name must contain at least one alphabetical character.",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500143" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE86", Value = GetValueOrEmpty(report.IEE86) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500143

            #region VR.ATO.GEN.500151

            /*  VR.ATO.GEN.500151
            First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE68, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^IEE68 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE68, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010436",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500151" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE68", Value = GetValueOrEmpty(report.IEE68) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500151

            #region VR.ATO.GEN.500151

            /*  VR.ATO.GEN.500151
            First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE86, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^IEE86 = RP:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE86, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010436",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500151" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE86", Value = GetValueOrEmpty(report.IEE86) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500151

            #region VR.ATO.GEN.500152

            /*  VR.ATO.GEN.500152
            Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE69, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^IEE69 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE69, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010448",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500152" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE69", Value = GetValueOrEmpty(report.IEE69) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500152

            #region VR.ATO.GEN.500152

            /*  VR.ATO.GEN.500152
            Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE87, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^IEE87 = RP:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE87, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010448",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500152" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE87", Value = GetValueOrEmpty(report.IEE87) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500152

            #region VR.ATO.GEN.500153

            /*  VR.ATO.GEN.500153
            Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE66, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^IEE66 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE66, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500153",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonNameDetails" + OccurrenceIndex(report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500153" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE66", Value = report.IEE66 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500153

            #region VR.ATO.GEN.500153

            /*  VR.ATO.GEN.500153
            Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^IEE84, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^IEE84 = RP:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.IEE84, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500153",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:PersonNameDetails" + OccurrenceIndex(report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500153" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE84", Value = report.IEE84 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.GEN.500153

            #region VR.ATO.IEE.410004

            /*  VR.ATO.IEE.410004
            Where the interposed entity is a Self Managed Superannuation Fund or Trust, the question regarding the central management and control of the trust must be answered.
    
            Legacy Rule Format:
            IF ([IEE116] = "E") AND ([IEE16] = SET("217","212")) AND ([IEE17] = NULL)
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
            [IEE17] = IEE:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustralia.Indicator
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
            (^IEE116 = 'E') AND (InSet(^IEE16, '"217","212"')) AND (^IEE17 = NULL)
    
            Data Elements:
    
            ^IEE17 = RP:Residency.CentralManagementAndControlOutsideAustralia.Indicator
    
            ^IEE16 = RP:Party.Type.Code
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
            */
            assertion = (report.IEE116 == @"E" && IsMatch(report.IEE16, @"^(217|212)$") && report.IEE17 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410004",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Central management and control indicator must be present if interposed entity is a SMSF or Trust",
                    Location = "/xbrli:xbrl/tns:Residency.CentralManagementAndControlOutsideAustralia.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410004" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE116", Value = report.IEE116 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE16", Value = report.IEE16 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE17", Value = GetValueOrEmpty(report.IEE17) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410004

            #region VR.ATO.IEE.410006

            /*  VR.ATO.IEE.410006
            Interposed entity election  is incomplete.
    
            Legacy Rule Format:
            IF ([IEE116] = "E") AND (([IEE16] = NULLORBLANK) OR ([IEE45] = NULLORBLANK) OR ([IEE46] = NULL) OR ([IEE52] = NULLORBLANK) OR ([IEE53] = NULLORBLANK) OR ([IEE54] = NULLORBLANK) OR ([IEE58] = NULLORBLANK) OR ([IEE59] = NULL) OR ([IEE64] = NULL) OR ([IEE66] = NULLORBLANK) OR ([IEE70] = NULLORBLANK) OR ([IEE71] = NULLORBLANK) OR  ([IEE119] = NULLORBLANK) OR ([IEE170] = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
            [IEE45] = IEE:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year
            [IEE46] = IEE:RP:rvctc3.02.00:Elections.DateSpecified.Date
            [IEE52] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE53] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE54] = IEE:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE58] = IEE:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE59] = IEE:RP:rvctc3.02.00:Elections.Commencement.Date IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE64] = IEE:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE66] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE70] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE71] = IEE:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
            [IEE119] = IEE:RP:pyde.02.18:OrganisationNameDetails.OrganisationalNameType.Code WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE170] = IEE:RP:pyde.02.00:OrganisationNameDetails.Currency.Code WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)

            Technical Business Rule Format:
            (^IEE116 = 'E') AND ((^IEE16 = BLANK) OR (^IEE45 = BLANK) OR (^IEE46 = NULL) OR (^IEE52 = BLANK) OR (^IEE53 = BLANK) OR (^IEE54 = BLANK) OR (^IEE58 = BLANK) OR (^IEE59 = NULL) OR (^IEE64 = NULL) OR (^IEE66 = BLANK) OR (^IEE70 = BLANK) OR (^IEE71 = BLANK) OR  (^IEE119 = BLANK) OR (^IEE170 = BLANK))
    
            Data Elements:
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
    
            ^IEE16 = RP:Party.Type.Code
    
            ^IEE45 = RP:Elections.InterposedEntityElectionStatus.Year
    
            ^IEE46 = RP:Elections.DateSpecified.Date
    
            ^IEE52 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^IEE53 = RP:FamilyTrust:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE54 = RP:FamilyTrust:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE58 = RP:FamilyTrust:Elections.FamilyTrustElectionStatus.Year
    
            ^IEE59 = RP:FamilyTrust:Elections.Commencement.Date
    
            ^IEE64 = RP:SpecifiedIndividual:PersonDemographicDetails.Birth.Date
    
            ^IEE66 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
    
            ^IEE70 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE71 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE119 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^IEE170 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.IEE116 == @"E" && (string.IsNullOrWhiteSpace(report.IEE16) == true || report.IEE45 == null || report.IEE46 == null || string.IsNullOrWhiteSpace(report.IEE52) == true || string.IsNullOrWhiteSpace(report.IEE53) == true || string.IsNullOrWhiteSpace(report.IEE54) == true || report.IEE58 == null || report.IEE59 == null || report.IEE64 == null || string.IsNullOrWhiteSpace(report.IEE66) == true || string.IsNullOrWhiteSpace(report.IEE70) == true || string.IsNullOrWhiteSpace(report.IEE71) == true || string.IsNullOrWhiteSpace(report.IEE119) == true || string.IsNullOrWhiteSpace(report.IEE170) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410006",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Interposed entity election schedule must be complete",
                    LongDescription = @"The full name and address of the family trust in respect of which the election is made, the income year, day specified, election commencement time, and/or date of birth, name and address of the specified individual must all be completed to make a valid Interposed entity election",
                    Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionOrRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410006" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE116", Value = report.IEE116 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE16", Value = report.IEE16 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE45", Value = GetValueOrEmpty(report.IEE45) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE46", Value = GetValueOrEmptyDate(report.IEE46) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE52", Value = report.IEE52 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE53", Value = report.IEE53 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE54", Value = report.IEE54 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE58", Value = GetValueOrEmpty(report.IEE58) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE59", Value = GetValueOrEmptyDate(report.IEE59) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE64", Value = GetValueOrEmptyDate(report.IEE64) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE66", Value = report.IEE66 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE70", Value = report.IEE70 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE71", Value = report.IEE71 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE119", Value = report.IEE119 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE170", Value = report.IEE170 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410006

            #region VR.ATO.IEE.410015

            /*  VR.ATO.IEE.410015
            Where the interposed entity is a Company or Partnership, the question regarding the central management and control of the trust must not be answered.
    
            Legacy Rule Format:
            IF ([IEE116] = "E") AND ([IEE16] = SET("051","161")) AND ([IEE17] <> NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
            [IEE17] = IEE:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustralia.Indicator
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
            (^IEE116 = 'E') AND (InSet(^IEE16, '"051","161"')) AND (^IEE17 <> NULL)
    
            Data Elements:
    
            ^IEE17 = RP:Residency.CentralManagementAndControlOutsideAustralia.Indicator
    
            ^IEE16 = RP:Party.Type.Code
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
            */
            assertion = (report.IEE116 == @"E" && IsMatch(report.IEE16, @"^(051|161)$") && report.IEE17 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410015",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Central mgmnt and control indicator must not be present if interposed entity is a Coy or Partnership",
                    Location = "/xbrli:xbrl/tns:Residency.CentralManagementAndControlOutsideAustralia.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410015" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE116", Value = report.IEE116 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE16", Value = report.IEE16 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE17", Value = GetValueOrEmpty(report.IEE17) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410015

            #region VR.ATO.IEE.410019

            /*  VR.ATO.IEE.410019
            Entity code must be a valid 3 digit code
    
            Legacy Rule Format:
            IF ([IEE16] <> NULLORBLANK) AND ([IEE16] <> SET("051","161","212","217"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code

            Technical Business Rule Format:
            (^IEE16 <> BLANK) AND (NotInSet(^IEE16, '"051","161","212","217"'))
    
            Data Elements:
    
            ^IEE16 = RP:Party.Type.Code
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE16) != true && !(IsMatch(report.IEE16, @"^(051|161|212|217)$", RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410019",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity code must use a 3 digit code - 161, 212, 051 or 217",
                    LongDescription = @"Entity code must use the code 161 for a Partnership, 212 for a Trust, 051 for a Company or 217 for a Fund",
                    Location = "/xbrli:xbrl/tns:Party.Type.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410019" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE16", Value = report.IEE16 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410019

            #region VR.ATO.IEE.410034

            /*  VR.ATO.IEE.410034
            Where the interposed entity is a Company, the question regarding non-residency for tax purposes must be answered
    
            Legacy Rule Format:
            IF ([IEE16] = "051") AND ([IEE21] = NULLORBLANK) AND ([IEE116] = "E")
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
            [IEE21] = IEE:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code

            Technical Business Rule Format:
            (^IEE16 = '051') AND (^IEE21 = BLANK) AND (^IEE116 = 'E')
    
            Data Elements:
    
            ^IEE21 = RP:Residency.TaxPurposesOrganisationStatus.Code
    
            ^IEE16 = RP:Party.Type.Code
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
            */
            assertion = (report.IEE16 == @"051" && string.IsNullOrWhiteSpace(report.IEE21) == true && report.IEE116 == @"E");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410034",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-residency for tax purposes question must be answered if interposed entity is a Company",
                    Location = "/xbrli:xbrl/tns:Residency.TaxPurposesOrganisationStatus.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410034" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE16", Value = report.IEE16 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE21", Value = GetValueOrEmpty(report.IEE21) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE116", Value = report.IEE116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410034

            #region VR.ATO.IEE.410035

            /*  VR.ATO.IEE.410035
            If the Interposed entity is a Self Managed Superannuation Fund, Partnership or Trust, the question regarding non-residency for tax purposes must not be answered.
    
            Legacy Rule Format:
            IF ([IEE21] <> NULL) AND ([IEE116] = "E") AND ([IEE16] = SET("161","217","212")) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
            [IEE21] = IEE:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
            (^IEE21 <> NULL) AND (^IEE116 = 'E') AND (InSet(^IEE16, '"161","217","212"'))
    
            Data Elements:
    
            ^IEE21 = RP:Residency.TaxPurposesOrganisationStatus.Code
    
            ^IEE16 = RP:Party.Type.Code
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
            */
            assertion = (report.IEE21 != null && report.IEE116 == @"E" && IsMatch(report.IEE16, @"^(161|217|212)$"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410035",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-res for tax purposes question must not be answered if entity is SMSF, Partnership or Trust",
                    Location = "/xbrli:xbrl/tns:Residency.TaxPurposesOrganisationStatus.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410035" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE21", Value = GetValueOrEmpty(report.IEE21) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE116", Value = report.IEE116 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE16", Value = report.IEE16 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410035

            #region VR.ATO.IEE.410041

            /*  VR.ATO.IEE.410041
            Family trust TFN must not equal TFN
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF ([IEE50] <> NULLORBLANK) AND ([IEE50] = IEE:RP:entity.identifier.TFN)
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE50] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)

            Technical Business Rule Format:
            (^IEE50 <> BLANK) AND (^IEE50 = ^TFN)
    
            Data Elements:
    
            ^IEE50 = RP:FamilyTrust:Identifiers.TaxFileNumber.Identifier
    
            ^TFN = RP
            */
            if (isStandalone)
            {
                assertion = (string.IsNullOrWhiteSpace(report.IEE50) != true && report.IEE50 == report.RPIdentifierTFN);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410041",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Family trust TFN must not equal TFN of trust, company or partnership",
                        LongDescription = @"An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the entity making the election",
                        Location = "/xbrli:xbrl/tns:FamilyTrust/tns:Identifiers.TaxFileNumber.Identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410041" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE50", Value = GetValueOrEmpty(report.IEE50) });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410041

            #region VR.ATO.IEE.410058

            /*  VR.ATO.IEE.410058
            The day specified for the IEE cannot be earlier than the first day of the FTE income year
    
            Legacy Rule Format:
            IF ([IEE58] <> NULLORBLANK) AND ([IEE61] <> NULL) AND ([IEE60] = NULL) AND ([IEE46] < ("0107"&([IEE58] - 1)))
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE46] = IEE:RP:rvctc3.02.00:Elections.DateSpecified.Date
            [IEE58] = IEE:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE60] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE61] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)

            Technical Business Rule Format:
            (^IEE58 <> BLANK) AND (^IEE61 <> NULL) AND (^IEE60 = NULL) AND (^IEE46 < AsDate(Concat(^IEE58 - 1, '-07-01')))
    
            Data Elements:
    
            ^IEE58 = RP:FamilyTrust:Elections.FamilyTrustElectionStatus.Year
    
            ^IEE46 = RP:Elections.DateSpecified.Date
    
            ^IEE60 = RP:FamilyTrust:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^IEE61 = RP:FamilyTrust:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
            */
            assertion = (report.IEE58 != null && report.IEE61 != null && report.IEE60 == null && report.IEE46.GetValueOrDefault() < AsDate(string.Concat(report.IEE58.GetValueOrDefault() - 1, @"-07-01")));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410058",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Day specified must not be before the FTE income year of substituted accounting period",
                    LongDescription = @"The day specified for the IEE cannot be earlier than the first day of the FTE income year of substituted accounting period",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:Elections.FamilyTrustElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410058" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE58", Value = GetValueOrEmpty(report.IEE58) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE61", Value = GetValueOrEmptyDate(report.IEE61) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE60", Value = GetValueOrEmptyDate(report.IEE60) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE46", Value = GetValueOrEmptyDate(report.IEE46) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410058

            #region VR.ATO.IEE.410059

            /*  VR.ATO.IEE.410059
            Election commencement time for the interposed entity cannot be earlier than the specified day for the interposed entity election.
    
            Legacy Rule Format:
            IF ([IEE49] <> NULL AND  [IEE46] <> NULL AND ([IEE49] < [IEE46])) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE46] = IEE:RP:rvctc3.02.00:Elections.DateSpecified.Date
            [IEE49] = IEE:RP:rvctc3.02.00:Elections.Commencement.Date

            Technical Business Rule Format:
            (^IEE49 <> NULL AND  ^IEE46 <> NULL AND (^IEE49 < ^IEE46))
    
            Data Elements:
    
            ^IEE46 = RP:Elections.DateSpecified.Date
    
            ^IEE49 = RP:Elections.Commencement.Date
            */
            assertion = (report.IEE49 != null && report.IEE46 != null && report.IEE49.GetValueOrDefault() < report.IEE46.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410059",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Election commencement time must not be before the specified day for the interposed entity election",
                    Location = "/xbrli:xbrl/tns:Elections.DateSpecified.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410059" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE49", Value = GetValueOrEmptyDate(report.IEE49) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE46", Value = GetValueOrEmptyDate(report.IEE46) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410059

            #region VR.ATO.IEE.410066

            /*  VR.ATO.IEE.410066
            For an election either a tax file number or 'No TFN' indicator must be present.
    
            Legacy Rule Format:
            IF ([IEE116] = "E") AND ([IEE50] = NULLORBLANK) AND ([IEE51] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE50] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE51] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
            (^IEE116 = 'E') AND (^IEE50 = BLANK) AND (^IEE51 = NULL)
    
            Data Elements:
    
            ^IEE50 = RP:FamilyTrust:Identifiers.TaxFileNumber.Identifier
    
            ^IEE51 = RP:FamilyTrust:Identifiers.TaxFileNumberInexistent.Indicator
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
            */
            assertion = (report.IEE116 == @"E" && string.IsNullOrWhiteSpace(report.IEE50) == true && report.IEE51 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410066",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Either a TFN or the No TFN indictor must be present for the Family Trust",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410066" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE116", Value = report.IEE116 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE50", Value = GetValueOrEmpty(report.IEE50) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE51", Value = GetValueOrEmpty(report.IEE51) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410066

            #region VR.ATO.IEE.410067

            /*  VR.ATO.IEE.410067
            Tax file number and 'No TFN' indicator are both absent and the surname is present. Either tax file number 'No TFN' indicator is required.
    
            Legacy Rule Format:
            IF ([IEE66] <> NULLORBLANK) AND ([IEE62] = NULLORBLANK) AND ([IEE63] = NULL) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE62] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE63] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE66] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)

            Technical Business Rule Format:
            (^IEE66 <> BLANK) AND (^IEE62 = BLANK) AND (^IEE63 = NULL)
    
            Data Elements:
    
            ^IEE62 = RP:SpecifiedIndividual:Identifiers.TaxFileNumber.Identifier
    
            ^IEE63 = RP:SpecifiedIndividual:Identifiers.TaxFileNumberInexistent.Indicator
    
            ^IEE66 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE66) != true && string.IsNullOrWhiteSpace(report.IEE62) == true && report.IEE63 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410067",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Either a TFN or the No TFN indictor must be present if Specified Individual name is present",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410067" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE66", Value = report.IEE66 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE62", Value = GetValueOrEmpty(report.IEE62) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE63", Value = GetValueOrEmpty(report.IEE63) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410067

            #region VR.ATO.IEE.410068

            /*  VR.ATO.IEE.410068
            End Date must be after a Start date
    
            Legacy Rule Format:
            IN TUPLE(period1.xx.xx:PeriodDetails)
            IF pyin.xx.xx:PeriodStart.Date >= pyin.xx.xx:PeriodEnd.Date
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PeriodDetails <> NULL) AND (^IEE60 >= ^IEE61)
    
            Data Elements:
    
            ^IEE60 = RP:FamilyTrust:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^IEE61 = RP:FamilyTrust:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^PeriodDetails = RP:FamilyTrust:PeriodDetails
            */
            assertion = (report.FamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPExists != false && report.IEE60.GetValueOrDefault() >= report.IEE61.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410068",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"End Date must be after a Start date",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:PeriodDetails" + OccurrenceIndex(report.FamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410068" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE60", Value = GetValueOrEmptyDate(report.IEE60) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE61", Value = GetValueOrEmptyDate(report.IEE61) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410068

            #region VR.ATO.IEE.410068

            /*  VR.ATO.IEE.410068
            End Date must be after a Start date
    
            Legacy Rule Format:
            IN TUPLE(period1.xx.xx:PeriodDetails)
            IF pyin.xx.xx:PeriodStart.Date >= pyin.xx.xx:PeriodEnd.Date
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PeriodDetails <> NULL) AND (^IEE47 >= ^IEE48)
    
            Data Elements:
    
            ^IEE47 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^IEE48 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^PeriodDetails = RP:PeriodDetails
            */
            assertion = (report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPExists != false && report.IEE47.GetValueOrDefault() >= report.IEE48.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410068",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"End Date must be after a Start date",
                    Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410068" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE47", Value = GetValueOrEmptyDate(report.IEE47) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE48", Value = GetValueOrEmptyDate(report.IEE48) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410068

            #region VR.ATO.IEE.410069

            /*  VR.ATO.IEE.410069
            Section B of the IEE should only be completed when the client is revoking an IEE.  This revocation is also indicated by the IEE revocation code on the main return form.
    
            Legacy Rule Format:
            WHERE PARENT RETURN EXISTS
            IF ([IEE82] <> NULLORBLANK) AND ([IEE116] = "E")
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE82] = IEE:RP:rvctc3.02.00:Elections.Revocation.Year
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
            (^IEE82 <> BLANK) AND (^IEE116 = 'E')
    
            Data Elements:
    
            ^IEE82 = RP:Elections.Revocation.Year
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
            */
            if (!isStandalone)
            {
                assertion = (report.IEE82 != null && report.IEE116 == @"E");
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410069",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Questions about revocation must not be answered if the form is used for an election",
                        Location = "/xbrli:xbrl/tns:Elections.Revocation.Year",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410069" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE82", Value = GetValueOrEmpty(report.IEE82) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE116", Value = report.IEE116 });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410069

            #region VR.ATO.IEE.410071

            /*  VR.ATO.IEE.410071
            As indicated on the main form, an interposed entity election is to be revoked; however information is missing from the revocation section on the schedule.  For the revocation to be valid Section B on the IEE schedule must be completed.
    
            Legacy Rule Format:
            WHERE PARENT RETURN EXISTS
            IF ([IEE116] = "R") AND (([IEE52] = NULLORBLANK) OR ([IEE53] = NULLORBLANK) OR ([IEE54] = NULLORBLANK) OR ([IEE119] = NULLORBLANK) OR ([IEE170] = NULLORBLANK) OR ([IEE82] = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE52] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE53] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE54] = IEE:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE82] = IEE:RP:rvctc3.02.00:Elections.Revocation.Year
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
            [IEE119] = IEE:RP:pyde.02.18:OrganisationNameDetails.OrganisationalNameType.Code WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE170] = IEE:RP:pyde.02.00:OrganisationNameDetails.Currency.Code WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)

            Technical Business Rule Format:
            (^IEE116 = 'R') AND ((^IEE52 = BLANK) OR (^IEE53 = BLANK) OR (^IEE54 = BLANK) OR (^IEE119 = BLANK) OR (^IEE170 = BLANK) OR (^IEE82 = BLANK))
    
            Data Elements:
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
    
            ^IEE52 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^IEE53 = RP:FamilyTrust:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE54 = RP:FamilyTrust:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^IEE82 = RP:Elections.Revocation.Year
    
            ^IEE119 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^IEE170 = RP:FamilyTrust:OrganisationNameDetails:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            if (!isStandalone)
            {
                assertion = (report.IEE116 == @"R" && (string.IsNullOrWhiteSpace(report.IEE52) == true || string.IsNullOrWhiteSpace(report.IEE53) == true || string.IsNullOrWhiteSpace(report.IEE54) == true || string.IsNullOrWhiteSpace(report.IEE119) == true || string.IsNullOrWhiteSpace(report.IEE170) == true || report.IEE82 == null));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410071",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"For a revocation all details about the family trust must be present",
                        LongDescription = @"You have nominated to revoke an interposed entity, however information is missing from the revocation section on the schedule. For the revocation to be valid Section B on the IEE schedule must be completed",
                        Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionOrRevocation.Code",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410071" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE116", Value = report.IEE116 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE52", Value = report.IEE52 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE53", Value = report.IEE53 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE54", Value = report.IEE54 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE119", Value = report.IEE119 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE170", Value = report.IEE170 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE82", Value = GetValueOrEmpty(report.IEE82) });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410071

            #region VR.ATO.IEE.410072

            /*  VR.ATO.IEE.410072
            The date of birth of the specified individual cannot be later than the election commencement time or the substituted accounting period for the family trust.
    
            Legacy Rule Format:
            IF ([IEE64] <> NULL) AND ((([IEE59] <> NULL) AND ([IEE64] > [IEE59])) OR (([IEE60] <> NULL) AND ([IEE64] > [IEE60]))) 
                 RETURN VALIDATION MESSAGE
             ENDIF
            
            [IEE59] = IEE:RP:rvctc3.02.00:Elections.Commencement.Date IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE60] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE64] = IEE:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)

            Technical Business Rule Format:
            (^IEE64 <> NULL) AND (((^IEE59 <> NULL) AND (^IEE64 > ^IEE59)) OR ((^IEE60 <> NULL) AND (^IEE64 > ^IEE60)))
    
            Data Elements:
    
            ^IEE64 = RP:SpecifiedIndividual:PersonDemographicDetails.Birth.Date
    
            ^IEE59 = RP:FamilyTrust:Elections.Commencement.Date
    
            ^IEE60 = RP:FamilyTrust:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
            */
            assertion = (report.IEE64 != null && (report.IEE59 != null && report.IEE64.GetValueOrDefault() > report.IEE59.GetValueOrDefault() || report.IEE60 != null && report.IEE64.GetValueOrDefault() > report.IEE60.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410072",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The date of birth of the specified individual cannot be later than the election commencement time",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:PersonDemographicDetails.Birth.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410072" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE64", Value = GetValueOrEmptyDate(report.IEE64) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE59", Value = GetValueOrEmptyDate(report.IEE59) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE60", Value = GetValueOrEmptyDate(report.IEE60) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410072

            #region VR.ATO.IEE.410073

            /*  VR.ATO.IEE.410073
            Day specified cannot be earlier than the substituted accounting period for the family trust election.
    
            Legacy Rule Format:
            IF ([IEE46] <> NULL) AND ([IEE60] <> NULL) AND ([IEE46] < [IEE60]) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE46] = IEE:RP:rvctc3.02.00:Elections.DateSpecified.Date
            [IEE60] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)

            Technical Business Rule Format:
            (^IEE46 <> NULL) AND (^IEE60 <> NULL) AND (^IEE46 < ^IEE60)
    
            Data Elements:
    
            ^IEE46 = RP:Elections.DateSpecified.Date
    
            ^IEE60 = RP:FamilyTrust:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
            */
            assertion = (report.IEE46 != null && report.IEE60 != null && report.IEE46.GetValueOrDefault() < report.IEE60.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410073",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Day specified must not be before the Substituted accounting period for the family trust election",
                    Location = "/xbrli:xbrl/tns:Elections.DateSpecified.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410073" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE46", Value = GetValueOrEmptyDate(report.IEE46) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE60", Value = GetValueOrEmptyDate(report.IEE60) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410073

            #region VR.ATO.IEE.410074

            /*  VR.ATO.IEE.410074
            The day specified for the purpose of the interposed entity election cannot be earlier than the first day of the income year specified or later than the last day of the income year specified or earlier or later than the dates supplied for the substituted accounting period for the interposed entity.
    
            Legacy Rule Format:
            IF ([IEE45] <> NULLORBLANK) AND ([IEE46] <> NULL) AND ((([IEE47] = NULL) AND ([IEE46] < ("0107"&([IEE45] - 1)))) OR (([IEE48] = NULL) AND ([IEE46] > ("3006"&([IEE45])))) OR (([IEE47] <> NULL) AND ([IEE46] < [IEE47])) OR (([IEE48] <> NULL) AND ([IEE46] > [IEE48]))) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE45] = IEE:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year
            [IEE46] = IEE:RP:rvctc3.02.00:Elections.DateSpecified.Date
            [IEE47] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
            [IEE48] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

            Technical Business Rule Format:
            (^IEE45 <> BLANK) AND (^IEE46 <> NULL) AND (((^IEE47 = NULL) AND (^IEE46 < AsDate(Concat(^IEE45 - 1, '-07-01')))) OR ((^IEE48 = NULL) AND (^IEE46 >  AsDate(Concat(^IEE45, '-06-30')))) OR ((^IEE47 <> NULL) AND (^IEE46 < ^IEE47)) OR ((^IEE48 <> NULL) AND (^IEE46 > ^IEE48)))
    
            Data Elements:
    
            ^IEE46 = RP:Elections.DateSpecified.Date
    
            ^IEE45 = RP:Elections.InterposedEntityElectionStatus.Year
    
            ^IEE47 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^IEE48 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
            */
            assertion = (report.IEE45 != null && report.IEE46 != null && (report.IEE47 == null && report.IEE46.GetValueOrDefault() < AsDate(string.Concat(report.IEE45.GetValueOrDefault() - 1, @"-07-01")) || report.IEE48 == null && report.IEE46.GetValueOrDefault() > AsDate(string.Concat(report.IEE45.GetValueOrDefault(), @"-06-30")) || report.IEE47 != null && report.IEE46.GetValueOrDefault() < report.IEE47.GetValueOrDefault() || report.IEE48 != null && report.IEE46.GetValueOrDefault() > report.IEE48.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410074",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Day specified must be within the income year and the substituted accounting period",
                    LongDescription = @"The day specified for the purpose of the interposed entity election cannot be earlier than the first day of the income year specified or later than the last day of the income year specified or earlier or later than the dates supplied for the substituted accounting period for the interposed entity",
                    Location = "/xbrli:xbrl/tns:Elections.DateSpecified.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410074" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE45", Value = GetValueOrEmpty(report.IEE45) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE46", Value = GetValueOrEmptyDate(report.IEE46) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE47", Value = GetValueOrEmptyDate(report.IEE47) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE48", Value = GetValueOrEmptyDate(report.IEE48) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410074

            #region VR.ATO.IEE.410076

            /*  VR.ATO.IEE.410076
            An IEE must not contain both election and revocation details
    
            Legacy Rule Format:
            WHERE PARENT RETURN EXISTS
            IF (([IEE58] <> NULLORBLANK) OR ([IEE59] <> NULL) OR ([IEE60] <> NULL) OR ([IEE61] <> NULL) OR ([IEE62] <> NULLORBLANK) OR ([IEE63] <> NULLORBLANK) OR ([IEE64] <> NULL) OR ([IEE65] <> NULLORBLANK) OR ([IEE66] <> NULLORBLANK) OR ([IEE67] <> NULLORBLANK) OR ([IEE68] <> NULLORBLANK) OR ([IEE69] <> NULLORBLANK) OR ([IEE70] <> NULLORBLANK) OR ([IEE71] <> NULLORBLANK) OR ([IEE72] <> NULLORBLANK) OR ([IEE73] <> NULLORBLANK)) AND ([IEE82] <> NULLORBLANK)   RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE58] = IEE:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE59] = IEE:RP:rvctc3.02.00:Elections.Commencement.Date IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE60] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE61] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE62] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE63] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE64] = IEE:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE65] = IEE:RP:pyde.02.00:PersonNameDetails.Title.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE66] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE67] = IEE:RP:pyde.02.00:PersonNameDetails.NameSuffix.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE68] = IEE:RP:pyde.02.00:PersonNameDetails.GivenName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE69] = IEE:RP:pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE70] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE71] = IEE:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE72] = IEE:RP:pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE73] = IEE:RP:pyde.02.00:AddressDetails.Postcode.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE82] = IEE:RP:rvctc3.02.00:Elections.Revocation.Year

            Technical Business Rule Format:
            ((^IEE58 <> BLANK) OR (^IEE59 <> NULL) OR (^IEE60 <> NULL) OR (^IEE61 <> NULL) OR (^IEE62 <> BLANK) OR (^IEE69 <> BLANK) OR (^IEE64 <> NULL) OR (^IEE65 <> BLANK) OR (^IEE66 <> BLANK) OR (^IEE67 <> BLANK) OR (^IEE68 <> BLANK) OR (^IEE69 <> BLANK) OR (^IEE70 <> BLANK) OR (^IEE71 <> BLANK) OR (^IEE72 <> BLANK) OR (^IEE73 <> BLANK)) AND (^IEE82 <> BLANK)
    
            Data Elements:
    
            ^IEE82 = RP:Elections.Revocation.Year
    
            ^IEE58 = RP:FamilyTrust:Elections.FamilyTrustElectionStatus.Year
    
            ^IEE59 = RP:FamilyTrust:Elections.Commencement.Date
    
            ^IEE60 = RP:FamilyTrust:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^IEE61 = RP:FamilyTrust:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
    
            ^IEE62 = RP:SpecifiedIndividual:Identifiers.TaxFileNumber.Identifier
    
            ^IEE63 = RP:SpecifiedIndividual:Identifiers.TaxFileNumberInexistent.Indicator

            ^IEE64 = RP:SpecifiedIndividual:PersonDemographicDetails.Birth.Date
    
            ^IEE65 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
    
            ^IEE66 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
    
            ^IEE67 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
    
            ^IEE68 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
    
            ^IEE69 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
    
            ^IEE70 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE71 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE72 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
    
            ^IEE73 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
            if (!isStandalone)
            {
                assertion = ((report.IEE58 != null || report.IEE59 != null || report.IEE60 != null || report.IEE61 != null || string.IsNullOrWhiteSpace(report.IEE62) != true || report.IEE63 != null || report.IEE64 != null || string.IsNullOrWhiteSpace(report.IEE65) != true || string.IsNullOrWhiteSpace(report.IEE66) != true || string.IsNullOrWhiteSpace(report.IEE67) != true || string.IsNullOrWhiteSpace(report.IEE68) != true || string.IsNullOrWhiteSpace(report.IEE69) != true || string.IsNullOrWhiteSpace(report.IEE70) != true || string.IsNullOrWhiteSpace(report.IEE71) != true || string.IsNullOrWhiteSpace(report.IEE72) != true || string.IsNullOrWhiteSpace(report.IEE73) != true) && report.IEE82 != null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410076",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"An IEE election must not contain revocation Income Year",
                        LongDescription = @"This schedule contains election details, and also the 'income year from which the revocation is to be effective'. If both an election and a revocation are needed, a second schedule is to be submitted with the main return form",
                        Location = "/xbrli:xbrl/tns:Elections.Revocation.Year",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410076" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE58", Value = GetValueOrEmpty(report.IEE58) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE59", Value = GetValueOrEmptyDate(report.IEE59) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE60", Value = GetValueOrEmptyDate(report.IEE60) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE61", Value = GetValueOrEmptyDate(report.IEE61) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE62", Value = GetValueOrEmpty(report.IEE62) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE63", Value = GetValueOrEmpty(report.IEE63) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE69", Value = GetValueOrEmpty(report.IEE69) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE64", Value = GetValueOrEmptyDate(report.IEE64) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE65", Value = GetValueOrEmpty(report.IEE65) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE66", Value = report.IEE66 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE67", Value = GetValueOrEmpty(report.IEE67) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE68", Value = GetValueOrEmpty(report.IEE68) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE70", Value = report.IEE70 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE71", Value = report.IEE71 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE72", Value = GetValueOrEmpty(report.IEE72) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE73", Value = GetValueOrEmpty(report.IEE73) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE82", Value = GetValueOrEmpty(report.IEE82) });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410076

            #region VR.ATO.IEE.410079

            /*  VR.ATO.IEE.410079
            Year must be 2005 or later
    
            Legacy Rule Format:
            IF ([IEE45] <> NULLORBLANK) AND ([IEE45] < "2005")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE45] = IEE:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year

            Technical Business Rule Format:
            (^IEE45 <> BLANK) AND (^IEE45 < 2005)
    
            Data Elements:
    
            ^IEE45 = RP:Elections.InterposedEntityElectionStatus.Year
            */
            assertion = (report.IEE45 != null && report.IEE45.GetValueOrDefault() < 2005);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410233",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Election status year specified for IEE must be 2005 or later",
                    Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410079" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE45", Value = GetValueOrEmpty(report.IEE45) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410079

            #region VR.ATO.IEE.410080

            /*  VR.ATO.IEE.410080
            Year must be 1995 or later
    
            Legacy Rule Format:
            IF ([IEE58] <> NULLORBLANK) AND  ([IEE58] < "1995")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE58] = IEE:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)

            Technical Business Rule Format:
            (^IEE58 <> BLANK) AND  (^IEE58 < 1995)
    
            Data Elements:
    
            ^IEE58 = RP:FamilyTrust:Elections.FamilyTrustElectionStatus.Year
            */
            assertion = (report.IEE58 != null && report.IEE58.GetValueOrDefault() < 1995);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410236",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family trust election specified income year must be 1995 or later",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:Elections.FamilyTrustElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410080" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE58", Value = GetValueOrEmpty(report.IEE58) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410080

            #region VR.ATO.IEE.410084

            /*  VR.ATO.IEE.410084
            Year must be 2008 or later
    
            Legacy Rule Format:
            WHERE PARENT RETURN EXISTS
            IF ([IEE82] <> NULLORBLANK) AND ([IEE82] < "2008")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE82] = IEE:RP:rvctc3.02.00:Elections.Revocation.Year

            Technical Business Rule Format:
            (^IEE82 <> BLANK) AND (^IEE82 < 2008)
    
            Data Elements:
    
            ^IEE82 = RP:Elections.Revocation.Year
            */
            if (!isStandalone)
            {
                assertion = (report.IEE82 != null && report.IEE82.GetValueOrDefault() < 2008);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410238",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Year specified for IEE revocation must be 2008 or later",
                        Location = "/xbrli:xbrl/tns:Elections.Revocation.Year",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410084" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE82", Value = GetValueOrEmpty(report.IEE82) });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410084

            #region VR.ATO.IEE.410086

            /*  VR.ATO.IEE.410086
            TFN of specified individual must be entered if TFN non existent indicator is blank or 'FALSE'
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF ([IEE63] <> TRUE) AND ([IEE62] = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE62] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE63] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)

            Technical Business Rule Format:
            (^IEE63 <> TRUE) AND (^IEE62 = BLANK)
    
            Data Elements:
    
            ^IEE62 = RP:SpecifiedIndividual:Identifiers.TaxFileNumber.Identifier
    
            ^IEE63 = RP:SpecifiedIndividual:Identifiers.TaxFileNumberInexistent.Indicator
            */
            if (isStandalone)
            {
                assertion = (report.IEE63 != true && string.IsNullOrWhiteSpace(report.IEE62) == true);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410086",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"TFN of specified individual must be present if TFN non existent indicator is not 'TRUE'",
                        LongDescription = @"TFN of specified individual must be present if TFN non existent indicator is blank or 'FALSE'",
                        Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:Identifiers.TaxFileNumber.Identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410086" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE63", Value = GetValueOrEmpty(report.IEE63) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE62", Value = GetValueOrEmpty(report.IEE62) });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410086

            #region VR.ATO.IEE.410104

            /*  VR.ATO.IEE.410104
            The 'specified individual does not have a tax file number indicator' is present and a tax file number has been supplied
    
            Legacy Rule Format:
            IF ([IEE63] <> NULL) AND ([IEE62] <> NULLORBLANK) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE62] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE63] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)

            Technical Business Rule Format:
            (^IEE63 <> NULL) AND (^IEE62 <> BLANK)
    
            Data Elements:
    
            ^IEE62 = RP:SpecifiedIndividual:Identifiers.TaxFileNumber.Identifier
    
            ^IEE63 = RP:SpecifiedIndividual:Identifiers.TaxFileNumberInexistent.Indicator
            */
            assertion = (report.IEE63 != null && string.IsNullOrWhiteSpace(report.IEE62) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410104",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The 'no TFN' indicator must not be completed if the TFN of the specified individual is present",
                    LongDescription = @"The 'specified individual does not have a tax file number' indicator' is present and a tax file number has been supplied",
                    Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410104" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE63", Value = GetValueOrEmpty(report.IEE63) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE62", Value = GetValueOrEmpty(report.IEE62) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410104

            #region VR.ATO.IEE.410136

            /*  VR.ATO.IEE.410136
            Current postal address must be entered
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF ([IEE5] = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE5] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails)

            Technical Business Rule Format:
            ^IEE5 = BLANK
    
            Data Elements:
    
            ^IEE5 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            if (isStandalone)
            {
                assertion = (string.IsNullOrWhiteSpace(report.IEE5) == true);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410136",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Current postal address - Line 1 must be present",
                        Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410136" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE5", Value = report.IEE5 });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410136

            #region VR.ATO.IEE.410137

            /*  VR.ATO.IEE.410137
            Current address must be entered
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF ([IEE6] = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE6] = IEE:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails)

            Technical Business Rule Format:
            ^IEE6 = BLANK
    
            Data Elements:
    
            ^IEE6 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            if (isStandalone)
            {
                assertion = (string.IsNullOrWhiteSpace(report.IEE6) == true);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410137",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Current postal address suburb must be present",
                        Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410137" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE6", Value = report.IEE6 });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410137

            #region VR.ATO.IEE.410138

            /*  VR.ATO.IEE.410138
            Election or revocation question must be answered
    
            Legacy Rule Format:
            IF ([IEE116] = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
            ^IEE116 = BLANK
    
            Data Elements:
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE116) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410138",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Election or revocation question must be answered",
                    Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionOrRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410138" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE116", Value = report.IEE116 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410138

            #region VR.ATO.IEE.410186

            /*  VR.ATO.IEE.410186
            Submission must not contain more than 10 trustees or partners
    
            Legacy Rule Format:
            IF COUNT(iee.0003.lodge.req.xx.xx:TrusteeOrPartner) > 10
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^TrusteeOrPartner) > 10
    
            Data Elements:
    
            ^TrusteeOrPartner = RP:TrusteeOrPartner
            */
            assertion = (Count(report.TrusteeOrPartnerCollectionCount) > 10);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410186",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Submission must not contain more than 10 trustees or partners",
                    Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(report.TrusteeOrPartnerCollection == null ? 0 : (report.TrusteeOrPartnerCollection.Count() == 0 ? 0 : report.TrusteeOrPartnerCollection.Last().OccurrenceIndex)) + "/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410186" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410186

            #region VR.ATO.IEE.410187

            /*  VR.ATO.IEE.410187
            More than four periods must not be specified
    
            Legacy Rule Format:
            IF COUNT([IEE19]) > 4
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE19] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

            Technical Business Rule Format:
            Count(^IEE19) > 4
    
            Data Elements:
    
            ^IEE19 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            */
            assertion = (Count(report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection == null ? new List<DateTime?>() : report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection.Select(f => f.IEE19)) > 4);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410187",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"More than four periods must not be specified",
                    LongDescription = @"More than four periods must not be specified for central management and control of the trust outside of Australia",
                    Location = "/xbrli:xbrl/tns:PeriodDetails/tns:Period.Start.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410187" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410187

            #region VR.ATO.IEE.410188

            /*  VR.ATO.IEE.410188
            More than four periods must not be specified
    
            Legacy Rule Format:
            IF COUNT([IEE23]) > 4
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE23] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

            Technical Business Rule Format:
            Count(^IEE23) > 4
    
            Data Elements:
    
            ^IEE23 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
            */
            assertion = (Count(report.PeriodDetailsNonResidentPartPeriodRPCollection == null ? new List<DateTime?>() : report.PeriodDetailsNonResidentPartPeriodRPCollection.Select(f => f.IEE23)) > 4);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410188",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"More than four periods must not be specified",
                    LongDescription = @"More than four periods must not be specified for company being a non-resident for tax purposes",
                    Location = "/xbrli:xbrl/tns:PeriodDetails/tns:Period.Start.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410188" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410188

            #region VR.ATO.IEE.410193

            /*  VR.ATO.IEE.410193
            For a revocation, Family Trust tax file number must be present
    
            Legacy Rule Format:
            WHERE PARENT RETURN EXISTS
            IF ([IEE116] = "R") AND ([IEE50] = NULLORBLANK) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE50] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
            (^IEE116 = 'R') AND (^IEE50 = BLANK)
    
            Data Elements:
    
            ^IEE50 = RP:FamilyTrust:Identifiers.TaxFileNumber.Identifier
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
            */
            if (!isStandalone)
            {
                assertion = (report.IEE116 == @"R" && string.IsNullOrWhiteSpace(report.IEE50) == true);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410193",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"For a revocation, Family Trust tax file number must be present",
                        Location = "/xbrli:xbrl/tns:FamilyTrust/tns:Identifiers.TaxFileNumber.Identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410193" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE116", Value = report.IEE116 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE50", Value = GetValueOrEmpty(report.IEE50) });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410193

            #region VR.ATO.IEE.410198

            /*  VR.ATO.IEE.410198
            TFN of specified individual must be entered if TFN non existent indicator is blank or 'FALSE'
    
            Legacy Rule Format:
            WHERE PARENT RETURN EXISTS
            IF ([IEE116] = "E") AND ([IEE62] = NULLORBLANK) AND ([IEE63] <> TRUE) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE62] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE63] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code

            Technical Business Rule Format:
            (^IEE116 = 'E') AND (^IEE62 = BLANK) AND (^IEE63 <> TRUE)
    
            Data Elements:
    
            ^IEE62 = RP:SpecifiedIndividual:Identifiers.TaxFileNumber.Identifier
    
            ^IEE63 = RP:SpecifiedIndividual:Identifiers.TaxFileNumberInexistent.Indicator
    
            ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
            */
            if (!isStandalone)
            {
                assertion = (report.IEE116 == @"E" && string.IsNullOrWhiteSpace(report.IEE62) == true && report.IEE63 != true);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410086",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"TFN of specified individual must be present if TFN non existent indicator is not 'TRUE'",
                        LongDescription = @"TFN of specified individual must be present if TFN non existent indicator is blank or 'FALSE'",
                        Location = "/xbrli:xbrl/tns:SpecifiedIndividual/tns:Identifiers.TaxFileNumber.Identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410198" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE116", Value = report.IEE116 });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE62", Value = GetValueOrEmpty(report.IEE62) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE63", Value = GetValueOrEmpty(report.IEE63) });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410198

            #region VR.ATO.IEE.410199

            /*  VR.ATO.IEE.410199
            Election Commencement Time must not be present if Income Year Specified in the election is not equal to Income Year of lodgment.
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF ([IEE49] <> NULL) AND ([IEE45] <> [IEE1]) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE1] = IEE:RP:pyin.02.00:Lodgment.PeriodYear.Year
            [IEE45] = IEE:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year
            [IEE49] = IEE:RP:rvctc3.02.00:Elections.Commencement.Date

            Technical Business Rule Format:
            (^IEE49 <> NULL) AND (^IEE45 <> ^IEE1)
    
            Data Elements:
    
            ^IEE49 = RP:Elections.Commencement.Date
    
            ^IEE1 = RP:Lodgment.PeriodYear.Year
    
            ^IEE45 = RP:Elections.InterposedEntityElectionStatus.Year
            */
            if (isStandalone)
            {
                assertion = (report.IEE49 != null && report.IEE45.GetValueOrDefault() != report.IEE1.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410051",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Election Commencement Time must not be present if Income Year Specified in the election is not equal to Income Year of lodgment",
                        Location = "/xbrli:xbrl/tns:Elections.Commencement.Date",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410199" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE49", Value = GetValueOrEmptyDate(report.IEE49) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE45", Value = GetValueOrEmpty(report.IEE45) });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IEE1", Value = GetValueOrEmpty(report.IEE1) });

                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410199

            #region VR.ATO.IEE.410221

            /*  VR.ATO.IEE.410221
            Name must not contain 'space hyphen space'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:Person UnstructuredName.FullName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^IEE186, ' - ')
    
            Data Elements:
    
            ^IEE186 = INT:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = (report.IEE186 == null ? false : report.IEE186.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000423",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Name cannot contain ""space hyphen space""",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410221" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE186", Value = report.IEE186 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410221

            #region VR.ATO.IEE.410227

            /*  VR.ATO.IEE.410227
            An ARBN and an ACN must not both be provided
    
            Legacy Rule Format:
            IF ([IEE100] <> NULLORBLANK) AND ([IEE101] <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE100] = IEE:RP:pyid.02.00:Identifiers.AustralianCompanyNumber.Identifier
            [IEE101] = IEE:RP:pyid.02.00:Identifiers.AustralianRegisteredBodyNumber.Identifier

            Technical Business Rule Format:
            (^IEE100 <> BLANK) AND (^IEE101 <> BLANK)
    
            Data Elements:
    
            ^IEE100 = RP:Identifiers.AustralianCompanyNumber.Identifier
    
            ^IEE101 = RP:Identifiers.AustralianRegisteredBodyNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.IEE100) != true && string.IsNullOrWhiteSpace(report.IEE101) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410222",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"An ARBN and an ACN must not both be provided",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianCompanyNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410227" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE100", Value = GetValueOrEmpty(report.IEE100) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE101", Value = GetValueOrEmpty(report.IEE101) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410227

            #region VR.ATO.IEE.410230

            /*  VR.ATO.IEE.410230
            Income year must be present
    
            Legacy Rule Format:
            IF (RP:pyin.xx.xx.Lodgment.PeriodYear.Year = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IEE1 = BLANK
    
            Data Elements:
    
            ^IEE1 = RP:Lodgment.PeriodYear.Year
            */
            assertion = (report.IEE1 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Lodgment.PeriodYear.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410230" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE1", Value = GetValueOrEmpty(report.IEE1) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410230

            #region VR.ATO.IEE.410231

            /*  VR.ATO.IEE.410231
            Income year must not be less than 2005
    
            Legacy Rule Format:
            IF (RP:pyin.xx.xx:Lodgment.PeriodYear.Year <> NULLORBLANK) AND (RP:pyin.xx.xx:Lodgment.PeriodYear.Year < "2005")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IEE1 <> BLANK) AND (^IEE1 < 2005)
    
            Data Elements:
    
            ^IEE1 = RP:Lodgment.PeriodYear.Year
            */
            assertion = (report.IEE1 != null && report.IEE1.GetValueOrDefault() < 2005);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410231",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year must not be less than 2005",
                    Location = "/xbrli:xbrl/tns:Lodgment.PeriodYear.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410231" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE1", Value = GetValueOrEmpty(report.IEE1) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410231

            #region VR.ATO.IEE.410234

            /*  VR.ATO.IEE.410234
            More than one subsituted accounting period must not be specified
    
            Legacy Rule Format:
            IF COUNT([IEE47]) > 1
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE47] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)

            Technical Business Rule Format:
            Count(^IEE47) > 1
    
            Data Elements:
    
            ^IEE47 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
            */
            assertion = (Count(report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410234",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"More than one period must not be specified",
                    LongDescription = @"More than one period must not be specified for substitued accounting period",
                    Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPLastOccurrenceIndex) + "/tns:Period.Start.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410234" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410234

            #region VR.ATO.IEE.410237

            /*  VR.ATO.IEE.410237
            More than one subsituted accounting period of family trust must not be specified
    
            Legacy Rule Format:
            IF COUNT([IEE60]) > 1
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE60] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "SubstitutedAccountingPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)

            Technical Business Rule Format:
            Count(^IEE60) > 1
    
            Data Elements:
    
            ^IEE60 = RP:FamilyTrust:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
            */
            assertion = (Count(report.FamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410237",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"More than one period must not be specified",
                    LongDescription = @"More than one period must not be specified for substitued accounting period of family trust",
                    Location = "/xbrli:xbrl/tns:FamilyTrust/tns:PeriodDetails" + OccurrenceIndex(report.FamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPLastOccurrenceIndex) + "/tns:Period.Start.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410237" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410237

            #region VR.ATO.IEE.410250

            /*  VR.ATO.IEE.410250
            Number of Trustee Declarations supplied must not exceed 10
    
            Legacy Rule Format:
            IF COUNT(iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner) > 10
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^DeclarationOfTrusteeOrPartner) > 10
    
            Data Elements:
    
            ^DeclarationOfTrusteeOrPartner = RP:DeclarationOfTrusteeOrPartner
            */
            assertion = (Count(report.DeclarationOfTrusteeOrPartnerCollectionCount) > 10);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410250",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of trustee or partner declarations supplied must not exceed 10",
                    Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(report.DeclarationOfTrusteeOrPartnerCollection == null ? 0 : (report.DeclarationOfTrusteeOrPartnerCollection.Count() == 0 ? 0 : report.DeclarationOfTrusteeOrPartnerCollection.Last().OccurrenceIndex)) + "/tns:Declaration/tns:Declaration.StatementType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410250" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410250

            #region VR.ATO.IEE.410251

            /*  VR.ATO.IEE.410251
            Number of Trustee/partner Declarations supplied must equal the  number of trustees/partners
    
            Legacy Rule Format:
            IF (INT = NULL) AND COUNT(iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner) > COUNT (iee.0003.lodge.request.xx.xx:TrusteeOrPartner)
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^INT = NULL) AND Count(^DeclarationOfTrusteeOrPartner) > Count(^TrusteeOrPartner)
    
            Data Elements:
    
            ^DeclarationOfTrusteeOrPartner = RP:DeclarationOfTrusteeOrPartner
    
            ^TrusteeOrPartner = RP:TrusteeOrPartner
    
            ^INT = INT
            */
            assertion = (report.INTExists == false && Count(report.DeclarationOfTrusteeOrPartnerCollectionCount) > Count(report.TrusteeOrPartnerCollectionCount));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410251",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of trustee or partner declarations supplied must be equal to the number of trustees or partners.",
                    Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(report.DeclarationOfTrusteeOrPartnerCollection == null ? 0 : (report.DeclarationOfTrusteeOrPartnerCollection.Count() == 0 ? 0 : report.DeclarationOfTrusteeOrPartnerCollection.Last().OccurrenceIndex)) + "/tns:Declaration/tns:Declaration.StatementType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410251" } },
                };
                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410251

            #region VR.ATO.IEE.410259

            /*  VR.ATO.IEE.410259
            If Entity type is Trust,Fund or Partnership, reporting party declaration must not be present.
    
            Legacy Rule Format:
            IF ([IEE16] = SET ("217","212","161") AND [IEE147] <> NULLORBLANK) 
                RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
            [IEE147] = IEE:RP:pyin.02.03:Declaration.StatementType.Code WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

            Technical Business Rule Format:
            (InSet(^IEE16, '"217","212","161"') AND ^IEE147 <> BLANK)
    
            Data Elements:
    
            ^IEE147 = RP:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']
    
            ^IEE16 = RP:Party.Type.Code
            */
            assertion = (IsMatch(report.IEE16, @"^(217|212|161)$") && string.IsNullOrWhiteSpace(report.IEE147) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410259",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If Entity type is Trust, Fund or Partnership, reporting party declaration must not be present",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:Declaration.StatementType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410259" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE16", Value = report.IEE16 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE147", Value = report.IEE147 });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410259

            #region VR.ATO.IEE.410264

            /*  VR.ATO.IEE.410264
            Declaration Statement Accepted Indicator must be 'TRUE'
    
            Legacy Rule Format:
            IF [IEE192] = FALSE
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE192] = IEE:RP:pyin.02.28:Declaration.StatementAccepted.Indicator WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

            Technical Business Rule Format:
            ^IEE192 = FALSE
    
            Data Elements:
    
            ^IEE192 = RP:Declaration:Declaration.StatementAccepted.Indicator WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']
            */
            assertion = (report.IEE192 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410264",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration Statement Accepted Indicator must be 'TRUE'",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:Declaration.StatementAccepted.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410264" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE192", Value = GetValueOrEmpty(report.IEE192) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410264

            #region VR.ATO.IEE.410267

            /*  VR.ATO.IEE.410267
            Declaration Statement Accepted Indicator must be 'TRUE'
    
            Legacy Rule Format:
            IF [IEE179] = FALSE
              RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE179] = IEE:INT:pyin.02.28:Declaration.StatementAccepted.Indicator WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

            Technical Business Rule Format:
            ^IEE179 = FALSE
    
            Data Elements:
    
            ^IEE179 = INT:Declaration:Declaration.StatementAccepted.Indicator WHERE [Declaration.StatementType.Code='TrueAndCorrect']
            */
            assertion = (report.IEE179 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IEE.410264",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration Statement Accepted Indicator must be 'TRUE'",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:Declaration.StatementAccepted.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410267" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IEE179", Value = GetValueOrEmpty(report.IEE179) });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IEE.410267

            #region VR.ATO.IEE.410271

            /*  VR.ATO.IEE.410271
            If intermediary is not present, then reporting party declaration must be supplied.
    
            Legacy Rule Format:
            WHERE PARENT RETURN DOES NOT EXIST
            IF INT = NULL AND COUNT ([IEE147]) = 0 AND COUNT ([IEE193]) = 0
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [IEE193] = IEE:RP:pyin.02.03:Declaration.StatementType.Code IN TUPLE(iee.0003.lodge.req.02.00:Declaration) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
            [IEE147] = IEE:RP:pyin.02.03:Declaration.StatementType.Code WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

            Technical Business Rule Format:
            ^INT = NULL AND Count(^IEE147) = 0 AND Count(^IEE193) = 0
    
            Data Elements:
    
            ^IEE193 = RP:DeclarationOfTrusteeOrPartner:Declaration:Declaration.StatementType.Code
    
            ^IEE147 = RP:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code = 'TrueAndCorrect']
    
            ^INT = INT
            */
            if (isStandalone)
            {
                assertion = (report.INTExists == false && Count(report.DeclarationCollection_TrueAndCorrect_RPCount) == 0 && Count(report.DeclarationOfTrusteeOrPartnerCollection == null ? new List<string>() : report.DeclarationOfTrusteeOrPartnerCollection.Select(f => f.IEE193)) == 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410271",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"If intermediary is not present, then reporting party declaration must be supplied.",

                        Location = $"/xbrli:xbrl/tns:Declaration{OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPLastOccurrenceIndex)}/tns:Declaration.StatementType.Code",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410271" } },
                    };
                    response.Add(processMessage);
                }
            }
            #endregion // VR.ATO.IEE.410271

            /*  VR.ATO.IEE.410270
           If intermediary is present, then intermediary declaration must be supplied.

       Legacy Rule Format:
           WHERE PARENT RETURN DOES NOT EXIST
           IF INT <> NULL AND COUNT [IEE159] = 0
              RETURN VALIDATION MESSAGE
           ENDIF

           [IEE159] = IEE:INT:pyin.02.03:Declaration.StatementType.Code WHERE (pyin.02.03:Declaration.StatementType.Code = "TrueAndCorrect") IN TUPLE(iee.0003.lodge.req.02.00:Declaration)

       Technical Business Rule Format:
           ^INT <> NULL AND Count(^IEE159) = 0

       Data Elements:

       ^IEE159 = INT:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code='TrueAndCorrect']

       ^INT = INT
       */
            if (isStandalone)
            {

                assertion = (report.INTExists != false && report.DeclarationCollection_TrueAndCorrect_INTCount == 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IEE.410270",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"If intermediary is present, then intermediary declaration must be supplied.",
                        Location = $"/xbrli:xbrl/tns:Declaration{OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTLastOccurrenceIndex)}/tns:Declaration.StatementType.Code",

                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410270" } },
                    };
                    response.Add(processMessage);
                }
            }

            #region Repeating report.TrusteeOrPartnerCollection
            if (report.TrusteeOrPartnerCollection != null)
            {
                int itemIndex1 = 0;
                foreach (IEE2018.TrusteeOrPartner trusteeOrPartner in report.TrusteeOrPartnerCollection)
                {
                    itemIndex1++;

                    #region VR.ATO.GEN.000405

                    /*  VR.ATO.GEN.000405
                    Address must not contain "UNKNOWN".
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE35, 'UNKNOWN')
            
                    Data Elements:
            
                    ^IEE35 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (trusteeOrPartner.IEE35 == null ? false : trusteeOrPartner.IEE35.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000405",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000405" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE35", Value = trusteeOrPartner.IEE35 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000405

                    #region VR.ATO.GEN.000406

                    /*  VR.ATO.GEN.000406
                    Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE35 <> BLANK) AND (StartsWithSet(^IEE35,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^IEE35, '" C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^IEE35 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE35) != true && IsMatch(trusteeOrPartner.IEE35, @"^(C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(trusteeOrPartner.IEE35, @" C/O | C/ | Care Of ", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000406",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE35", Value = trusteeOrPartner.IEE35 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000406

                    #region VR.ATO.GEN.000407

                    /*  VR.ATO.GEN.000407
                    Address Line 2 cannot contain 'UNKNOWN'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE36, 'UNKNOWN')
            
                    Data Elements:
            
                    ^IEE36 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (trusteeOrPartner.IEE36 == null ? false : trusteeOrPartner.IEE36.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000407",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 2 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000407" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE36", Value = GetValueOrEmpty(trusteeOrPartner.IEE36) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000407

                    #region VR.ATO.GEN.000408

                    /*  VR.ATO.GEN.000408
                    Care of' is not valid for address line 2
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE36 <> BLANK) AND (StartsWithSet(^IEE36,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^IEE36, '" C/- "," C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^IEE36 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE36) != true && (IsMatch(trusteeOrPartner.IEE36, @"^(C/- |C/O |C/ |Care Of |CO )", RegexOptions.IgnoreCase) || IsMatch(trusteeOrPartner.IEE36, @" C/- | C/O | C/ | Care Of ", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000408",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Care of' is not valid for address line 2",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE36", Value = GetValueOrEmpty(trusteeOrPartner.IEE36) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000408

                    #region VR.ATO.GEN.000409

                    /*  VR.ATO.GEN.000409
                    Address must not contain "UNKNOWN".
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE37, 'UNKNOWN')
            
                    Data Elements:
            
                    ^IEE37 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (trusteeOrPartner.IEE37 == null ? false : trusteeOrPartner.IEE37.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000409",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000409" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE37", Value = trusteeOrPartner.IEE37 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000409

                    #region VR.ATO.GEN.000410

                    /*  VR.ATO.GEN.000410
                    Address - Locality must not contain numerics.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ContainsSet(^IEE37, '0-9')
            
                    Data Elements:
            
                    ^IEE37 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE37, @"[0-9]", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000410",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain numeric characters",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000410" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE37", Value = trusteeOrPartner.IEE37 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000410

                    #region VR.ATO.GEN.000413

                    /*  VR.ATO.GEN.000413
                    Orqanisation Name cannot contain 'space hyphen space' .
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE32, ' - ')
            
                    Data Elements:
            
                    ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (trusteeOrPartner.IEE32 == null ? false : trusteeOrPartner.IEE32.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000413",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain space hyphen space combinations",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000413" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE32", Value = trusteeOrPartner.IEE32 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000413

                    #region VR.ATO.GEN.000414

                    /*  VR.ATO.GEN.000414
                    Organisation Name must not contain 'P/L'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE32, 'P/L')
            
                    Data Elements:
            
                    ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (trusteeOrPartner.IEE32 == null ? false : trusteeOrPartner.IEE32.ToUpperInvariant().Contains(@"P/L"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000414",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""P/L""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000414" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE32", Value = trusteeOrPartner.IEE32 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000414

                    #region VR.ATO.GEN.000416

                    /*  VR.ATO.GEN.000416
                    Organisation Name must not contain repeated hyphen, apostrophe, or space characters.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ContainsSet(^IEE32, '"--","''","  "')
            
                    Data Elements:
            
                    ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE32, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000416",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000416" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE32", Value = trusteeOrPartner.IEE32 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000416

                    #region VR.ATO.GEN.000417

                    /*  VR.ATO.GEN.000417
                    Non-individualname must not end with a leading space followed by 'T/A', 'T/A P'ship', 'T/A Pship', 'T/A P/Ship' or 'T/A Partnership'.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    EndsWithSet(^IEE32, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"')
            
                    Data Elements:
            
                    ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE32, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000417",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000417" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE32", Value = trusteeOrPartner.IEE32 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000417

                    #region VR.ATO.GEN.000423

                    /*  VR.ATO.GEN.000423
                    Family Name must not contain 'space hyphen space' .
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE28, ' - ')
            
                    Data Elements:
            
                    ^IEE28 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (trusteeOrPartner.IEE28 == null ? false : trusteeOrPartner.IEE28.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000423",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name cannot contain ""space hyphen space""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000423" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE28", Value = trusteeOrPartner.IEE28 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000423

                    #region VR.ATO.GEN.000427

                    /*  VR.ATO.GEN.000427
                    Individualname must not contain repeated hyphen, apostrophe, or space characters.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ContainsSet(^IEE28, '"--","''","  "')
            
                    Data Elements:
            
                    ^IEE28 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE28, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000427",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000427" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE28", Value = trusteeOrPartner.IEE28 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000427

                    #region VR.ATO.GEN.000434

                    /*  VR.ATO.GEN.000434
                    Given Name must not contain 'space hyphen space' .
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE30, ' - ')
            
                    Data Elements:
            
                    ^IEE30 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (trusteeOrPartner.IEE30 == null ? false : trusteeOrPartner.IEE30.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000434",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000434" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE30", Value = GetValueOrEmpty(trusteeOrPartner.IEE30) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000434

                    #region VR.ATO.GEN.000439

                    /*  VR.ATO.GEN.000439
                    Given name must not contain repeated hyphen, apostrophe, or space characters.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ContainsSet(^IEE30, '"--","''","  "')
            
                    Data Elements:
            
                    ^IEE30 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE30, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000439",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000439" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE30", Value = GetValueOrEmpty(trusteeOrPartner.IEE30) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000439

                    #region VR.ATO.GEN.000446

                    /*  VR.ATO.GEN.000446
                    Other Given Name must not contain 'space hyphen space' .
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE31, ' - ')
            
                    Data Elements:
            
                    ^IEE31 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (trusteeOrPartner.IEE31 == null ? false : trusteeOrPartner.IEE31.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000446",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000446" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE31", Value = GetValueOrEmpty(trusteeOrPartner.IEE31) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000446

                    #region VR.ATO.GEN.000451

                    /*  VR.ATO.GEN.000451
                    Individual name must not contain repeated hyphen, apostrophe, or space characters.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ContainsSet(^IEE31, '"--","''","  "')
            
                    Data Elements:
            
                    ^IEE31 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE31, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000451",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000451" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE31", Value = GetValueOrEmpty(trusteeOrPartner.IEE31) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000451

                    #region VR.ATO.GEN.000458

                    /*  VR.ATO.GEN.000458
                    Suffix must equal a code in the Individual Suffix Code table
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE29 <> BLANK) AND (NotInDomain(^IEE29, 'SUFFIX CODES'))
            
                    Data Elements:
            
                    ^IEE29 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE29) != true && !(IsMatch(trusteeOrPartner.IEE29, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000458",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suffix is invalid",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.NameSuffix.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE29", Value = GetValueOrEmpty(trusteeOrPartner.IEE29) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000458

                    #region VR.ATO.GEN.000459

                    /*  VR.ATO.GEN.000459
                    Title must equal a code in the Individual Title Code table
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE27 <> BLANK) AND (NotInDomain(^IEE27, 'TITLE CODES'))
            
                    Data Elements:
            
                    ^IEE27 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE27) != true && !(IsMatch(trusteeOrPartner.IEE27, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000459",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Title is invalid",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.Title.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE27", Value = GetValueOrEmpty(trusteeOrPartner.IEE27) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000459

                    #region VR.ATO.GEN.000460

                    /*  VR.ATO.GEN.000460
                    Postcode must be a valid code
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0200-0299)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(0800-7499)AND pyde.xx.xx:AddressDetails.Postcode.Text  <> SET(7800-9799))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^IEE39 <> BLANK AND (IsNumeric(^IEE39) = 'FALSE' OR ((AsNumeric(^IEE39) < 200 OR AsNumeric(^IEE39) > 299) AND (AsNumeric(^IEE39) < 800 OR AsNumeric(^IEE39) > 7499) AND (AsNumeric(^IEE39) < 7800 OR AsNumeric(^IEE39) > 9799)) )
            
                    Data Elements:
            
                    ^IEE39 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE39) != true && (IsNumeric(trusteeOrPartner.IEE39) == false || (AsNumeric(trusteeOrPartner.IEE39) < 200 || AsNumeric(trusteeOrPartner.IEE39) > 299) && (AsNumeric(trusteeOrPartner.IEE39) < 800 || AsNumeric(trusteeOrPartner.IEE39) > 7499) && (AsNumeric(trusteeOrPartner.IEE39) < 7800 || AsNumeric(trusteeOrPartner.IEE39) > 9799)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000460",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Postcode is invalid",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000460" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE39", Value = GetValueOrEmpty(trusteeOrPartner.IEE39) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000460

                    #region VR.ATO.GEN.410002

                    /*  VR.ATO.GEN.410002
                    If Address Line 1 contains C/-,  Address line 2 must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ((Contains(^IEE35, 'C/-')) AND (^IEE36 = BLANK))
            
                    Data Elements:
            
                    ^IEE35 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE36 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((trusteeOrPartner.IEE35 == null ? false : trusteeOrPartner.IEE35.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE36) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410002",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE35", Value = trusteeOrPartner.IEE35 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE36", Value = GetValueOrEmpty(trusteeOrPartner.IEE36) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410002

                    #region VR.ATO.GEN.410003

                    /*  VR.ATO.GEN.410003
                    If State or Postcode are present, Suburb/town must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) OR (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK)) AND (pyde.xx.xx:AddressDetails.LocalityName.Text = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ((^IEE38 <> BLANK) OR (^IEE39 <> BLANK)) AND (^IEE37 = BLANK)
            
                    Data Elements:
            
                    ^IEE37 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE38 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE39 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((string.IsNullOrWhiteSpace(trusteeOrPartner.IEE38) != true || string.IsNullOrWhiteSpace(trusteeOrPartner.IEE39) != true) && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE37) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410003",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If State or Postcode are present, Suburb/town must be present",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410003" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE38", Value = GetValueOrEmpty(trusteeOrPartner.IEE38) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE39", Value = GetValueOrEmpty(trusteeOrPartner.IEE39) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE37", Value = trusteeOrPartner.IEE37 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410003

                    #region VR.ATO.GEN.410008

                    /*  VR.ATO.GEN.410008
                    Address Line 3 is not required
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^IEE131 <> BLANK
            
                    Data Elements:
            
                    ^IEE131 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE131) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410008",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 3 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE131", Value = GetValueOrEmpty(trusteeOrPartner.IEE131) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410008

                    #region VR.ATO.GEN.410013

                    /*  VR.ATO.GEN.410013
                    Address Line 4 is not required
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^IEE132 <> BLANK
            
                    Data Elements:
            
                    ^IEE132 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE132) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410013",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 4 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE132", Value = GetValueOrEmpty(trusteeOrPartner.IEE132) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410013

                    #region VR.ATO.GEN.410017

                    /*  VR.ATO.GEN.410017
                    If Postcode is present, State must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Postcode.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE39 <> BLANK) AND (^IEE38 = BLANK)
            
                    Data Elements:
            
                    ^IEE38 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE39 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE39) != true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE38) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410017",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"State must be present if Postcode is present",
                            LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410017" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE39", Value = GetValueOrEmpty(trusteeOrPartner.IEE39) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE38", Value = GetValueOrEmpty(trusteeOrPartner.IEE38) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410017

                    #region VR.ATO.GEN.410018

                    /*  VR.ATO.GEN.410018
                    State and postcode or country must be entered.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND ((^IEE38 = BLANK) AND (^IEE39 = BLANK) AND (^IEE113 = BLANK))
            
                    Data Elements:
            
                    ^IEE38 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE39 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE113 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:TrusteeOrPartner:AddressDetails
                    */
                    assertion = (trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPExists != false && (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE38) == true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE39) == true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE113) == true));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410018",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Either state and postcode, or country must be present",
                            LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country must be present",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410018" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE38", Value = GetValueOrEmpty(trusteeOrPartner.IEE38) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE39", Value = GetValueOrEmpty(trusteeOrPartner.IEE39) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE113", Value = GetValueOrEmpty(trusteeOrPartner.IEE113) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410018

                    #region VR.ATO.GEN.410031

                    /*  VR.ATO.GEN.410031
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier <> NULLORBLANK) AND (TFNALGORITHM (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier) = FALSE)
              RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    (^IEE25 <> BLANK) AND (FailsTFNAlgorithm(^IEE25))
            
                    Data Elements:
            
                    ^IEE25 = RP:TrusteeOrPartner:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE25) != true && FailsTFNAlgorithm(trusteeOrPartner.IEE25));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410031",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"TFN must be a valid TFN",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410031" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE25", Value = GetValueOrEmpty(trusteeOrPartner.IEE25) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410031

                    #region VR.ATO.GEN.410038

                    /*  VR.ATO.GEN.410038
                    Non-individual Name cannot start with 'T/A' followed by a space and end with a space followed by 'Pship', 'P'ship or 'P/Ship'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (StartsWith(^IEE32, 'T/A ')) AND (EndsWithSet(^IEE32, '" Pship"," P'ship"," P/ship"'))
            
                    Data Elements:
            
                    ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (IsMatch(trusteeOrPartner.IEE32, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(trusteeOrPartner.IEE32, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410038",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE32", Value = trusteeOrPartner.IEE32 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410038

                    #region VR.ATO.GEN.410039

                    /*  VR.ATO.GEN.410039
                    Organisation name must contain at least one alpha or numeric character.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE32  <> BLANK) AND (NotContainsSet(^IEE32, '"a-z", "A-Z", "0-9"'))
            
                    Data Elements:
            
                    ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE32) != true && !(IsMatch(trusteeOrPartner.IEE32, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410039",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must contain at least one alpha or numeric character",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE32", Value = trusteeOrPartner.IEE32 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410039

                    #region VR.ATO.GEN.410053

                    /*  VR.ATO.GEN.410053
                    Postcode, State and Country must not all be blank.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND ((pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code = NULLORBLANK))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE35 <> BLANK) AND ((^IEE38 = BLANK) AND (^IEE39 = BLANK) AND (^IEE113 = BLANK))
            
                    Data Elements:
            
                    ^IEE35 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE38 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE39 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE113 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE35) != true && (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE38) == true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE39) == true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE113) == true));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410053",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Postcode and State, or Country code must be present",
                            LongDescription = @"If the address is an Australian address, then state and postcode must be present. If the address is an overseas address, then country code must be present",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410053" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE35", Value = trusteeOrPartner.IEE35 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE38", Value = GetValueOrEmpty(trusteeOrPartner.IEE38) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE39", Value = GetValueOrEmpty(trusteeOrPartner.IEE39) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE113", Value = GetValueOrEmpty(trusteeOrPartner.IEE113) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410053

                    #region VR.ATO.GEN.410055

                    /*  VR.ATO.GEN.410055
                    If State is present, Postcode must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Postcode.Text = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE38 <> BLANK) AND (^IEE39 = BLANK)
            
                    Data Elements:
            
                    ^IEE39 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE38 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE38) != true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE39) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410055",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Postcode must be present if State is present",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Postcode.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410055" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE38", Value = GetValueOrEmpty(trusteeOrPartner.IEE38) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE39", Value = GetValueOrEmpty(trusteeOrPartner.IEE39) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410055

                    #region VR.ATO.GEN.410105

                    /*  VR.ATO.GEN.410105
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianRegisteredBodyNumber.Identifier <> NULLORBLANK) AND (ARBNALGORITHM (pyid.xx.xx:Identifiers.AustralianRegisteredBodyNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    (^IEE95 <> BLANK) AND (FailsARBNAlgorithm(^IEE95))
            
                    Data Elements:
            
                    ^IEE95 = RP:TrusteeOrPartner:Identifiers.AustralianRegisteredBodyNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE95) != true && FailsACNAlgorithm(trusteeOrPartner.IEE95));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410105",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"ARBN must be a valid number that passes the algorithm",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Identifiers.AustralianRegisteredBodyNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410105" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE95", Value = GetValueOrEmpty(trusteeOrPartner.IEE95) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410105

                    #region VR.ATO.GEN.410131

                    /*  VR.ATO.GEN.410131
                    Individual name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE31 <> BLANK) AND (NotContainsSet(^IEE31, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^IEE31 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE31) != true && !(IsMatch(trusteeOrPartner.IEE31, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410131",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE31", Value = GetValueOrEmpty(trusteeOrPartner.IEE31) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410131

                    #region VR.ATO.GEN.410194

                    /*  VR.ATO.GEN.410194
                    Address Line 1 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                         RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^IEE35) > 38
            
                    Data Elements:
            
                    ^IEE35 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(trusteeOrPartner.IEE35) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410194",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE35", Value = trusteeOrPartner.IEE35 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410194

                    #region VR.ATO.GEN.410195

                    /*  VR.ATO.GEN.410195
                    Address Line 2 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                         RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^IEE36) > 38
            
                    Data Elements:
            
                    ^IEE36 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(trusteeOrPartner.IEE36) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410195",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE36", Value = GetValueOrEmpty(trusteeOrPartner.IEE36) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410195

                    #region VR.ATO.GEN.410200

                    /*  VR.ATO.GEN.410200
                    Individualname must not contain 'Exec for', 'Rep for' or 'Trustee for'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE28, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^IEE28 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE28, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000424",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE28", Value = trusteeOrPartner.IEE28 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410200

                    #region VR.ATO.GEN.410201

                    /*  VR.ATO.GEN.410201
                    IndividualNamefield must not contain MR, MRS, MISS, MS with space before and after.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE28, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^IEE28 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE28, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000426",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE28", Value = trusteeOrPartner.IEE28 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410201

                    #region VR.ATO.GEN.410203

                    /*  VR.ATO.GEN.410203
                    Given Name must not contain trustee reference
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE30, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^IEE30 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE30, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000437",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE30", Value = GetValueOrEmpty(trusteeOrPartner.IEE30) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410203

                    #region VR.ATO.GEN.410204

                    /*  VR.ATO.GEN.410204
                    Given Name must not contain a title
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE30, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^IEE30 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE30, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000438",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE30", Value = GetValueOrEmpty(trusteeOrPartner.IEE30) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410204

                    #region VR.ATO.GEN.410205

                    /*  VR.ATO.GEN.410205
                    Address Line 1 cannot contain 'as above'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE35, '"AS ABOVE"'))
            
                    Data Elements:
            
                    ^IEE35 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE35, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410001",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not contain 'AS ABOVE'",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE35", Value = trusteeOrPartner.IEE35 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410205

                    #region VR.ATO.GEN.410208

                    /*  VR.ATO.GEN.410208
                    Individualname must not contain 'Exec for', 'Rep for' or 'Trustee for'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE31, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^IEE31 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE31, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000449",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE31", Value = GetValueOrEmpty(trusteeOrPartner.IEE31) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410208

                    #region VR.ATO.GEN.410209

                    /*  VR.ATO.GEN.410209
                    Other Given Name must not contain a title
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE31, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^IEE31 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE31, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000450",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE31", Value = GetValueOrEmpty(trusteeOrPartner.IEE31) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410209

                    #region VR.ATO.GEN.410213

                    /*  VR.ATO.GEN.410213
                    Country Name must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.CountryName.Text <> NULLORBLANK)
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^IEE40 <> BLANK
            
                    Data Elements:
            
                    ^IEE40 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.CountryName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE40) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410213",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Country Name must be left blank",
                            LongDescription = @"For SBR submissions to the ATO, the Country Name must be left blank. To supply the country for an overseas address please supply a Country Code",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.CountryName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410213" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE40", Value = GetValueOrEmpty(trusteeOrPartner.IEE40) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410213

                    #region VR.ATO.GEN.410218

                    /*  VR.ATO.GEN.410218
                    Firstname must be entered if OtherGivenNames is entered
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                        RETURN VALIDATION MESSAGE 
                     ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND ((^IEE31 <> BLANK) AND (^IEE30 = BLANK))
            
                    Data Elements:
            
                    ^IEE31 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            
                    ^IEE30 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:TrusteeOrPartner:PersonNameDetails
                    */
                    assertion = (trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPExists != false && (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE31) != true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE30) == true));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410218",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"FirstName must be entered if OtherGivenNames is entered",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE31", Value = GetValueOrEmpty(trusteeOrPartner.IEE31) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE30", Value = GetValueOrEmpty(trusteeOrPartner.IEE30) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410218

                    #region VR.ATO.GEN.428021

                    /*  VR.ATO.GEN.428021
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianCompanyNumber.Identifier <> NULLORBLANK) AND (ACNALGORITHM(pyid.xx.xx:Identifiers.AustralianCompanyNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    (^IEE34 <> BLANK) AND (FailsACNAlgorithm(^IEE34))
            
                    Data Elements:
            
                    ^IEE34 = RP:TrusteeOrPartner:Identifiers.AustralianCompanyNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE34) != true && FailsACNAlgorithm(trusteeOrPartner.IEE34));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.428021",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid ACN",
                            LongDescription = @"The Australian Company Number (ACN) has failed the algorithm check",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Identifiers.AustralianCompanyNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428021" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE34", Value = GetValueOrEmpty(trusteeOrPartner.IEE34) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428021

                    #region VR.ATO.GEN.428230

                    /*  VR.ATO.GEN.428230
                    Address - Locality must not contain a state code
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE37, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
            
                    Data Elements:
            
                    ^IEE37 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE37, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000411",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain state code",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE37", Value = trusteeOrPartner.IEE37 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.428230

                    #region VR.ATO.GEN.500132

                    /*  VR.ATO.GEN.500132
                    Organisation name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^OrganisationNameDetails <> NULL) AND (^IEE32 = BLANK)
            
                    Data Elements:
            
                    ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^OrganisationNameDetails = RP:TrusteeOrPartner:OrganisationNameDetails
                    */
                    assertion = (trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE32) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500132",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must be present.",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE32", Value = trusteeOrPartner.IEE32 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500132

                    #region VR.ATO.GEN.500133

                    /*  VR.ATO.GEN.500133
                    Address Line 1 must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^IEE35 = BLANK)
            
                    Data Elements:
            
                    ^IEE35 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:TrusteeOrPartner:AddressDetails
                    */
                    assertion = (trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE35) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500133",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must be present.",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE35", Value = trusteeOrPartner.IEE35 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500133

                    #region VR.ATO.GEN.500141

                    /*  VR.ATO.GEN.500141
                    For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
                    AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                        RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE38 <> BLANK) AND (NotInSet(^IEE38, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
            
                    Data Elements:
            
                    ^IEE38 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE38) != true && !(IsMatch(trusteeOrPartner.IEE38, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500141",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE38", Value = GetValueOrEmpty(trusteeOrPartner.IEE38) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500141

                    #region VR.ATO.GEN.500143

                    /*  VR.ATO.GEN.500143
                    Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND (NotContainsSet(^IEE30, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^IEE30 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:TrusteeOrPartner:PersonNameDetails
                    */
                    assertion = (trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPExists != false && !(IsMatch(trusteeOrPartner.IEE30, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500143",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Given Name must contain at least one alphabetical character.",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500143" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE30", Value = GetValueOrEmpty(trusteeOrPartner.IEE30) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500143

                    #region VR.ATO.GEN.500151

                    /*  VR.ATO.GEN.500151
                    First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE30, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^IEE30 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE30, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010436",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500151" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE30", Value = GetValueOrEmpty(trusteeOrPartner.IEE30) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500151

                    #region VR.ATO.GEN.500152

                    /*  VR.ATO.GEN.500152
                    Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE31, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^IEE31 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE31, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010448",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500152" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE31", Value = GetValueOrEmpty(trusteeOrPartner.IEE31) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500152

                    #region VR.ATO.GEN.500153

                    /*  VR.ATO.GEN.500153
                    Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE28, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^IEE28 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(trusteeOrPartner.IEE28, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500153",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500153" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE28", Value = trusteeOrPartner.IEE28 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500153

                    #region VR.ATO.IEE.410025

                    /*  VR.ATO.IEE.410025
                    If Entity type is Company, Address Details of trustee or partner must be blank.
    
                    Legacy Rule Format:
                    IF ([IEE116] = "E") AND ([IEE16] = "051") AND ([IEE35] <> NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                    [IEE35] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        
                    Technical Business Rule Format:
                    (^IEE116 = 'E') AND (^IEE16 = '051') AND (^IEE35 <> BLANK)
            
                    Data Elements:
            
                    ^IEE35 = RP:TrusteeOrPartner:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    ^IEE16 = RP:Party.Type.Code
            
                    ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
                    */
                    assertion = (report.IEE116 == @"E" && report.IEE16 == @"051" && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE35) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410025",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If Entity type is Company, Address Details of trustee or partner must be blank",
                            LongDescription = @"If the Entity Type is a Company, you are not required to supply details of trustee or partner",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410025" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE116", Value = report.IEE116 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE16", Value = report.IEE16 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE35", Value = trusteeOrPartner.IEE35 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410025

                    #region VR.ATO.IEE.410026

                    /*  VR.ATO.IEE.410026
                    If Entity type is Company, ACN of trustee or partner must be blank.
    
                    Legacy Rule Format:
                    IF ([IEE116] = "E") AND ([IEE16] = "051") AND ([IEE34] <> NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                    [IEE34] = IEE:RP:pyid.02.00:Identifiers.AustralianCompanyNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        
                    Technical Business Rule Format:
                    (^IEE116 = 'E') AND (^IEE16 = '051') AND (^IEE34 <> BLANK)
            
                    Data Elements:
            
                    ^IEE34 = RP:TrusteeOrPartner:Identifiers.AustralianCompanyNumber.Identifier
            
                    ^IEE16 = RP:Party.Type.Code
            
                    ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
                    */
                    assertion = (report.IEE116 == @"E" && report.IEE16 == @"051" && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE34) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410026",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If Entity type is Company, company ACN of trustee or partner must be blank",
                            LongDescription = @"If the Entity Type is a Company, you are not required to supply details of trustee or partner",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Identifiers.AustralianCompanyNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410026" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE116", Value = report.IEE116 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE16", Value = report.IEE16 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE34", Value = GetValueOrEmpty(trusteeOrPartner.IEE34) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410026

                    #region VR.ATO.IEE.410027

                    /*  VR.ATO.IEE.410027
                    If Entity type is Company, Organisation name of trustee or partner must be blank.
    
                    Legacy Rule Format:
                    IF ([IEE116] = "E") AND ([IEE16] = "051") AND ([IEE32] <> NULLORBLANK)
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                    [IEE32] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        
                    Technical Business Rule Format:
                    (^IEE116 = 'E') AND (^IEE16 = '051') AND (^IEE32 <> BLANK)
            
                    Data Elements:
            
                    ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^IEE16 = RP:Party.Type.Code
            
                    ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
                    */
                    assertion = (report.IEE116 == @"E" && report.IEE16 == @"051" && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE32) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410027",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If Entity type is Company, Organisation Name of trustee or partner must be blank",
                            LongDescription = @"If the Entity Type is a Company, you are not required to enter a Name under the 'If the trustee or partner is a company' question",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410027" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE116", Value = report.IEE116 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE16", Value = report.IEE16 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE32", Value = trusteeOrPartner.IEE32 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410027

                    #region VR.ATO.IEE.410028

                    /*  VR.ATO.IEE.410028
                    If Entity type is Company, Residency indicator  of trustee or partner must be blank.
    
                    Legacy Rule Format:
                    IF ([IEE116] = "E") AND ([IEE16] = "051") AND ([IEE41] <> NULL)
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                    [IEE41] = IEE:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        
                    Technical Business Rule Format:
                    (^IEE116 = 'E') AND (^IEE16 = '051') AND (^IEE41 <> NULL)
            
                    Data Elements:
            
                    ^IEE41 = RP:TrusteeOrPartner:Residency.TaxPurposesPersonStatus.Indicator
            
                    ^IEE16 = RP:Party.Type.Code
            
                    ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
                    */
                    assertion = (report.IEE116 == @"E" && report.IEE16 == @"051" && trusteeOrPartner.IEE41 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410028",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If Entity type is Company, Residency Indicator of trustee or partner must be blank",
                            LongDescription = @"If the Entity Type is a Company, you are not required to supply details of trustee or partner",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Residency.TaxPurposesPersonStatus.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410028" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE116", Value = report.IEE116 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE16", Value = report.IEE16 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE41", Value = GetValueOrEmpty(trusteeOrPartner.IEE41) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410028

                    #region VR.ATO.IEE.410029

                    /*  VR.ATO.IEE.410029
                    If Entity type is Company, details of residency of trustee or partner must be blank.
    
                    Legacy Rule Format:
                    IF ([IEE116] = "E") AND ([IEE16] = "051") AND ([IEE42] <> NULL)
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                    [IEE42] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        
                    Technical Business Rule Format:
                    (^IEE116 = 'E') AND (^IEE16 = '051') AND (^IEE42 <> NULL)
            
                    Data Elements:
            
                    ^IEE42 = RP:TrusteeOrPartner:Residency.NonResidentFullPeriod.Indicator
            
                    ^IEE16 = RP:Party.Type.Code
            
                    ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
                    */
                    assertion = (report.IEE116 == @"E" && report.IEE16 == @"051" && trusteeOrPartner.IEE42 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410029",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If Entity type is Company, details of residency of trustee or partner must be blank",
                            LongDescription = @"If the Entity Type is a Company, you are not required to supply details of trustee or partner",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Residency.NonResidentFullPeriod.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410029" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE116", Value = report.IEE116 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE16", Value = report.IEE16 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE42", Value = GetValueOrEmpty(trusteeOrPartner.IEE42) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410029

                    #region VR.ATO.IEE.410032

                    /*  VR.ATO.IEE.410032
                    If Entity type is Company, TFN non-existent indicator of trustee or partner must be blank.
    
                    Legacy Rule Format:
                    IF ([IEE116] = "E") AND ([IEE16] = "051") AND ([IEE26] <> NULL)
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                    [IEE26] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        
                    Technical Business Rule Format:
                    (^IEE116 = 'E') AND (^IEE16 = '051') AND (^IEE26 <> NULL)
            
                    Data Elements:
            
                    ^IEE26 = RP:TrusteeOrPartner:Identifiers.TaxFileNumberInexistent.Indicator
            
                    ^IEE16 = RP:Party.Type.Code
            
                    ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
                    */
                    assertion = (report.IEE116 == @"E" && report.IEE16 == @"051" && trusteeOrPartner.IEE26 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410032",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If Entity type is Company, TFN non-existent indicator of trustee or partner must not be provided",
                            LongDescription = @"If the Entity Type is a Company, you are not required to supply details of trustee or partner",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumberInexistent.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410032" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE116", Value = report.IEE116 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE16", Value = report.IEE16 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE26", Value = GetValueOrEmpty(trusteeOrPartner.IEE26) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410032

                    #region VR.ATO.IEE.410042

                    /*  VR.ATO.IEE.410042
                    Trustee FamilyNameand Non-individualname must not both be entered.  Only one name is allowed.
    
                    Legacy Rule Format:
                    IF ([IEE28] <> NULLORBLANK) AND ([IEE32] <> NULLORBLANK) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE28] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Identifier" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE32] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
        
                    Technical Business Rule Format:
                    (^IEE28 <> BLANK) AND (^IEE32 <> BLANK)
            
                    Data Elements:
            
                    ^IEE28 = RP:TrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            
                    ^IEE32 = RP:TrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE28) != true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE32) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410042",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee Family Name and Non-individual Name must not both be present. Only one name is allowed",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410042" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE28", Value = trusteeOrPartner.IEE28 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE32", Value = trusteeOrPartner.IEE32 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410042

                    #region VR.ATO.IEE.410085

                    /*  VR.ATO.IEE.410085
                    TFN of Trustee must not be entered if TFN non existent indicator is entered
    
                    Legacy Rule Format:
                    IF ([IEE26] = TRUE) AND [IEE25] <> NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE25] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE26] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
        
                    Technical Business Rule Format:
                    (^IEE26 = TRUE) AND ^IEE25 <> BLANK
            
                    Data Elements:
            
                    ^IEE26 = RP:TrusteeOrPartner:Identifiers.TaxFileNumberInexistent.Indicator
            
                    ^IEE25 = RP:TrusteeOrPartner:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (trusteeOrPartner.IEE26 == true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE25) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410085",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If TFN non existent indicator is 'true', TFN of Trustee must not be present",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumberInexistent.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410085" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE26", Value = GetValueOrEmpty(trusteeOrPartner.IEE26) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE25", Value = GetValueOrEmpty(trusteeOrPartner.IEE25) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410085

                    #region VR.ATO.IEE.410094

                    /*  VR.ATO.IEE.410094
                    Both full period indicator and specific dates have been supplied for non-resident trustee or partner
    
                    Legacy Rule Format:
                    IF ([IEE42] = TRUE) AND ([IEE43] <> NULL) AND ([IEE44] <> NULL) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE42] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE43] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE44] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
        
                    Technical Business Rule Format:
                    (^IEE42 = TRUE) AND AnyOccurrence(^IEE43, ^IEE43 <> NULL) AND AnyOccurrence(^IEE44 , ^IEE44 <> NULL)
            
                    Data Elements:
            
                    ^IEE42 = RP:TrusteeOrPartner:Residency.NonResidentFullPeriod.Indicator
            
                    ^IEE43 = RP:TrusteeOrPartner:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
            
                    ^IEE44 = RP:TrusteeOrPartner:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                    */
                    assertion = (trusteeOrPartner.IEE42 == true && (trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection == null ? false : trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Any(IEE43Repeat => IEE43Repeat.IEE43 != null)) && (trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection == null ? false : trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Any(IEE44Repeat => IEE44Repeat.IEE44 != null)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410094",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Full period indicator and dates trustee or partner was a non-resident must not both be present",
                            LongDescription = @"Both full period indicator and specific dates have been supplied for non-resident trustee or partner",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Residency.NonResidentFullPeriod.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410094" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE42", Value = GetValueOrEmpty(trusteeOrPartner.IEE42) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE43", Value = "IEE43" });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE44", Value = "IEE44" });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410094

                    #region VR.ATO.IEE.410099

                    /*  VR.ATO.IEE.410099
                    The interposed entity code is '161', '212' or '217' and the field indicating that the trustee or partner does not have a tax file number is blank and no tax file number has been supplied
    
                    Legacy Rule Format:
                    WHERE PARENT RETURN DOES NOT EXIST
                    IF ([IEE16] = SET("217","161","212")) AND ([IEE26] <> TRUE) AND ([IEE25] = NULLORBLANK)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                    [IEE25] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE26] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
        
                    Technical Business Rule Format:
                    (InSet(^IEE16, '"217","161","212"')) AND (^IEE26 <> TRUE) AND (^IEE25 = BLANK)
            
                    Data Elements:
            
                    ^IEE25 = RP:TrusteeOrPartner:Identifiers.TaxFileNumber.Identifier
            
                    ^IEE16 = RP:Party.Type.Code
            
                    ^IEE26 = RP:TrusteeOrPartner:Identifiers.TaxFileNumberInexistent.Indicator
                    */
                    if (isStandalone)
                    {
                        assertion = (IsMatch(report.IEE16, @"^(217|161|212)$") && trusteeOrPartner.IEE26 != true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE25) == true);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IEE.410099",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"The TFN of the trustee or partner or the no TFN indicator must be present",
                                LongDescription = @"The interposed entity code is '217', '161' or '212' and the field indicating that the trustee or partner does not have a tax file number is blank and no tax file number has been supplied",
                                Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410099" } },
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IEE16", Value = report.IEE16 });

                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IEE26", Value = GetValueOrEmpty(trusteeOrPartner.IEE26) });

                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IEE25", Value = GetValueOrEmpty(trusteeOrPartner.IEE25) });

                            response.Add(processMessage);
                        }
                    }
                    #endregion // VR.ATO.IEE.410099

                    #region VR.ATO.IEE.410100

                    /*  VR.ATO.IEE.410100
                    The interposed entity code is '161', '212' or '217' and the field indicating that the trustee or partner does not have a tax file number is blank and no tax file number has been supplied
    
                    Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF ([IEE116] = "E") AND ([IEE16] = SET ("217","212","161")) AND ([IEE26] <> TRUE) AND ([IEE25] = NULLORBLANK) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                    [IEE25] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE26] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        
                    Technical Business Rule Format:
                    (^IEE116 = 'E') AND (InSet(^IEE16, '"217","212","161"')) AND (^IEE26 <> TRUE) AND (^IEE25 = BLANK)
            
                    Data Elements:
            
                    ^IEE25 = RP:TrusteeOrPartner:Identifiers.TaxFileNumber.Identifier
            
                    ^IEE16 = RP:Party.Type.Code
            
                    ^IEE26 = RP:TrusteeOrPartner:Identifiers.TaxFileNumberInexistent.Indicator
            
                    ^IEE116 = RP:Elections.InterposedEntityElectionOrRevocation.Code
                    */
                    if (!isStandalone)
                    {
                        assertion = (report.IEE116 == @"E" && IsMatch(report.IEE16, @"^(217|212|161)$") && trusteeOrPartner.IEE26 != true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE25) == true);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IEE.410100",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"The TFN of the trustee or partner or the no TFN indicator must be present",
                                LongDescription = @"The interposed entity code is '161', '217' or '212' and the field indicating that the 'trustee or partner does not have a tax file number' is blank, and no tax file number has been supplied",
                                Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410100" } },
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IEE116", Value = report.IEE116 });

                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IEE16", Value = report.IEE16 });

                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IEE26", Value = GetValueOrEmpty(trusteeOrPartner.IEE26) });

                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IEE25", Value = GetValueOrEmpty(trusteeOrPartner.IEE25) });

                            response.Add(processMessage);
                        }
                    }
                    #endregion // VR.ATO.IEE.410100

                    #region VR.ATO.IEE.410222

                    /*  VR.ATO.IEE.410222
                    An ARBN and an ACN must not both be provided
    
                    Legacy Rule Format:
                    IF ([IEE34] <> NULLORBLANK) AND ([IEE95] <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE34] = IEE:RP:pyid.02.00:Identifiers.AustralianCompanyNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                    [IEE95] = IEE:RP:pyid.02.00:Identifiers.AustralianRegisteredBodyNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
        
                    Technical Business Rule Format:
                    (^IEE34 <> BLANK) AND (^IEE95 <> BLANK)
            
                    Data Elements:
            
                    ^IEE34 = RP:TrusteeOrPartner:Identifiers.AustralianCompanyNumber.Identifier
            
                    ^IEE95 = RP:TrusteeOrPartner:Identifiers.AustralianRegisteredBodyNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(trusteeOrPartner.IEE34) != true && string.IsNullOrWhiteSpace(trusteeOrPartner.IEE95) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410222",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"An ARBN and an ACN must not both be provided",
                            Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:Identifiers.AustralianCompanyNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410222" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE34", Value = GetValueOrEmpty(trusteeOrPartner.IEE34) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE95", Value = GetValueOrEmpty(trusteeOrPartner.IEE95) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410222
                }
            }

            #endregion // Foreach loop

            #region Repeating report.DeclarationOfTrusteeOrPartnerCollection
            if (report.DeclarationOfTrusteeOrPartnerCollection != null)
            {
                int itemIndex1 = 0;
                foreach (IEE2018.DeclarationOfTrusteeOrPartner declarationOfTrusteeOrPartner in report.DeclarationOfTrusteeOrPartnerCollection)
                {
                    itemIndex1++;

                    #region VR.ATO.GEN.000413

                    /*  VR.ATO.GEN.000413
                    Orqanisation Name cannot contain 'space hyphen space' .
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE88, ' - ')
            
                    Data Elements:
            
                    ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrusteeOrPartner.IEE88 == null ? false : declarationOfTrusteeOrPartner.IEE88.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000413",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain space hyphen space combinations",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000413" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE88", Value = declarationOfTrusteeOrPartner.IEE88 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000413

                    #region VR.ATO.GEN.000414

                    /*  VR.ATO.GEN.000414
                    Organisation Name must not contain 'P/L'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE88, 'P/L')
            
                    Data Elements:
            
                    ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrusteeOrPartner.IEE88 == null ? false : declarationOfTrusteeOrPartner.IEE88.ToUpperInvariant().Contains(@"P/L"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000414",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""P/L""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000414" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE88", Value = declarationOfTrusteeOrPartner.IEE88 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000414

                    #region VR.ATO.GEN.000416

                    /*  VR.ATO.GEN.000416
                    Organisation Name must not contain repeated hyphen, apostrophe, or space characters.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ContainsSet(^IEE88, '"--","''","  "')
            
                    Data Elements:
            
                    ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE88, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000416",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000416" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE88", Value = declarationOfTrusteeOrPartner.IEE88 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000416

                    #region VR.ATO.GEN.000417

                    /*  VR.ATO.GEN.000417
                    Non-individualname must not end with a leading space followed by 'T/A', 'T/A P'ship', 'T/A Pship', 'T/A P/Ship' or 'T/A Partnership'.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    EndsWithSet(^IEE88, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"')
            
                    Data Elements:
            
                    ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE88, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000417",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000417" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE88", Value = declarationOfTrusteeOrPartner.IEE88 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000417

                    #region VR.ATO.GEN.000423

                    /*  VR.ATO.GEN.000423
                    Family Name must not contain 'space hyphen space' .
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE203, ' - ')
            
                    Data Elements:
            
                    ^IEE203 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrusteeOrPartner.IEE203 == null ? false : declarationOfTrusteeOrPartner.IEE203.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000423",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name cannot contain ""space hyphen space""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000423" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE203", Value = declarationOfTrusteeOrPartner.IEE203 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000423

                    #region VR.ATO.GEN.000427

                    /*  VR.ATO.GEN.000427
                    Individualname must not contain repeated hyphen, apostrophe, or space characters.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ContainsSet(^IEE203, '"--","''","  "')
            
                    Data Elements:
            
                    ^IEE203 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE203, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000427",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000427" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE203", Value = declarationOfTrusteeOrPartner.IEE203 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000427

                    #region VR.ATO.GEN.000434

                    /*  VR.ATO.GEN.000434
                    Given Name must not contain 'space hyphen space' .
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE204, ' - ')
            
                    Data Elements:
            
                    ^IEE204 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrusteeOrPartner.IEE204 == null ? false : declarationOfTrusteeOrPartner.IEE204.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000434",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000434" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE204", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE204) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000434

                    #region VR.ATO.GEN.000439

                    /*  VR.ATO.GEN.000439
                    Given name must not contain repeated hyphen, apostrophe, or space characters.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ContainsSet(^IEE204, '"--","''","  "')
            
                    Data Elements:
            
                    ^IEE204 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE204, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000439",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000439" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE204", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE204) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000439

                    #region VR.ATO.GEN.000446

                    /*  VR.ATO.GEN.000446
                    Other Given Name must not contain 'space hyphen space' .
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^IEE205, ' - ')
            
                    Data Elements:
            
                    ^IEE205 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrusteeOrPartner.IEE205 == null ? false : declarationOfTrusteeOrPartner.IEE205.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000446",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000446" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE205", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE205) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000446

                    #region VR.ATO.GEN.000451

                    /*  VR.ATO.GEN.000451
                    Individual name must not contain repeated hyphen, apostrophe, or space characters.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ContainsSet(^IEE205, '"--","''","  "')
            
                    Data Elements:
            
                    ^IEE205 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE205, @"--|''|  ", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000451",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000451" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE205", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE205) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000451

                    #region VR.ATO.GEN.000458

                    /*  VR.ATO.GEN.000458
                    Suffix must equal a code in the Individual Suffix Code table
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE202 <> BLANK) AND (NotInDomain(^IEE202, 'SUFFIX CODES'))
            
                    Data Elements:
            
                    ^IEE202 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrusteeOrPartner.IEE202) != true && !(IsMatch(declarationOfTrusteeOrPartner.IEE202, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000458",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Suffix is invalid",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.NameSuffix.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE202", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE202) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000458

                    #region VR.ATO.GEN.000459

                    /*  VR.ATO.GEN.000459
                    Title must equal a code in the Individual Title Code table
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE201 <> BLANK) AND (NotInDomain(^IEE201, 'TITLE CODES'))
            
                    Data Elements:
            
                    ^IEE201 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrusteeOrPartner.IEE201) != true && !(IsMatch(declarationOfTrusteeOrPartner.IEE201, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000459",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Title is invalid",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.Title.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE201", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE201) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.000459

                    #region VR.ATO.GEN.410038

                    /*  VR.ATO.GEN.410038
                    Non-individual Name cannot start with 'T/A' followed by a space and end with a space followed by 'Pship', 'P'ship or 'P/Ship'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (StartsWith(^IEE88, 'T/A ')) AND (EndsWithSet(^IEE88, '" Pship"," P'ship"," P/ship"'))
            
                    Data Elements:
            
                    ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (IsMatch(declarationOfTrusteeOrPartner.IEE88, @"^(T/A )", RegexOptions.IgnoreCase) && IsMatch(declarationOfTrusteeOrPartner.IEE88, @"( Pship| P'ship| P/ship)$", RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410038",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE88", Value = declarationOfTrusteeOrPartner.IEE88 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410038

                    #region VR.ATO.GEN.410039

                    /*  VR.ATO.GEN.410039
                    Organisation name must contain at least one alpha or numeric character.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE88  <> BLANK) AND (NotContainsSet(^IEE88, '"a-z", "A-Z", "0-9"'))
            
                    Data Elements:
            
                    ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrusteeOrPartner.IEE88) != true && !(IsMatch(declarationOfTrusteeOrPartner.IEE88, @"[a-z]|[A-Z]|[0-9]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410039",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must contain at least one alpha or numeric character",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE88", Value = declarationOfTrusteeOrPartner.IEE88 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410039

                    #region VR.ATO.GEN.410131

                    /*  VR.ATO.GEN.410131
                    Individual name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE205 <> BLANK) AND (NotContainsSet(^IEE205, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^IEE205 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrusteeOrPartner.IEE205) != true && !(IsMatch(declarationOfTrusteeOrPartner.IEE205, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410131",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE205", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE205) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410131

                    #region VR.ATO.GEN.410200

                    /*  VR.ATO.GEN.410200
                    Individualname must not contain 'Exec for', 'Rep for' or 'Trustee for'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE203, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^IEE203 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE203, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000424",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE203", Value = declarationOfTrusteeOrPartner.IEE203 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410200

                    #region VR.ATO.GEN.410201

                    /*  VR.ATO.GEN.410201
                    IndividualNamefield must not contain MR, MRS, MISS, MS with space before and after.
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE203, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^IEE203 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE203, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000426",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE203", Value = declarationOfTrusteeOrPartner.IEE203 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410201

                    #region VR.ATO.GEN.410203

                    /*  VR.ATO.GEN.410203
                    Given Name must not contain trustee reference
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE204, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^IEE204 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE204, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000437",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE204", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE204) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410203

                    #region VR.ATO.GEN.410204

                    /*  VR.ATO.GEN.410204
                    Given Name must not contain a title
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE204, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^IEE204 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE204, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000438",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE204", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE204) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410204

                    #region VR.ATO.GEN.410208

                    /*  VR.ATO.GEN.410208
                    Individualname must not contain 'Exec for', 'Rep for' or 'Trustee for'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE205, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^IEE205 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE205, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000449",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE205", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE205) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410208

                    #region VR.ATO.GEN.410209

                    /*  VR.ATO.GEN.410209
                    Other Given Name must not contain a title
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE205, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^IEE205 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE205, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000450",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE205", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE205) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410209

                    #region VR.ATO.GEN.410218

                    /*  VR.ATO.GEN.410218
                    Firstname must be entered if OtherGivenNames is entered
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                        RETURN VALIDATION MESSAGE 
                     ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND ((^IEE205 <> BLANK) AND (^IEE204 = BLANK))
            
                    Data Elements:
            
                    ^IEE205 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^IEE204 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails
                    */
                    assertion = (declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists != false && (string.IsNullOrWhiteSpace(declarationOfTrusteeOrPartner.IEE205) != true && string.IsNullOrWhiteSpace(declarationOfTrusteeOrPartner.IEE204) == true));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410218",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"FirstName must be entered if OtherGivenNames is entered",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE205", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE205) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE204", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE204) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.410218

                    #region VR.ATO.GEN.430255

                    /*  VR.ATO.GEN.430255
    
            Legacy Rule Format:
            IF pyin.xx.xx:Declaration.Signature.Date > DATE(TODAY)
               RETURN VALIDATION MESSAGE
            ENDIF
        
                    Technical Business Rule Format:
                    ^IEE90 > Today()
            
                    Data Elements:
            
                    ^IEE90 = RP:DeclarationOfTrusteeOrPartner:Declaration:Declaration.Signature.Date
                    */
                    assertion = (declarationOfTrusteeOrPartner.IEE90.GetValueOrDefault() > DateTime.Now.Date);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430255",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Signature date must not be later than today",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:Declaration/tns:Declaration.Signature.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430255" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE90", Value = GetValueOrEmptyDate(declarationOfTrusteeOrPartner.IEE90) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.430255

                    #region VR.ATO.GEN.500132

                    /*  VR.ATO.GEN.500132
                    Organisation name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^OrganisationNameDetails <> NULL) AND (^IEE88 = BLANK)
            
                    Data Elements:
            
                    ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^OrganisationNameDetails = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails
                    */
                    assertion = (declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(declarationOfTrusteeOrPartner.IEE88) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500132",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must be present.",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE88", Value = declarationOfTrusteeOrPartner.IEE88 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500132

                    #region VR.ATO.GEN.500143

                    /*  VR.ATO.GEN.500143
                    Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND (NotContainsSet(^IEE204, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^IEE204 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^PersonNameDetails = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails
                    */
                    assertion = (declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists != false && !(IsMatch(declarationOfTrusteeOrPartner.IEE204, @"[A-Z]|[a-z]", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500143",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Given Name must contain at least one alphabetical character.",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500143" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE204", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE204) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500143

                    #region VR.ATO.GEN.500151

                    /*  VR.ATO.GEN.500151
                    First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE204, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^IEE204 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE204, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010436",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500151" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE204", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE204) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500151

                    #region VR.ATO.GEN.500152

                    /*  VR.ATO.GEN.500152
                    Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE205, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^IEE205 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE205, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010448",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500152" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE205", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE205) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500152

                    #region VR.ATO.GEN.500153

                    /*  VR.ATO.GEN.500153
                    Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^IEE203, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^IEE203 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(declarationOfTrusteeOrPartner.IEE203, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)", RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500153",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.500153" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE203", Value = declarationOfTrusteeOrPartner.IEE203 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.GEN.500153

                    #region VR.ATO.IEE.410054

                    /*  VR.ATO.IEE.410054
                    Individual name and non-individual name of a trustee declaration must not both be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner)
                    IF ([IEE203] <> NULL) AND ([IEE88] <> NULL)
                      RETURN VALIDATION MESSAGE 
                    ENDIF
                    
                    [IEE88] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
                    [IEE203] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
        
                    Technical Business Rule Format:
                    (^IEE203 <> NULL) AND (^IEE88 <> NULL)
            
                    Data Elements:
            
                    ^IEE203 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrusteeOrPartner.IEE203 != null && declarationOfTrusteeOrPartner.IEE88 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410246",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Individual and non-individual name must not both be present in trustee or partner declaration",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410054" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE203", Value = declarationOfTrusteeOrPartner.IEE203 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE88", Value = declarationOfTrusteeOrPartner.IEE88 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410054

                    #region VR.ATO.IEE.410247

                    /*  VR.ATO.IEE.410247
                    Individual name and non-individual name of a trustee declaration must not both be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner)
                    IF ([IEE203] = NULL) AND ([IEE88] = NULL)
                      RETURN VALIDATION MESSAGE 
                    ENDIF
                    
                    [IEE88] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
                    [IEE203] = IEE:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "DeclarationSignatory" AND pyde.02.00:PersonNameDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:PersonNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
        
                    Technical Business Rule Format:
                    (^IEE203 = NULL) AND (^IEE88 = NULL)
            
                    Data Elements:
            
                    ^IEE203 = RP:DeclarationOfTrusteeOrPartner:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            
                    ^IEE88 = RP:DeclarationOfTrusteeOrPartner:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (declarationOfTrusteeOrPartner.IEE203 == null && declarationOfTrusteeOrPartner.IEE88 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410247",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee or partner declaration must contain either an individual name or a non-individual name",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410247" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE203", Value = declarationOfTrusteeOrPartner.IEE203 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE88", Value = declarationOfTrusteeOrPartner.IEE88 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410247

                    #region VR.ATO.IEE.410252

                    /*  VR.ATO.IEE.410252
                    Statement Accepted Indicator must be  'TRUE'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner\declaration2.xx.xx:Declaration)
                    IF [IEE146] = FALSE
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE146] = IEE:RP:pyin.02.28:Declaration.StatementAccepted.Indicator IN TUPLE(iee.0003.lodge.req.02.00:Declaration) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
        
                    Technical Business Rule Format:
                    ^IEE146 = FALSE
            
                    Data Elements:
            
                    ^IEE146 = RP:DeclarationOfTrusteeOrPartner:Declaration:Declaration.StatementAccepted.Indicator
                    */
                    assertion = (declarationOfTrusteeOrPartner.IEE146 == false);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410252",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee or partner declaration - Statement Accepted Indicator must be 'TRUE'",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:Declaration/tns:Declaration.StatementAccepted.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410252" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE146", Value = GetValueOrEmpty(declarationOfTrusteeOrPartner.IEE146) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410252

                    #region VR.ATO.IEE.410254

                    /*  VR.ATO.IEE.410254
                    Declaration Statement Type Code must be either 'HardCopy' 'TrueAndCorrect'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(iee.0003.lodge.req.xx.xx:DeclarationOfTrusteeOrPartner\declaration2.xx.xx:Declaration)
                    IF (pyin.xx.xx:Declaration.StatementType.Code <> NULLORBLANK)
                    AND (pyin.xx.xx:Declaration.StatementType.Code  <> SET ("HardCopy", "TrueAndCorrect" ))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IEE193 <> BLANK) AND NotInSet(^IEE193, '"HardCopy", "TrueAndCorrect"')
            
                    Data Elements:
            
                    ^IEE193 = RP:DeclarationOfTrusteeOrPartner:Declaration:Declaration.StatementType.Code
                    */
                    assertion = (string.IsNullOrWhiteSpace(declarationOfTrusteeOrPartner.IEE193) != true && !(IsMatch(declarationOfTrusteeOrPartner.IEE193, @"^(HardCopy|TrueAndCorrect)$", RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410254",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee or partner declaration -Statement Type Code must be either 'TrueAndCorrect' or 'HardCopy'",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:Declaration/tns:Declaration.StatementType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410254" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE193", Value = declarationOfTrusteeOrPartner.IEE193 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410254

                    #region VR.ATO.IEE.410260

                    /*  VR.ATO.IEE.410260
                    If Entity type is Company, trustee or partner declaration must not be present.
    
                    Legacy Rule Format:
                    IF ([IEE16] = "051" AND [IEE193] <> NULLORBLANK) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE16] = IEE:RP:pyde.02.21:Party.Type.Code
                    [IEE193] = IEE:RP:pyin.02.03:Declaration.StatementType.Code IN TUPLE(iee.0003.lodge.req.02.00:Declaration) IN TUPLE(iee.0003.lodge.req.02.00:DeclarationOfTrusteeOrPartner)
        
                    Technical Business Rule Format:
                    ^IEE16 = '051' AND ^IEE193 <> BLANK
            
                    Data Elements:
            
                    ^IEE193 = RP:DeclarationOfTrusteeOrPartner:Declaration:Declaration.StatementType.Code
            
                    ^IEE16 = RP:Party.Type.Code
                    */
                    assertion = (report.IEE16 == @"051" && string.IsNullOrWhiteSpace(declarationOfTrusteeOrPartner.IEE193) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410260",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If Entity type is Company, trustee or partner declaration must not be present",
                            Location = "/xbrli:xbrl/tns:DeclarationOfTrusteeOrPartner" + OccurrenceIndex(declarationOfTrusteeOrPartner.OccurrenceIndex) + "/tns:Declaration/tns:Declaration.StatementType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410260" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE16", Value = report.IEE16 });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE193", Value = declarationOfTrusteeOrPartner.IEE193 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410260
                }
            }

            #endregion // Foreach loop

            #region Repeating report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection
            if (report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection != null)
            {
                int itemIndex1 = 0;
                foreach (IEE2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP periodDetailsControlOutsideAustraliaPartPeriod in report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection)
                {
                    itemIndex1++;

                    #region VR.ATO.IEE.410050

                    /*  VR.ATO.IEE.410050
                    From Date for Central Mngmnt must not be before Election Cmcmnt or Day Specified for the election
    
                    Legacy Rule Format:
                    IF ([IEE19] < [IEE46]) OR ([IEE19] < [IEE49]) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE19] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
                    [IEE46] = IEE:RP:rvctc3.02.00:Elections.DateSpecified.Date
                    [IEE49] = IEE:RP:rvctc3.02.00:Elections.Commencement.Date
        
                    Technical Business Rule Format:
                    (^IEE19 < ^IEE46) OR (^IEE19 < ^IEE49)
            
                    Data Elements:
            
                    ^IEE19 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            
                    ^IEE46 = RP:Elections.DateSpecified.Date
            
                    ^IEE49 = RP:Elections.Commencement.Date
                    */
                    assertion = (periodDetailsControlOutsideAustraliaPartPeriod.IEE19.GetValueOrDefault() < report.IEE46.GetValueOrDefault() || periodDetailsControlOutsideAustraliaPartPeriod.IEE19.GetValueOrDefault() < report.IEE49.GetValueOrDefault());
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410050",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"From Date for Central Mngmnt must not be before Election Cmcmnt or Day Specified for the election",
                            LongDescription = @"The from date for central management and control of trust was outside Australia during the relevant period (Section A Item 5a) is earlier than the election commencement date (Section A Item 8) or the day specified (Section A Item 7) for the interposed entity election",
                            Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(periodDetailsControlOutsideAustraliaPartPeriod.PeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410050" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE19", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.IEE19) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE46", Value = GetValueOrEmptyDate(report.IEE46) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE49", Value = GetValueOrEmptyDate(report.IEE49) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410050

                    #region VR.ATO.IEE.410068

                    /*  VR.ATO.IEE.410068
                    End Date must be after a Start date
    
                    Legacy Rule Format:
                    IN TUPLE(period1.xx.xx:PeriodDetails)
                    IF pyin.xx.xx:PeriodStart.Date >= pyin.xx.xx:PeriodEnd.Date
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^IEE19 >= ^IEE20
            
                    Data Elements:
            
                    ^IEE19 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            
                    ^IEE20 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
                    */
                    assertion = (periodDetailsControlOutsideAustraliaPartPeriod.IEE19.GetValueOrDefault() >= periodDetailsControlOutsideAustraliaPartPeriod.IEE20.GetValueOrDefault());
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410068",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"End Date must be after a Start date",
                            Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(periodDetailsControlOutsideAustraliaPartPeriod.PeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410068" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE19", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.IEE19) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE20", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.IEE20) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410068

                    #region VR.ATO.IEE.410088

                    /*  VR.ATO.IEE.410088
                    Full period indicator is 'True' and specific dates have been entered for central management and control of trust was outside Australia.
    
                    Legacy Rule Format:
                    IF ([IEE18] = "TRUE" AND ([IEE19] <> NULL) AND ([IEE20] <> NULL) )
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE18] = IEE:RP:pyde.02.00:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
                    [IEE19] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
                    [IEE20] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "ControlOutsideAustraliaPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
        
                    Technical Business Rule Format:
                    (^IEE18 = 'TRUE') AND (^IEE19 <> NULL) AND (^IEE20 <> NULL)
            
                    Data Elements:
            
                    ^IEE18 = RP:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator
            
                    ^IEE19 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
            
                    ^IEE20 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='ControlOutsideAustraliaPartPeriod']
                    */
                    assertion = (report.IEE18 == true && periodDetailsControlOutsideAustraliaPartPeriod.IEE19 != null && periodDetailsControlOutsideAustraliaPartPeriod.IEE20 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410088",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"If Full period indicator is 'True', specific dates must not be present",
                            LongDescription = @"Full period indicator is 'true' and and specific dates have been entered for central management and control of trust was outside Australia. Either delete dates or the full period indicator",
                            Location = "/xbrli:xbrl/tns:Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410088" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE18", Value = GetValueOrEmpty(report.IEE18) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE19", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.IEE19) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE20", Value = GetValueOrEmptyDate(periodDetailsControlOutsideAustraliaPartPeriod.IEE20) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410088
                }
            }

            #endregion // Foreach loop

            #region Repeating report.PeriodDetailsNonResidentPartPeriodRPCollection
            if (report.PeriodDetailsNonResidentPartPeriodRPCollection != null)
            {
                int itemIndex1 = 0;
                foreach (IEE2018.PeriodDetailsNonResidentPartPeriodRP periodDetailsNonResidentPartPeriod in report.PeriodDetailsNonResidentPartPeriodRPCollection)
                {
                    itemIndex1++;

                    #region VR.ATO.IEE.410056

                    /*  VR.ATO.IEE.410056
                    Date company was non-resident must not be before the Day Specified or the Election Commencement Time
    
                    Legacy Rule Format:
                    IF (([IEE23] < [IEE46]) OR ([IEE23] < [IEE49])) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE23] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
                    [IEE46] = IEE:RP:rvctc3.02.00:Elections.DateSpecified.Date
                    [IEE49] = IEE:RP:rvctc3.02.00:Elections.Commencement.Date
        
                    Technical Business Rule Format:
                    ((^IEE23 < ^IEE46) OR (^IEE23 < ^IEE49))
            
                    Data Elements:
            
                    ^IEE23 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
            
                    ^IEE46 = RP:Elections.DateSpecified.Date
            
                    ^IEE49 = RP:Elections.Commencement.Date
                    */
                    assertion = (periodDetailsNonResidentPartPeriod.IEE23.GetValueOrDefault() < report.IEE46.GetValueOrDefault() || periodDetailsNonResidentPartPeriod.IEE23.GetValueOrDefault() < report.IEE49.GetValueOrDefault());
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410056",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Date company was non-resident must not be before the Day Specified or the Election Commencement Time",
                            Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(periodDetailsNonResidentPartPeriod.PeriodDetailsCollection_NonResidentPartPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410056" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE23", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.IEE23) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE46", Value = GetValueOrEmptyDate(report.IEE46) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE49", Value = GetValueOrEmptyDate(report.IEE49) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410056

                    #region VR.ATO.IEE.410068

                    /*  VR.ATO.IEE.410068
                    End Date must be after a Start date
    
                    Legacy Rule Format:
                    IN TUPLE(period1.xx.xx:PeriodDetails)
                    IF pyin.xx.xx:PeriodStart.Date >= pyin.xx.xx:PeriodEnd.Date
                      RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^IEE23 >= ^IEE24
            
                    Data Elements:
            
                    ^IEE23 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
            
                    ^IEE24 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                    */
                    assertion = (periodDetailsNonResidentPartPeriod.IEE23.GetValueOrDefault() >= periodDetailsNonResidentPartPeriod.IEE24.GetValueOrDefault());
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410068",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"End Date must be after a Start date",
                            Location = "/xbrli:xbrl/tns:PeriodDetails" + OccurrenceIndex(periodDetailsNonResidentPartPeriod.PeriodDetailsCollection_NonResidentPartPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410068" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE23", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.IEE23) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE24", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.IEE24) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410068

                    #region VR.ATO.IEE.410091

                    /*  VR.ATO.IEE.410091
                    Both full period indicator and specific dates company was a non resident for tax purposes are present when only one is allowed.
    
                    Legacy Rule Format:
                    IF ([IEE22] = TRUE) AND ([IEE23] <> NULL) AND ([IEE24] <> NULL) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE22] = IEE:RP:pyde.02.00:Residency.NonResidentFullPeriod.Indicator
                    [IEE23] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
                    [IEE24] = IEE:RP:pyin.02.00:Period.End.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails)
        
                    Technical Business Rule Format:
                    (^IEE22 = TRUE) AND (^IEE23 <> NULL) AND (^IEE24 <> NULL)
            
                    Data Elements:
            
                    ^IEE22 = RP:Residency.NonResidentFullPeriod.Indicator
            
                    ^IEE23 = RP:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
            
                    ^IEE24 = RP:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                    */
                    assertion = (report.IEE22 == true && periodDetailsNonResidentPartPeriod.IEE23 != null && periodDetailsNonResidentPartPeriod.IEE24 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410091",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Full period indicator and specific dates company was a non resident must not both be present",
                            Location = "/xbrli:xbrl/tns:Residency.NonResidentFullPeriod.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410091" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE22", Value = GetValueOrEmpty(report.IEE22) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE23", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.IEE23) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE24", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.IEE24) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IEE.410091
                }
            }

            #endregion // Foreach loop

            #region Repeating report.TrusteeOrPartnerCollection
            if (report.TrusteeOrPartnerCollection != null)
            {
                int itemIndex1 = 0;
                foreach (IEE2018.TrusteeOrPartner trusteeOrPartner in report.TrusteeOrPartnerCollection)
                {
                    itemIndex1++;

                    #region Repeating trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection
                    if (trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection != null)
                    {
                        int itemIndex2 = 0;
                        foreach (IEE2018.TrusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRP periodDetailsNonResidentPartPeriod in trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection)
                        {
                            itemIndex2++;

                            #region VR.ATO.IEE.410057

                            /*  VR.ATO.IEE.410057
                            The from date of the relevant period at which the trustee or partner was a non-resident for tax purposes (Section A Item 6) cannot be earlier than either the day specified for the interposed entity election (Section A Item 7) or the election commencement date (Section A Item 8).
            
                            Legacy Rule Format:
                            IF ([IEE43] < [IEE46]) OR ([IEE43] < [IEE49]) 
                                RETURN VALIDATION MESSAGE
                            ENDIF
                            
                            [IEE43] = IEE:RP:pyin.02.00:Period.Start.Date WHERE (pyin.02.05:Period.Type.Code = "NonResidentPartPeriod") IN TUPLE(iee.0003.lodge.req.02.00:PeriodDetails) IN TUPLE(iee.0003.lodge.req.02.00:TrusteeOrPartner)
                            [IEE46] = IEE:RP:rvctc3.02.00:Elections.DateSpecified.Date
                            [IEE49] = IEE:RP:rvctc3.02.00:Elections.Commencement.Date
                
                            Technical Business Rule Format:
                            (^IEE43 < ^IEE46) OR (^IEE43 < ^IEE49)
                    
                            Data Elements:
                    
                            ^IEE43 = RP:TrusteeOrPartner:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                    
                            ^IEE46 = RP:Elections.DateSpecified.Date
                    
                            ^IEE49 = RP:Elections.Commencement.Date
                            */
                            assertion = (periodDetailsNonResidentPartPeriod.IEE43.GetValueOrDefault() < report.IEE46.GetValueOrDefault() || periodDetailsNonResidentPartPeriod.IEE43.GetValueOrDefault() < report.IEE49.GetValueOrDefault());
                            if (assertion)
                            {
                                processMessage = new ProcessMessageDocument()
                                {
                                    Code = "CMN.ATO.IEE.410057",
                                    Severity = ProcessMessageSeverity.Error,
                                    Description = @"Date trustee/partner was non-resident must not be before Day Specified or the Election Commence Date",
                                    LongDescription = @"The from date of the relevant period at which the trustee or partner was a non-resident for tax purposes (Section A Item 6) cannot be earlier than the either the day specified for the interposed entity election (Section A Item 7) or the election commencement date (Section A Item 8)",
                                    Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PeriodDetails" + OccurrenceIndex(periodDetailsNonResidentPartPeriod.TrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410057" } },
                                };
                                processMessage.Parameters.Add(new ProcessMessageParameter
                                { Name = "IEE43", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.IEE43) });

                                processMessage.Parameters.Add(new ProcessMessageParameter
                                { Name = "IEE46", Value = GetValueOrEmptyDate(report.IEE46) });

                                processMessage.Parameters.Add(new ProcessMessageParameter
                                { Name = "IEE49", Value = GetValueOrEmptyDate(report.IEE49) });

                                response.Add(processMessage);
                            }
                            #endregion // VR.ATO.IEE.410057

                            #region VR.ATO.IEE.410068

                            /*  VR.ATO.IEE.410068
                            End Date must be after a Start date
            
                            Legacy Rule Format:
                            IN TUPLE(period1.xx.xx:PeriodDetails)
                            IF pyin.xx.xx:PeriodStart.Date >= pyin.xx.xx:PeriodEnd.Date
                              RETURN VALIDATION MESSAGE
                            ENDIF
                
                            Technical Business Rule Format:
                            ^IEE43 >= ^IEE44
                    
                            Data Elements:
                    
                            ^IEE43 = RP:TrusteeOrPartner:PeriodDetails:Period.Start.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                    
                            ^IEE44 = RP:TrusteeOrPartner:PeriodDetails:Period.End.Date WHERE [Period.Type.Code='NonResidentPartPeriod']
                            */
                            assertion = (periodDetailsNonResidentPartPeriod.IEE43.GetValueOrDefault() >= periodDetailsNonResidentPartPeriod.IEE44.GetValueOrDefault());
                            if (assertion)
                            {
                                processMessage = new ProcessMessageDocument()
                                {
                                    Code = "CMN.ATO.IEE.410068",
                                    Severity = ProcessMessageSeverity.Error,
                                    Description = @"End Date must be after a Start date",
                                    Location = "/xbrli:xbrl/tns:TrusteeOrPartner" + OccurrenceIndex(trusteeOrPartner.OccurrenceIndex) + "/tns:PeriodDetails" + OccurrenceIndex(periodDetailsNonResidentPartPeriod.TrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPOccurrenceIndex) + "/tns:Period.Start.Date",
                                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410068" } },
                                };
                                processMessage.Parameters.Add(new ProcessMessageParameter
                                { Name = "IEE43", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.IEE43) });

                                processMessage.Parameters.Add(new ProcessMessageParameter
                                { Name = "IEE44", Value = GetValueOrEmptyDate(periodDetailsNonResidentPartPeriod.IEE44) });

                                response.Add(processMessage);
                            }
                            #endregion // VR.ATO.IEE.410068
                        }
                    }

                    #endregion // Foreach loop
                }
            }

            #endregion // Foreach loop

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Value = _emptyParameterValue;
                        }

                        if (currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }


    }
}
