using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Text.RegularExpressions;
using Au.Gov.Sbr.Xbrl.Document;
using DataContracts;
using Au.Gov.Sbr.Xbrl.Metadata;

namespace Ato.EN.IntegrationServices.CodeGenerationFTER
{
    public class FTER2018Consumer
    {

        private static Regex StripWhitespace = new Regex(@"\s*");

        #region Helper/Validation Methods
        private static bool? ToBoolean(string str)
        {
            bool returnValue;
            string value = (str ?? "").Trim();
            if (value == "0" || value == "1")
                return Convert.ToBoolean(Convert.ToInt32(value));
            
            if (Boolean.TryParse(value, out returnValue))
                return returnValue;
            return null;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(DateTime? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Derives the SBR element location.
        /// </summary>
        /// <param name="sbrElement">The SBR element.</param>
        /// <param name="addOccurrenceIndex">if set to <c>true</c> [add occurrence the occurrence index to the path].</param>
        /// <returns></returns>
        private static string DeriveSbrElementLocation(SBRElement sbrElement, bool addOccurrenceIndex = false)
        {
            if (sbrElement == null) return "/xbrli:xbrl";
            int xpathOccurrence = sbrElement.OccurrenceIndex + 1;
            string location;

            if (addOccurrenceIndex)
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name + "[" +
                           xpathOccurrence + "]";
            }
            else
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name;
            }

            if (sbrElement.IsInTuple)
            {
                location = DeriveSbrElementLocation(sbrElement.ParentTuple, true) + location;
            }
            else
            {
                location = "/xbrli:xbrl" + location;
            }

            return location;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

         #endregion

        public FTER2018 Consume(SBRDocument sbrDocument)
        {
            List<ProcessMessageDocument> errors;
            return Consume(sbrDocument, false, out errors);
        }

        /// <summary>
        /// Allows for validation of rules that can not be generated and must run at a lower level.
        /// A sub-class will need to be created with an override for this method - which does nothing in its base implementation.
        /// </summary>
        /// <param name="sbrDocument">The SBR document.</param>
        /// <param name="populatedReportElementsMap">The populated report elements map can be used to find data elements and tuples.</param>
        /// <param name="contextArray">The context array can be used to find context values.</param>
        /// <param name="errors">The errors collection is to be populated with validation results.</param>
        protected virtual void ValidateUnsupportedRules(SBRDocument sbrDocument, IDictionary<string, SBRElement> populatedReportElementsMap, SBRContext[] contextArray, List<ProcessMessageDocument> errors)
        {

        }

        public FTER2018 Consume(SBRDocument sbrDocument, bool validationMode, out List<ProcessMessageDocument> errors)
        {
            FTER2018 report;
            report = new FTER2018();
            errors = new List<ProcessMessageDocument>();

            SBRContext[] contextArray;
            IDictionary<string, SBRElement> populatedReportElementsMap;

            contextArray = sbrDocument.ContextsMap.Values.ToArray();
            populatedReportElementsMap = sbrDocument.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

            if (validationMode) ValidateUnsupportedRules(sbrDocument, populatedReportElementsMap, contextArray, errors);

            /// <summary>
            /// Current element being extracted
            /// </summary>
            SBRElement currentValue = null;
            /// <summary>
            /// Current elements being extracted
            /// </summary>
            IList<SBRElement> currentValues = null;
            /// <summary>
            /// Current occurrences being extracted
            /// </summary>
            IList<SBRElement> currentOccurrences = null;
            /// <summary>
            /// Working Variable for date time variables
            /// </summary>
            DateTime currentDateTimeValue;
            /// <summary>
            /// Working Variable for boolean variables
            /// </summary>
            bool currentBooleanValue;
            /// <summary>
            /// Working Variable for decimal variables
            /// </summary>
            decimal currentDecimalValue;
            /// <summary>
            /// Working Variable for integer variables
            /// </summary>
            int currentIntValue;
            /// <summary>
            /// Working Variable for long variables
            /// </summary>
            long currentLongValue;
            XmlDocumentFragment currentSegment;
            XmlNamespaceManager currentNamespaceManager;
            XmlNode currentNode;
    
            #region sbrINTContext
            IEnumerable<SBRContext> sbrINTContexts = null;
            SBRContext sbrINTContext = null;
    
            sbrINTContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Intermediary")) ));
            Dictionary<string, IList<SBRElement>> sbrPopulatedINTElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrINTContexts != null && sbrINTContexts.Count() > 0)
            {
                sbrINTContext = sbrINTContexts.First();
        
                report.INTId = sbrINTContext.Id;
                report.INTIdentifier = sbrINTContext.EntityIdentifier;
                report.INTIdentifierScheme = sbrINTContext.EntityScheme;
        
                report.INTCount = sbrINTContexts.Count();
                report.INTExists = true;
                report.INTOccurrenceIndex = Array.IndexOf(contextArray, sbrINTContext) + 1;
                report.INTLastOccurrenceIndex = Array.IndexOf(contextArray, sbrINTContexts.Last()) + 1;
        

                if (report.INTIdentifierScheme == "http://www.ato.gov.au/abn") report.INTIdentifierABN = report.INTIdentifier;
    
                report.INTStartDate = ((SBRDurationPeriod)sbrINTContext.Period).DurationStartAsDateObject;
                report.INTEndDate = ((SBRDurationPeriod)sbrINTContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedINTElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrINTContexts.Count(); i++)
                {
                    sbrINTContext = sbrINTContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrINTContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedINTElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedINTElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedINTElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
        
                    IEnumerable<SBRElement> sbrDeclarationCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration", out currentValue))
                        sbrDeclarationCollection = currentValue.Occurrences;
                    else
                        sbrDeclarationCollection = null;
            
                    #region sbrDeclarationCollection
            
                    if (sbrDeclarationCollection != null)
                    {
                        report.DeclarationCollectionExists = true;
                        report.DeclarationCollectionCount = sbrDeclarationCollection.Count();
                
                        IEnumerable<SBRElement> sbrDeclarationCollection_TrueAndCorrect_INTCollection;
                        sbrDeclarationCollection_TrueAndCorrect_INTCollection = sbrDeclarationCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "Declaration.StatementType.Code" && te.Value == "TrueAndCorrect" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrDeclarationCollection_TrueAndCorrect_INTCollection != null && sbrDeclarationCollection_TrueAndCorrect_INTCollection.Count() > 0)
                        {
                            SBRElement sbrDeclarationCollection_TrueAndCorrect_INT;
                            sbrDeclarationCollection_TrueAndCorrect_INT = sbrDeclarationCollection_TrueAndCorrect_INTCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap;
                            sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap = sbrDeclarationCollection_TrueAndCorrect_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.DeclarationCollection_TrueAndCorrect_INTExists = true;
                            report.DeclarationCollection_TrueAndCorrect_INTCount = sbrDeclarationCollection_TrueAndCorrect_INTCollection.Count();
                            report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_INT.OccurrenceIndex + 1;
                            report.DeclarationCollection_TrueAndCorrect_INTLastOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_INTCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.StatementType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER159 = currentValue.Value;
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.StatementAccepted.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER160 = ToBoolean(currentValue.Value);
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.Statement.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER161 = currentValue.Value;
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.Signature.Date", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER162 = DateTime.Parse(currentValue.Value);
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.SignatoryIdentifier.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER163 = currentValue.Value;
                            }
                
                            IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection;
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName", out currentValue))
                                sbrDeclaration_PersonUnstructuredNameCollection = currentValue.Occurrences;
                            else
                                sbrDeclaration_PersonUnstructuredNameCollection = null;
                    
                            #region sbrDeclaration_PersonUnstructuredNameCollection
                    
                            if (sbrDeclaration_PersonUnstructuredNameCollection != null)
                            {
                                report.Declaration_PersonUnstructuredNameCollectionExists = true;
                                report.Declaration_PersonUnstructuredNameCollectionCount = sbrDeclaration_PersonUnstructuredNameCollection.Count();
                        
                                IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection;
                                sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection = sbrDeclaration_PersonUnstructuredNameCollection.Where(
                                    ce => ce.ChildElements != null && 
                                    ce.ChildElements.Any(te => te.Name == "PersonUnstructuredName.Usage.Code" && te.Value == "DeclarationSignatory" &&
                                    sbrINTContexts.Contains(te.Context)));
                                if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection != null && sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.Count() > 0)
                                {
                                    SBRElement sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.ElementAt(0);
                        
                                    IDictionary<string, SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                        
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTExists = true;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCount = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.Count();
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT.OccurrenceIndex + 1;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTLastOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.Last().OccurrenceIndex + 1;
                            
                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.Usage.Code", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.FTER164 = currentValue.Value;
                                    }
                            
                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.FullName.Text", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.FTER165 = currentValue.Value;
                                    }
                                }  
                            } 
                            #endregion End of Tuple sbrDeclaration_PersonUnstructuredNameCollection
                            if (validationMode) ValidateINTDeclaration_PersonUnstructuredNameCollection(errors, sbrDeclaration_PersonUnstructuredNameCollection, sbrINTContexts, report);
                    
                        }  
                    } 
                    #endregion End of Tuple sbrDeclarationCollection
                    if (validationMode) ValidateINTDeclarationCollection(errors, sbrDeclarationCollection, sbrINTContexts, report);
            
            
                    if (sbrPopulatedINTElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.TaxAgentNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER2 = currentValues[0].Value;
                    }
        
                    IEnumerable<SBRElement> sbrPersonUnstructuredNameCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName", out currentValue))
                        sbrPersonUnstructuredNameCollection = currentValue.Occurrences;
                    else
                        sbrPersonUnstructuredNameCollection = null;
            
                    #region sbrPersonUnstructuredNameCollection
            
                    if (sbrPersonUnstructuredNameCollection != null)
                    {
                        report.PersonUnstructuredNameCollectionExists = true;
                        report.PersonUnstructuredNameCollectionCount = sbrPersonUnstructuredNameCollection.Count();
                
                        IEnumerable<SBRElement> sbrPersonUnstructuredNameCollection_Contact_INTCollection;
                        sbrPersonUnstructuredNameCollection_Contact_INTCollection = sbrPersonUnstructuredNameCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "PersonUnstructuredName.Usage.Code" && te.Value == "Contact" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrPersonUnstructuredNameCollection_Contact_INTCollection != null && sbrPersonUnstructuredNameCollection_Contact_INTCollection.Count() > 0)
                        {
                            SBRElement sbrPersonUnstructuredNameCollection_Contact_INT;
                            sbrPersonUnstructuredNameCollection_Contact_INT = sbrPersonUnstructuredNameCollection_Contact_INTCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrPersonUnstructuredNameCollection_Contact_INTTupleElementMap;
                            sbrPersonUnstructuredNameCollection_Contact_INTTupleElementMap = sbrPersonUnstructuredNameCollection_Contact_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.PersonUnstructuredNameCollection_Contact_INTExists = true;
                            report.PersonUnstructuredNameCollection_Contact_INTCount = sbrPersonUnstructuredNameCollection_Contact_INTCollection.Count();
                            report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex = sbrPersonUnstructuredNameCollection_Contact_INT.OccurrenceIndex + 1;
                            report.PersonUnstructuredNameCollection_Contact_INTLastOccurrenceIndex = sbrPersonUnstructuredNameCollection_Contact_INTCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrPersonUnstructuredNameCollection_Contact_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER156 = currentValue.Value;
                            }
                    
                            if (sbrPersonUnstructuredNameCollection_Contact_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.FullName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER82 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrPersonUnstructuredNameCollection
                    if (validationMode) ValidateINTPersonUnstructuredNameCollection(errors, sbrPersonUnstructuredNameCollection, sbrINTContexts, report);
            
        
                    IEnumerable<SBRElement> sbrElectronicContactTelephoneCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}ElectronicContactTelephone", out currentValue))
                        sbrElectronicContactTelephoneCollection = currentValue.Occurrences;
                    else
                        sbrElectronicContactTelephoneCollection = null;
            
                    #region sbrElectronicContactTelephoneCollection
            
                    if (sbrElectronicContactTelephoneCollection != null)
                    {
                        report.ElectronicContactTelephoneCollectionExists = true;
                        report.ElectronicContactTelephoneCollectionCount = sbrElectronicContactTelephoneCollection.Count();
                
                        IEnumerable<SBRElement> sbrElectronicContactTelephoneCollection_03_INTCollection;
                        sbrElectronicContactTelephoneCollection_03_INTCollection = sbrElectronicContactTelephoneCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "ElectronicContact.Telephone.Usage.Code" && te.Value == "03" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrElectronicContactTelephoneCollection_03_INTCollection != null && sbrElectronicContactTelephoneCollection_03_INTCollection.Count() > 0)
                        {
                            SBRElement sbrElectronicContactTelephoneCollection_03_INT;
                            sbrElectronicContactTelephoneCollection_03_INT = sbrElectronicContactTelephoneCollection_03_INTCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrElectronicContactTelephoneCollection_03_INTTupleElementMap;
                            sbrElectronicContactTelephoneCollection_03_INTTupleElementMap = sbrElectronicContactTelephoneCollection_03_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.ElectronicContactTelephoneCollection_03_INTExists = true;
                            report.ElectronicContactTelephoneCollection_03_INTCount = sbrElectronicContactTelephoneCollection_03_INTCollection.Count();
                            report.ElectronicContactTelephoneCollection_03_INTOccurrenceIndex = sbrElectronicContactTelephoneCollection_03_INT.OccurrenceIndex + 1;
                            report.ElectronicContactTelephoneCollection_03_INTLastOccurrenceIndex = sbrElectronicContactTelephoneCollection_03_INTCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}ElectronicContact.Telephone.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER157 = currentValue.Value;
                            }
                    
                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}ElectronicContact.Telephone.ServiceLine.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER158 = currentValue.Value;
                            }
                    
                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}ElectronicContact.Telephone.Area.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER83 = currentValue.Value;
                            }
                    
                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}ElectronicContact.Telephone.Minimal.Number", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER84 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrElectronicContactTelephoneCollection
                    if (validationMode) ValidateINTElectronicContactTelephoneCollection(errors, sbrElectronicContactTelephoneCollection, sbrINTContexts, report);
            
                } 
            } 
            #endregion End of Context sbrINTContext
    
            #region sbrRPContext
            IEnumerable<SBRContext> sbrRPContexts = null;
            SBRContext sbrRPContext = null;
    
            sbrRPContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) ));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPContexts != null && sbrRPContexts.Count() > 0)
            {
                sbrRPContext = sbrRPContexts.First();
        
                report.RPId = sbrRPContext.Id;
                report.RPIdentifier = sbrRPContext.EntityIdentifier;
                report.RPIdentifierScheme = sbrRPContext.EntityScheme;
        
                report.RPCount = sbrRPContexts.Count();
                report.RPExists = true;
                report.RPOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContext) + 1;
                report.RPLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContexts.Last()) + 1;
        

                if (report.RPIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPIdentifierTFN = report.RPIdentifier;
    
                report.RPStartDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationStartAsDateObject;
                report.RPEndDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPContexts.Count(); i++)
                {
                    sbrRPContext = sbrRPContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
                    report.FTER3 = report.RPIdentifier;
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.AustralianBusinessNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER170 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Lodgment.PeriodYear.Year", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER1 = int.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Elections.FamilyTrustElectionRevocation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER95 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Report.CompletionHours.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER85 = decimal.Parse(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails", out currentValue))
                        sbrOrganisationNameDetailsCollection = currentValue.Occurrences;
                    else
                        sbrOrganisationNameDetailsCollection = null;
            
                    #region sbrOrganisationNameDetailsCollection
            
                    if (sbrOrganisationNameDetailsCollection != null)
                    {
                        report.OrganisationNameDetailsCollectionExists = true;
                        report.OrganisationNameDetailsCollectionCount = sbrOrganisationNameDetailsCollection.Count();
                
                        IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection_MN_C_RPCollection;
                        sbrOrganisationNameDetailsCollection_MN_C_RPCollection = sbrOrganisationNameDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrOrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrOrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                        {
                            SBRElement sbrOrganisationNameDetailsCollection_MN_C_RP;
                            sbrOrganisationNameDetailsCollection_MN_C_RP = sbrOrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                            sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrOrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.OrganisationNameDetailsCollection_MN_C_RPExists = true;
                            report.OrganisationNameDetailsCollection_MN_C_RPCount = sbrOrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                            report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                            report.OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER103 = currentValue.Value;
                            }
                    
                            if (sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER102 = currentValue.Value;
                            }
                    
                            if (sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER4 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrOrganisationNameDetailsCollection
                    if (validationMode) ValidateRPOrganisationNameDetailsCollection(errors, sbrOrganisationNameDetailsCollection, sbrRPContexts, report);
            
        
                    IEnumerable<SBRElement> sbrAddressDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails", out currentValue))
                        sbrAddressDetailsCollection = currentValue.Occurrences;
                    else
                        sbrAddressDetailsCollection = null;
            
                    #region sbrAddressDetailsCollection
            
                    if (sbrAddressDetailsCollection != null)
                    {
                        report.AddressDetailsCollectionExists = true;
                        report.AddressDetailsCollectionCount = sbrAddressDetailsCollection.Count();
                
                        IEnumerable<SBRElement> sbrAddressDetailsCollection_POS_C_RPCollection;
                        sbrAddressDetailsCollection_POS_C_RPCollection = sbrAddressDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "POS") &&
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "C" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrAddressDetailsCollection_POS_C_RPCollection != null && sbrAddressDetailsCollection_POS_C_RPCollection.Count() > 0)
                        {
                            SBRElement sbrAddressDetailsCollection_POS_C_RP;
                            sbrAddressDetailsCollection_POS_C_RP = sbrAddressDetailsCollection_POS_C_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrAddressDetailsCollection_POS_C_RPTupleElementMap;
                            sbrAddressDetailsCollection_POS_C_RPTupleElementMap = sbrAddressDetailsCollection_POS_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.AddressDetailsCollection_POS_C_RPExists = true;
                            report.AddressDetailsCollection_POS_C_RPCount = sbrAddressDetailsCollection_POS_C_RPCollection.Count();
                            report.AddressDetailsCollection_POS_C_RPOccurrenceIndex = sbrAddressDetailsCollection_POS_C_RP.OccurrenceIndex + 1;
                            report.AddressDetailsCollection_POS_C_RPLastOccurrenceIndex = sbrAddressDetailsCollection_POS_C_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.OverseasAddress.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER114 = ToBoolean(currentValue.Value);
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER105 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER104 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line1.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER5 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line2.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER110 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line3.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER111 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line4.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER112 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.LocalityName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER6 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Postcode.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER8 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.StateOrTerritory.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER7 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.CountryName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER9 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Country.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER113 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrAddressDetailsCollection
                    if (validationMode) ValidateRPAddressDetailsCollection(errors, sbrAddressDetailsCollection, sbrRPContexts, report);
            
            
                    #region sbrOrganisationNameDetailsCollection
            
                    if (sbrOrganisationNameDetailsCollection != null)
                    {
                
                        IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection_MN_P_RPCollection;
                        sbrOrganisationNameDetailsCollection_MN_P_RPCollection = sbrOrganisationNameDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "P" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrOrganisationNameDetailsCollection_MN_P_RPCollection != null && sbrOrganisationNameDetailsCollection_MN_P_RPCollection.Count() > 0)
                        {
                            SBRElement sbrOrganisationNameDetailsCollection_MN_P_RP;
                            sbrOrganisationNameDetailsCollection_MN_P_RP = sbrOrganisationNameDetailsCollection_MN_P_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap;
                            sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap = sbrOrganisationNameDetailsCollection_MN_P_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.OrganisationNameDetailsCollection_MN_P_RPExists = true;
                            report.OrganisationNameDetailsCollection_MN_P_RPCount = sbrOrganisationNameDetailsCollection_MN_P_RPCollection.Count();
                            report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_P_RP.OccurrenceIndex + 1;
                            report.OrganisationNameDetailsCollection_MN_P_RPLastOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_P_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER107 = currentValue.Value;
                            }
                    
                            if (sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER106 = currentValue.Value;
                            }
                    
                            if (sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER10 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrOrganisationNameDetailsCollection
                    if (validationMode) ValidateRPOrganisationNameDetailsCollection1(errors, sbrOrganisationNameDetailsCollection, sbrRPContexts, report);
            
            
                    #region sbrAddressDetailsCollection
            
                    if (sbrAddressDetailsCollection != null)
                    {
                
                        IEnumerable<SBRElement> sbrAddressDetailsCollection_POS_P_RPCollection;
                        sbrAddressDetailsCollection_POS_P_RPCollection = sbrAddressDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "POS") &&
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "P" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrAddressDetailsCollection_POS_P_RPCollection != null && sbrAddressDetailsCollection_POS_P_RPCollection.Count() > 0)
                        {
                            SBRElement sbrAddressDetailsCollection_POS_P_RP;
                            sbrAddressDetailsCollection_POS_P_RP = sbrAddressDetailsCollection_POS_P_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrAddressDetailsCollection_POS_P_RPTupleElementMap;
                            sbrAddressDetailsCollection_POS_P_RPTupleElementMap = sbrAddressDetailsCollection_POS_P_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.AddressDetailsCollection_POS_P_RPExists = true;
                            report.AddressDetailsCollection_POS_P_RPCount = sbrAddressDetailsCollection_POS_P_RPCollection.Count();
                            report.AddressDetailsCollection_POS_P_RPOccurrenceIndex = sbrAddressDetailsCollection_POS_P_RP.OccurrenceIndex + 1;
                            report.AddressDetailsCollection_POS_P_RPLastOccurrenceIndex = sbrAddressDetailsCollection_POS_P_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.OverseasAddress.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER119 = ToBoolean(currentValue.Value);
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER109 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER108 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line1.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER11 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line2.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER115 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line3.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER116 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line4.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER117 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.LocalityName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER12 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Postcode.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER14 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.StateOrTerritory.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER13 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.CountryName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER15 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Country.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER118 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrAddressDetailsCollection
                    if (validationMode) ValidateRPAddressDetailsCollection1(errors, sbrAddressDetailsCollection, sbrRPContexts, report);
            
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Residency.CentralManagementAndControlOutsideAustralia.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER16 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER17 = ToBoolean(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrPeriodDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PeriodDetails", out currentValue))
                        sbrPeriodDetailsCollection = currentValue.Occurrences;
                    else
                        sbrPeriodDetailsCollection = null;
            
                    #region sbrPeriodDetailsCollection
            
                    if (sbrPeriodDetailsCollection != null)
                    {
                        report.PeriodDetailsCollectionExists = true;
                        report.PeriodDetailsCollectionCount = sbrPeriodDetailsCollection.Count();
                        if (sbrPeriodDetailsCollection.Count() > 0)
                        {
                    
                            IEnumerable<SBRElement> sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPCollection;
                            sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPCollection = sbrPeriodDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "Period.Type.Code" && te.Value == "ControlOutsideAustraliaPartPeriod" &&
                                sbrRPContexts.Contains(te.Context)));
                
                            report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection = new List<FTER2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP>();
                
                            for (int explicitTupleIndex = 0; explicitTupleIndex < sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPCollection.Count(); explicitTupleIndex++)
                            {
                                SBRElement sbrPeriodDetails;
                                sbrPeriodDetails = sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPCollection.ElementAt(explicitTupleIndex);
                
                                IDictionary<string, SBRElement> sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap;
                                sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap = sbrPeriodDetails.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                FTER2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP periodDetailsControlOutsideAustraliaPartPeriod = new FTER2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP();
                                report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection.Add(periodDetailsControlOutsideAustraliaPartPeriod);
                        
                                if (sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.Type.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) periodDetailsControlOutsideAustraliaPartPeriod.FTER88 = currentValue.Value;
                                }
                        
                                if (sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.Start.Date", out currentValue))
                                {
                                    if (!currentValue.IsNil) periodDetailsControlOutsideAustraliaPartPeriod.FTER18 = DateTime.Parse(currentValue.Value);
                                }
                        
                                if (sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.End.Date", out currentValue))
                                {
                                    if (!currentValue.IsNil) periodDetailsControlOutsideAustraliaPartPeriod.FTER19 = DateTime.Parse(currentValue.Value);
                                }
                            }
                        }
                    } 
                    #endregion End of Tuple sbrPeriodDetailsCollection
        
                    IEnumerable<SBRElement> sbrTrusteeCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Trustee", out currentValue))
                        sbrTrusteeCollection = currentValue.Occurrences;
                    else
                        sbrTrusteeCollection = null;
            
                    #region sbrTrusteeCollection
            
                    if (sbrTrusteeCollection != null)
                    {
                        report.TrusteeCollectionExists = true;
                        report.TrusteeCollectionCount = sbrTrusteeCollection.Count();
                        if (sbrTrusteeCollection != null && sbrTrusteeCollection.Count() > 0)
                        {
                            report.TrusteeCollection = new List<FTER2018.Trustee>();
                            for (int tupleIndex = 0; tupleIndex < sbrTrusteeCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrTrustee;
                                sbrTrustee = sbrTrusteeCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrTrusteeTupleElementMap;
                                sbrTrusteeTupleElementMap = sbrTrustee.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                FTER2018.Trustee trustee = new FTER2018.Trustee();
                                report.TrusteeCollection.Add(trustee);
                                trustee.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.TaxFileNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) trustee.FTER20 = currentValue.Value;
                                }
                        
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.TaxFileNumberInexistent.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) trustee.FTER21 = ToBoolean(currentValue.Value);
                                }
                    
                                IEnumerable<SBRElement> sbrTrustee_PersonNameDetailsCollection;
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails", out currentValue))
                                    sbrTrustee_PersonNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrTrustee_PersonNameDetailsCollection = null;
                        
                                #region sbrTrustee_PersonNameDetailsCollection
                        
                                if (sbrTrustee_PersonNameDetailsCollection != null)
                                {
                                    trustee.Trustee_PersonNameDetailsCollectionExists = true;
                                    trustee.Trustee_PersonNameDetailsCollectionCount = sbrTrustee_PersonNameDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPCollection;
                                    sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPCollection = sbrTrustee_PersonNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.PersonNameType.Code" && te.Value == "LGL") &&
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Usage.Code" && te.Value == "Contact") &&
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPCollection != null && sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RP;
                                        sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RP = sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap;
                                        sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap = sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPExists = true;
                                        trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPCount = sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPCollection.Count();
                                        trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex = sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RP.OccurrenceIndex + 1;
                                        trustee.Trustee_PersonNameDetailsCollection_LGL_Contact_C_RPLastOccurrenceIndex = sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.PersonNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER120 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Usage.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER121 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER122 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Title.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER22 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.NameSuffix.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER24 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.FamilyName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER23 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.GivenName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER25 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.OtherGivenName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER26 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrTrustee_PersonNameDetailsCollection
                                if (validationMode) ValidateRPTrustee_PersonNameDetailsCollection(errors, sbrTrustee_PersonNameDetailsCollection, sbrRPContexts, report);
                        
                    
                                IEnumerable<SBRElement> sbrTrustee_OrganisationNameDetailsCollection;
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails", out currentValue))
                                    sbrTrustee_OrganisationNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrTrustee_OrganisationNameDetailsCollection = null;
                        
                                #region sbrTrustee_OrganisationNameDetailsCollection
                        
                                if (sbrTrustee_OrganisationNameDetailsCollection != null)
                                {
                                    trustee.Trustee_OrganisationNameDetailsCollectionExists = true;
                                    trustee.Trustee_OrganisationNameDetailsCollectionCount = sbrTrustee_OrganisationNameDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection;
                                    sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection = sbrTrustee_OrganisationNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrTrustee_OrganisationNameDetailsCollection_MN_C_RP;
                                        sbrTrustee_OrganisationNameDetailsCollection_MN_C_RP = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                                        sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPExists = true;
                                        trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPCount = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                                        trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                                        trustee.Trustee_OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER132 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER133 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER27 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrTrustee_OrganisationNameDetailsCollection
                                if (validationMode) ValidateRPTrustee_OrganisationNameDetailsCollection(errors, sbrTrustee_OrganisationNameDetailsCollection, sbrRPContexts, report);
                        
                        
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.AustralianCompanyNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) trustee.FTER86 = currentValue.Value;
                                }
                        
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.AustralianRegisteredBodyNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) trustee.FTER29 = currentValue.Value;
                                }
                    
                                IEnumerable<SBRElement> sbrTrustee_AddressDetailsCollection;
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails", out currentValue))
                                    sbrTrustee_AddressDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrTrustee_AddressDetailsCollection = null;
                        
                                #region sbrTrustee_AddressDetailsCollection
                        
                                if (sbrTrustee_AddressDetailsCollection != null)
                                {
                                    trustee.Trustee_AddressDetailsCollectionExists = true;
                                    trustee.Trustee_AddressDetailsCollectionCount = sbrTrustee_AddressDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrTrustee_AddressDetailsCollection_POS_C_RPCollection;
                                    sbrTrustee_AddressDetailsCollection_POS_C_RPCollection = sbrTrustee_AddressDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "POS") &&
                                        ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrTrustee_AddressDetailsCollection_POS_C_RPCollection != null && sbrTrustee_AddressDetailsCollection_POS_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrTrustee_AddressDetailsCollection_POS_C_RP;
                                        sbrTrustee_AddressDetailsCollection_POS_C_RP = sbrTrustee_AddressDetailsCollection_POS_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap;
                                        sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap = sbrTrustee_AddressDetailsCollection_POS_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        trustee.Trustee_AddressDetailsCollection_POS_C_RPExists = true;
                                        trustee.Trustee_AddressDetailsCollection_POS_C_RPCount = sbrTrustee_AddressDetailsCollection_POS_C_RPCollection.Count();
                                        trustee.Trustee_AddressDetailsCollection_POS_C_RPOccurrenceIndex = sbrTrustee_AddressDetailsCollection_POS_C_RP.OccurrenceIndex + 1;
                                        trustee.Trustee_AddressDetailsCollection_POS_C_RPLastOccurrenceIndex = sbrTrustee_AddressDetailsCollection_POS_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.OverseasAddress.Indicator", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER126 = ToBoolean(currentValue.Value);
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Usage.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER128 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER127 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line1.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER30 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line2.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER31 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line3.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER123 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line4.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER124 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.LocalityName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER32 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Postcode.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER34 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.StateOrTerritory.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER33 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.CountryName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER35 = currentValue.Value;
                                        }
                                
                                        if (sbrTrustee_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Country.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trustee.FTER125 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrTrustee_AddressDetailsCollection
                                if (validationMode) ValidateRPTrustee_AddressDetailsCollection(errors, sbrTrustee_AddressDetailsCollection, sbrRPContexts, report);
                        
                        
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Residency.TaxPurposesPersonStatus.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) trustee.FTER36 = ToBoolean(currentValue.Value);
                                }
                        
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Residency.NonResidentFullPeriod.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) trustee.FTER37 = ToBoolean(currentValue.Value);
                                }
                    
                                IEnumerable<SBRElement> sbrTrustee_PeriodDetailsCollection;
                                if (sbrTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PeriodDetails", out currentValue))
                                    sbrTrustee_PeriodDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrTrustee_PeriodDetailsCollection = null;
                        
                                #region sbrTrustee_PeriodDetailsCollection
                        
                                if (sbrTrustee_PeriodDetailsCollection != null)
                                {
                                    trustee.Trustee_PeriodDetailsCollectionExists = true;
                                    trustee.Trustee_PeriodDetailsCollectionCount = sbrTrustee_PeriodDetailsCollection.Count();
                                    if (sbrTrustee_PeriodDetailsCollection.Count() > 0)
                                    {
                                
                                        IEnumerable<SBRElement> sbrTrustee_PeriodDetailsCollection_NonResidentPartPeriod_RPCollection;
                                        sbrTrustee_PeriodDetailsCollection_NonResidentPartPeriod_RPCollection = sbrTrustee_PeriodDetailsCollection.Where(
                                            ce => ce.ChildElements != null && 
                                            ce.ChildElements.Any(te => te.Name == "Period.Type.Code" && te.Value == "NonResidentPartPeriod" &&
                                            sbrRPContexts.Contains(te.Context)));
                            
                                        trustee.Trustee_PeriodDetailsNonResidentPartPeriodRPCollection = new List<FTER2018.Trustee.Trustee_PeriodDetailsNonResidentPartPeriodRP>();
                            
                                        for (int explicitTupleIndex = 0; explicitTupleIndex < sbrTrustee_PeriodDetailsCollection_NonResidentPartPeriod_RPCollection.Count(); explicitTupleIndex++)
                                        {
                                            SBRElement sbrTrustee_PeriodDetails;
                                            sbrTrustee_PeriodDetails = sbrTrustee_PeriodDetailsCollection_NonResidentPartPeriod_RPCollection.ElementAt(explicitTupleIndex);
                            
                                            IDictionary<string, SBRElement> sbrTrustee_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap;
                                            sbrTrustee_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap = sbrTrustee_PeriodDetails.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                            FTER2018.Trustee.Trustee_PeriodDetailsNonResidentPartPeriodRP periodDetailsNonResidentPartPeriod = new FTER2018.Trustee.Trustee_PeriodDetailsNonResidentPartPeriodRP();
                                            trustee.Trustee_PeriodDetailsNonResidentPartPeriodRPCollection.Add(periodDetailsNonResidentPartPeriod);
                                    
                                            if (sbrTrustee_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.Type.Code", out currentValue))
                                            {
                                                if (!currentValue.IsNil) periodDetailsNonResidentPartPeriod.FTER90 = currentValue.Value;
                                            }
                                    
                                            if (sbrTrustee_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.Start.Date", out currentValue))
                                            {
                                                if (!currentValue.IsNil) periodDetailsNonResidentPartPeriod.FTER38 = DateTime.Parse(currentValue.Value);
                                            }
                                    
                                            if (sbrTrustee_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.End.Date", out currentValue))
                                            {
                                                if (!currentValue.IsNil) periodDetailsNonResidentPartPeriod.FTER39 = DateTime.Parse(currentValue.Value);
                                            }
                                        }
                                    }
                                } 
                                #endregion End of Tuple sbrTrustee_PeriodDetailsCollection
                    } 
                    #endregion End of Tuple sbrTrusteeCollection
                        } 
                    } 
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Elections.FamilyTrustElectionStatus.Year", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER40 = int.Parse(currentValues[0].Value);
                    }
            
                    #region sbrPeriodDetailsCollection
            
                    if (sbrPeriodDetailsCollection != null)
                    {
                
                        IEnumerable<SBRElement> sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection;
                        sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection = sbrPeriodDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "Period.Type.Code" && te.Value == "SubstitutedAccountingPeriod" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection != null && sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.Count() > 0)
                        {
                            SBRElement sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RP;
                            sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RP = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap;
                            sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPExists = true;
                            report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCount = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.Count();
                            report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPOccurrenceIndex = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RP.OccurrenceIndex + 1;
                            report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPLastOccurrenceIndex = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.Type.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER89 = currentValue.Value;
                            }
                    
                            if (sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.Start.Date", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER41 = DateTime.Parse(currentValue.Value);
                            }
                    
                            if (sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.End.Date", out currentValue))
                            {
                                if (!currentValue.IsNil) report.FTER42 = DateTime.Parse(currentValue.Value);
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrPeriodDetailsCollection
                    if (validationMode) ValidateRPPeriodDetailsCollection1(errors, sbrPeriodDetailsCollection, sbrRPContexts, report);
            
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Elections.Commencement.Date", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER43 = DateTime.Parse(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrSpecifiedIndividualCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}SpecifiedIndividual", out currentValue))
                        sbrSpecifiedIndividualCollection = currentValue.Occurrences;
                    else
                        sbrSpecifiedIndividualCollection = null;
            
                    #region sbrSpecifiedIndividualCollection
            
                    if (sbrSpecifiedIndividualCollection != null)
                    {
                        report.SpecifiedIndividualCollectionExists = true;
                        report.SpecifiedIndividualCollectionCount = sbrSpecifiedIndividualCollection.Count();
                        SBRElement sbrSpecifiedIndividual;
                        sbrSpecifiedIndividual = sbrSpecifiedIndividualCollection.ElementAt(0);
                
                        IDictionary<string, SBRElement> sbrSpecifiedIndividualTupleElementMap;
                        sbrSpecifiedIndividualTupleElementMap = sbrSpecifiedIndividual.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.TaxFileNumber.Identifier", out currentValue))
                        {
                            if (!currentValue.IsNil) report.FTER44 = currentValue.Value;
                        }
                
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.TaxFileNumberInexistent.Indicator", out currentValue))
                        {
                            if (!currentValue.IsNil) report.FTER45 = ToBoolean(currentValue.Value);
                        }
                
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonDemographicDetails.Birth.Date", out currentValue))
                        {
                            if (!currentValue.IsNil) report.FTER46 = DateTime.Parse(currentValue.Value);
                        }
            
                        IEnumerable<SBRElement> sbrSpecifiedIndividual_PersonNameDetailsCollection;
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails", out currentValue))
                            sbrSpecifiedIndividual_PersonNameDetailsCollection = currentValue.Occurrences;
                        else
                            sbrSpecifiedIndividual_PersonNameDetailsCollection = null;
                
                        #region sbrSpecifiedIndividual_PersonNameDetailsCollection
                
                        if (sbrSpecifiedIndividual_PersonNameDetailsCollection != null)
                        {
                            report.SpecifiedIndividual_PersonNameDetailsCollectionExists = true;
                            report.SpecifiedIndividual_PersonNameDetailsCollectionCount = sbrSpecifiedIndividual_PersonNameDetailsCollection.Count();
                    
                            IEnumerable<SBRElement> sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPCollection;
                            sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPCollection = sbrSpecifiedIndividual_PersonNameDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "PersonNameDetails.PersonNameType.Code" && te.Value == "LGL") &&
                                ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Usage.Code" && te.Value == "Contact") &&
                                ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Currency.Code" && te.Value == "C" &&
                                sbrRPContexts.Contains(te.Context)));
                            if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPCollection != null && sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPCollection.Count() > 0)
                            {
                                SBRElement sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RP;
                                sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RP = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPCollection.ElementAt(0);
                    
                                IDictionary<string, SBRElement> sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap;
                                sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                    
                                report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPExists = true;
                                report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPCount = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPCollection.Count();
                                report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPOccurrenceIndex = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RP.OccurrenceIndex + 1;
                                report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPLastOccurrenceIndex = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPCollection.Last().OccurrenceIndex + 1;
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.PersonNameType.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER129 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Usage.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER130 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Currency.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER131 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Title.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER47 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.NameSuffix.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER51 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.FamilyName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER50 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.GivenName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER48 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.OtherGivenName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER49 = currentValue.Value;
                                }
                            }  
                        } 
                        #endregion End of Tuple sbrSpecifiedIndividual_PersonNameDetailsCollection
                        if (validationMode) ValidateRPSpecifiedIndividual_PersonNameDetailsCollection(errors, sbrSpecifiedIndividual_PersonNameDetailsCollection, sbrRPContexts, report);
                
            
                        IEnumerable<SBRElement> sbrSpecifiedIndividual_AddressDetailsCollection;
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails", out currentValue))
                            sbrSpecifiedIndividual_AddressDetailsCollection = currentValue.Occurrences;
                        else
                            sbrSpecifiedIndividual_AddressDetailsCollection = null;
                
                        #region sbrSpecifiedIndividual_AddressDetailsCollection
                
                        if (sbrSpecifiedIndividual_AddressDetailsCollection != null)
                        {
                            report.SpecifiedIndividual_AddressDetailsCollectionExists = true;
                            report.SpecifiedIndividual_AddressDetailsCollectionCount = sbrSpecifiedIndividual_AddressDetailsCollection.Count();
                    
                            IEnumerable<SBRElement> sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection;
                            sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection = sbrSpecifiedIndividual_AddressDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "RES") &&
                                ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "C" &&
                                sbrRPContexts.Contains(te.Context)));
                            if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection != null && sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection.Count() > 0)
                            {
                                SBRElement sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RP;
                                sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RP = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection.ElementAt(0);
                    
                                IDictionary<string, SBRElement> sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap;
                                sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                    
                                report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPExists = true;
                                report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPCount = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection.Count();
                                report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RP.OccurrenceIndex + 1;
                                report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPLastOccurrenceIndex = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection.Last().OccurrenceIndex + 1;
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.OverseasAddress.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER138 = ToBoolean(currentValue.Value);
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Usage.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER139 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Currency.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER140 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line1.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER52 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line2.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER134 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line3.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER135 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line4.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER136 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.LocalityName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER53 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Postcode.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER55 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.StateOrTerritory.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER54 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.CountryName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER56 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Country.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.FTER137 = currentValue.Value;
                                }
                            }  
                        } 
                        #endregion End of Tuple sbrSpecifiedIndividual_AddressDetailsCollection
                        if (validationMode) ValidateRPSpecifiedIndividual_AddressDetailsCollection(errors, sbrSpecifiedIndividual_AddressDetailsCollection, sbrRPContexts, report);
                
                    } 
                    #endregion End of Tuple sbrSpecifiedIndividualCollection
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Elections.Revocation.Date", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER57 = DateTime.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Elections.FamilyTrustRevocationStatusEffective.Year", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER96 = int.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Elections.NotificationOfElectionBeingRevoked.Year", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER58 = int.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.TaxFileNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER60 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.TaxFileNumberInexistent.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.FTER61 = ToBoolean(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrInterposedEntityCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}InterposedEntity", out currentValue))
                        sbrInterposedEntityCollection = currentValue.Occurrences;
                    else
                        sbrInterposedEntityCollection = null;
            
                    #region sbrInterposedEntityCollection
            
                    if (sbrInterposedEntityCollection != null)
                    {
                        report.InterposedEntityCollectionExists = true;
                        report.InterposedEntityCollectionCount = sbrInterposedEntityCollection.Count();
                        if (sbrInterposedEntityCollection != null && sbrInterposedEntityCollection.Count() > 0)
                        {
                            report.InterposedEntityCollection = new List<FTER2018.InterposedEntity>();
                            for (int tupleIndex = 0; tupleIndex < sbrInterposedEntityCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrInterposedEntity;
                                sbrInterposedEntity = sbrInterposedEntityCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrInterposedEntityTupleElementMap;
                                sbrInterposedEntityTupleElementMap = sbrInterposedEntity.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                FTER2018.InterposedEntity interposedEntity = new FTER2018.InterposedEntity();
                                report.InterposedEntityCollection.Add(interposedEntity);
                                interposedEntity.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrInterposedEntityTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Party.Type.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) interposedEntity.FTER62 = currentValue.Value;
                                }
                        
                                if (sbrInterposedEntityTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.TaxFileNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) interposedEntity.FTER64 = currentValue.Value;
                                }
                        
                                if (sbrInterposedEntityTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.TaxFileNumberInexistent.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) interposedEntity.FTER63 = ToBoolean(currentValue.Value);
                                }
                    
                                IEnumerable<SBRElement> sbrInterposedEntity_OrganisationNameDetailsCollection;
                                if (sbrInterposedEntityTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails", out currentValue))
                                    sbrInterposedEntity_OrganisationNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrInterposedEntity_OrganisationNameDetailsCollection = null;
                        
                                #region sbrInterposedEntity_OrganisationNameDetailsCollection
                        
                                if (sbrInterposedEntity_OrganisationNameDetailsCollection != null)
                                {
                                    interposedEntity.InterposedEntity_OrganisationNameDetailsCollectionExists = true;
                                    interposedEntity.InterposedEntity_OrganisationNameDetailsCollectionCount = sbrInterposedEntity_OrganisationNameDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPCollection;
                                    sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPCollection = sbrInterposedEntity_OrganisationNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RP;
                                        sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RP = sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                                        sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPExists = true;
                                        interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPCount = sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                                        interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                                        interposedEntity.InterposedEntity_OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER151 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER152 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER65 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrInterposedEntity_OrganisationNameDetailsCollection
                                if (validationMode) ValidateRPInterposedEntity_OrganisationNameDetailsCollection(errors, sbrInterposedEntity_OrganisationNameDetailsCollection, sbrRPContexts, report);
                        
                        
                                if (sbrInterposedEntityTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.AustralianCompanyNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) interposedEntity.FTER87 = currentValue.Value;
                                }
                        
                                if (sbrInterposedEntityTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Identifiers.AustralianRegisteredBodyNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) interposedEntity.FTER67 = currentValue.Value;
                                }
                    
                                IEnumerable<SBRElement> sbrInterposedEntity_AddressDetailsCollection;
                                if (sbrInterposedEntityTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails", out currentValue))
                                    sbrInterposedEntity_AddressDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrInterposedEntity_AddressDetailsCollection = null;
                        
                                #region sbrInterposedEntity_AddressDetailsCollection
                        
                                if (sbrInterposedEntity_AddressDetailsCollection != null)
                                {
                                    interposedEntity.InterposedEntity_AddressDetailsCollectionExists = true;
                                    interposedEntity.InterposedEntity_AddressDetailsCollectionCount = sbrInterposedEntity_AddressDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrInterposedEntity_AddressDetailsCollection_POS_C_RPCollection;
                                    sbrInterposedEntity_AddressDetailsCollection_POS_C_RPCollection = sbrInterposedEntity_AddressDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "POS") &&
                                        ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPCollection != null && sbrInterposedEntity_AddressDetailsCollection_POS_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrInterposedEntity_AddressDetailsCollection_POS_C_RP;
                                        sbrInterposedEntity_AddressDetailsCollection_POS_C_RP = sbrInterposedEntity_AddressDetailsCollection_POS_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap;
                                        sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap = sbrInterposedEntity_AddressDetailsCollection_POS_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPExists = true;
                                        interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPCount = sbrInterposedEntity_AddressDetailsCollection_POS_C_RPCollection.Count();
                                        interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPOccurrenceIndex = sbrInterposedEntity_AddressDetailsCollection_POS_C_RP.OccurrenceIndex + 1;
                                        interposedEntity.InterposedEntity_AddressDetailsCollection_POS_C_RPLastOccurrenceIndex = sbrInterposedEntity_AddressDetailsCollection_POS_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.OverseasAddress.Indicator", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER146 = ToBoolean(currentValue.Value);
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Usage.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER147 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER155 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line1.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER68 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line2.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER69 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line3.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER143 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line4.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER144 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.LocalityName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER70 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Postcode.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER72 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.StateOrTerritory.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER71 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.CountryName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER73 = currentValue.Value;
                                        }
                                
                                        if (sbrInterposedEntity_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Country.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) interposedEntity.FTER145 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrInterposedEntity_AddressDetailsCollection
                                if (validationMode) ValidateRPInterposedEntity_AddressDetailsCollection(errors, sbrInterposedEntity_AddressDetailsCollection, sbrRPContexts, report);
                        
                    } 
                    #endregion End of Tuple sbrInterposedEntityCollection
                        } 
                    } 
        
                    IEnumerable<SBRElement> sbrDeclarationOfTrusteeCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}DeclarationOfTrustee", out currentValue))
                        sbrDeclarationOfTrusteeCollection = currentValue.Occurrences;
                    else
                        sbrDeclarationOfTrusteeCollection = null;
            
                    #region sbrDeclarationOfTrusteeCollection
            
                    if (sbrDeclarationOfTrusteeCollection != null)
                    {
                        report.DeclarationOfTrusteeCollectionExists = true;
                        report.DeclarationOfTrusteeCollectionCount = sbrDeclarationOfTrusteeCollection.Count();
                        if (sbrDeclarationOfTrusteeCollection != null && sbrDeclarationOfTrusteeCollection.Count() > 0)
                        {
                            report.DeclarationOfTrusteeCollection = new List<FTER2018.DeclarationOfTrustee>();
                            for (int tupleIndex = 0; tupleIndex < sbrDeclarationOfTrusteeCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrDeclarationOfTrustee;
                                sbrDeclarationOfTrustee = sbrDeclarationOfTrusteeCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrDeclarationOfTrusteeTupleElementMap;
                                sbrDeclarationOfTrusteeTupleElementMap = sbrDeclarationOfTrustee.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                FTER2018.DeclarationOfTrustee declarationOfTrustee = new FTER2018.DeclarationOfTrustee();
                                report.DeclarationOfTrusteeCollection.Add(declarationOfTrustee);
                                declarationOfTrustee.OccurrenceIndex = tupleIndex + 1;
                    
                                IEnumerable<SBRElement> sbrDeclarationOfTrustee_PersonNameDetailsCollection;
                                if (sbrDeclarationOfTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails", out currentValue))
                                    sbrDeclarationOfTrustee_PersonNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrDeclarationOfTrustee_PersonNameDetailsCollection = null;
                        
                                #region sbrDeclarationOfTrustee_PersonNameDetailsCollection
                        
                                if (sbrDeclarationOfTrustee_PersonNameDetailsCollection != null)
                                {
                                    declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollectionExists = true;
                                    declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollectionCount = sbrDeclarationOfTrustee_PersonNameDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection;
                                    sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection = sbrDeclarationOfTrustee_PersonNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.PersonNameType.Code" && te.Value == "LGL") &&
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Usage.Code" && te.Value == "DeclarationSignatory") &&
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection != null && sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP;
                                        sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP = sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap;
                                        sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap = sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists = true;
                                        declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCount = sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.Count();
                                        declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex = sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP.OccurrenceIndex + 1;
                                        declarationOfTrustee.DeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPLastOccurrenceIndex = sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.PersonNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER148 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Usage.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER149 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER150 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Title.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER74 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.NameSuffix.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER76 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.FamilyName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER75 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.GivenName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER77 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.OtherGivenName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER78 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrDeclarationOfTrustee_PersonNameDetailsCollection
                                if (validationMode) ValidateRPDeclarationOfTrustee_PersonNameDetailsCollection(errors, sbrDeclarationOfTrustee_PersonNameDetailsCollection, sbrRPContexts, report);
                        
                    
                                IEnumerable<SBRElement> sbrDeclarationOfTrustee_OrganisationNameDetailsCollection;
                                if (sbrDeclarationOfTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails", out currentValue))
                                    sbrDeclarationOfTrustee_OrganisationNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrDeclarationOfTrustee_OrganisationNameDetailsCollection = null;
                        
                                #region sbrDeclarationOfTrustee_OrganisationNameDetailsCollection
                        
                                if (sbrDeclarationOfTrustee_OrganisationNameDetailsCollection != null)
                                {
                                    declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollectionExists = true;
                                    declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollectionCount = sbrDeclarationOfTrustee_OrganisationNameDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection;
                                    sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection = sbrDeclarationOfTrustee_OrganisationNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RP;
                                        sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RP = sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                                        sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPExists = true;
                                        declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPCount = sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                                        declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                                        declarationOfTrustee.DeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER153 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER154 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER79 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrDeclarationOfTrustee_OrganisationNameDetailsCollection
                                if (validationMode) ValidateRPDeclarationOfTrustee_OrganisationNameDetailsCollection(errors, sbrDeclarationOfTrustee_OrganisationNameDetailsCollection, sbrRPContexts, report);
                        
                    
                                IEnumerable<SBRElement> sbrDeclarationOfTrustee_DeclarationCollection;
                                if (sbrDeclarationOfTrusteeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration", out currentValue))
                                    sbrDeclarationOfTrustee_DeclarationCollection = currentValue.Occurrences;
                                else
                                    sbrDeclarationOfTrustee_DeclarationCollection = null;
                        
                                #region sbrDeclarationOfTrustee_DeclarationCollection
                        
                                if (sbrDeclarationOfTrustee_DeclarationCollection != null)
                                {
                                    declarationOfTrustee.DeclarationOfTrustee_DeclarationCollectionExists = true;
                                    declarationOfTrustee.DeclarationOfTrustee_DeclarationCollectionCount = sbrDeclarationOfTrustee_DeclarationCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPCollection;
                                    sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPCollection = sbrDeclarationOfTrustee_DeclarationCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "Declaration.StatementType.Code" && te.Value == "TrueAndCorrect" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPCollection != null && sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RP;
                                        sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RP = sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPTupleElementMap;
                                        sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPTupleElementMap = sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        declarationOfTrustee.DeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPExists = true;
                                        declarationOfTrustee.DeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPCount = sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPCollection.Count();
                                        declarationOfTrustee.DeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex = sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RP.OccurrenceIndex + 1;
                                        declarationOfTrustee.DeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPLastOccurrenceIndex = sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.StatementType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER99 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.StatementAccepted.Indicator", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER169 = ToBoolean(currentValue.Value);
                                        }
                                
                                        if (sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.Statement.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER101 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.Signature.Date", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER168 = DateTime.Parse(currentValue.Value);
                                        }
                                
                                        if (sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.SignatoryIdentifier.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrustee.FTER100 = currentValue.Value;
                                        }
                            
                                        IEnumerable<SBRElement> sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection;
                                        if (sbrDeclarationOfTrustee_DeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName", out currentValue))
                                            sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection = currentValue.Occurrences;
                                        else
                                            sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection = null;
                                
                                        #region sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection
                                
                                        if (sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection != null)
                                        {
                                            declarationOfTrustee.DeclarationOfTrustee_Declaration_PersonUnstructuredNameCollectionExists = true;
                                            declarationOfTrustee.DeclarationOfTrustee_Declaration_PersonUnstructuredNameCollectionCount = sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection.Count();
                                    
                                            IEnumerable<SBRElement> sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection;
                                            sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection = sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection.Where(
                                                ce => ce.ChildElements != null && 
                                                ce.ChildElements.Any(te => te.Name == "PersonUnstructuredName.Usage.Code" && te.Value == "DeclarationSignatory" &&
                                                sbrRPContexts.Contains(te.Context)));
                                            if (sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection != null && sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Count() > 0)
                                            {
                                                SBRElement sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP;
                                                sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP = sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.ElementAt(0);
                                    
                                                IDictionary<string, SBRElement> sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap;
                                                sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap = sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                                    
                                                declarationOfTrustee.DeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPExists = true;
                                                declarationOfTrustee.DeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCount = sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Count();
                                                declarationOfTrustee.DeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex = sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP.OccurrenceIndex + 1;
                                                declarationOfTrustee.DeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPLastOccurrenceIndex = sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Last().OccurrenceIndex + 1;
                                        
                                                if (sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.Usage.Code", out currentValue))
                                                {
                                                    if (!currentValue.IsNil) declarationOfTrustee.FTER167 = currentValue.Value;
                                                }
                                        
                                                if (sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.FullName.Text", out currentValue))
                                                {
                                                    if (!currentValue.IsNil) declarationOfTrustee.FTER166 = currentValue.Value;
                                                }
                                            }  
                                        } 
                                        #endregion End of Tuple sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection
                                        if (validationMode) ValidateRPDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection(errors, sbrDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection, sbrRPContexts, report);
                                
                                    }  
                                } 
                                #endregion End of Tuple sbrDeclarationOfTrustee_DeclarationCollection
                                if (validationMode) ValidateRPDeclarationOfTrustee_DeclarationCollection(errors, sbrDeclarationOfTrustee_DeclarationCollection, sbrRPContexts, report);
                        
                    } 
                    #endregion End of Tuple sbrDeclarationOfTrusteeCollection
                        } 
                    } 
                } 
            } 
            #endregion End of Context sbrRPContext
            return report;

        } // Of Consume Method
        #region ValidateINTDeclaration_PersonUnstructuredNameCollection
        protected virtual void ValidateINTDeclaration_PersonUnstructuredNameCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTExists = true;

                    SBRElement FTER164Value = null;
                    string FTER164 = null;
                    SBRElement FTER165Value = null;
                    string FTER165 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.Usage.Code", out FTER164Value))
                    {
                        if (!FTER164Value.IsNil) FTER164 = FTER164Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.FullName.Text", out FTER165Value))
                    {
                        if (!FTER165Value.IsNil) FTER165 = FTER165Value.Value;
                    }
        
                    #region VR.ATO.GEN.436279

        /*  VR.ATO.GEN.436279

        Legacy Rule Format:
            WHERE IN TUPLE (personunstructuredname1.xx.xx:PersonUnstructuredName) IN TUPLE(declaration2.xx.xx:Declaration)
            IF pyde.xx.xx:PersonUnstructuredName.Usage.Code <> "DeclarationSignatory"
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^PersonUnstructuredName <> NULL) AND (^FTER164 <> 'DeclarationSignatory')

        Data Elements:

        ^FTER164 = INT:Declaration:PersonUnstructuredName:PersonUnstructuredName.Usage.Code WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']

        ^PersonUnstructuredName = INT:Declaration:PersonUnstructuredName
        */
                    assertion = (Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTExists != false && FTER164 != @"DeclarationSignatory");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.436279", Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration name must have a usage code of 'DeclarationSignatory'",
                            Location = FTER165Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonUnstructuredName.FullName.Text" : DeriveSbrElementLocation(FTER165Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.436279"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER164", Value = FTER164 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateINTDeclarationCollection
        protected virtual void ValidateINTDeclarationCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER159Value = null;
                    string FTER159 = null;
                    SBRElement FTER160Value = null;
                    bool? FTER160 = null;
                    SBRElement FTER161Value = null;
                    string FTER161 = null;
                    SBRElement FTER162Value = null;
                    DateTime? FTER162 = null;
                    SBRElement FTER163Value = null;
                    string FTER163 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.StatementType.Code", out FTER159Value))
                    {
                        if (!FTER159Value.IsNil) FTER159 = FTER159Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.StatementAccepted.Indicator", out FTER160Value))
                    {
                        if (!FTER160Value.IsNil) FTER160 = ToBoolean(FTER160Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.Statement.Text", out FTER161Value))
                    {
                        if (!FTER161Value.IsNil) FTER161 = FTER161Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.Signature.Date", out FTER162Value))
                    {
                        if (!FTER162Value.IsNil) FTER162 = DateTime.Parse(FTER162Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.SignatoryIdentifier.Text", out FTER163Value))
                    {
                        if (!FTER163Value.IsNil) FTER163 = FTER163Value.Value;
                    }
        
                    #region VR.ATO.GEN.438061

        /*  VR.ATO.GEN.438061
            If an Intermediary is present, Intermediary declaration statement type code must be supplied and must be 
            “TrueAndCorrect”.

        Legacy Rule Format:
            IF INT<>NULL AND INT:pyin.xx.xx:Declaration.StatementType.Code <> "TrueAndCorrect"
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            ^INT <> NULL AND ^FTER159 <> 'TrueAndCorrect'

        Data Elements:

        ^FTER159 = INT:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code='TrueAndCorrect']

        ^INT = INT
        */
                    assertion = (report.INTExists != false && FTER159 != @"TrueAndCorrect");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.438061", Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration statement type code is required.",
                            LongDescription = @"If an Intermediary is present, Intermediary declaration statement type code must be supplied and must be ""TrueAndCorrect"".",
                            Location = FTER159Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:Declaration.StatementType.Code" : DeriveSbrElementLocation(FTER159Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438061"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER159", Value = FTER159 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateINTPersonUnstructuredNameCollection
        protected virtual void ValidateINTPersonUnstructuredNameCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool PersonUnstructuredNameCollection_Contact_INTExists = true;

                    SBRElement FTER156Value = null;
                    string FTER156 = null;
                    SBRElement FTER82Value = null;
                    string FTER82 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.Usage.Code", out FTER156Value))
                    {
                        if (!FTER156Value.IsNil) FTER156 = FTER156Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.FullName.Text", out FTER82Value))
                    {
                        if (!FTER82Value.IsNil) FTER82 = FTER82Value.Value;
                    }
        
                    #region VR.ATO.GEN.432391

        /*  VR.ATO.GEN.432391

        Legacy Rule Format:
            WHERE IN TUPLE (xbrli\personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF INT:pyde.xx.xx:PersonUnstructuredName.Usage.Code <> "Contact"
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^PersonUnstructuredName <> NULL) AND (^FTER156 <> 'Contact')

        Data Elements:

        ^FTER156 = INT:PersonUnstructuredName:PersonUnstructuredName.Usage.Code WHERE [PersonUnstructuredName.Usage.Code='Contact']

        ^PersonUnstructuredName = INT:PersonUnstructuredName
        */
                    assertion = (PersonUnstructuredNameCollection_Contact_INTExists != false && FTER156 != @"Contact");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.432391", Severity = ProcessMessageSeverity.Error,
                            Description = @"Tax agent contact name must have a Usage code of 'Contact'",
                            LongDescription = @"The unstructured person name for the tax agent or intermediary contact must include a Usage Code of 'Contact'. Please contact your software vendor in order to rectify this error.",
                            Location = FTER82Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonUnstructuredName.FullName.Text" : DeriveSbrElementLocation(FTER82Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.432391"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER156", Value = FTER156 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateINTElectronicContactTelephoneCollection
        protected virtual void ValidateINTElectronicContactTelephoneCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool ElectronicContactTelephoneCollection_03_INTExists = true;

                    SBRElement FTER157Value = null;
                    string FTER157 = null;

                    SBRElement FTER158Value = null;
                    string FTER158 = null;
                    SBRElement FTER83Value = null;
                    string FTER83 = null;
                    SBRElement FTER84Value = null;
                    string FTER84 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}ElectronicContact.Telephone.Usage.Code", out FTER157Value))
                    {
                        if (!FTER157Value.IsNil) FTER157 = FTER157Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}ElectronicContact.Telephone.ServiceLine.Code", out FTER158Value))
                    {
                        if (!FTER158Value.IsNil) FTER158 = FTER158Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}ElectronicContact.Telephone.Area.Code", out FTER83Value))
                    {
                        if (!FTER83Value.IsNil) FTER83 = FTER83Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}ElectronicContact.Telephone.Minimal.Number", out FTER84Value))
                    {
                        if (!FTER84Value.IsNil) FTER84 = FTER84Value.Value;
                    }
        
                    #region VR.ATO.GEN.500131

        /*  VR.ATO.GEN.500131
            Telephone must include usage code '03' and service line code '01' or '02'.

        Legacy Rule Format:
            WHERE IN TUPLE(electroniccontacttelephone1.xx.xx:ElectronicContactTelephone)
            IF pyde.xx.xx:ElectronicContact.Telephone.Usage.Code <> "03" OR pyde.xx.xx:ElectronicContact.Telephone.ServiceLine.Code <> SET ("01", "02")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^ElectronicContactTelephone <> NULL) AND (^FTER157 <> '03' OR NotInSet(^FTER158, '"01", "02"'))

        Data Elements:

        ^FTER157 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.Usage.Code WHERE [ElectronicContact.Telephone.Usage.Code='03']

        ^FTER158 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.ServiceLine.Code WHERE [ElectronicContact.Telephone.Usage.Code='03']

        ^ElectronicContactTelephone = INT:ElectronicContactTelephone
        */
                    assertion = (ElectronicContactTelephoneCollection_03_INTExists != false && (FTER157 != @"03" || !(IsMatch(FTER158, @"^(01|02)$",RegexOptions.IgnoreCase))));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500131", Severity = ProcessMessageSeverity.Error,
                            Description = @"Telephone must include usage code '03' and service line code '01' or '02'.",
                            Location = FTER84Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:ElectronicContact.Telephone.Minimal.Number" : DeriveSbrElementLocation(FTER84Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500131"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER157", Value = FTER157 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER158", Value = FTER158 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPOrganisationNameDetailsCollection
        protected virtual void ValidateRPOrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER103Value = null;
                    string FTER103 = null;
                    SBRElement FTER102Value = null;
                    string FTER102 = null;
                    SBRElement FTER4Value = null;
                    string FTER4 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalNameType.Code", out FTER103Value))
                    {
                        if (!FTER103Value.IsNil) FTER103 = FTER103Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.Currency.Code", out FTER102Value))
                    {
                        if (!FTER102Value.IsNil) FTER102 = FTER102Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalName.Text", out FTER4Value))
                    {
                        if (!FTER4Value.IsNil) FTER4 = FTER4Value.Value;
                    }
        
                    #region VR.ATO.FTER.408009

        /*  VR.ATO.FTER.408009
            The name of trust must have Type Code - 'MN'

        Legacy Rule Format:
            WHERE IN TUPLE(xbrli\organisationname2.xx.xx:OrganisationNameDetails)
            IF pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> "MN" 
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            ^FTER103 <> 'MN'

        Data Elements:

        ^FTER103 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
        */
                    assertion = (FTER103 != @"MN");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408009", Severity = ProcessMessageSeverity.Error,
                            Description = @"The name of trust must have Type Code - 'MN'",
                            LongDescription = @"The name of trust must have Type Code - 'MN' (Main Name)",
                            Location = FTER4Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalName.Text" : DeriveSbrElementLocation(FTER4Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408009"} },
                        };
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPAddressDetailsCollection
        protected virtual void ValidateRPAddressDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPOrganisationNameDetailsCollection1
        protected virtual void ValidateRPOrganisationNameDetailsCollection1(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPAddressDetailsCollection1
        protected virtual void ValidateRPAddressDetailsCollection1(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPTrustee_PersonNameDetailsCollection
        protected virtual void ValidateRPTrustee_PersonNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER120Value = null;
                    string FTER120 = null;

                    SBRElement FTER121Value = null;
                    string FTER121 = null;

                    SBRElement FTER122Value = null;
                    string FTER122 = null;
                    SBRElement FTER22Value = null;
                    string FTER22 = null;
                    SBRElement FTER24Value = null;
                    string FTER24 = null;
                    SBRElement FTER23Value = null;
                    string FTER23 = null;
                    SBRElement FTER25Value = null;
                    string FTER25 = null;
                    SBRElement FTER26Value = null;
                    string FTER26 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.PersonNameType.Code", out FTER120Value))
                    {
                        if (!FTER120Value.IsNil) FTER120 = FTER120Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Usage.Code", out FTER121Value))
                    {
                        if (!FTER121Value.IsNil) FTER121 = FTER121Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Currency.Code", out FTER122Value))
                    {
                        if (!FTER122Value.IsNil) FTER122 = FTER122Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Title.Text", out FTER22Value))
                    {
                        if (!FTER22Value.IsNil) FTER22 = FTER22Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.NameSuffix.Text", out FTER24Value))
                    {
                        if (!FTER24Value.IsNil) FTER24 = FTER24Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.FamilyName.Text", out FTER23Value))
                    {
                        if (!FTER23Value.IsNil) FTER23 = FTER23Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.GivenName.Text", out FTER25Value))
                    {
                        if (!FTER25Value.IsNil) FTER25 = FTER25Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.OtherGivenName.Text", out FTER26Value))
                    {
                        if (!FTER26Value.IsNil) FTER26 = FTER26Value.Value;
                    }
        
                    #region VR.ATO.FTER.408157

        /*  VR.ATO.FTER.408157
            Trustee Name Type Code must be 'LGL', Currency Code -  'C' and Usage Code - 'Contact'

        Legacy Rule Format:
            WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:Trustee/personstructuredname3.xx.xx:PersonNameDetails)
            IF (PersonNameDetails.PersonNameType.Code <> "LGL") OR (PersonNameDetails.Usage.Code <> "Contact") OR (PersonNameDetails.Currency.Code <> "C") 
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^FTER120 <> 'LGL') OR (^FTER121 <> 'Contact') OR (^FTER122 <> 'C')

        Data Elements:

        ^FTER120 = RP:Trustee:PersonNameDetails:PersonNameDetails.PersonNameType.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']

        ^FTER121 = RP:Trustee:PersonNameDetails:PersonNameDetails.Usage.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']

        ^FTER122 = RP:Trustee:PersonNameDetails:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
        */
                    assertion = (FTER120 != @"LGL" || FTER121 != @"Contact" || FTER122 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.432435", Severity = ProcessMessageSeverity.Error,
                            Description = @"Individual name Type code, Currency code and Usage code must be present and correct",
                            LongDescription = @"Each person name must have the Name Type code 'LGL' (legal), Currency Code 'C' (current) and Usage code 'Contact'. Please contact your software vendor in order to rectify this error",
                            Location = FTER23Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.FamilyName.Text" : DeriveSbrElementLocation(FTER23Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408157"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER120", Value = FTER120 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER121", Value = FTER121 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER122", Value = FTER122 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPTrustee_OrganisationNameDetailsCollection
        protected virtual void ValidateRPTrustee_OrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER132Value = null;
                    string FTER132 = null;

                    SBRElement FTER133Value = null;
                    string FTER133 = null;
                    SBRElement FTER27Value = null;
                    string FTER27 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalNameType.Code", out FTER132Value))
                    {
                        if (!FTER132Value.IsNil) FTER132 = FTER132Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.Currency.Code", out FTER133Value))
                    {
                        if (!FTER133Value.IsNil) FTER133 = FTER133Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalName.Text", out FTER27Value))
                    {
                        if (!FTER27Value.IsNil) FTER27 = FTER27Value.Value;
                    }
        
                    #region VR.ATO.FTER.408007

        /*  VR.ATO.FTER.408007
            Trustee Name Type Code must be 'MN' and Currency Code must be 'C'

        Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:Trustee\organisationname2.xx.xx:OrganisationNameDetails) 
            IF (OrganisationNameDetails.OrganisationalNameType.Code <> "MN") OR (OrganisationNameDetails.Currency.Code <> "C")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^FTER132 <> 'MN') OR (^FTER133 <> 'C')

        Data Elements:

        ^FTER132 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

        ^FTER133 = RP:Trustee:OrganisationNameDetails:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
        */
                    assertion = (FTER132 != @"MN" || FTER133 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408007", Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee Non-individual Name Type Code must be 'MN' and Currency Code must be 'C'",
                            LongDescription = @"Trustee Non-individual Name Type Code must be 'MN' (Main Name) and Currency Code must be 'C' (Current)",
                            Location = FTER27Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalName.Text" : DeriveSbrElementLocation(FTER27Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408007"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER132", Value = FTER132 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER133", Value = FTER133 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPTrustee_AddressDetailsCollection
        protected virtual void ValidateRPTrustee_AddressDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER128Value = null;
                    string FTER128 = null;

                    SBRElement FTER127Value = null;
                    string FTER127 = null;
                    SBRElement FTER126Value = null;
                    bool? FTER126 = null;
                    SBRElement FTER30Value = null;
                    string FTER30 = null;
                    SBRElement FTER31Value = null;
                    string FTER31 = null;
                    SBRElement FTER123Value = null;
                    string FTER123 = null;
                    SBRElement FTER124Value = null;
                    string FTER124 = null;
                    SBRElement FTER32Value = null;
                    string FTER32 = null;
                    SBRElement FTER34Value = null;
                    string FTER34 = null;
                    SBRElement FTER33Value = null;
                    string FTER33 = null;
                    SBRElement FTER35Value = null;
                    string FTER35 = null;
                    SBRElement FTER125Value = null;
                    string FTER125 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Usage.Code", out FTER128Value))
                    {
                        if (!FTER128Value.IsNil) FTER128 = FTER128Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Currency.Code", out FTER127Value))
                    {
                        if (!FTER127Value.IsNil) FTER127 = FTER127Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.OverseasAddress.Indicator", out FTER126Value))
                    {
                        if (!FTER126Value.IsNil) FTER126 = ToBoolean(FTER126Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line1.Text", out FTER30Value))
                    {
                        if (!FTER30Value.IsNil) FTER30 = FTER30Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line2.Text", out FTER31Value))
                    {
                        if (!FTER31Value.IsNil) FTER31 = FTER31Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line3.Text", out FTER123Value))
                    {
                        if (!FTER123Value.IsNil) FTER123 = FTER123Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line4.Text", out FTER124Value))
                    {
                        if (!FTER124Value.IsNil) FTER124 = FTER124Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.LocalityName.Text", out FTER32Value))
                    {
                        if (!FTER32Value.IsNil) FTER32 = FTER32Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Postcode.Text", out FTER34Value))
                    {
                        if (!FTER34Value.IsNil) FTER34 = FTER34Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.StateOrTerritory.Code", out FTER33Value))
                    {
                        if (!FTER33Value.IsNil) FTER33 = FTER33Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.CountryName.Text", out FTER35Value))
                    {
                        if (!FTER35Value.IsNil) FTER35 = FTER35Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Country.Code", out FTER125Value))
                    {
                        if (!FTER125Value.IsNil) FTER125 = FTER125Value.Value;
                    }
        
                    #region VR.ATO.FTER.408075

        /*  VR.ATO.FTER.408075
            Trustee postal address Usage Code must be 'POS' and Currency Code must be 'C'.

        Legacy Rule Format:
            WHERE IN TUPLE(fter.0002.lodge.req.xx.xx:Trustee\address2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Usage.Code <> "POS") OR (pyde.xx.xx:AddressDetails.Currency.Code <> "C")
              RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^FTER128 <> 'POS') OR (^FTER127 <> 'C')

        Data Elements:

        ^FTER128 = RP:Trustee:AddressDetails:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']

        ^FTER127 = RP:Trustee:AddressDetails:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
        */
                    assertion = (FTER128 != @"POS" || FTER127 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408075", Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee postal address Usage Code must be 'POS' and Currency Code must be 'C'.",
                            LongDescription = @"Trustee postal address Usage Code must be 'POS' (Postal address) and Currency Code must be 'C' (Current address).",
                            Location = FTER30Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Line1.Text" : DeriveSbrElementLocation(FTER30Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408075"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER128", Value = FTER128 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER127", Value = FTER127 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPPeriodDetailsCollection1
        protected virtual void ValidateRPPeriodDetailsCollection1(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER89Value = null;
                    string FTER89 = null;
                    SBRElement FTER41Value = null;
                    DateTime? FTER41 = null;
                    SBRElement FTER42Value = null;
                    DateTime? FTER42 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.Type.Code", out FTER89Value))
                    {
                        if (!FTER89Value.IsNil) FTER89 = FTER89Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.Start.Date", out FTER41Value))
                    {
                        if (!FTER41Value.IsNil) FTER41 = DateTime.Parse(FTER41Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Period.End.Date", out FTER42Value))
                    {
                        if (!FTER42Value.IsNil) FTER42 = DateTime.Parse(FTER42Value.Value);
                    }
        
                    #region VR.ATO.FTER.408091

        /*  VR.ATO.FTER.408091
            Number of Substituted Accounting Period details supplied cannot exceed 1

        Legacy Rule Format:
            IF COUNT (TUPLE(xbrli\perioddetails1.xx.xx.PeriodDetails WHERE TUPLE ELEMENT EXPLICIT pyin.xx.xx:Period.Type.Code = "SubstitutedAccountingPeriod")) > 1
               RETURN VALIDATION  MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            Count(^FTER89) > 1

        Data Elements:

        ^FTER89 = RP:PeriodDetails:Period.Type.Code WHERE [Period.Type.Code='SubstitutedAccountingPeriod']
        */
                    assertion = (!alreadyChecked.Contains("VR.ATO.FTER.408091") && (report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCount > 1));
                    if (assertion)
                    {
                        alreadyChecked.Add("VR.ATO.FTER.408091");
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408091", Severity = ProcessMessageSeverity.Error,
                            Description = @"Number of separate Substituted Accounting Periods must not exceed 1",
                            Location = DeriveSbrElementLocation(tupleElement) + "/tns:Period.Start.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408091"} },
                        };
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPSpecifiedIndividual_PersonNameDetailsCollection
        protected virtual void ValidateRPSpecifiedIndividual_PersonNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER129Value = null;
                    string FTER129 = null;
                    SBRElement FTER130Value = null;
                    string FTER130 = null;
                    SBRElement FTER131Value = null;
                    string FTER131 = null;
                    SBRElement FTER47Value = null;
                    string FTER47 = null;
                    SBRElement FTER51Value = null;
                    string FTER51 = null;
                    SBRElement FTER50Value = null;
                    string FTER50 = null;
                    SBRElement FTER48Value = null;
                    string FTER48 = null;
                    SBRElement FTER49Value = null;
                    string FTER49 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.PersonNameType.Code", out FTER129Value))
                    {
                        if (!FTER129Value.IsNil) FTER129 = FTER129Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Usage.Code", out FTER130Value))
                    {
                        if (!FTER130Value.IsNil) FTER130 = FTER130Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Currency.Code", out FTER131Value))
                    {
                        if (!FTER131Value.IsNil) FTER131 = FTER131Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Title.Text", out FTER47Value))
                    {
                        if (!FTER47Value.IsNil) FTER47 = FTER47Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.NameSuffix.Text", out FTER51Value))
                    {
                        if (!FTER51Value.IsNil) FTER51 = FTER51Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.FamilyName.Text", out FTER50Value))
                    {
                        if (!FTER50Value.IsNil) FTER50 = FTER50Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.GivenName.Text", out FTER48Value))
                    {
                        if (!FTER48Value.IsNil) FTER48 = FTER48Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.OtherGivenName.Text", out FTER49Value))
                    {
                        if (!FTER49Value.IsNil) FTER49 = FTER49Value.Value;
                    }
        
                    #region VR.ATO.FTER.408089

        /*  VR.ATO.FTER.408089
            A name for the Specified Individual must be present (one only)

        Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:SpecifiedIndividual)
            IF COUNT (TUPLE personstructuredname3.xx.xx.PersonNameDetails WHERE TUPLE ELEMENT EXPLICIT PersonNameDetails.PersonNameType.Code = "LGL" AND WHERE TUPLE ELEMENT EXPLICIT PersonNameDetails.Usage.Code = "Contact" AND WHERE TUPLE ELEMENT EXPLICIT PersonNameDetails.Currency.Code = "C") <> 1
                RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            Count(^FTER129) <> 1

        Data Elements:

        ^FTER129 = RP:SpecifiedIndividual:PersonNameDetails:PersonNameDetails.PersonNameType.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact'].[PersonNameDetails.Currency.Code='C']
        */
                    assertion = (!alreadyChecked.Contains("VR.ATO.FTER.408089") && (report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Contact_C_RPCount != 1));
                    if (assertion)
                    {
                        alreadyChecked.Add("VR.ATO.FTER.408089");
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408089", Severity = ProcessMessageSeverity.Error,
                            Description = @"A name for the Specified Individual must be present (one only)",
                            LongDescription = @"A name for the Specified Individual must be present (one only)",
                            Location = DeriveSbrElementLocation(tupleElement) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408089"} },
                        };
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPSpecifiedIndividual_AddressDetailsCollection
        protected virtual void ValidateRPSpecifiedIndividual_AddressDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER139Value = null;
                    string FTER139 = null;

                    SBRElement FTER140Value = null;
                    string FTER140 = null;
                    SBRElement FTER138Value = null;
                    bool? FTER138 = null;
                    SBRElement FTER52Value = null;
                    string FTER52 = null;
                    SBRElement FTER134Value = null;
                    string FTER134 = null;
                    SBRElement FTER135Value = null;
                    string FTER135 = null;
                    SBRElement FTER136Value = null;
                    string FTER136 = null;
                    SBRElement FTER53Value = null;
                    string FTER53 = null;
                    SBRElement FTER55Value = null;
                    string FTER55 = null;
                    SBRElement FTER54Value = null;
                    string FTER54 = null;
                    SBRElement FTER56Value = null;
                    string FTER56 = null;
                    SBRElement FTER137Value = null;
                    string FTER137 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Usage.Code", out FTER139Value))
                    {
                        if (!FTER139Value.IsNil) FTER139 = FTER139Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Currency.Code", out FTER140Value))
                    {
                        if (!FTER140Value.IsNil) FTER140 = FTER140Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.OverseasAddress.Indicator", out FTER138Value))
                    {
                        if (!FTER138Value.IsNil) FTER138 = ToBoolean(FTER138Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line1.Text", out FTER52Value))
                    {
                        if (!FTER52Value.IsNil) FTER52 = FTER52Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line2.Text", out FTER134Value))
                    {
                        if (!FTER134Value.IsNil) FTER134 = FTER134Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line3.Text", out FTER135Value))
                    {
                        if (!FTER135Value.IsNil) FTER135 = FTER135Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line4.Text", out FTER136Value))
                    {
                        if (!FTER136Value.IsNil) FTER136 = FTER136Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.LocalityName.Text", out FTER53Value))
                    {
                        if (!FTER53Value.IsNil) FTER53 = FTER53Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Postcode.Text", out FTER55Value))
                    {
                        if (!FTER55Value.IsNil) FTER55 = FTER55Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.StateOrTerritory.Code", out FTER54Value))
                    {
                        if (!FTER54Value.IsNil) FTER54 = FTER54Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.CountryName.Text", out FTER56Value))
                    {
                        if (!FTER56Value.IsNil) FTER56 = FTER56Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Country.Code", out FTER137Value))
                    {
                        if (!FTER137Value.IsNil) FTER137 = FTER137Value.Value;
                    }
        
                    #region VR.ATO.FTER.408108

        /*  VR.ATO.FTER.408108
            Specified Individual Address Usage Code must be 'RES' and Currency Code must be 'C'

        Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:SpecifiedIndividual\address2.xx.xx.AddressDetails)
            IF (AddressDetails.Usage.Code <> "RES") OR (AddressDetails.Currency.Code <> "C")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^FTER139 <> 'RES') OR (^FTER140 <> 'C')

        Data Elements:

        ^FTER139 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']

        ^FTER140 = RP:SpecifiedIndividual:AddressDetails:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
        */
                    assertion = (FTER139 != @"RES" || FTER140 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408108", Severity = ProcessMessageSeverity.Error,
                            Description = @"Specified Individual Address Usage Code must be 'RES' and Currency Code must be 'C'",
                            LongDescription = @"Specified Individual Address Usage Code must be 'RES' (Residential Address) and Currency Code must be 'C' (Current address).",
                            Location = FTER52Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Line1.Text" : DeriveSbrElementLocation(FTER52Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408108"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER139", Value = FTER139 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER140", Value = FTER140 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPInterposedEntity_OrganisationNameDetailsCollection
        protected virtual void ValidateRPInterposedEntity_OrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER151Value = null;
                    string FTER151 = null;

                    SBRElement FTER152Value = null;
                    string FTER152 = null;
                    SBRElement FTER65Value = null;
                    string FTER65 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalNameType.Code", out FTER151Value))
                    {
                        if (!FTER151Value.IsNil) FTER151 = FTER151Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.Currency.Code", out FTER152Value))
                    {
                        if (!FTER152Value.IsNil) FTER152 = FTER152Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalName.Text", out FTER65Value))
                    {
                        if (!FTER65Value.IsNil) FTER65 = FTER65Value.Value;
                    }
        
                    #region VR.ATO.FTER.408120

        /*  VR.ATO.FTER.408120
            Interposed Entity Name Type Code must be 'MN' and Currency Code must be 'C'

        Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:InterposedEntity\organisationname2.xx.xx:OrganisationNameDetails) 
            IF (OrganisationNameDetails.OrganisationalNameType.Code <> "MN") OR (OrganisationNameDetails.Currency.Code <> "C")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^FTER151 <> 'MN') OR (^FTER152 <> 'C')

        Data Elements:

        ^FTER151 = RP:InterposedEntity:OrganisationNameDetails:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

        ^FTER152 = RP:InterposedEntity:OrganisationNameDetails:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
        */
                    assertion = (FTER151 != @"MN" || FTER152 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408120", Severity = ProcessMessageSeverity.Error,
                            Description = @"Interposed Entity Name Type Code must be 'MN' and Currency Code must be 'C'",
                            LongDescription = @"Interposed Entity Name Type Code must be 'MN' (Main Name) and Currency Code must be 'C' (Current name).",
                            Location = FTER65Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalName.Text" : DeriveSbrElementLocation(FTER65Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408120"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER151", Value = FTER151 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER152", Value = FTER152 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPInterposedEntity_AddressDetailsCollection
        protected virtual void ValidateRPInterposedEntity_AddressDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER147Value = null;
                    string FTER147 = null;

                    SBRElement FTER155Value = null;
                    string FTER155 = null;
                    SBRElement FTER146Value = null;
                    bool? FTER146 = null;
                    SBRElement FTER68Value = null;
                    string FTER68 = null;
                    SBRElement FTER69Value = null;
                    string FTER69 = null;
                    SBRElement FTER143Value = null;
                    string FTER143 = null;
                    SBRElement FTER144Value = null;
                    string FTER144 = null;
                    SBRElement FTER70Value = null;
                    string FTER70 = null;
                    SBRElement FTER72Value = null;
                    string FTER72 = null;
                    SBRElement FTER71Value = null;
                    string FTER71 = null;
                    SBRElement FTER73Value = null;
                    string FTER73 = null;
                    SBRElement FTER145Value = null;
                    string FTER145 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Usage.Code", out FTER147Value))
                    {
                        if (!FTER147Value.IsNil) FTER147 = FTER147Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Currency.Code", out FTER155Value))
                    {
                        if (!FTER155Value.IsNil) FTER155 = FTER155Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.OverseasAddress.Indicator", out FTER146Value))
                    {
                        if (!FTER146Value.IsNil) FTER146 = ToBoolean(FTER146Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line1.Text", out FTER68Value))
                    {
                        if (!FTER68Value.IsNil) FTER68 = FTER68Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line2.Text", out FTER69Value))
                    {
                        if (!FTER69Value.IsNil) FTER69 = FTER69Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line3.Text", out FTER143Value))
                    {
                        if (!FTER143Value.IsNil) FTER143 = FTER143Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Line4.Text", out FTER144Value))
                    {
                        if (!FTER144Value.IsNil) FTER144 = FTER144Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.LocalityName.Text", out FTER70Value))
                    {
                        if (!FTER70Value.IsNil) FTER70 = FTER70Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Postcode.Text", out FTER72Value))
                    {
                        if (!FTER72Value.IsNil) FTER72 = FTER72Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.StateOrTerritory.Code", out FTER71Value))
                    {
                        if (!FTER71Value.IsNil) FTER71 = FTER71Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.CountryName.Text", out FTER73Value))
                    {
                        if (!FTER73Value.IsNil) FTER73 = FTER73Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}AddressDetails.Country.Code", out FTER145Value))
                    {
                        if (!FTER145Value.IsNil) FTER145 = FTER145Value.Value;
                    }
        
                    #region VR.ATO.FTER.408123

        /*  VR.ATO.FTER.408123
            Interposed Entity address Usage Code must be 'POS' and Currency Code must be 'C'

        Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:InterposedEntity\address2.xx.xx.AddressDetails)
            IF (AddressDetails.Usage.Code <> "POS") OR (AddressDetails.Currency.Code <> "C")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^FTER147 <> 'POS') OR (^FTER155 <> 'C')

        Data Elements:

        ^FTER147 = RP:InterposedEntity:AddressDetails:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']

        ^FTER155 = RP:InterposedEntity:AddressDetails:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
        */
                    assertion = (FTER147 != @"POS" || FTER155 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408123", Severity = ProcessMessageSeverity.Error,
                            Description = @"Interposed Entity address Usage Code must be 'POS' and Currency Code must be 'C'",
                            LongDescription = @"Interposed Entity address Usage Code must be 'POS' (Postal address) and Currency Code must be 'C' (Current address).",
                            Location = FTER68Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Line1.Text" : DeriveSbrElementLocation(FTER68Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408123"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER147", Value = FTER147 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER155", Value = FTER155 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPDeclarationOfTrustee_PersonNameDetailsCollection
        protected virtual void ValidateRPDeclarationOfTrustee_PersonNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER148Value = null;
                    string FTER148 = null;

                    SBRElement FTER149Value = null;
                    string FTER149 = null;

                    SBRElement FTER150Value = null;
                    string FTER150 = null;
                    SBRElement FTER74Value = null;
                    string FTER74 = null;
                    SBRElement FTER76Value = null;
                    string FTER76 = null;
                    SBRElement FTER75Value = null;
                    string FTER75 = null;
                    SBRElement FTER77Value = null;
                    string FTER77 = null;
                    SBRElement FTER78Value = null;
                    string FTER78 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.PersonNameType.Code", out FTER148Value))
                    {
                        if (!FTER148Value.IsNil) FTER148 = FTER148Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Usage.Code", out FTER149Value))
                    {
                        if (!FTER149Value.IsNil) FTER149 = FTER149Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Currency.Code", out FTER150Value))
                    {
                        if (!FTER150Value.IsNil) FTER150 = FTER150Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.Title.Text", out FTER74Value))
                    {
                        if (!FTER74Value.IsNil) FTER74 = FTER74Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.NameSuffix.Text", out FTER76Value))
                    {
                        if (!FTER76Value.IsNil) FTER76 = FTER76Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.FamilyName.Text", out FTER75Value))
                    {
                        if (!FTER75Value.IsNil) FTER75 = FTER75Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.GivenName.Text", out FTER77Value))
                    {
                        if (!FTER77Value.IsNil) FTER77 = FTER77Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonNameDetails.OtherGivenName.Text", out FTER78Value))
                    {
                        if (!FTER78Value.IsNil) FTER78 = FTER78Value.Value;
                    }
        
                    #region VR.ATO.FTER.408131

        /*  VR.ATO.FTER.408131
            Trustee declaration Name Type Code must be 'LGL', Usage Code  'DeclarationSignatory' and Currency Code 'C'.

        Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:DeclarationOfTrustee\personstructuredname3.xx.xx:PersonNameDetails) 
            IF (PersonNameDetails.PersonNameType.Code <> "LGL") OR (PersonNameDetails.Usage.Code <> "DeclarationSignatory") OR (PersonNameDetails.Currency.Code <> "C")
              RETURN VALIDATION MESSAGE 
            ENDIF
    
        Technical Business Rule Format:
            (^FTER148 <> 'LGL') OR (^FTER149 <> 'DeclarationSignatory') OR (^FTER150 <> 'C')

        Data Elements:

        ^FTER148 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.PersonNameType.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']

        ^FTER149 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.Usage.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']

        ^FTER150 = RP:DeclarationOfTrustee:PersonNameDetails:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
        */
                    assertion = (FTER148 != @"LGL" || FTER149 != @"DeclarationSignatory" || FTER150 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408131", Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee declaration name must be Type -'LGL', Usage - 'DeclarationSignatory', Currency - 'C'",
                            LongDescription = @"Each trustee declaration individual name must include a Name Type Code of 'LGL', a Usage Code of  'DeclarationSignatory' and a Currency Code of 'C'",
                            Location = FTER75Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.FamilyName.Text" : DeriveSbrElementLocation(FTER75Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408131"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER148", Value = FTER148 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER149", Value = FTER149 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER150", Value = FTER150 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPDeclarationOfTrustee_OrganisationNameDetailsCollection
        protected virtual void ValidateRPDeclarationOfTrustee_OrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER153Value = null;
                    string FTER153 = null;

                    SBRElement FTER154Value = null;
                    string FTER154 = null;
                    SBRElement FTER79Value = null;
                    string FTER79 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalNameType.Code", out FTER153Value))
                    {
                        if (!FTER153Value.IsNil) FTER153 = FTER153Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.Currency.Code", out FTER154Value))
                    {
                        if (!FTER154Value.IsNil) FTER154 = FTER154Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}OrganisationNameDetails.OrganisationalName.Text", out FTER79Value))
                    {
                        if (!FTER79Value.IsNil) FTER79 = FTER79Value.Value;
                    }
        
                    #region VR.ATO.FTER.408137

        /*  VR.ATO.FTER.408137
            Non-individual declaration Name Type Code must be 'MN' and Currency Code must be 'C'

        Legacy Rule Format:
            WHERE IN TUPLE (fter.0002.lodge.req.xx.xx:DeclarationOfTrustee\organisationname2.xx.xx:OrganisationNameDetails) 
            IF (OrganisationNameDetails.OrganisationalNameType.Code <> "MN") OR (OrganisationNameDetails.Currency.Code <> "C")
              RETURN VALIDATION MESSAGE 
            ENDIF
    
        Technical Business Rule Format:
            (^FTER153 <> 'MN') OR (^FTER154 <> 'C')

        Data Elements:

        ^FTER153 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']

        ^FTER154 = RP:DeclarationOfTrustee:OrganisationNameDetails:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
        */
                    assertion = (FTER153 != @"MN" || FTER154 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408137", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual declaration Name Type Code must be 'MN' and Currency Code must be 'C'",
                            Location = FTER79Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalName.Text" : DeriveSbrElementLocation(FTER79Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408137"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER153", Value = FTER153 });
        
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER154", Value = FTER154 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection
        protected virtual void ValidateRPDeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                    bool DeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPExists = true;

                    SBRElement FTER167Value = null;
                    string FTER167 = null;
                    SBRElement FTER166Value = null;
                    string FTER166 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.Usage.Code", out FTER167Value))
                    {
                        if (!FTER167Value.IsNil) FTER167 = FTER167Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}PersonUnstructuredName.FullName.Text", out FTER166Value))
                    {
                        if (!FTER166Value.IsNil) FTER166 = FTER166Value.Value;
                    }
        
                    #region VR.ATO.GEN.436279

        /*  VR.ATO.GEN.436279

        Legacy Rule Format:
            WHERE IN TUPLE (personunstructuredname1.xx.xx:PersonUnstructuredName) IN TUPLE(declaration2.xx.xx:Declaration)
            IF pyde.xx.xx:PersonUnstructuredName.Usage.Code <> "DeclarationSignatory"
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^PersonUnstructuredName <> NULL) AND (^FTER167 <> 'DeclarationSignatory')

        Data Elements:

        ^FTER167 = RP:DeclarationOfTrustee:Declaration:PersonUnstructuredName:PersonUnstructuredName.Usage.Code WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']

        ^PersonUnstructuredName = RP:DeclarationOfTrustee:Declaration:PersonUnstructuredName
        */
                    assertion = (DeclarationOfTrustee_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPExists != false && FTER167 != @"DeclarationSignatory");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.436279", Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration name must have a usage code of 'DeclarationSignatory'",
                            Location = FTER166Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonUnstructuredName.FullName.Text" : DeriveSbrElementLocation(FTER166Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.436279"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER167", Value = FTER167 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPDeclarationOfTrustee_DeclarationCollection
        protected virtual void ValidateRPDeclarationOfTrustee_DeclarationCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, FTER2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement FTER99Value = null;
                    string FTER99 = null;
                    SBRElement FTER169Value = null;
                    bool? FTER169 = null;
                    SBRElement FTER101Value = null;
                    string FTER101 = null;
                    SBRElement FTER168Value = null;
                    DateTime? FTER168 = null;
                    SBRElement FTER100Value = null;
                    string FTER100 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.StatementType.Code", out FTER99Value))
                    {
                        if (!FTER99Value.IsNil) FTER99 = FTER99Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.StatementAccepted.Indicator", out FTER169Value))
                    {
                        if (!FTER169Value.IsNil) FTER169 = ToBoolean(FTER169Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.Statement.Text", out FTER101Value))
                    {
                        if (!FTER101Value.IsNil) FTER101 = FTER101Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.Signature.Date", out FTER168Value))
                    {
                        if (!FTER168Value.IsNil) FTER168 = DateTime.Parse(FTER168Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/fter}Declaration.SignatoryIdentifier.Text", out FTER100Value))
                    {
                        if (!FTER100Value.IsNil) FTER100 = FTER100Value.Value;
                    }
        
                    #region VR.ATO.FTER.408163

        /*  VR.ATO.FTER.408163
            Declaration Statement Type Code must be either 'HardCopy' 'TrueAndCorrect'

        Legacy Rule Format:
            WHERE IN TUPLE(fter.000.lodge.req.xx.xx:DeclarationOfTrustee\declaration2.xx.xx:Declaration)
            IF (pyin.xx.xx:Declaration.StatementType.Code <> NULLORBLANK)
            AND (pyin.xx.xx:Declaration.StatementType.Code  <> SET ("HardCopy", "TrueAndCorrect" ))
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^FTER99 <> BLANK) AND NotInSet(^FTER99, '"HardCopy", "TrueAndCorrect"')

        Data Elements:

        ^FTER99 = RP:DeclarationOfTrustee:Declaration:Declaration.StatementType.Code WHERE [Declaration.StatementType.Code='TrueAndCorrect']
        */
                    assertion = (string.IsNullOrWhiteSpace(FTER99) != true && !(IsMatch(FTER99, @"^(HardCopy|TrueAndCorrect)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.FTER.408163", Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee declaration statement type code must be either 'TrueAndCorrect' or 'HardCopy'",
                            Location = FTER99Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:Declaration.StatementType.Code" : DeriveSbrElementLocation(FTER99Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.FTER.408163"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FTER99", Value = FTER99 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

    } // Of Class
} // Of Namespace

